C Corrected version of CONTINS.f with correct ionization potentials
C for the REE elements, as well as corrected for the computation of
C 3d ions. 13.04.1998
       SUBROUTINE CONTOP(IWHERE)
C
C  THIS SUBROUTINE FILLS COMMON BLOCK /OPAC/
C  WITH THE CONTINUOUS OPACITY IN THE CENTER OF
C  THE SPECTRAL INTERVAL (IWHERE=1). IF IWHERE=0, OPACITY
C  IS CALCULATED FOR STANDARD WAVELENGTH.
C
C   AUTHOR: N.Piskunov
C
C   LAST UPDATE: January 12, 1992
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
c      dimension XNFEU(MOSIZE,4)
c      dimension ANSWER(MOSIZE,3)
C
C  IF MOTYPE = 0 - KURUZC TYPE MODEL WITH RHOX AS A DEPTH PARAMETER
C            = 1 - DEPTH PARAMETER IS TAUSTD
C
C  IWHERE    = 0 - IS FOR STANDARD WAVELENGTH
C            > 0 - IS FOR BLUE AND RED ENDS OF SPECTAL INTERVAL
C
      IF(IWHERE.EQ.0) THEN
C
C  CALCULATE CONTINUOUS OPACITY IN STANDARD WAVELENGTH
C
        FREQ=2.997925E18/WLSTD
        FREQLG=LOG(FREQ)
        DO 1 I=1,NRHOX
        TKEV(I)=8.6171E-5*T(I)
        TK(I)=1.38054E-16*T(I)
        HKT(I)=6.6256E-27/TK(I)
        TLOG(I)=LOG(T(I))
        EHVKT(I)=EXP(-FREQ*HKT(I))
        FREQ15=FREQ*1.E-15
        STIM(I)=1.-EHVKT(I)
   1    BNU(I)=1.47439E-2*FREQ15**3*EHVKT(I)/STIM(I)
        CALL ALAM(0)
      ELSE
C
C  CALCULATE CONTINUOUS OPACITY IN BOTH SIDES OF SPECTRAL INTERVAL
C
        DO 3 IWL=1,2
        IF(IWL.EQ.1) WLCONT=WFIRST
        IF(IWL.EQ.2) WLCONT=WLAST
        FREQ=2.997925E18/WLCONT
        FREQLG=LOG(FREQ)
        DO 2 I=1,NRHOX
        TKEV(I)=8.6171E-5*T(I)
        TK(I)=1.38054E-16*T(I)
        HKT(I)=6.6256E-27/TK(I)
        TLOG(I)=LOG(T(I))
        EHVKT(I)=EXP(-FREQ*HKT(I))
        FREQ15=FREQ/1.E15
        STIM(I)=1.-EHVKT(I)
c        CALL PFSAHA(I,58,3,13,ANSWER)
c        write(*,'(F10.2,7E10.3)') T(I),XNE(I),(answer(I,II),II=1,3)
   2    BNU(I)=1.47439E-2*FREQ15**3*EHVKT(I)/STIM(I)
        CALL ALAM(IWL)
   3    CONTINUE
      END IF
C
c      CALL POPS(63.03,12,XNFEU)
c      do 4 i=1,NRHOX
c   4  write(*,'(A2,2X,4E12.5)') ELEMEN(63),(XNFEU(i,j),j=1,4)
C
      RETURN
      END

      SUBROUTINE ALAM(ITYPE)
C
C  THIS SUBROUTINE COMPUTES CONTINUOUS OPACITY USING
C  KURUCZ's ATLAS-9 SUBROUTINES.
C
C  ITYPE  = 0 - OPACITY AT STANDARD WAVELENGTH
C         = 1 - BLUE END OF SPECTRAL INTERVAL
C         = 2 - RED END OF SPECTRAL INTERVAL
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      IF(IFOP(1).EQ.1) CALL POPS(1.01,11,XNFPH)
      IF(IFOP(5)+IFOP(6)+IFOP(7).GT.0) CALL POPS(2.02,11,XNFPHE)
C
C  CLEAR OPACITY ACCUMULATORS
C
      DO 1 J=1,NRHOX
      AHYD(J)=0.
      AH2P(J)=0.
      AHMIN(J)=0.
      SIGH(J)=0.
      AHE1(J)=0.
      AHE2(J)=0.
      AHEMIN(J)=0.
      SIGHE(J)=0.
      ACOOL(J)=0.
      ALUKE(J)=0.
      AHOT(J)=0.
      SIGEL(J)=0.
      SIGH2(J)=0.
   1  CONTINUE
      IF(IFOP(1).EQ.1) CALL HOP
      IF(IFOP(2).EQ.1) CALL H2PLOP
      IF(IFOP(3).EQ.1) CALL HMINOP
      IF(IFOP(4).EQ.1) CALL HRAYOP
      IF(IFOP(5).EQ.1) CALL HE1OP
      IF(IFOP(6).EQ.1) CALL HE2OP
      IF(IFOP(7).EQ.1) CALL HEMIOP
      IF(IFOP(8).EQ.1) CALL HERAOP
      IF(IFOP(9).EQ.1) CALL COOLOP
      IF(IFOP(10).EQ.1) CALL LUKEOP
      IF(IFOP(11).EQ.1) CALL HOTOP
      IF(IFOP(12).EQ.1) CALL ELECOP
      IF(IFOP(13).EQ.1) CALL H2RAOP
C
C  CALCULATE THE TOTAL CONTINUOUS OPACITY
C
      DO 3 J=1,NRHOX
      A=AH2P(J)+AHE1(J)+AHE2(J)+AHEMIN(J)+ACOOL(J)+ALUKE(J)+AHOT(J)
      B=A+AHYD(J)+AHMIN(J)+SIGH(J)+SIGHE(J)+SIGEL(J)+SIGH2(J)
      IF(ITYPE.EQ.0) THEN
        COPSTD(J)=B
      ELSE IF(ITYPE.EQ.1) THEN
        COPBLU(J)=B
      ELSE IF(ITYPE.EQ.2) THEN
        COPRED(J)=B
      END IF
c      write(*,'(I2,F10.1,13E11.4)') J,T(J),AH2P(J),AHE1(J),AHE2(J),
c     *   AHEMIN(J),ACOOL(J),ALUKE(J),AHOT(J),AHYD(J),AHMIN(J)
c      write(*,'(I2,F10.1,6E11.4)') J,T(J),B,AHYD(J),AHMIN(J),
c     *   AH2P(J),AHE1(J),AHE2(J)
   3  CONTINUE
c      stop
C
      RETURN
      END

      SUBROUTINE POPS(CODE,MODE,NUMBER)
C
C  THIS SUBROUTINE IS BASED ON KURUCZ ATLAS-6 PROGRAM
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
      REAL NUMBER(MOSIZE,*)
C

      IF(CODE.EQ.0) RETURN
      IZ=CODE
      NION=(CODE-FLOAT(IZ))*100.+1.5
      DO 1 J=1,NRHOX
      CALL PFSAHA(J,IZ,NION,MODE,NUMBER)
C
C     PFSAHA RETURNS IONIZATION FRACTIONS OR IONIZATION FRACTIONS/
C     PARTITION FUNCTIONS SO CONVERTED TO NUMBER DENSITIES
C
      NNN=NION
      IF(MODE.LT.10) NNN=1
      DO 1 IONN=1,NNN
      NUMBER(J,IONN)=NUMBER(J,IONN)*XNA(J)*ABUND(IZ)
   1  CONTINUE
C
      RETURN
      END

      SUBROUTINE PFSAHA(J,IZ,NION,MODE,ANSWER)
C
C     MODE 1 RETURNS IONIZATION FRACTION/PARTITION FUNCTION
C     MODE 2 RETURNS IONIZATION FRACTION
C     MODE 3 RETURNS PARTITION FUNCTION
C     MODE 4 RETURNS NUMBER OF ELECTRONS PRODUCED
C     MODE + 10 RETURN ALL IONS TO NION. MODE ALONE RETURN NION ONLY
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
      DOUBLE PRECISION F(6)
      REAL ANSWER(MOSIZE,NION),IP(6),PART(6),POTLO(6),SCALE(4),
     *     FRCT(IONSIZ),POTI(IONSIZ)
      INTEGER LOCZ(29)
      INTEGER NNNPFN(6,369)
      INTEGER NNN01(54),NNN02(54),NNN03(54),NNN04(54),NNN05(54)
      INTEGER NNN06(54),NNN07(54),NNN08(54),NNN09(54),NNN10(54)
      INTEGER NNN11(54),NNN12(54),NNN13(54),NNN14(54),NNN15(54)
      INTEGER NNN16(54),NNN17(54),NNN18(54),NNN19(54),NNN20(54)
      INTEGER NNN21(54),NNN22(54),NNN23(54),NNN24(54),NNN25(54)
      INTEGER NNN26(54),NNN27(54),NNN28(54),NNN29(54),NNN30(54)
      INTEGER NNN31(54),NNN32(54),NNN33(54),NNN34(54),NNN35(54)
      INTEGER NNN36(54),NNN37(54),NNN38(54),NNN39(54),NNN40(12)
      INTEGER NNN67(96)
      EQUIVALENCE (NNNPFN(1,  1),NNN01(1)),(NNNPFN(1, 10),NNN02(1))
      EQUIVALENCE (NNNPFN(1, 19),NNN03(1)),(NNNPFN(1, 28),NNN04(1))
      EQUIVALENCE (NNNPFN(1, 37),NNN05(1)),(NNNPFN(1, 46),NNN06(1))
      EQUIVALENCE (NNNPFN(1, 55),NNN07(1)),(NNNPFN(1, 64),NNN08(1))
      EQUIVALENCE (NNNPFN(1, 73),NNN09(1)),(NNNPFN(1, 82),NNN10(1))
      EQUIVALENCE (NNNPFN(1, 91),NNN11(1)),(NNNPFN(1,100),NNN12(1))
      EQUIVALENCE (NNNPFN(1,109),NNN13(1)),(NNNPFN(1,118),NNN14(1))
      EQUIVALENCE (NNNPFN(1,127),NNN15(1)),(NNNPFN(1,136),NNN16(1))
      EQUIVALENCE (NNNPFN(1,145),NNN17(1)),(NNNPFN(1,154),NNN18(1))
      EQUIVALENCE (NNNPFN(1,163),NNN19(1)),(NNNPFN(1,172),NNN20(1))
      EQUIVALENCE (NNNPFN(1,181),NNN21(1)),(NNNPFN(1,190),NNN22(1))
      EQUIVALENCE (NNNPFN(1,199),NNN23(1)),(NNNPFN(1,208),NNN24(1))
      EQUIVALENCE (NNNPFN(1,217),NNN25(1)),(NNNPFN(1,226),NNN26(1))
      EQUIVALENCE (NNNPFN(1,235),NNN27(1)),(NNNPFN(1,244),NNN28(1))
      EQUIVALENCE (NNNPFN(1,253),NNN29(1)),(NNNPFN(1,262),NNN30(1))
      EQUIVALENCE (NNNPFN(1,271),NNN31(1)),(NNNPFN(1,280),NNN32(1))
      EQUIVALENCE (NNNPFN(1,289),NNN33(1)),(NNNPFN(1,298),NNN34(1))
      EQUIVALENCE (NNNPFN(1,307),NNN35(1)),(NNNPFN(1,316),NNN36(1))
      EQUIVALENCE (NNNPFN(1,325),NNN37(1)),(NNNPFN(1,334),NNN38(1))
      EQUIVALENCE (NNNPFN(1,343),NNN39(1)),(NNNPFN(1,352),NNN40(1))
      EQUIVALENCE (NNNPFN(1,354),NNN67(1))
      SAVE NNNPFN,LOCZ,SCALE
C      ( 1)( 2)   ( 3)( 4)   ( 5)( 6)   ( 7)( 8)   ( 9)(10)   ( IP ) G  REF
      DATA NNN01/
     1 200020001, 200020011, 201620881, 231228281, 378953411,  1359502, D+F 1.00
     2 100010001, 100010001, 100010001, 100010001, 100010001,  1359500, G   1.01
     3 100010001, 100010011, 102111241, 145022061, 363059451,  2458104, D+F 2.00
     4 200020001, 200020071, 208524971, 382669341, 128222452,  5440302, D+F 2.01
     5 100010001, 100010001, 100010001, 100010001, 100010001,  5440300, G   2.02
     6 200020011, 201220481, 212922881, 258731081, 394251691,   538901, D+F 3.00
     7 100010001, 100010201, 126225521,  67216512, 351165562,  7561907, D+F 3.01
     8 200020001, 200020211, 227936571,  69610342, 137217102, 12241800, D+F 3.02
     9 100010001, 100010001, 100010001, 100010001, 100010001, 12241800/ G   3.03
      DATA NNN02/
     1 100010051, 104311441, 131615641, 190623681, 298037691,   931900, AEL 4.00
     2 200120231, 211422771, 249627631, 309034911, 398545051,  1820600, AEL 4.01
     3 100010001, 100010201, 126225521,  67216512, 351165562, 15385000, AEL 4.02
     4 200020001, 200020011, 201220661, 223426161, 332644691, 21765700, AEL 4.03
     5 600060001, 600560281, 608761991, 637466191, 693973361,   829500, AEL 5.00
     6 100310831, 132016901, 214226411, 315736741, 419147071,  2514900, AEL 5.01
     7 200721061, 233526401, 297533311, 369040481, 440747651,  3792000, AEL 5.02
     8 100010001, 100010001, 100010001, 100010001, 100010001, 25929800, G   5.03
     9 893292271,  96110042, 105311262, 126315202, 196126432,  1125508/ D+F 6.00
      DATA NNN03/
     1 595060251, 620865751, 713280191,  95712292, 167623542,  2437501, D+F 6.01
     2 105513201, 180324851, 341851341,  88416332, 296550722,  4787101, D+F 6.02
     3 204922771, 262630421, 350941931, 494556971, 644872001,  6447600, D+F 6.03
     4 403141851, 457051681, 594071181,  92913362, 203331152,  1452915, D+F 7.00
     5 919899541, 107211512, 124914302, 182526232, 403762662,  2959202, D+F 7.01
     6 596862721, 684177081,  88110342, 128317062, 239334312,  4742501, D+F 7.02
     7 112816481, 240733751, 462068491, 116419932, 283736822,  7744900, D+F 7.03
     8 210124681, 293634211, 391145791, 539862151, 703178471,  9786200, D+F 7.04
     9 874789691, 924795711,  99410492, 115213492, 169022242,  1361307/ D+F 8.00
      DATA NNN04/
     1 424151091, 622874781,  91312832, 221842502,  79914013,  3510711, D+F 8.01
     2  95610702, 118113032, 149619922, 329761642, 101914173,  5488500, D+F 8.02
     3 603567171, 775391141, 106612482, 143716252, 181420032,  7739300, D+F 8.03
     4 124420321, 306943181, 606281181, 101712232, 142916342, 11387300, D+F 8.04
     5 215026541, 323137551, 421546491, 508255151, 594863811, 13807900, AEL 8.05
     6 575958511, 589859231, 595860671, 636470031, 815199581,  1741802, D+F 9.00
     7 900296401, 102610802, 113912542, 152921152, 318348952,  3498003, D+F 9.01
     8 469162651, 791295541, 121419552, 402686872, 154822203,  6264500, D+F 9.02
     9  99511422, 129214572, 170523002, 320140922, 498458762,  8713900/ D+F 9.03
      DATA NNN05/
     1 615472711,  87710602, 127215002, 172919582, 218624152, 11421300, D+F 9.04
     2 135324181, 377252001, 661580261,  94410852, 122613672, 15711700, AEL 9.05
     3 100010001, 100010051, 105313051, 210239461,  74013022,  2155808, D+F10.00
     4 580158751, 591759741, 642687101, 159332652,  64111533,  4106907, D+F10.01
     5  93510272, 110411662, 127116062, 257647882,  75110223,  6350000, D+F10.02
     6 529774371,  94611322, 135816202, 188221442, 240626682,  9701900, D+F10.03
     7 103312152, 140616092, 181320182, 222224262, 263128352, 12630000, AEL10.04
     8 629178711,  98311802, 136715512, 173619202, 210422892, 15790900, AEL10.05
     9 200020001, 200320211, 207322131, 253031421, 417657451,   513802/ D+F11.00
      DATA NNN06/
     1 100010001, 100010161, 119621261,  50711872, 246445382,  4728901, D+F11.01
     2 580158751, 591860351,  71813142, 321968812, 106014333,  7165000, D+F11.02
     3  96910772, 116012242, 130714232, 153916552, 177118872,  9888000, D+F11.03
     4 601386081, 108812932, 148916832, 187820722, 226624612, 13836900, AEL11.04
     5 105712442, 144616652, 189221182, 234425702, 279630222, 17209000, AEL11.05
     6 100010011, 101410621, 118414581, 204831781, 509479731,   764404, D+F12.00
     7 200120051, 202921001, 226926901, 368457091,  92814872,  1503101, D+F12.01
     8 100010001, 100110611, 177455431, 176546012,  99718753,  8011905, D+F12.02
     9 579758751, 591459501, 600560591, 611461681, 622362781, 10928900/ AEL12.03
      DATA NNN07/
     1 100611232, 120612752, 134214102, 147815462, 161416822, 14122900, AEL12.04
     2 674896701, 121814462, 167018942, 211723412, 256527892, 18648900, AEL12.05
     3 558857701, 583558761, 593260591, 635969541, 796790971,   598400, D+F13.00
     4 100310211, 110313021, 172828201,  55311252, 215637942,  1882203, D+F13.01
     5 200320201, 208622331, 250530971, 410251081, 611571211,  2844000, D+F13.02
     6 100010001, 100210881, 207436531, 523168101, 838999681, 11996000, D+F13.03
     7 577758651, 591259631, 604461351, 622563161, 640764981, 15377000, AEL13.04
     8 103511582, 124713242, 140014772, 155316292, 170517812, 19042000, AEL13.05
     9 825189211,  95210052, 106211532, 134317202, 237934082,   814913/ D+F14.00
      DATA NNN08/
     1 563057761, 588160311, 631768671, 791097651, 127817282,  1634000, D+F14.01
     2 101110771, 126716471, 232438081,  71914052, 262045302,  3346001, D+F14.02
     3 200720521, 217224081, 284439171, 551370951,  86810262,  4513000, D+F14.03
     4 100010001, 100210881, 207436531, 523168101, 838999681, 16672900, FAK14.04
     5 575458521, 591459851, 610063201, 672674071, 843698661, 20510900, AEL14.05
     6 402643441, 496757481, 658274401, 833492941, 103511532,  1048300, AEL15.00
     7 874497931, 106011282, 119812802, 138415142, 164717802,  1972000, AEL15.01
     8 564058061, 604164611, 709579551,  90410172, 112912422,  3015500, AEL15.02
     9 100811411, 149720221, 280936121, 441552181, 602168241,  5135400/ AEL15.03
      DATA NNN09/
     1 200420781, 227025361, 281430911, 336936471, 392542021,  6500700, AEL15.04
     2 100010001, 100010001, 100010001, 100010001, 100010001, 22041300, G  15.05
     3 822887891, 930697831, 102610932, 121614492, 185124742,  1035708, D+F16.00
     4 443056011, 694982961,  96911522, 144218572, 227326892,  2339900, D+F16.01
     5  91610392, 113512242, 136416942, 233429882, 364242962,  3500000, D+F16.02
     6 560058861, 633871081,  82410062, 123314602, 168619132,  4728900, D+F16.03
     7 104512901, 177025421, 375163021, 122420462, 286036742,  7250000, D+F16.04
     8 202321571, 241428261, 358355061,  78310152, 124814802,  8802800, D+F16.05
     9 538155931, 571657911, 598067191,  89013782, 227737172,  1300916/ D+F17.00
      DATA NNN10/
     1 873396771, 104411072, 118513532, 175525872, 406763932,  2379903, D+F17.01
     2 506569571,  87610522, 134421682, 439092662, 182132573,  3990006, D+F17.02
     3  95110872, 120013232, 154921252, 345149322, 641378942,  5350000, D+F17.03
     4 558960371, 677779341,  95311692, 138816082, 182720472,  6780000, D+F17.04
     5 100010001, 100010051, 106913911, 240147261,  90716112,  1575411, D+F18.00
     6 550256831, 578158781, 636585461, 151530162,  58010303,  2762007, D+F18.01
     7  92110362, 112412002, 133216772, 254443722,  76512833,  4090003, D+F18.02
     8 582082081, 103112292, 149920212, 309750502, 720793642,  5978900, D+F18.03
     9  97111072, 123213982, 172625622, 463976582, 106413633,  7500000/ D+F18.04
      DATA NNN11/
     1 200020011, 200720361, 211923291, 280137141, 525575741,   433803, D+F19.00
     2 100010001, 100110341, 135929551,  79119282, 405274892,  3180905, D+F19.01
     3 554657081, 581260301,  73012702, 285363872, 129023363,  4600005, D+F19.02
     4  96010862, 118413212, 180836632,  90321023, 416863253,  6090000, D+F19.03
     5 657793361, 119515082, 195826322, 352944302, 533162332,  8259900, D+F19.04
     6 100110061, 104311741, 145919971, 294345051,  69010322,   611003, D+F20.00
     7 205822781, 279234761, 427553061, 688994901, 136319772,  1186701, D+F20.01
     8 100010001, 100510821, 168744821, 130232522,  69012813,  5121003, D+F20.02
     9 555157161, 585662471,  82816862,  42510013, 168423663,  6700000/ D+F20.03
      DATA NNN12/
     1  99411262, 123814062, 182930402, 484766392,  84310223,  8438900, D+F20.04
     2 924696691, 105212282, 151219062, 240530032, 368944512,   653900, AEL21.00
     3 190424662, 297634542, 391743752, 482952832, 573761912,  1280000, AEL21.01
     4 976799291, 101110322, 105810882, 111911502, 118112122,  2475000, AEL21.02
     5 100010001, 100510821, 168744821, 130232522,  69012813,  7390000, FAK21.03
     6 555157161, 585662471,  82816862,  42510013, 168423663,  9200000, FAK21.04
     7 181021172, 260333222, 430155582, 710089242, 110213293,   681900, D+F22.00
     8 474659872, 721284672,  98211413, 134515623, 177919963,  1356900, D+F22.01
     9 228327012, 308134272, 381143862, 534563472, 734983512,  2747000/ D+F22.02
      DATA NNN13/
     1 971498311,  99210032, 102610572, 108711172, 114711782,  4324000, D+F22.03
     2 100010001, 100510821, 168744821, 130232522,  69012813,  9980000, FAK22.04
     3 272835172, 425851532, 632278322,  97212013, 146817723,   674000, AEL23.00
     4 373954132, 743597002, 121414713, 173920143, 229225713,  1464900, AEL23.01
     5 323142642, 519660272, 679975352, 824789522,  96610363,  2930900, AEL23.02
     6 248329302, 324234952, 373439752, 421744582, 469949412,  4800000, AEL23.03
     7 970698231, 990699881, 100710152, 102410322, 104010482,  6500000, AEL23.04
     8 717277611,  92911652, 152620872, 295141952, 550468122,   676400, D+F24.00
     9  71611552, 205635512, 558281952, 115315823, 205625293,  1649000/ D+F24.01
      DATA NNN14/
     1 280639822, 538369722,  87610823, 129115003, 170919183,  3095000, D+F24.02
     2 377150952, 616070292, 791788382,  97610683, 116012523,  5000000, D+F24.03
     3 264730962, 341436462, 394042872, 463549832, 533056782,  7300000, D+F24.04
     4 600060321, 629270891,  86911302, 151020222, 267534752,   743100, AEL25.00
     5 739594821, 139921212, 309342852, 567372412,  97112553,  1563600, AEL25.01
     6  98417472, 265535782, 454754842, 641973532, 828792212,  3369000, AEL25.02
     7 328847052, 586668342, 771785912,  94710343, 112112093,  5300000, AEL25.03
     8 422055132, 636770792, 779285062, 921999322, 106411363,  7600000, AEL25.04
     9 197023222, 274433302, 416753952, 723799822, 139419053,   787038/ D+F26.00
      DATA NNN15/
     1 409453722, 686687452, 110213823, 174322233, 286437043,  1617902, D+F26.01
     2 262136422, 501167232,  87911303, 138916483, 190721673,  3064300, D+F26.02
     3  98723522, 420363072,  87011423, 145117913, 215925463,  5700000, AEL26.03
     4 388854482, 666275742, 846693572, 102511143, 120312923,  7900000, D+F26.04
     5 199427202, 335740022, 474957182, 708090462, 118315403,   786000, D+F27.00
     6 279739202, 490858232, 684582472, 104713233, 159818733,  1704900, D+F27.01
     7 279836622, 461857562, 720693022, 124915873, 192522633,  3349000, D+F27.02
     8 262136422, 501167232,  87911303, 138916483, 190821673,  5300000, FAK27.03
     9  98723522, 420363072,  87011423, 145117913, 215925463,  8300000/ FAK27.04
      DATA NNN16/
     1 227027622, 306233052, 356839222, 446052912, 652382292,   763314, D+F28.00
     2 108416342, 222428472, 353944332, 577378932, 110314303,  1814900, D+F28.01
     3 198724282, 293236452, 468362702,  86511123, 136016073,  3516000, D+F28.02
     4 279836622, 461857562, 720693022, 124915873, 192522633,  5600000, FAK28.03
     5 262136422, 501167232,  87911303, 138916483, 190721673,  7900000, FAK28.04
     6 201620781, 231026761, 314737361, 450555381, 692386911,   772301, D+F29.00
     7 109415761, 247938311,  58910042, 190937022,  68311693,  2028903, D+F29.01
     8 897195961, 107212972, 165021182, 260230862, 356940532,  3682900, D+F29.02
     9 100010001, 100410231, 108712611, 167124841, 388460411,   939102/ D+F30.00
      DATA NNN17/
     1 200020021, 201620761, 223726341, 351352061,  80812472,  1796001, D+F30.01
     2 100610471, 122617301, 300566361, 149924112, 332342352,  3970000, D+F30.02
     3 403245601, 493151431, 529654331, 559358091, 611065171,   600000, AEL31.00
     4  99710051, 104511541, 135016501, 208226431, 321837921,  2050900, AEL31.01
     5 199820071, 204521391, 229124761, 266028451, 302932131,  3070000, AEL31.02
     6 502665261, 755183501, 901496201, 102410942, 117912812,   787900, AEL32.00
     7 422848161, 512153401, 557458941, 636270361, 794489061,  1593000, AEL32.01
     8 100010261, 114613921, 175221251, 249828711, 324436181,  3421000, AEL32.02
     9 403143241, 491856701, 649173781, 840396751, 113013392,   981000/ AEL33.00
      DATA NNN18/
     1 593676641, 884697521, 105911572, 129515012, 180322212,  1858700, AEL33.01
     2 484470541,  91510972, 125614082, 157017612, 199722912,  2829900, AEL33.02
     3 630172361, 799686381, 919797221, 102810942, 117712832,   975000, AEL34.00
     4 438055511, 691582151,  94510732, 121413672, 152016732,  2150000, AEL34.01
     5 651982921,  94610382, 113212492, 139515462, 169718482,  3200000, AEL34.02
     6 437347431, 498951671, 538559501,  74710812, 169126672,  1183910, D+F35.00
     7 705183611,  93510092, 111614162, 222932532, 427652992,  2160000, D+F35.01
     8 510869921,  87410312, 123116552, 236530712, 377744832,  3590000, D+F35.02
     9 100010001, 100010051, 105012781, 198535971,  65911422,  1399507/ D+F36.00
      DATA NNN19/
     1 461049811, 522254261, 609088131, 168935052,  68612253,  2455908, D+F36.01
     2 759990901, 101911142, 129017782, 302856642,  99414333,  3690000, D+F36.02
     3 200020011, 200720361, 211523021, 269434141, 459163351,   417502, D+F37.00
     4 100010001, 100110321, 129524961,  61014202, 291753192,  2750004, D+F37.01
     5 473650891, 533156051,  66810932, 232950852,  99915303,  4000000, D+F37.02
     6 100110041, 104111741, 146019721, 281941411, 607785251,   569202, D+F38.00
     7 202621931, 255331271, 384347931, 624085761, 122417632,  1102600, D+F38.01
     8 100010001, 100110321, 129524961,  61014202, 291753192,  4300000, FAK38.02
     9 791587851, 100012192, 155119942, 254031782, 389946932,   637900/ AEL39.00
      DATA NNN20/
     1 118217102, 220827002, 319036792, 416646512, 513256072,  1223000, AEL39.01
     2  92510012, 104710862, 112311612, 120212472, 132814282,  2050000, AEL39.02
     3 141320802, 291439702, 531170262,  92712273, 162521053,   684000, D+F40.00
     4 354454352, 724689652, 107212643, 148517093, 193321573,  1312900, D+F40.01
     5 209727032, 324537052, 415446282, 510255752, 604965222,  2298000, D+F40.02
     6 256636022, 465759302, 749693962, 116514243, 171520333,   687900, AEL41.00
     7 335157222,  84511463, 147718363, 221826083, 299933893,  1431900, AEL41.01
     8 223725352, 280830972, 340937362, 406844002, 473150632,  2503900, AEL41.02
     9 703972941,  82610822, 154822682, 327244912, 571469372,   709900/ D+F42.00
      DATA NNN21/
     1  75714552, 274347322, 718897632, 123414913, 174920063,  1614900, D+F42.01
     2 267645462, 669890262, 115514323, 173620673, 242528083,  2714900, AEL42.02
     3  90613732, 184823562, 291735332, 419949102, 565764332,   728000, AEL43.00
     4 131318312, 227126932, 311735452, 397644072, 483852692,  1525900, AEL43.01
     5 204721673, 234725733, 284031463, 348738613, 426546943,  3000000, AEL43.02
     6 176824122, 318941082, 515263202, 761790472, 106112303,   736400, AEL44.00
     7 221934642, 501968372,  88911173, 136316243, 189221613,  1675900, AEL44.01
     8 210622722, 241025422, 267928262, 297731272, 327834282,  2846000, AEL44.02
     9 148520202, 255230902, 364942462, 489656082, 638872352,   746000/ AEL45.00
      DATA NNN22/
     1 153421292, 288137912, 484660322, 720187062, 101011483,  1807000, AEL45.01
     2 254537212, 492362292, 770592182, 107312243, 137615273,  3104900, AEL45.02
     3 115919651, 320746011, 607576761,  95011642, 141817172,   832900, AEL46.00
     4 755087211, 105913442, 173122222, 282034722, 412247732,  1941900, AEL46.01
     5 180223462, 289735212, 414247632, 538460052, 662672472,  3292000, AEL46.02
     6 200020001, 200220141, 206422141, 257633021, 455164681,   757403, D+F47.00
     7 100810581, 125817401, 260641031,  66210072, 135316982,  2148000, D+F47.01
     8 795887491,  97711762, 156620252, 248329422, 340038582,  3481900, D+F47.02
     9 100010001, 100410241, 109212891, 176827421, 444268771,   899003/ D+F48.00
      DATA NNN23/
     1 200020021, 201720921, 233329881, 451475371, 127520782,  1690301, D+F48.01
     2 100310281, 114815371, 246138311, 519265531, 791492761,  3747000, D+F48.02
     3 252431921, 368440461, 433746521, 512259221, 723389021,   578400, D+F49.00
     4 100110071, 104611651, 146118581, 225426511, 304734431,  1886000, D+F49.01
     5 200120111, 205021611, 243628031, 317035371, 390442701,  2802900, D+F49.02
     6 232637101, 488058571, 669074381, 816189091,  97210632,   734200, AEL50.00
     7 286335941, 408144471, 479351961, 571862901, 686274341,  1462700, AEL50.01
     8 100010251, 114013811, 175321601, 256829751, 338337901,  3049000, AEL50.02
     9 404043481, 494656811, 646772781, 813490751, 101411372,   863900/ AEL51.00
      DATA NNN24/
     1 303147981, 618472951, 827392621, 103711702, 131214532,  1650000, AEL51.01
     2 313037601, 429347901, 536260591, 689477591, 862494881,  2529900, AEL51.02
     3 526258801, 657372351, 784284071, 897095741, 102711082,   900900, AEL52.00
     4 440855541, 686481251,  93810792, 125414792, 176321132,  1860000, AEL52.01
     5 349054751, 699883081,  96611302, 134216202, 197724212,  2800000, AEL52.02
     6 405342041, 438645621, 475751071, 587974491, 102214572,  1045404, D+F53.00
     7 568567471, 773485861,  94510362, 112712182, 130914002,  1909000, D+F53.01
     8 514269581,  86910562, 130716652, 215327742, 351843662,  3200000, AEL53.02
     9 100010001, 100010091, 109515351, 291060661, 119621482,  1212716/ D+F54.00
      DATA NNN25/
     1 414844131, 465649111, 538464651,  87112232, 158019362,  2120000, D+F54.01
     2 615475101, 867797531, 112213462, 157618062, 203622662,  3209900, D+F54.02
     3 200020001, 201020501, 215623871, 283536181, 462756261,   389300, D+F55.00
     4 100010001, 100310371, 119016501, 269146361,  77912412,  2510000, D+F55.01
     5 424445601, 481750061, 516953311, 549356551, 581759791,  3500000, D+F55.02
     6 101210791, 135119351, 282340571, 574580391, 111015062,   521002, D+F56.00
     7 262638611, 504160621, 698579371,  91010692, 129115952,  1000000, D+F56.01
     8 100010001, 100310351, 118416321, 264945521,  76512182,  3700000, FAK56.02
     9  71111992, 172323592, 312540402, 510763182, 765791012,   557700/ AEL57.00
      DATA NNN26/
     1 204529582, 383647882, 582469262, 807992692, 104911723,  1106000, AEL57.01
     2  94712552, 148416582, 179819212, 203621522, 227424042,  1917700, AEL57.02
     3 295959132, 103515693, 215527593, 335939413, 449650223,   553870, AEL58.00
c     4  79718153, 289639443, 495159253, 686877533, 863794813,  1085000, AEL58.01
c     5 298640242, 475053692, 596965912, 725379692, 872094692,  2198000, AEL58.02
     4  80118633, 304342383, 541765723, 769387773,  98210814,  1085000, MZH58.01
     5 506183092, 108612923, 146416133, 174418603, 196520603,  2020000, CCB58.02
     6 460693672, 158523823, 327242303, 519661563, 709379783,   546400, FAK59.00
     7 455480232, 114014653, 178521013, 240927073, 299232633,  1055000, AEL59.01
     8  46410533, 183826893, 354443773, 518459633, 674375243,  2162400, AEL59.02
     9 139623042, 364860002,  96114603, 209828633, 373446973,   552500/ AEL60.00
      DATA NNN27/
     1 460493692, 158523823, 327142303, 519661563, 709279783,  1073000, AEL60.01
     2 455480232, 114014653, 178521013, 240927073, 299232633,  2218000, BOR60.02
     3 131720482, 280535692, 441254492, 676583972, 103412583,   555400, AEL61.00
     4 139623042, 364860002,  96114603, 209828633, 373446973,  1090000, FAK61.01
     5 460493682, 158523823, 327142303, 519661563, 709279783,  2230000, FAK61.02
     6  92915672, 222431062, 444763802,  89612173, 159520253,   564370, AEL62.00
     7 315059662,  97114563, 204627093, 342541693, 490556383,  1106900, AEL62.01
     8 269037812, 520270372,  91111273, 133915483, 172719093,  2340000, AEL62.02
     9 800080571, 851699301, 127617362, 240433032, 444958442,   567045/ AEL63.00
      DATA NNN28/
     1 125416052, 211828182, 375549622, 644381732, 101112213,  1124100, AEL63.01
     2 800080571, 851699301, 127617362, 240433032, 444958442,  2492000, FAK63.02
     3 240432982, 427555202, 708489962, 112613853, 167319843,   615000, AEL64.00
     4 534793262, 139219123, 247730843, 371043333, 495055893,  1209000, AEL64.01
     5 364145232, 514756362, 604864112, 673870372, 732276072,  2063000, AEL64.02
     6 480767202,  89011393, 144118243, 230028753, 354142883,   586390, AEL65.00
     7 480767192,  89011393, 144118243, 230028753, 354142883,  1151900, FAK65.01
     8 480767202,  89011393, 144118243, 230028753, 354142883,  2191000, FAK65.02
     9 343147532, 645887152, 115314793, 183322063, 257729373,   593890/ FAK66.00
      DATA NNN29/
     1 343147532, 645887142, 115314793, 183322063, 257729373,  1167000, AEL66.01
     2 343147532, 645887142, 115314793, 183322063, 257729373,  2280000, FAK66.02
     3 222635002, 542276772, 100312353, 145716713, 187020703,   602160, FAK67.00
     4 222635002, 542276772, 100312353, 145716713, 187020703,  1180000, FAK67.01
     5 222635002, 542276772, 100312353, 145716713, 187020703,  2284000, AEL67.02
     6 133715382, 209130152, 429859382,  79410293, 129815983,   610780, AEL68.00
     7 265934782, 497877532, 120517733, 245032063, 400448073,  1193000, AEL68.01
     8 265934782, 497877532, 120517733, 245032063, 400448073,  2274000, FAK68.02
     9 800381111,  87510702, 147621462, 310343462, 585475982,   618436/ AEL69.00
      DATA NNN30/
     1 156718872, 279244452, 678196342, 128316243, 197823443,  1205000, AEL69.01
     2  93517192, 364666132, 103414613, 192624193, 293334613,  2368000, AEL69.02
     3 100010011, 101310651, 118613951, 169120661, 250629971,   625394, AEL70.00
     4 200120901, 270345231,  81714042, 223533112, 461959862,  1218400, AEL70.01
     5 100312561, 250851931,  91914182, 198626022, 323638692,  2505000, AEL70.02
     6 514664441, 759086851,  99211442, 133315612, 182721252,   542589, AEL71.00
     7 125924831, 438667801,  98714112, 199727872, 380850742,  1389900, AEL71.01
     8 323948621, 661297271, 158626482, 426865032,  93712843,  2095960, AEL71.02
     9 659294081, 128016962, 222528952, 372047062, 585171462,   700000/ AEL72.00
      DATA NNN31/
     1  99117882, 274638812, 520867322,  84410313, 123314453,  1489900, AEL72.01
     2 187427702, 343739872, 448049452, 539358282, 625266642,  2329900, AEL72.02
     3  65210892, 171325762, 373552252, 705192012, 116414343,   787900, AEL73.00
     4 192837842, 600784802, 111113823, 165419233, 218524383,  1620000, AEL73.01
     5  99117872, 274638812, 520867312,  84410313, 123314453,  2400000, FAK73.02
     6 398981651, 130019172, 273438022, 516168382,  88411163,   797900, AEL74.00
     7 131429482, 523279952, 111414623, 183422233, 262130233,  1770000, AEL74.01
     8 192837842, 600784792, 111113823, 165419233, 218524383,  2500000, FAK74.02
     9 600963001,  75910412, 150121572, 301940972, 539168952,   787000/ AEL75.00
      DATA NNN32/
     1  73710852, 190731262, 464964142,  83810503, 127315053,  1660000, AEL75.01
     2 131429482, 523279952, 111414623, 183422233, 262130233,  2600000, FAK75.02
     3 110815502, 216829732, 398752322, 672484682, 104612673,   850000, AEL76.00
     4 168225972, 362046562, 566766422, 757484612,  93010103,  1700000, AEL76.01
     5  73710852, 190731262, 464964142,  83810503, 127315053,  2700000, FAK76.02
     6 129117892, 239430882, 388748292, 596173252,  89510843,   910000, AEL77.00
     7 110815502, 216829732, 398752322, 672484682, 104612673,  2000000, FAK77.01
     8 168225972, 362046562, 566766422, 757484612,  93010103,  2800000, FAK77.02
     9 158918512, 207523002, 254328242, 316335762, 407246582,   900000/ AEL78.00
      DATA NNN33/
     1  98115462, 224930742, 401150612, 623475412,  89910583,  1855900, AEL78.01
     2 110815502, 216829732, 398752322, 672484682, 104612673,  2900000, FAK78.02
     3 203222611, 265731251, 364042301, 494958601, 702084731,   922000, AEL79.00
     4 120521331, 357753801,  75310062, 130516572, 206925452,  2050000, AEL79.01
     5 651780821, 108814772, 195925252, 316338622, 460853882,  3000000, AEL79.02
     6 100010001, 100110111, 105211851, 152122101, 341552811,  1043002, D+F80.00
     7 200320211, 210023021, 268834231, 480472341, 111416912,  1875000, D+F80.01
     8 104012871, 186129471, 458664151,  82410072, 119013732,  3420000, D+F80.02
     9 200420711, 222424271, 265429161, 325637371, 442853911,   610500/ AEL81.00
      DATA NNN34/
     1 100010021, 101910801, 121414641, 189525811, 358949721,  2041900, AEL81.01
     2 200020311, 216624611, 296337451, 489064791,  85711212,  2979900, AEL81.02
     3 103411711, 147819101, 244331781, 434862751,  93113762,   741404, D+F82.00
     4 204122231, 248227841, 311535621, 429153941, 651976431,  1502800, D+F82.01
     5 100210131, 106812201, 154522671, 381665951,  95512512,  3192900, D+F82.02
     6 400140351, 416944121, 474851591, 564362181, 690477231,   728700, AEL83.00
     7 106814451, 204427341, 350744811, 586879131, 108314772,  1667900, AEL83.01
     8 205523051, 264830231, 345439921, 469156001, 675281671,  2555900, AEL83.02
     9 500950661, 518153561, 559058941, 628968071, 748483501,   843000/ AEL84.00
      DATA NNN35/
     1 443756241, 696282451,  95411012, 128615262, 182922012,  1900000, FAK84.01
     2 336953201, 682481011,  93810882, 127915272, 184622442,  2700000, FAK84.02
     3 402841621, 431544771, 463148311, 520059491, 734896851,   930000, FAK85.00
     4 576168741, 788387631,  96910642, 116012552, 135014462,  2000000, FAK85.01
     5 490265341, 812797201, 116614322, 179622692, 285035302,  2900000, FAK85.02
     6 100010001, 100010031, 102311051, 133018071, 264539391,  1074500, AEL86.00
     7 402841621, 431544771, 463148311, 520059491, 734996851,  2000000, FAK86.01
     8 576168741, 788387631,  96910642, 116012552, 135014462,  3000000, FAK86.02
     9 200020011, 201220591, 218124481, 296538611, 488859141,   400000/ FAK87.00
      DATA NNN36/
     1 100010001, 100010031, 102311051, 133018071, 264539401,  2200000, FAK87.01
     2 421645151, 477449611, 511852711, 542455761, 572958821,  3300000, FAK87.02
     3 100010041, 105212131, 153220271, 270435641, 460258111,   527600, AEL88.00
     4 201221791, 258131471, 381645781, 546365131, 777592781,  1014400, AEL88.01
     5 100010001, 100010031, 102311051, 133018071, 264539391,  3400000, FAK88.02
     6 510064491,  82710872, 142718412, 232328712, 348341572,   690000, AEL89.00
     7 228951571,  88513232, 183324132, 305537492, 448152402,  1210000, AEL89.01
     8 723989131, 103511752, 130814352, 155416652, 177018682,  2000000, AEL89.02
     9 620099241, 162725772, 391457072,  80110833, 141818023,   600000/ AEL90.00
      DATA NNN37/
     1 620099241, 162725772, 391457072,  80110833, 141818023,  1200000, FAK90.01
     2 620099251, 162725772, 391457072,  80110833, 141818023,  2000000, FAK90.02
     3 347877992, 129318323, 240730533, 380546863, 570368573,   600000, AEL91.00
     4 347877992, 129318323, 240730533, 380546863, 570368573,  1200000, FAK91.01
     5 347777992, 129318323, 240730533, 380546863, 570368573,  2000000, FAK91.02
     6 209530092, 450866762,  96613623, 186524763, 318839893,   600000, AEL92.00
     7 209530092, 450866762,  96613623, 186524763, 318839893,  1200000, FAK92.01
     8 209530092, 450866762,  96613623, 186524763, 318839893,  2000000, FAK92.02
     9 209530092, 450866762,  96613623, 186524763, 318839893,   600000/ FAK93.00
      DATA NNN38/
     1 209530092, 450866762,  96613623, 186524763, 318839893,  1200000, FAK93.01
     2 209530092, 450866762,  96613623, 186524763, 318839893,  2000000, FAK93.02
     3 209530092, 450866762,  96613623, 186524763, 318839893,   600000, FAK94.00
     4 209530092, 450866762,  96613623, 186524763, 318839893,  1200000, FAK94.01
     5 209530092, 450866762,  96613623, 186524763, 318839893,  2000000, FAK94.02
     6 209530092, 450866762,  96613623, 186524763, 318839893,   600000, FAK95.00
     7 209530092, 450866762,  96613623, 186524763, 318839893,  1200000, FAK95.01
     8 209530092, 450866762,  96613623, 186524763, 318839893,  2000000, FAK95.02
     9 209530092, 450866762,  96613623, 186524763, 318839893,   600000/ FAK96.00
      DATA NNN39/
     1 209530092, 450866762,  96613623, 186524763, 318839893,  1200000, FAK96.01
     2 209530092, 450866762,  96613623, 186524763, 318839893,  2000000, FAK96.02
     3 209530092, 450866762,  96613623, 186524763, 318839893,   600000, FAK97.00
     4 209530092, 450866762,  96613623, 186524763, 318839893,  1200000, FAK97.01
     5 209530092, 450866762,  96613623, 186524763, 318839893,  2000000, FAK97.02
     6 209530092, 450866762,  96613623, 186524763, 318839893,   600000, FAK98.00
     7 209530092, 450866762,  96613623, 186524763, 318839893,  1200000, FAK98.01
     8 209530092, 450866762,  96613623, 186524763, 318839893,  2000000, FAK98.02
     9 209530092, 450866762,  96613623, 186524763, 318839893,   600000/ FAK99.00
      DATA NNN40/
     1 209530092, 450866762,  96613623, 186524763, 318839893,  1200000, FAK99.01
     2 209530092, 450866762,  96613623, 186524763, 318839893,  2000000/ FAK99.02
      DATA NNN67/
     1 893292271,  96110042, 105311262, 126315202, 196126432,  1125508, D+F 6.00
     2 595060251, 620865751, 713280191,  95712292, 167623542,  2437501, D+F 6.01
     3 105513201, 180324851, 341851341,  88416332, 296550722,  4787101, D+F 6.02
     4 204922771, 262630421, 350941931, 494556971, 644872001,  6447600, D+F 6.03
     5 100010001, 100010001, 100010001, 100010001, 100010001, 39207700, G   6.04
     6 200020001, 200020001, 200020001, 200020001, 200020001, 48998100, G   6.05
     7 403141851, 457051681, 594071181,  92913362, 203331152,  1452915, D+F 7.00
     8 919899541, 107211512, 124914302, 182526232, 403762662,  2959202, D+F 7.01
     9 596862721, 684177081,  88110342, 128317062, 239334312,  4742501, D+F 7.02
     T 112816481, 240733751, 462068491, 116419932, 283736822,  7744900, D+F 7.03
     1 210124681, 293634211, 391145791, 539862151, 703178471,  9786200, D+F 7.04
     2 100010001, 100010001, 100010001, 100010001, 100010001, 55205700, G   7.05
     9 800080571, 851699301, 127617362, 240433032, 444958442,   567045, AEL63.00
     1 125416052, 211828182, 375549622, 644381732, 101112213,  1124100, AEL63.01
     2 800080571, 851699301, 127617362, 240433032, 444958442,  2492000, FAK63.02
     3 490049002, 490049002, 490049002, 490049002, 490049002,  4000000/ TAR63.03
      DATA LOCZ/1,3,6,10,14,18,22,27,33,39,45,51,57,63,69,75,81,86,91,
     1          96,101,106,111,116,121,126,131,136,141/
      DATA SCALE/0.001,0.01,0.1,1.0/
C
      MODE1=MODE
      IF(MODE1.GT.10) MODE1=MODE1-10
C
C   LOWERING OF THE IONIZATION POTENTIAL IN VOLTS FOR UNIT ZEFF
C
      CHARGE=XNE(J)*2.
      EXCESS=XNE(J)-XNA(J)
C
C   ALLOWANCE FOR DOUBLY IONIZED HELIUM
C
      IF(EXCESS.GT.0.) CHARGE=CHARGE-EXCESS+4.*(2.*EXCESS)
      DEBYE=SQRT(TK(J)/2.8965E-18/CHARGE)
      POTLOW=MIN(1.,1.44E-7/DEBYE)
      TV=TKEV(J)
      IF(IZ.EQ.6) THEN
        N=355
        NIONS=6
      ELSE IF(IZ.EQ.63) THEN
        N=366
        NIONS=4
      ELSE IF(IZ.EQ.7) THEN
        N=361
        NIONS=6
      ELSE IF(IZ.LE.28) THEN
        N=LOCZ(IZ)
        NIONS=LOCZ(IZ+1)-N
      ELSE
        N=3*IZ+54
        NIONS=3
      END IF
      NION2=MIN(NION+2,NIONS)
      N=N-1
C
      DO 1 IONN=1,NION2
      Z=IONN
      POTLO(IONN)=POTLOW*Z
      N=N+1
      NNN100=NNNPFN(6,N)/100
      IP(IONN)=FLOAT(NNN100)/1000.
      G=NNNPFN(6,N)-NNN100*100
      IF(N.EQ.1) THEN
        PART(1)=2.
        IF(T(J).LT.9000.) GO TO 1
        PART(1)=PART(1)+8.*EXP(-10.196/TV)+18.*EXP(-12.084/TV)+32.*
     1  EXP(-12.745/TV)+50.*EXP(-13.051/TV)+72.*EXP(-13.217/TV)
        D1=13.595/6.5/6.5/TV
        D2=POTLO(1)/TV
      ELSE
        T2000=IP(IONN)*2000./11.
        IT=MAX(1,MIN(9,INT(T(J)/T2000-.5)))
        DT=T(J)/T2000-FLOAT(IT)-.5
        PMIN=1.
        I=(IT+1)/2
        K1=NNNPFN(I,N)/100000
        K2=NNNPFN(I,N)-K1*100000
        K3=K2/10
        KSCALE=K2-K3*10
        IF(MOD(IT,2).EQ.0) THEN
          P1=FLOAT(K3)*SCALE(KSCALE)
          K1=NNNPFN(I+1,N)/100000
          KSCALE=MOD(NNNPFN(I+1,N),10)
          P2=K1*SCALE(KSCALE)
        ELSE
          P1=K1*SCALE(KSCALE)
          P2=K3*SCALE(KSCALE)
          IF(DT.LT.0..AND.KSCALE.LE.1) KP1=P1
          IF(DT.LT.0..AND.KSCALE.LE.1.AND.KP1.EQ.INT(P2+.5)) PMIN=KP1
        END IF
        PART(IONN)=MAX(PMIN,P1+(P2-P1)*DT)
        IF(G.EQ.0..OR.POTLO(IONN).LT..1.OR.T(J).LT.T2000*4.) GO TO 1
        IF(T(J).GT.(T2000*11.)) TV=(T2000*11.)*8.6171E-5
        D1=.1/TV
      END IF
      D2=POTLO(IONN)/TV
      PART(IONN)=PART(IONN)+G*EXP(-IP(IONN)/TV)*
     *(SQRT(13.595*Z*Z/TV/D2)**3*
     *(1./3.+(1.-(.5+(1./18.+D2/120.)*D2)*D2)*D2)-
     -SQRT(13.595*Z*Z/TV/D1)**3*
     *(1./3.+(1.-(.5+(1./18.+D1/120.)*D1)*D1)*D1))
      TV=TKEV(J)
   1  CONTINUE
C
      IF(MODE1.EQ.3) GO TO 5
C
      N=N-NION2
      CF=2.*2.4148E15*T(J)*SQRT(T(J))/XNE(J)
      DO 2 IONN=2,NION2
      N=N+1
C
C   THE MIN IS FOR ANY UNFORTUNATE WHO HAS A 360
C
      FEXARG=(IP(IONN-1)-POTLO(IONN-1))/TV
      IF(FEXARG.GT.80.) THEN
        F(IONN)=0.
      ELSE
        F(IONN)=CF*PART(IONN)/PART(IONN-1)*EXP(-FEXARG)
      END IF
   2  CONTINUE
      F(1)=1.
      DO 3 IONN=NION2,2,-1
   3  F(1)=1.+F(IONN)*F(1)
      F(1)=1./F(1)
      DO 4 IONN=2,NION2
   4  F(IONN) =F(IONN-1)*F(IONN)
C
   5  IF(MODE.LT.10) GO TO 14
      GO TO (6,8,10,12),MODE1
   6  DO 7 IONN=1,NION
   7  ANSWER(J,IONN)=F(IONN)/PART(IONN)
c      write(*,'(F10.2,5E10.3)') T(J),XNE(J),F(1),F(2),F(3),F(4)
c      write(*,'(F10.2,5E10.3)') T(J),XNE(J),PART(1),PART(2),PART(3)
      RETURN
   8  DO 9 IONN=1,NION
   9  ANSWER(J,IONN)=F(IONN)
      RETURN
  10  DO 11 IONN=1,NION
  11  ANSWER(J,IONN)=PART(IONN)
c      write(*,'(F10.2,5E10.3)') T(J),XNE(J),PART(1),PART(2),PART(3)
      RETURN
  12  ANSWER(J,1)=0.
      DO 13 IONN=2,NION2
  13  ANSWER(J,1)=ANSWER(J,1)+F(IONN)*FLOAT(IONN-1)
      RETURN
  14  GO TO (15,16,17,12),MODE1
  15  ANSWER(J,1)=F(NION)/PART(NION)
      RETURN
  16  ANSWER(J,1)=F(NION)
      RETURN
  17  ANSWER(J,1)=PART(NION)
      RETURN
C
C  ENTRY POINT FOR DIRECT ACCESS FROM FUNCTION "IONTBL"
C
      ENTRY XSAHA(IEL,TT,XNELEC,XNATOM,MAXION,POTI,FRCT)
C
      TTKEV=8.6171E-5*TT
      TV=TTKEV
      TTK=1.38054E-16*TT
C
C   LOWERING OF THE IONIZATION POTENTIAL IN VOLTS FOR UNIT ZEFF
C
      CHARGE=2.*XNELEC
      EXCESS=XNELEC-XNATOM
C
C   ALLOWANCE FOR DOUBLY IONIZED HELIUM
C
      IF(EXCESS.GT.0.) CHARGE=CHARGE-EXCESS+4.*(2.*EXCESS)
      DEBYE=SQRT(TTK/(2.8965E-18*CHARGE))
      POTLOW=MIN(1.,1.44E-7/DEBYE)
C
C  SOLVE SAHA EQUATION
C
      IF(IEL.EQ.6) THEN
        N=354
        NIONS=6
      ELSE IF(IEL.EQ.63) THEN
        N=366
        NIONS=4
      ELSE IF(IEL.EQ.7) THEN
        N=360
        NIONS=6
      ELSE IF(IEL.LE.28) THEN
        N=LOCZ(IEL)
        NIONS=LOCZ(IEL+1)-N
      ELSE
        N=3*IEL+54
        NIONS=3
      END IF
      NION2=MIN(MAXION,NIONS)
      N=N-1
      DO 19 IONN=1,NION2
      Z=IONN
      POTLO(IONN)=POTLOW*Z
      N=N+1
      NNN100=NNNPFN(6,N)/100
      IP(IONN)=FLOAT(NNN100)/1000.
      G=NNNPFN(6,N)-NNN100*100
      IF(N.EQ.1) THEN
        PART(1)=2.
        IF(TT.LT.9000.) GO TO 19
        PART(1)=PART(1)+8.*EXP(-10.196/TV)+18.*EXP(-12.084/TV)+32.*
     1          EXP(-12.745/TV)+50.*EXP(-13.051/TV)+72.*EXP(-13.217/TV)
        D1=13.595/6.5/6.5/TV
        D2=POTLO(1)/TV
      ELSE
        T2000=IP(IONN)*2000./11.
        IT=MAX(1,MIN(9,INT(TT/T2000-.5)))
        DT=TT/T2000-FLOAT(IT)-.5
        PMIN=1.
        I=(IT+1)/2
        K1=NNNPFN(I,N)/100000
        K2=NNNPFN(I,N)-K1*100000
        K3=K2/10
        KSCALE=K2-K3*10
        IF(MOD(IT,2).EQ.0) THEN
          P1=K3*SCALE(KSCALE)
          K1=NNNPFN(I+1,N)/100000
          KSCALE=MOD(NNNPFN(I+1,N),10)
          P2=K1*SCALE(KSCALE)
        ELSE
          P1=K1*SCALE(KSCALE)
          P2=K3*SCALE(KSCALE)
          IF(DT.LT.0.AND.KSCALE.LE.1) KP1=P1
          IF(DT.LT.0.AND.KSCALE.LE.1.AND.KP1.EQ.INT(P2+.5)) PMIN=KP1
        END IF
        PART(IONN)=MAX(PMIN,P1+(P2-P1)*DT)
        IF(G.EQ.0.0.OR.POTLO(IONN).LT.0.1.OR.TT.LT.T2000*4.0) GO TO 19
        IF(TT.GT.(T2000*11.)) TV=(T2000*11.)*8.6171E-5
        D1=.1/TV
      END IF
      D2=POTLO(IONN)/TV
      PART(IONN)=PART(IONN)+G*EXP(-IP(IONN)/TV)*
     *           (SQRT(13.595*Z*Z/TV/D2)**3*
     *           (1./3.+(1.-(.5+(1./18.+D2/120.)*D2)*D2)*D2)-
     -           SQRT(13.595*Z*Z/TV/D1)**3*
     *           (1./3.+(1.-(.5+(1./18.+D1/120.)*D1)*D1)*D1))
      TV=TTKEV
  19  CONTINUE
C      write(*,'(I5,F8.3,E10.3)') IONN,IP(IONN),PART(IONN)
C
      N=N-NION2
      CF=2.*2.4148E15*TT*SQRT(TT)/XNELEC
      DO 20 IONN=2,NION2
      N=N+1
C
C   MIN IS FOR ANY UNFORTUNATE WHO HAS A 360
C
      FEXARG=(IP(IONN-1)-POTLO(IONN-1))/TV
      IF(FEXARG.GT.80.) THEN
        F(IONN)=0.
      ELSE
        F(IONN)=CF*PART(IONN)/PART(IONN-1)*EXP(-FEXARG)
      END IF
  20  CONTINUE
      F(1)=1.
      DO 21 IONN=NION2,2,-1
  21  F(1)=1.+F(IONN)*F(1)
      F(1)=1./F(1)
      FRCT(1)=F(1)/PART(1)
      POTI(1)=IP(1)
      DO 22 IONN=2,NION2
      POTI(IONN)=IP(IONN)
      F(IONN)=F(IONN-1)*F(IONN)
      FRCT(IONN)=F(IONN)/PART(IONN)
  22  CONTINUE
c      write(*,*) NION2     
c      write(*,'(2F8.3,2E10.3)') IP(1),IP(2),F(1),F(2)
c      write(*,'(I5,F8.3,4E10.3)') IONN,IP(IONN),F(IONN),FRCT(IONN)
C
C  SPECIAL PROVISION FOR NEUTRAL HYDROGEN AND HELIUM:
C  STORE FRACTIONS WITHOUT DIVIDING BY PF
C
c      if(IEL.eq.2) write(*,'(F10.0,1P5E11.3)') TT,XNELEC,
c     *  XNATOM*F(1)*0.1,XNATOM*F(2)*0.1,XNATOM*F(3)*0.1,PART(1)
      IF(IEL.EQ.1) FRCT(3)=F(1)
      IF(IEL.EQ.2) FRCT(4)=F(1)
C
      RETURN
      END


      SUBROUTINE HOP
C
C     REQUIRES FUNCTIONS COULX AND COULFF
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      DOUBLE PRECISION H,XR,C,EX,CFREE
      DOUBLE PRECISION CONT(8),BOLT(MOSIZE,8),EXLIM(MOSIZE),
     *          FREET(MOSIZE),BOLTEX(MOSIZE)
C
      DO 2 J=1,NRHOX
      DO 1 N=1,8
   1  BOLT(J,N)=EXP(-13.595*(1.-1./FLOAT(N*N))/TKEV(J))*2.*FLOAT(N*N)*
     *XNFPH(J,1)/RHO(J)
      FREET(J)=XNE(J)*XNFPH(J,2)/(SQRT(T(J))*RHO(J))
C
C A bug found by OK in Aug. 2000 was this factor 4 in the line below
C      XR=XNFPH(J,1)*4./13.595*TKEV(J)/RHO(J)
C this should be:
C      XR=XNFPH(J,1)*(2/2/13.595)*TKEV(J)/RHO(J)
      XR=XNFPH(J,1)/13.595*TKEV(J)/RHO(J)
      BOLTEX(J)=EXP(-13.427/TKEV(J))*XR
   2  EXLIM(J)=EXP(-13.595/TKEV(J))*XR
      DO 3 N=1,8
   3  CONT(N)=COULX(N,FREQ,1.)
      CFREE=3.6919D8/FREQ**2
      C=((2.815D29/FREQ)/FREQ)/FREQ
      DO 5 J=1,NRHOX
      EX=BOLTEX(J)
      IF(FREQ.LT.4.05933E13) EX=EXLIM(J)/EHVKT(J)
      H=(CONT(7)*BOLT(J,7)+CONT(8)*BOLT(J,8)+(EX-EXLIM(J))*C+COULFF(J,1)
     *    *FREET(J)/FREQ*CFREE)*STIM(J)
      DO 4 N=1,6
   4  H=H+CONT(N)*BOLT(J,N)*(1.-EHVKT(J))
   5  AHYD(J)=H
      RETURN
      END

      FUNCTION COULX(N,FREQ,Z)
      DIMENSION A(6),B(6),C(6)
      SAVE A,B,C
      DATA A/0.9916,1.105,1.101,1.101,1.102,1.0986/
      DATA B/2.719E3,-2.375E4,-9.863E3,-5.765E3,-3.909E3,-2.704E3/
      DATA C/-2.268E10,4.077E8,1.035E8,4.593E7,2.371E7,1.229E7/
C
      IF(FREQ.LT.Z*Z*3.28805E15/FLOAT(N*N)) GO TO 1
      FREQ1=FREQ*1.E-10
      COULX=0.2815/FREQ1/FREQ1/FREQ1/FLOAT(N**5)*Z**4
      IF(N.GT.6) RETURN
      COULX=COULX*(A(N)+(B(N)+C(N)*(Z*Z/FREQ1))*(Z*Z/FREQ1))
      RETURN
 1    COULX=0.
      RETURN
      END

      FUNCTION COULFF(J,NZ)
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      DIMENSION Z4LOG(6),A(11,12)
      SAVE Z4LOG,A
      DATA Z4LOG/0.,1.20412,1.90849,2.40824,2.79588,3.11261/
      DATA A/
     1 5.53,5.49,5.46,5.43,5.40,5.25,5.00,4.69,4.48,4.16,3.85,
     2 4.91,4.87,4.84,4.80,4.77,4.63,4.40,4.13,3.87,3.52,3.27,
     3 4.29,4.25,4.22,4.18,4.15,4.02,3.80,3.57,3.27,2.98,2.70,
     4 3.64,3.61,3.59,3.56,3.54,3.41,3.22,2.97,2.70,2.45,2.20,
     5 3.00,2.98,2.97,2.95,2.94,2.81,2.65,2.44,2.21,2.01,1.81,
     6 2.41,2.41,2.41,2.41,2.41,2.32,2.19,2.02,1.84,1.67,1.50,
     7 1.87,1.89,1.91,1.93,1.95,1.90,1.80,1.68,1.52,1.41,1.30,
     8 1.33,1.39,1.44,1.49,1.55,1.56,1.51,1.42,1.33,1.25,1.17,
     9 0.90,0.95,1.00,1.08,1.17,1.30,1.32,1.30,1.20,1.15,1.11,
     A 0.55,0.58,0.62,0.70,0.85,1.01,1.15,1.18,1.15,1.11,1.08,
     B 0.33,0.36,0.39,0.46,0.59,0.76,0.97,1.09,1.13,1.10,1.08,
     C 0.19,0.21,0.24,0.28,0.38,0.53,0.76,0.96,1.08,1.09,1.09/
C
C     GAMLOG=LOG10(158000*Z*Z/T)*2
C
      GAMLOG=10.39638-TLOG(J)/1.15129+Z4LOG(NZ)
      IGAM=MAX(MIN(INT(GAMLOG+7.),10),1)
C
C     HVKTLG=2*LOG10(HVKT)
C
      HVKTLG=(FREQLG-TLOG(J))/1.15129-20.63764
      IHVKT=MAX(MIN(INT(HVKTLG+9.),11),1)
      P=GAMLOG-FLOAT(IGAM-7)
      Q=HVKTLG-FLOAT(IHVKT-9)
      COULFF=(1.-P)*((1.-Q)*A(IGAM,IHVKT)+Q*A(IGAM,IHVKT+1))+
     +P*((1.-Q)*A(IGAM+1,IHVKT)+Q*A(IGAM+1,IHVKT+1))
      RETURN
      END

      SUBROUTINE HRAYOP
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      WAVE=2.997925E18/MIN(FREQ,2.463E15)
      WW=WAVE**2
      SIG=(5.799E-13+1.422E-6/WW+2.784/(WW*WW))/(WW*WW)
      DO 1 J=1,NRHOX
   1  SIGH(J)=SIG*XNFPH(J,1)*2./RHO(J)
      RETURN
      END

      SUBROUTINE H2PLOP
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
      DOUBLE PRECISION ES,FR
C
      IF(FREQ.GT.3.28805E15) RETURN
      FR=-3.0233D3+(3.7797D2+(-1.82496D1+(3.9207D-1-3.1672D-3*FREQLG)*
     *FREQLG)*FREQLG)*FREQLG
      FREQ15=FREQ*1.E-15
      ES=-7.342E-3+(-2.409+(1.028+(-0.4230+(0.1224-0.01351*FREQ15)*
     *   FREQ15)*FREQ15)*FREQ15)*FREQ15
      DO 1 J=1,NRHOX
      AH2P(J)=EXP(-ES/TKEV(J)+FR)*2.*XNFPH(J,1)*XNFPH(J,2)/
     /        RHO(J)*STIM(J)
   1  CONTINUE
C
      RETURN
      END
 
      SUBROUTINE HMINOP
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      DIMENSION XHMIN(MOSIZE)
      DOUBLE PRECISION H,HMINBF,HMINFF
C
      DO 1 J=1,NRHOX
   1  XHMIN(J)=EXP(0.7552/TKEV(J))/(2.*2.4148E15*T(J)*SQRT(T(J)))*
     *XNFPH(J,1)*XNE(J)
      FREQ1=FREQ*1.E-10
      B=(1.3727E-15+4.3748/FREQ)/FREQ1
      C=-2.5993E-7/FREQ1**2
      IF(FREQ.LE.1.8259E14) GO TO 3
      IF(FREQ.GE.2.111E14) GO TO 2
      HMINBF=3.695E-6+(-1.251E-1+1.052E3/FREQ1)/FREQ1
      GO TO 4
   2  HMINBF=6.801E-10+(5.358E-3+(1.481E3+(-5.519E7+4.808E11/FREQ1)/
     /FREQ1)/FREQ1)/FREQ1
      GO TO 4
   3  HMINBF=0.
   4  DO 5 J=1,NRHOX
      HMINFF=(B+C/T(J))*XNFPH(J,1)*XNE(J)*2.E-20/RHO(J)
      H=HMINBF*(1-EHVKT(J))*XHMIN(J)/RHO(J)*1.E-10
   5  AHMIN(J)=H+HMINFF
      RETURN
      END
 
      SUBROUTINE HE1OP
C
C     REQUIRES FUNCTIONS COULX AND COULFF
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      DOUBLE PRECISION CHI(10),BOLT(MOSIZE,10),EXLIM(MOSIZE),HEFREQ(10),
     *          TRANS(10),G(10),BOLTEX(MOSIZE),FREET(MOSIZE),
     *          CFREE,FREQ3,C,EX,XRLOG,HE1
      SAVE G, HEFREQ,CHI
C
      DATA G/1.,3.,1.,9.,3.,3.,1.,9.,20.,3./
      DATA HEFREQ/5.945209E15,1.152844E15,.9803331E15,.8761076E15,
     1             .814710E15,.4519048E15,.4030971E15,.8321191E15,
     2             .3660215E15,.3627891E15/
      DATA CHI/0.,19.819,20.615,20.964,21.217,22.718,22.920,23.006,
     1            23.073,23.086/
C
      DO 2 J=1,NRHOX
      DO 1 N=1,10
   1  BOLT(J,N)=EXP(-CHI(N)/TKEV(J)+LOG(XNFPHE(J,1))-LOG(RHO(J)))*G(N)
      FREET(J)=XNE(J)*1.E-10*XNFPHE(J,2)*1.E-10/
     /         RHO(J)/SQRT(T(J))*1.E-10
c      XRLOG=LOG(XNFPHE(J,1)*(4/2/13.595)*TKEV(J)/RHO(J))
      XRLOG=LOG(XNFPHE(J,1)*(2./13.595)*TKEV(J)/RHO(J))
      BOLTEX(J)=EXP(-23.730/TKEV(J)+XRLOG)
   2  EXLIM(J) =EXP(-24.587/TKEV(J)+XRLOG)
      FREQ3=(FREQ*1.D-10)**3
      CFREE=3.6919D8/FREQ3
      C=2.815D-1/FREQ3
      DO 3 NMIN=1,10
      TRANS(NMIN)=0.
      IF(HEFREQ(NMIN).LE.FREQ) GO TO 4
   3  CONTINUE
      GO TO 15
   4  GO TO(5,6,7,8,9,10,11,12,13,14),NMIN
   5  TRANS(1)=EXP(33.32-2.*FREQLG)
   6  TRANS(2)=EXP(-390.026+(21.035-0.318*FREQLG)*FREQLG)
   7  TRANS(3)=EXP(26.83-1.91*FREQLG)
   8  TRANS(4)=EXP(61.21-2.9*FREQLG)
   9  TRANS(5)=EXP(81.35-3.5*FREQLG)
  10  TRANS(6)=EXP(12.69-1.54*FREQLG)
  11  TRANS(7)=EXP(23.85-1.86*FREQLG)
  12  TRANS(8)=EXP(49.30-2.60*FREQLG)
  13  TRANS(9)=EXP(85.20-3.69*FREQLG)
  14  TRANS(10)=EXP(58.81-2.89*FREQLG)
  15  DO 17 J=1,NRHOX
      EX=BOLTEX(J)
      IF(FREQ.LT.2.055E14) EX=EXLIM(J)/EHVKT(J)
      HE1=(EX-EXLIM(J))*C
      DO 16 N=1,10
  16  HE1=HE1+TRANS(N)*BOLT(J,N)
  17  AHE1(J)=(HE1+COULFF(J,1)*FREET(J)*CFREE)*STIM(J)
      RETURN
      END
 
      SUBROUTINE HE2OP
C
C     REQUIRES FUNCTIONS COULX AND COULFF
C     FREQUENCIES ARE 4X HYDROGEN,CHI ARE FOR ION POT=54.403
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      DOUBLE PRECISION HE2,HETMP,C,XRLOG,FREQ3,EX
      DIMENSION CONT(9),BOLT(MOSIZE,9),EXLIM(MOSIZE),
     *          FREET(MOSIZE),BOLTEX(MOSIZE)
C
      DO 2 J=1,NRHOX
      DO 1 N=1,9
      BLTARG=(54.403-54.403/FLOAT(N*N))/TKEV(J)+LOG(RHO(J))
      IF(XNFPHE(J,2).EQ.0.0.OR.BLTARG.GT.80.) THEN
        BOLT(J,N)=0.
      ELSE
        BOLT(J,N)=EXP(-BLTARG)*2.*FLOAT(N*N)*XNFPHE(J,2)
      END IF
   1  CONTINUE
      FREET(J)=XNE(J)*XNFPHE(J,3)/SQRT(T(J))/RHO(J)
c      XRLOG=LOG(TKEV(J)*(2/2/13.595)/RHO(J))
      XRLOG=LOG(TKEV(J)/13.595/RHO(J))
      BLTLOG=53.859/TKEV(J)-XRLOG
      IF(XNFPHE(J,2).EQ.0.0.OR.BLTLOG.GT.80.) THEN
        BOLTEX(J)=0.
      ELSE
        BOLTEX(J)=XNFPHE(J,2)*EXP(-BLTLOG)
      END IF
      EXLLOG=54.403/TKEV(J)-XRLOG
      IF(XNFPHE(J,2).EQ.0.0.OR.EXLLOG.GT.80.) THEN
        EXLIM(J)=0.
      ELSE
        EXLIM(J)=XNFPHE(J,2)*EXP(-EXLLOG)
      END IF
   2  CONTINUE
      DO 3 N=1,9
   3  CONT(N)=COULX(N,FREQ,2.)
      FREQ3=(FREQ*1.D-05)**3
      CFREE=3.6919D-07/FREQ3*4.
      C=2.815E+14*2.*2./FREQ3
      DO 5 J=1,NRHOX
      EX=BOLTEX(J)
      IF(FREQ.LT.1.31522E14) EX=EXLIM(J)/EHVKT(J)
      HE2=(EX-EXLIM(J))*C
      DO 4 N=1,9
      HETMP=CONT(N)
   4  HE2=HE2+HETMP*BOLT(J,N)
      HE2=(HE2+COULFF(J,2)*CFREE*FREET(J))*STIM(J)
      IF(HE2.LT.1.E-30) THEN
        AHE2(J)=0.
      ELSE
        AHE2(J)=HE2
      END IF
   5  CONTINUE
      RETURN
      END
 
      SUBROUTINE HEMIOP
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      A= 3.397E-26+(-5.216E-11+7.039E05/FREQ)/FREQ
      B=-4.116E-22+( 1.067E-06+8.135E09/FREQ)/FREQ
      C= 5.081E-17+(-8.724E-03-5.659E12/FREQ)/FREQ
      DO 1 J=1,NRHOX
   1  AHEMIN(J)=(A*T(J)+B+C/T(J))*XNE(J)*XNFPHE(J,1)/RHO(J)*1.E-20
      RETURN
      END
 
      SUBROUTINE HERAOP
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      WAVE=2.997925E+03/MIN(FREQ*1.E-15,5.15)
      WW=WAVE*WAVE
      SIG=5.484E-14/WW/WW*(1.+(2.44E5+5.94E10/(WW-2.90E5))/WW)**2
      DO 1 J=1,NRHOX
   1  SIGHE(J)=SIG*XNFPHE(J,1)/RHO(J)
      RETURN
      END
 
      SUBROUTINE COOLOP
C
C     SI1,MG1,AL1,C1
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      DIMENSION XNFPC(MOSIZE),XNFPMG(MOSIZE),XNFPAL(MOSIZE),
     *          XNFPSI(MOSIZE),XNFPFE(MOSIZE)
      REAL C1OP,MG1OP,AL1OP,SI1OP
C
      CALL POPS( 6.,11,XNFPC)
      CALL POPS(12.,11,XNFPMG)
      CALL POPS(13.,11,XNFPAL)
      CALL POPS(14.,11,XNFPSI)
      CALL POPS(26.,11,XNFPFE)
      DO 1 J=1,NRHOX
  1   ACOOL(J)=(C1OP(J)*XNFPC(J)+MG1OP(J)*XNFPMG(J)+AL1OP(J)*XNFPAL(J)+
     +          SI1OP(J)*XNFPSI(J)+FE1OP(J)*XNFPFE(J))*STIM(J)/RHO(J)
      RETURN
      END
 
      FUNCTION C1OP(J)
C
C     CROSS-SECTION TIMES THE PARTITION FUNCTION
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      C1240=5.*EXP(-1.264/TKEV(J))
      C1444=EXP(-2.683/TKEV(J))
      X1444=0.
      X1240=0.
      X1100=0.
      IF(FREQ.GE.2.7254E15) X1100=SEATON(2.7254E15,1.219E-17,2.,3.317)
      IF(FREQ.GE.2.4196E15) X1240=SEATON(2.4196E15,1.03E-17,1.5,2.789)
      IF(FREQ.GE.2.0761E15) X1444=SEATON(2.0761E15,9.59E-18,1.5,3.501)
      C1OP=X1100*9.+X1240*C1240+X1444*C1444
C
      RETURN
      END
 
      REAL FUNCTION MG1OP(J)
C
C     CROSS-SECTION TIMES THE PARTITION FUNCTION
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      DIMENSION FLOG(9),FREQMG(7),PEACH(7,15),TLG(7)
      SAVE PEACH,FREQMG,FLOG,TLG
      DATA PEACH/
C  TEMP:  4000     5000     6000     7000     8000     9000    10000     WAVE(A)
     1 -42.474, -42.350, -42.109, -41.795, -41.467, -41.159, -40.883,     1500
     2 -41.808, -41.735, -41.582, -41.363, -41.115, -40.866, -40.631,     1550
     2 -41.273, -41.223, -41.114, -40.951, -40.755, -40.549, -40.347,     1621
     3 -45.583, -44.008, -42.957, -42.205, -41.639, -41.198, -40.841,     1622
     3 -44.324, -42.747, -41.694, -40.939, -40.370, -39.925, -39.566,     2513
     4 -50.969, -48.388, -46.630, -45.344, -44.355, -43.568, -42.924,     2514
     4 -50.633, -48.026, -46.220, -44.859, -43.803, -42.957, -42.264,     3756
     5 -53.028, -49.643, -47.367, -45.729, -44.491, -43.520, -42.736,     3757
     5 -51.785, -48.352, -46.050, -44.393, -43.140, -42.157, -41.363,     6549
     6 -52.285, -48.797, -46.453, -44.765, -43.486, -42.480, -41.668,     6550
     6 -52.028, -48.540, -46.196, -44.507, -43.227, -42.222, -41.408,     7234
     7 -52.384, -48.876, -46.513, -44.806, -43.509, -42.488, -41.660,     7235
     7 -52.363, -48.856, -46.493, -44.786, -43.489, -42.467, -41.639,     7291
     8 -54.704, -50.772, -48.107, -46.176, -44.707, -43.549, -42.611,     7292
     9 -54.359, -50.349, -47.643, -45.685, -44.198, -43.027, -42.418/     9000
      DATA FREQMG/1.9341452E15,1.8488510E15,1.1925797E15,7.9804046E14,
     1            4.5772110E14,4.1440977E14,4.1113514E14/
      DATA FLOG/35.32123,35.19844,35.15334,34.71490,34.31318,33.75728,
     1          33.65788,33.64994,33.43947/
      DATA TLG/8.29405,8.51719,8.69951,8.85367,8.98720,9.10498,9.21034/
C
      NT=MAX(MIN(6,INT(T(J)/1000.)-3),1)
      DT=(TLOG(J)-TLG(NT))/(TLG(NT+1)-TLG(NT))
      DO 1 N=1,7
      IF(FREQ.GT.FREQMG(N)) GO TO 2
   1  CONTINUE
      N=8
   2  D=(FREQLG-FLOG(N))/(FLOG(N+1)-FLOG(N))
      IF(N.GT.2) N=2*N-2
      D1=1.0-D
      XWL1=PEACH(NT  ,N+1)*D+PEACH(NT  ,N)*D1
      XWL2=PEACH(NT+1,N+1)*D+PEACH(NT+1,N)*D1
      MG1OP=EXP(XWL1*(1.0-DT)+XWL2*DT)
C
      RETURN
      END
 
      FUNCTION AL1OP(J)
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      AL1OP=0.0
      IF(FREQ.GE.1.443E15) AL1OP=2.1E-17*(1.443E15/FREQ)**3*6.
      RETURN
      END
 
      FUNCTION SI1OP(J)
C
C     CROSS-SECTION TIMES THE PARTITION FUNCTION
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      DIMENSION FLOG(11),FREQSI(9),TLG(9),PEACH(9,19)
      SAVE PEACH,FREQSI,FLOG,TLG
C
      DATA PEACH/
C TEMP:  4000   5000   6000   7000   8000   9000  10000  11000  12000    WAVE(A)
     1 38.136,38.138,38.140,38.141,38.143,38.144,38.144,38.145,38.145,   1200
     2 37.834,37.839,37.843,37.847,37.850,37.853,37.855,37.857,37.858,   1400
     2 37.898,37.898,37.897,37.897,37.897,37.896,37.895,37.895,37.894,   1519
     3 40.737,40.319,40.047,39.855,39.714,39.604,39.517,39.445,39.385,   1520
     3 40.581,40.164,39.893,39.702,39.561,39.452,39.366,39.295,39.235,   1676
     4 45.521,44.456,43.753,43.254,42.878,42.580,42.332,42.119,41.930,   1677
     4 45.520,44.455,43.752,43.251,42.871,42.569,42.315,42.094,41.896,   1978
     5 55.068,51.783,49.553,47.942,46.723,45.768,44.997,44.360,43.823,   1979
     5 53.868,50.369,48.031,46.355,45.092,44.104,43.308,42.652,42.100,   5379
     6 54.133,50.597,48.233,46.539,45.261,44.262,43.456,42.790,42.230,   5380
     6 54.051,50.514,48.150,46.454,45.176,44.175,43.368,42.702,42.141,   5624
     7 54.442,50.854,48.455,46.733,45.433,44.415,43.592,42.912,42.340,   5625
     7 54.320,50.722,48.313,46.583,45.277,44.251,43.423,42.738,42.160,   6260
     8 55.691,51.965,49.444,47.615,46.221,45.119,44.223,43.478,42.848,   6261
     8 55.661,51.933,49.412,47.582,46.188,45.085,44.189,43.445,42.813,   6349
     9 55.973,52.193,49.630,47.769,46.349,45.226,44.314,43.555,42.913,   6350
     9 55.922,52.141,49.577,47.715,46.295,45.172,44.259,43.500,42.858,   6491
     A 56.828,52.821,50.110,48.146,46.654,45.477,44.522,43.730,43.061,   6492
     A 56.657,52.653,49.944,47.983,46.491,45.315,44.360,43.569,42.901/   6900
C     3P,1D,1S,1D,3D,3F,1D,3P
      DATA FREQSI/2.1413750E15,1.9723165E15,1.7879689E15,1.5152920E15,
     1 .55723927E15,5.3295914E14,4.7886458E14,4.7216422E14,4.6185133E14/
      DATA FLOG/35.45438,35.30022,35.21799,35.11986,34.95438,33.95402,
     1          33.90947,33.80244,33.78835,33.76626,33.70518/
      DATA TLG/8.29405,8.51719,8.69951,8.85367,8.98720,9.10498,
     1         9.21034,9.30565,9.39266/
C
      NT=MAX(MIN(8,INT(T(J)/1000.)-3),1)
      DT=(TLOG(J)-TLG(NT))/(TLG(NT+1)-TLG(NT))
      DO 1 N=1,9
      IF(FREQ.GT.FREQSI(N)) GO TO 2
   1  CONTINUE
      N=10
   2  D=(FREQLG-FLOG(N))/(FLOG(N+1)-FLOG(N))
      IF(N.GT.2) N=2*N-2
      DD=1.-D
      XWL1=PEACH(NT  ,N+1)*D+PEACH(NT  ,N)*DD
      XWL2=PEACH(NT+1,N+1)*D+PEACH(NT+1,N)*DD
      SI1OP=EXP(-(XWL1*(1.-DT)+XWL2*DT))*9.
C
      RETURN
      END

      FUNCTION FE1OP(J)
C
C     CROSS-SECTION TIMES PARTITION FUNCTION
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      DIMENSION BOLT(48),G(48),E(48),WNO(48),XSECT(48)
      SAVE G,E,WNO
      DATA G/25.,35.,21.,15.,9.,35.,33.,21.,27.,49.,9.,21.,27.,9.,9.,
     1 25.,33.,15.,35.,3.,5.,11.,15.,13.,15.,9.,21.,15.,21.,25.,35.,
     2 9.,5.,45.,27.,21.,15.,21.,15.,25.,21.,35.,5.,15.,45.,35.,55.,25./
      DATA E/    500., 7500.,12500.,17500.,19000.,19500.,19500.,21000.,
     1  22000.,23000.,23000.,24000.,24000.,24500.,24500.,26000.,26500.,
     2  26500.,27000.,27500.,28500.,29000.,29500.,29500.,29500.,30000.,
     3  31500.,31500.,33500.,33500.,34000.,34500.,34500.,35000.,35500.,
     4  37000.,37000.,37000.,38500.,40000.,40000.,41000.,41000.,43000.,
     5  43000.,43000.,43000.,44000./
      DATA WNO/63500.,58500.,53500.,59500.,45000.,44500.,44500.,43000.,
     1  58000.,41000.,54000.,40000.,40000.,57500.,55500.,38000.,57500.,
     2  57500.,37000.,54500.,53500.,55000.,34500.,34500.,34500.,34000.,
     3  32500.,32500.,32500.,32500.,32000.,29500.,29500.,31000.,30500.,
     4  29000.,27000.,54000.,27500.,24000.,47000.,23000.,44000.,42000.,
     5  42000.,21000.,42000.,42000./
C
      WAVENO=FREQ/2.99792458E10
      FE1OP=0.
      IF(WAVENO.LT.21000.) RETURN
      DO 1 I=1,48
   1  BOLT(I)=G(I)*EXP(-E(I)*2.99792458E10*HKT(J))
      DO 2 I=1,48
      XSECT(I)=0.
      IF(WNO(I).LT.WAVENO) XSECT(I)=3.E-18/
     /                         (1.+((WNO(I)+3000.-WAVENO)/WNO(I)/.1)**4)
   2  CONTINUE
      DO 3 I=1,48
   3  FE1OP=FE1OP+XSECT(I)*BOLT(I)
C
      RETURN
      END
 
      SUBROUTINE LUKEOP
C
C     SI2,MG2,CA2,N1,O1
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      DIMENSION XNFPN(MOSIZE),XNFPO(MOSIZE),XNFPMG(MOSIZE),
     *          XNFPSI(MOSIZE),XNFPCA(MOSIZE)
      REAL N1OP,MG2OP
C
      CALL POPS( 7.00,1,XNFPN)
      CALL POPS( 8.00,1,XNFPO)
      CALL POPS(12.01,1,XNFPMG)
      CALL POPS(14.01,1,XNFPSI)
      CALL POPS(20.01,1,XNFPCA)
      DO 1 J=1,NRHOX
   1  ALUKE(J)=(N1OP(J)*XNFPN(J)+O1OP(J)*XNFPO(J)+MG2OP(J)*XNFPMG(J)+
     +SI2OP(J)*XNFPSI(J)+CA2OP(J)*XNFPCA(J))*STIM(J)/RHO(J)
      RETURN
      END
 
      REAL FUNCTION N1OP(J)
C
C     CROSS-SECTION TIMES PARTITION FUNCTION
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      C1130=6.*EXP(-3.575/TKEV(J))
      C1020=10.*EXP(-2.384/TKEV(J))
      X1130=0.
      X1020=0.
      X853=0.
      IF(FREQ.GE.3.517915E15) X853=SEATON(3.517915E15,1.142E-17,2.,4.29)
      IF(FREQ.GE.2.941534E15)X1020=SEATON(2.941534E15,4.41E-18,1.5,3.85)
      IF(FREQ.GE.2.653317E15)X1130=SEATON(2.653317E15,4.20E-18,1.5,4.34)
      N1OP=X853*4.+X1020*C1020+X1130*C1130
C
      RETURN
      END
 
      FUNCTION O1OP(J)
C
C     CROSS-SECTION TIMES PARTITION FUNCTION
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      IF(FREQ.GE.3.28805E15) THEN
        O1OP=9.*SEATON(3.28805E15,2.94E-18,1.,2.66)
      ELSE
        O1OP=0.
      END IF
      RETURN
      END
 
      REAL FUNCTION MG2OP(J)
C
C     CROSS-SECTION TIMES PARTITION FUNCTION
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      C1169=6.*EXP(-4.43/TKEV(J))
      X1169=0.
      X824=0.
      IF(FREQ.GE.3.635492E15) X824=SEATON(3.635492E15,1.40E-19,4.,6.7)
      IF(FREQ.GE.2.564306E15) X1169=5.11E-19*(2.564306E15/FREQ)**3
      MG2OP=X824*2.+X1169*C1169
C
      RETURN
      END
 
      FUNCTION SI2OP(J)
C
C     CROSS-SECTION TIMES THE PARTITION FUNCTION
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      DIMENSION FLOG(9),FREQSI(7),PEACH(6,14),TLG(6)
      SAVE PEACH,FREQSI,FLOG,TLG
C
      DATA PEACH/
C         10000     12000     14000     16000     18000     20000       WAVE(A)
     1  -43.8941, -43.8941, -43.8941, -43.8941, -43.8941, -43.8941,      500
     2  -42.2444, -42.2444, -42.2444, -42.2444, -42.2444, -42.2444,      600
     3  -40.6054, -40.6054, -40.6054, -40.6054, -40.6054, -40.6054,      759
     4  -54.2389, -52.2906, -50.8799, -49.8033, -48.9485, -48.2490,      760
     5  -50.4108, -48.4892, -47.1090, -46.0672, -45.2510, -44.5933,     1905
     6  -52.0936, -50.0741, -48.5999, -47.4676, -46.5649, -45.8246,     1906
     7  -51.9548, -49.9371, -48.4647, -47.3340, -46.4333, -45.6947,     1975
     8  -54.2407, -51.7319, -49.9178, -48.5395, -47.4529, -46.5709,     1976
     9  -52.7355, -50.2218, -48.4059, -47.0267, -45.9402, -45.0592,     3245
     T  -53.5387, -50.9189, -49.0200, -47.5750, -46.4341, -45.5082,     3246
     1  -53.2417, -50.6234, -48.7252, -47.2810, -46.1410, -45.2153,     3576
     2  -53.5097, -50.8535, -48.9263, -47.4586, -46.2994, -45.3581,     3577
     3  -54.0561, -51.2365, -49.1980, -47.6497, -46.4302, -45.4414,     3900
     4  -53.8469, -51.0256, -48.9860, -47.4368, -46.2162, -45.2266/     4200
      DATA FREQSI/4.9965417E15,3.9466738E15,1.5736321E15,1.5171539E15,
     1            9.2378947E14,8.3825004E14,7.6869872E14/
C     2P,2D,2P,2D,2P
      DATA FLOG/36.32984,36.14752,35.91165,34.99216,34.95561,34.45941,
     1          34.36234,34.27572,34.20161/
      DATA TLG/9.21034,9.39266,9.54681,9.68034,9.79813,9.90349/
C
      NT=MAX(MIN(5,INT(T(J)/2000.)-4),1)
      DT=(TLOG(J)-TLG(NT))/(TLG(NT+1)-TLG(NT))
      DO 1 N=1,7
      IF(FREQ.GT.FREQSI(N)) GO TO 2
   1  CONTINUE
      N=8
   2  D=(FREQLG-FLOG(N))/(FLOG(N+1)-FLOG(N))
      IF(N.GT.2) N=2*N-2
      IF(N.EQ.14) N=13
      D1=1.-D
      XWL1=PEACH(NT  ,N+1)*D+PEACH(NT  ,N)*D1
      XWL2=PEACH(NT+1,N+1)*D+PEACH(NT+1,N)*D1
      SI2OP=EXP(XWL1*(1.-DT)+XWL2*DT)*6.
C
      RETURN
      END
 
      FUNCTION CA2OP(J)
C
C     CROSS-SECTION TIMES THE PARTITION FUNCTION
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      C1218=10.*EXP(-1.697/TKEV(J))
      C1420=6.*EXP(-3.142/TKEV(J))
      X1420=0.
      X1218=0.
      X1044=0.
      IF(FREQ.GE.2.870454E15) X1044=1.08E-19*(2.870454E15/FREQ)**3
      IF(FREQ.GE.2.460127E15) X1218=1.64E-17*SQRT(2.460127E15/FREQ)
      IF(FREQ.GE.2.110779E15) X1420=SEATON(2.110779E15,4.13E-18,3.,0.69)
      CA2OP=X1044+X1218*C1218+X1420*C1420
C
      RETURN
      END
 
      FUNCTION SEATON(FREQ0,XSECT,POWER,A)
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      SEATON=XSECT*(A+(1.-A)*(FREQ0/FREQ))*
     *SQRT((FREQ0/FREQ)**(INT(2.*POWER+0.01)))
      RETURN
      END
 
      SUBROUTINE HOTOP
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
C   DUMMY
C
      RETURN
      END
 
      SUBROUTINE ELECOP
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
C
      DO 1 J=1,NRHOX
 1    SIGEL(J)=0.6653E-24*XNE(J)/RHO(J)
      RETURN
      END
 
      SUBROUTINE H2RAOP
C
      INCLUDE 'SIZES.SYN'
      INCLUDE 'COMMONS.SYN'
      DOUBLE PRECISION ARG,H1,WAVE,WW,SIG,XNH2
C
      WAVE=2.997925D18/MIN(FREQ,2.922E15)
      WW=WAVE**2
      SIG=(8.14E-13+1.28E-6/WW+1.61/(WW*WW))/(WW*WW)
      DO 1 J=1,NRHOX
      ARG=4.477/TKEV(J)-4.6628E1+(1.8031E-3+(-5.023E-7+
     +    (8.1424E-11-5.0501E-15*T(J))*T(J))*T(J))*T(J)-1.5*TLOG(J)
      H1=XNFPH(J,1)*2.
      XNH2=EXP(ARG)/RHO(J)*H1*H1
   1  SIGH2(J)=SIG*XNH2
C
      RETURN
      END




