C
C   Table of contents:
C
C     SUBROUTINE HLINPROF(WAVE,WAVE0,TEMP,XXNE,NLOW,NUP,
C    *                   H1FRC,HE1FRC,DOP,PROF)
C     SUBROUTINE STEHLE(HFILE,WAVE,WAVE0,TEMP,XXNE,NLOW,NUP,HLIN)
C     SUBROUTINE HGRID(HFILE,WAVE,WAVE0,TEMP,NHYD,NLOW,NUP,HLIN)
C     SUBROUTINE HSELF_PDWIDTH(LINE,TEMP,H1FRC,WIDTH)
C     SUBROUTINE HE_PDWIDTH(LINE,TEMP,HE1FRC,HEWID)
C     SUBROUTINE RAD_WIDTH(LINE,WIDTH)
C     SUBROUTINE ALI_GRIEM(LINE,WAVE0,H1FRC,WIDTH)
C     SUBROUTINE VCS(PR,XNE,T,DEL,II,N,M)                                      
C
C
C  Hydrogen Line Routines
C  All routines by Paul Barklem unless otherwise stated
C
      SUBROUTINE HLINPROF(WAVE,WAVE0,TEMP,XXNE,NLOW,NUP,
     *                   H1FRC,HE1FRC,DOP,PROF)
C
C  Returns H line Profile at specific point
C
C  Inputs: 
C          WAVE   - wavelength in AA
C          WAVE0  - line centre in AA
C          TEMP   - temperature in K
C          XXNE   - electron number density in cm^-3
C          NLOW   - n quantum number of lower level
C          NUP    - n quantum number of upper level
C          H1FRC  - neutral hydrogen number density in cm^-3
C          HE1FRC - neutral helium number density in cm^-3
C          DOP    - the doppler width delta lambda / lambda
C                   (only used if HLINOP is called instead!)
C  Outputs:
C          PROF   - area normalised (in AA) profile for WAVE
C  Common Inputs: 
C     (CURRENTLY SET HARD!!!! see below)
C          ISTARK - switch for different recipes
C                  0=none
C                  1=Stehle
C                  2=Vidal, Cooper & Smith
C          ISELF  - switch for self broadening
C                  0=none
C                  1=Barklem, Piskunov and O'Mara (grid)
C                  2=Barklem, Piskunov and O'Mara (p-d)
C                  3=Ali & Griem
C          ICONV  - switch for convolution method
C                  1=adding
C                  2=trapezoidal
C          IHE    - switch for broadening by He
C                  0=none
C                  1=estimated from Barklem et al (1997)
C          NB: ICONV=2 is incompatible with either 0 option 
C  
C  The trapezoidal convolution is only for testing.  Use at own risk!
C  It's left here in as it might be useful to someone.
C      
      IMPLICIT NONE

      INTEGER NKMAX
      PARAMETER (NKMAX = 10001)
      INTEGER NLOW,NUP,LINE
      INTEGER ISTARK,ISELF,ICONV
      INTEGER IHE
      INTEGER NLOWW,NHIGH,NKRN,IKRN,J,NMID,I1,I2
      REAL*4 TEMP,XXNE,PROF,H1FRC,HE1FRC
      REAL*4 HLIN,STARK
      REAL*4 HEWID,RAD,DOP
      REAL*4 HLINOP
      REAL*8 TD,XNED,H1FRCD
      REAL*8 WAVE,WAVE0,DEL,HHLIN
      REAL*8 KRNSTEP,WLKRN(NKMAX),KERN(NKMAX),KMAX,WV2
      REAL*8 DEL1,DDEL,XNEXP,F1,HLIND
      REAL*8 STARKD
      REAL*8 LOR,HWID,RES
      REAL*8 C,PI
      CHARACTER*80 HFILE,HSELFFILE
      LOGICAL FIRST
C
      SAVE FIRST 
C
      DATA FIRST/.TRUE./
      PARAMETER (C = 2.997925E+18, PI = 3.14159265)
C
C  Set the switches here 
C
      ISTARK = 1
      ISELF = 2
      ICONV = 1
      IHE = 1
C
C We check we have a line which we can handle
C  if not then pass to HLINOP
C
      IF ((NLOW .NE. 2) .OR. (NUP .GT. 5)) THEN
        IF (FIRST) THEN
          FIRST = .FALSE.
          PRINT*,' Using Hlinop code!'
        ENDIF
        PROF = HLINOP(WAVE,NLOW,NUP,WAVE0,TEMP,XXNE,H1FRC,
     ;                                          HE1FRC,DOP)
        PROF = PROF * C/WAVE/WAVE
      RETURN
      END IF 
C
C For historical reasons copy some variables to real*8
C
      XNED = XXNE
      TD = TEMP
      H1FRCD = H1FRC
C      
C Precomputed things here
C
      DEL = DABS(WAVE0 - WAVE)
      LINE = NUP - NLOW
C
C Adding for convolution of Stark and self
C
      IF (ICONV .EQ. 1) THEN
        HLIN = 0. 
        HHLIN = 0.
        IF (ISTARK .EQ. 1) THEN
          CALL STEHLE(HFILE,WAVE,WAVE0,TEMP,XXNE,NLOW,NUP,HLIN)
C
C If outside table range use VCS
C
          IF (HLIN.LT.0.) THEN
            CALL VCS(HLIND,XNED,TD,DEL,1,NLOW,NUP)
            HLIN = HLIND
          ENDIF
        END IF
        IF (ISTARK .EQ. 2) THEN
          CALL VCS(HLIND,XNED,TD,DEL,1,NLOW,NUP)
          HLIN = HLIND
        END IF
        IF ((DEL .GT. .2d0) .AND. (H1FRC .GT. 0.)) THEN
          IF (ISELF .EQ. 1) THEN
             CALL HGRID(HSELFFILE,
     ;            WAVE,WAVE0,TD,H1FRCD,NLOW,NUP,HHLIN)
C
C If outside table range use p-d approx
C
            IF (HHLIN.LT.0.) THEN
              CALL HSELF_PDWIDTH(LINE,TEMP,H1FRC,HWID)
              HWID = HWID * WAVE0 * WAVE0 / C / 2. / PI
              HHLIN = HWID / (HWID * HWID + DEL * DEL) / PI
            ENDIF
          ENDIF
          IF (ISELF .EQ. 2) THEN
            CALL HSELF_PDWIDTH(LINE,TEMP,H1FRC,HWID)
            HWID = HWID * WAVE0 * WAVE0 / C / 2. / PI
            HHLIN = HWID / (HWID * HWID + DEL * DEL) / PI
          END IF
          IF (ISELF .EQ. 3) THEN
            CALL ALI_GRIEM(LINE,WAVE0,H1FRC,RES)
            HHLIN = RES / (RES * RES + DEL * DEL) / PI
          END IF 
          HLIN = HLIN + HHLIN
        END IF
      END IF
C
C Convolution by simple trapezoidal method
C
C   Notes:  Should have Stark profile > Lorentzian
C   If ever this is broken you should reverse the two or 
C   increase NKRN points significantly.
C
      IF (ICONV.EQ.2) THEN
        IF (H1FRC .EQ. 0.) THEN
          PRINT*,'STOPPING : ZERO H1FRC, CONVOLVING WILL FAIL'
          STOP
        END IF
C
C Use a logarithmic spread of kernel points 
C
        KMAX = 100.     !we go out KMAX AA into wings
        NKRN = 1001     !use an odd number, testing may be 
C                       !needed to see what this number should 
C                       !for a given case
        NMID = (NKRN + 1) / 2
        NLOWW = NMID - 1
        NHIGH = NMID + 1
        XNEXP = DLOG(1.d-4)   !this is the low limit of detuning 
C                             !(the log of it in AA)
C                             !should be smaller than smallest half width
C                             !which are typically ~0.01AA for solar
        KRNSTEP = (LOG(KMAX) - XNEXP) / DBLE(NLOWW - 1)
        DO J = NKRN, NHIGH, -1
          WLKRN(J) = EXP(LOG(KMAX) - KRNSTEP * DBLE(NKRN - J))
        END DO
        WLKRN(NMID) = 0.d0
        DO J = 1, NLOWW, 1
          WLKRN(J) = - EXP(LOG(KMAX) - KRNSTEP * DBLE(J - 1))
        END DO
C
        DO IKRN = 1, NKRN
          WV2 = WAVE0 + WLKRN(IKRN)
            HHLIN = 0.
            IF (ISELF .EQ. 1) CALL HGRID(HSELFFILE,
     ;              WV2,WAVE0,TD,H1FRCD,NLOW,NUP,HHLIN)
            IF (ISELF .EQ. 2) THEN
              CALL HSELF_PDWIDTH(LINE,TEMP,H1FRC,HWID)
              HWID = HWID * WAVE0 * WAVE0 / C / 2. / PI
              HHLIN = HWID / (HWID * HWID + WLKRN(IKRN) * WLKRN(IKRN)) 
     ,                     / PI
            END IF
            IF (ISELF .EQ. 3) THEN
              CALL ALI_GRIEM(LINE,WAVE0,H1FRC,RES)
              HHLIN = RES / (RES * RES + WLKRN(IKRN) * WLKRN(IKRN)) / PI
            END IF 
          KERN(IKRN) = HHLIN
        END DO
C
C  Do Convolution -- Integration is simple Trapezoidal here
C
        HLIND = 0.d0
        DO IKRN = 1, NKRN
          DEL1 = DEL + WLKRN(IKRN)
          I1 = MAX0(   1, IKRN - 1)
          I2 = MIN0(NKRN, IKRN + 1)
		  DDEL = WLKRN(I2) - WLKRN(I1)
          IF (ISTARK .EQ. 1) THEN
            CALL STEHLE(HFILE,WAVE0+DEL1,WAVE0,TEMP,XXNE,NLOW,NUP,STARK)
          END IF
          IF (ISTARK .EQ. 2) THEN
            CALL VCS(STARKD,XNED,TD,DEL1,1,NLOW,NUP)
            STARK = STARKD
          END IF
          F1 = KERN(IKRN) * STARK
          HLIND = HLIND + DDEL * F1
        END DO
        HLIND= HLIND * .5d0
        HLIN = HLIND
      END IF
C
C The He and radiative broadening are always assumed small enough to 
C convolve by adding 
C
      HEWID = 0.
      IF (IHE .EQ. 1 ) CALL HE_PDWIDTH(LINE,TEMP,HE1FRC,HEWID)
      CALL RAD_WIDTH(LINE,RAD)
C
C  Compute the width in lambda units (AA)
C
      LOR = RAD + HEWID
      LOR = LOR * WAVE0 * WAVE0 / C / 2. / PI
C
      IF (DEL .GT. .2)
     ;  HLIN = HLIN + LOR / (LOR * LOR + DEL * DEL) / PI
C
      PROF = HLIN
      RETURN
C
      END

      SUBROUTINE STEHLE(HFILE,WAVE,WAVE0,TEMP,XXNE,NLOW,NUP,HLIN)
C
C  N Piskunov's code to return Stehle Stark profiles
C
      IMPLICIT NONE
      DOUBLE PRECISION WAVE,WAVE0
      REAL TEMP,XXNE,HLIN
      INTEGER NWL,NTEMP,NNE,NLINE,NLOW,NUP
      REAL SQRTPI
      PARAMETER (NWL=46,NTEMP=3,NNE=13,NLINE=6,SQRTPI=1.77245385)
      REAL T(NTEMP),XNE(NNE)
      REAL WING(NNE,NLINE),F0(NNE,NLINE),WIDTH(NTEMP,NNE,NLINE),
     *     ALPHA(NWL,NTEMP,NNE,NLINE),SPROF(NWL,NTEMP,NNE,NLINE)
      INTEGER NALPHA(NTEMP,NNE,NLINE),I,J,K,L,LINE,IT,IN,ITEMP,INE
      INTEGER IERR
      DOUBLE PRECISION EXP10
      REAL ALP,ALP1,ALP2,S11,S12,S21,S22,DT,DT1,DT2,ANE,DNE,DNE1,DNE2,
     *     DAL,DAL1,DAL2,X
      INTEGER LINDEX(2,5)
      LOGICAL FIRST
      CHARACTER HFILE*(*)
c      real talpha(NWL,NTEMP,NNE,NLINE),tprof(NWL,NTEMP,NNE,NLINE)
      SAVE T,XNE,WING,F0,WIDTH,ALPHA,SPROF,NALPHA,
     *     FIRST,LINDEX
      DATA LINDEX/0,0,
     *            1,0,
     *            2,4,
     *            3,5,
     *            0,6/
      DATA T/2500.,5000.,10000./
      DATA XNE/10.0,10.5,11.0,11.5,12.0,12.5,13.0,13.5,
     *         14.0,14.5,15.0,15.5,16.0/
      DATA F0/.005808,.012510,.026960,.058080,.125100,.269600,.580800,
     *        1.25100,2.69600,5.80800,12.5100,26.9600,58.0800,
     *        .005808,.012510,.026960,.058080,.125100,.269600,.580800,
     *        1.25100,2.69600,5.80800,12.5100,26.9600,58.0800,
     *        .005808,.012510,.026960,.058080,.125100,.269600,.580800,
     *        1.25100,2.69600,5.80800,12.5100,26.9600,58.0800,
     *        .005808,.012510,.026960,.058080,.125100,.269600,.580800,
     *        1.25100,2.69600,5.80800,12.5100,26.9600,58.0800,
     *        .005808,.012510,.026960,.058080,.125100,.269600,.580800,
     *        1.25100,2.69600,5.80800,12.5100,26.9600,58.0800,
     *        .005808,.012510,.026960,.058080,.125100,.269600,.580800,
     *        1.25100,2.69600,5.80800,12.5100,26.9600,58.0800/
      DATA WING/
     * .6731E-05,.6731E-05,.6731E-05,.6731E-05,.6731E-05,.6731E-05,
     * .6731E-05,.6731E-05,.6731E-05,.6731E-05,.6731E-05,.6731E-05,
     * .6731E-05,
     * .3586E-04,.3586E-04,.3586E-04,.3586E-04,.3586E-04,.3586E-04,
     * .3586E-04,.3586E-04,.3586E-04,.3586E-04,.3586E-04,.3586E-04,
     * .3586E-04,
     * .7965E-04,.7965E-04,.7965E-04,.7965E-04,.7965E-04,.7965E-04,
     * .7965E-04,.7965E-04,.7965E-04,.7965E-04,.7965E-04,.7965E-04,
     * .7965E-04,
     * .2564E-02,.2564E-02,.2564E-02,.2564E-02,.2564E-02,.2564E-02,
     * .2564E-02,.2564E-02,.2564E-02,.2564E-02,.2564E-02,.2564E-02,
     * .2564E-02,
     * .7062E-02,.7062E-02,.7062E-02,.7062E-02,.7062E-02,.7062E-02,
     * .7062E-02,.7062E-02,.7062E-02,.7062E-02,.7062E-02,.7062E-02,
     * .7062E-02,
     * .1189E-01,.1189E-01,.1189E-01,.1189E-01,.1189E-01,.1189E-01,
     * .1189E-01,.1189E-01,.1189E-01,.1189E-01,.1189E-01,.1189E-01,
     * .1189E-01/
      DATA WIDTH/
     * .3759E+01,.5347E+01,.7607E+01,.1764E+01,.2464E+01,.3501E+01,
     * .8166E+00,.1159E+01,.1624E+01,.3758E+00,.5348E+00,.7607E+00,
     * .1765E+00,.2464E+00,.3502E+00,.8169E-01,.1160E+00,.1625E+00,
     * .3762E-01,.5351E-01,.7610E-01,.1772E-01,.2470E-01,.3506E-01,
     * .8264E-02,.1168E-01,.1633E-01,.3884E-02,.5460E-02,.7710E-02,
     * .1921E-02,.2631E-02,.3632E-02,.9697E-03,.1325E-02,.1810E-02,
     * .5419E-03,.7162E-03,.9357E-03,
     * .3200E+01,.4541E+01,.6338E+01,.1473E+01,.2096E+01,.2979E+01,
     * .6904E+00,.9653E+00,.1373E+01,.3202E+00,.4539E+00,.6339E+00,
     * .1474E+00,.2098E+00,.2981E+00,.6929E-01,.9671E-01,.1374E+00,
     * .3233E-01,.4567E-01,.6366E-01,.1518E-01,.2134E-01,.3013E-01,
     * .7570E-02,.1025E-01,.1418E-01,.4024E-02,.5295E-02,.7098E-02,
     * .2645E-02,.3180E-02,.3870E-02,.2243E-02,.2444E-02,.2746E-02,
     * .2147E-02,.2325E-02,.2473E-02,
     * .3042E+01,.4291E+01,.6011E+01,.1401E+01,.1994E+01,.2824E+01,
     * .6505E+00,.9174E+00,.1306E+01,.3045E+00,.4294E+00,.6013E+00,
     * .1404E+00,.1997E+00,.2827E+00,.6570E-01,.9218E-01,.1310E+00,
     * .3123E-01,.4367E-01,.6067E-01,.1507E-01,.2087E-01,.2910E-01,
     * .8039E-02,.1056E-01,.1415E-01,.4942E-02,.6002E-02,.7696E-02,
     * .3618E-02,.4193E-02,.4954E-02,.3103E-02,.3467E-02,.3836E-02,
     * .2879E-02,.3157E-02,.3416E-02,
     * .2047E+02,.2906E+02,.4068E+02,.9422E+01,.1341E+02,.1907E+02,
     * .4423E+01,.6177E+01,.8777E+01,.2050E+01,.2909E+01,.4071E+01,
     * .9454E+00,.1343E+01,.1909E+01,.4473E+00,.6217E+00,.8810E+00,
     * .2110E+00,.2963E+00,.4124E+00,.1028E+00,.1411E+00,.1971E+00,
     * .5393E-01,.7214E-01,.9584E-01,.3163E-01,.3941E-01,.5156E-01,
     * .2132E-01,.2501E-01,.3123E-01,.1686E-01,.1928E-01,.2187E-01,
     * .1516E-01,.1640E-01,.1809E-01,
     * .1504E+02,.2139E+02,.3042E+02,.7064E+01,.9858E+01,.1401E+02,
     * .3271E+01,.4642E+01,.6507E+01,.1511E+01,.2146E+01,.3048E+01,
     * .7194E+00,.9959E+00,.1409E+01,.3435E+00,.4789E+00,.6648E+00,
     * .1772E+00,.2340E+00,.3216E+00,.1042E+00,.1296E+00,.1657E+00,
     * .8109E-01,.8859E-01,.1007E+00,.7836E-01,.8170E-01,.8384E-01,
     * .7547E-01,.8048E-01,.8329E-01,.7581E-01,.7976E-01,.8187E-01,
     * .7557E-01,.8065E-01,.8362E-01,
     * .1350E+02,.1921E+02,.2710E+02,.6230E+01,.8846E+01,.1259E+02,
     * .2940E+01,.4116E+01,.5799E+01,.1366E+01,.1935E+01,.2723E+01,
     * .6496E+00,.9035E+00,.1275E+01,.3259E+00,.4431E+00,.6023E+00,
     * .1823E+00,.2302E+00,.3065E+00,.1235E+00,.1430E+00,.1745E+00,
     * .1003E+00,.1120E+00,.1244E+00,.9324E-01,.9949E-01,.1072E+00,
     * .9011E-01,.9389E-01,.1005E+00,.9092E-01,.9586E-01,.9874E-01,
     * .9103E-01,.9779E-01,.1023E+00/
      DATA NALPHA/
     * 43,44,46,42,44,45,40,41,43,37,38,40,35,37,38,33,
     * 34,36,30,31,33,28,30,31,27,28,30,25,26,28,23,25,
     * 26,22,24,25,22,22,24,
     * 36,38,40,35,36,38,33,35,36,30,32,34,29,30,32,26,
     * 28,29,24,26,28,22,23,25,21,22,23,18,20,21,18,18,
     * 20,16,17,18,17,17,18,
     * 33,34,34,32,32,34,29,28,30,28,29,29,26,26,28,24,
     * 23,25,22,23,23,19,20,21,19,20,20,18,15,19,18,18,
     * 18,18,18,18,17,18,18,
     * 42,43,44,39,40,42,36,38,39,35,36,38,33,34,36,30,
     * 32,33,29,30,32,25,26,28,23,25,26,22,24,26,21,22,
     * 24,20,22,23,20,20,21,
     * 35,37,38,31,32,34,29,31,32,28,30,31,26,27,29,24,
     * 26,27,21,23,24,20,21,23,19,19,21,16,18,19,17,17,
     * 17,17,17,17,17,17,17,
     * 31,33,34,29,30,32,26,28,29,23,25,26,22,23,25,20,
     * 21,23,18,20,21,17,20,21,18,13,19,18,18,18,17,17,
     * 17,17,17,17,17,17,17/

      DATA ((ALPHA(1,I,J,1),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(2,I,J,1),I=1,NTEMP),J=1,NNE)/3*3.9810e-06,
     *3*2.5120e-06,3*3.9810e-06,6*6.3100e-06,3*1.0000e-05,18*1.5850e-05,
     *3*1.0000e-05/
      DATA ((ALPHA(3,I,J,1),I=1,NTEMP),J=1,NNE)/3*6.3100e-06,
     *3*3.9810e-06,3*6.3100e-06,6*1.0000e-05,3*1.5850e-05,18*2.5120e-05,
     *3*1.5850e-05/
      DATA ((ALPHA(4,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.0000e-05,
     *3*6.3100e-06,3*1.0000e-05,6*1.5850e-05,3*2.5120e-05,18*3.9810e-05,
     *3*2.5120e-05/
      DATA ((ALPHA(5,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.5850e-05,
     *3*1.0000e-05,3*1.5850e-05,6*2.5120e-05,3*3.9810e-05,18*6.3100e-05,
     *3*3.9810e-05/
      DATA ((ALPHA(6,I,J,1),I=1,NTEMP),J=1,NNE)/3*2.5120e-05,
     *3*1.5850e-05,3*2.5120e-05,6*3.9810e-05,3*6.3100e-05,18*1.0000e-04,
     *3*6.3100e-05/
      DATA ((ALPHA(7,I,J,1),I=1,NTEMP),J=1,NNE)/3*3.9810e-05,
     *3*2.5120e-05,3*3.9810e-05,6*6.3100e-05,3*1.0000e-04,18*1.5850e-04,
     *3*1.0000e-04/
      DATA ((ALPHA(8,I,J,1),I=1,NTEMP),J=1,NNE)/3*6.3100e-05,
     *3*3.9810e-05,3*6.3100e-05,6*1.0000e-04,3*1.5850e-04,18*2.5120e-04,
     *3*1.5850e-04/
      DATA ((ALPHA(9,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.0000e-04,
     *3*6.3100e-05,3*1.0000e-04,6*1.5850e-04,3*2.5120e-04,18*3.9810e-04,
     *3*2.5120e-04/
      DATA ((ALPHA(10,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.5850e-04,
     *3*1.0000e-04,3*1.5850e-04,6*2.5120e-04,3*3.9810e-04,18*6.3100e-04,
     *3*3.9810e-04/
      DATA ((ALPHA(11,I,J,1),I=1,NTEMP),J=1,NNE)/3*2.5120e-04,
     *3*1.5850e-04,3*2.5120e-04,6*3.9810e-04,3*6.3100e-04,18*1.0000e-03,
     *3*6.3100e-04/
      DATA ((ALPHA(12,I,J,1),I=1,NTEMP),J=1,NNE)/3*3.9810e-04,
     *3*2.5120e-04,3*3.9810e-04,6*6.3100e-04,3*1.0000e-03,18*1.5850e-03,
     *3*1.0000e-03/
      DATA ((ALPHA(13,I,J,1),I=1,NTEMP),J=1,NNE)/3*6.3100e-04,
     *3*3.9810e-04,3*6.3100e-04,6*1.0000e-03,3*1.5850e-03,18*2.5120e-03,
     *3*1.5850e-03/
      DATA ((ALPHA(14,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.0000e-03,
     *3*6.3100e-04,3*1.0000e-03,6*1.5850e-03,3*2.5120e-03,18*3.9810e-03,
     *3*2.5120e-03/
      DATA ((ALPHA(15,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.5850e-03,
     *3*1.0000e-03,3*1.5850e-03,6*2.5120e-03,3*3.9810e-03,18*6.3100e-03,
     *3*3.9810e-03/
      DATA ((ALPHA(16,I,J,1),I=1,NTEMP),J=1,NNE)/3*2.5120e-03,
     *3*1.5850e-03,3*2.5120e-03,6*3.9810e-03,3*6.3100e-03,18*1.0000e-02,
     *3*6.3100e-03/
      DATA ((ALPHA(17,I,J,1),I=1,NTEMP),J=1,NNE)/3*3.9810e-03,
     *3*2.5120e-03,3*3.9810e-03,6*6.3100e-03,3*1.0000e-02,18*1.5850e-02,
     *3*1.0000e-02/
      DATA ((ALPHA(18,I,J,1),I=1,NTEMP),J=1,NNE)/3*6.3100e-03,
     *3*3.9810e-03,3*6.3100e-03,6*1.0000e-02,3*1.5850e-02,18*2.5120e-02,
     *3*1.5850e-02/
      DATA ((ALPHA(19,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.0000e-02,
     *3*6.3100e-03,3*1.0000e-02,6*1.5850e-02,3*2.5120e-02,18*3.9810e-02,
     *3*2.5120e-02/
      DATA ((ALPHA(20,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.5850e-02,
     *3*1.0000e-02,3*1.5850e-02,6*2.5120e-02,3*3.9810e-02,18*6.3100e-02,
     *3*3.9810e-02/
      DATA ((ALPHA(21,I,J,1),I=1,NTEMP),J=1,NNE)/3*2.5120e-02,
     *3*1.5850e-02,3*2.5120e-02,6*3.9810e-02,3*6.3100e-02,18*1.0000e-01,
     *3*6.3100e-02/
      DATA ((ALPHA(22,I,J,1),I=1,NTEMP),J=1,NNE)/3*3.9810e-02,
     *3*2.5120e-02,3*3.9810e-02,6*6.3100e-02,3*1.0000e-01,18*1.5850e-01,
     *3*1.0000e-01/
      DATA ((ALPHA(23,I,J,1),I=1,NTEMP),J=1,NNE)/3*6.3100e-02,
     *3*3.9810e-02,3*6.3100e-02,6*1.0000e-01,3*1.5850e-01,15*2.5120e-01,
     *0.0000e+00,2*2.5120e-01,2*0.0000e+00,1.5850e-01/
      DATA ((ALPHA(24,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.0000e-01,
     *3*6.3100e-02,3*1.0000e-01,6*1.5850e-01,3*2.5120e-01,12*3.9810e-01,
     *0.0000e+00,2*3.9810e-01,0.0000e+00,2*3.9810e-01,2*0.0000e+00,
     *2.5120e-01/
      DATA ((ALPHA(25,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.5850e-01,
     *3*1.0000e-01,3*1.5850e-01,6*2.5120e-01,3*3.9810e-01,12*6.3100e-01,
     *0.0000e+00,2*6.3100e-01,2*0.0000e+00,6.3100e-01,3*0.0000e+00/
      DATA ((ALPHA(26,I,J,1),I=1,NTEMP),J=1,NNE)/3*2.5120e-01,
     *3*1.5850e-01,3*2.5120e-01,6*3.9810e-01,3*6.3100e-01,9*1.0000e+00,
     *0.0000e+00,2*1.0000e+00,2*0.0000e+00,1.0000e+00,6*0.0000e+00/
      DATA ((ALPHA(27,I,J,1),I=1,NTEMP),J=1,NNE)/3*3.9810e-01,
     *3*2.5120e-01,3*3.9810e-01,6*6.3100e-01,3*1.0000e+00,9*1.5850e+00,
     *2*0.0000e+00,1.5850e+00,9*0.0000e+00/
      DATA ((ALPHA(28,I,J,1),I=1,NTEMP),J=1,NNE)/3*6.3100e-01,
     *3*3.9810e-01,3*6.3100e-01,6*1.0000e+00,3*1.5850e+00,6*2.5120e+00,
     *0.0000e+00,2*2.5120e+00,2*0.0000e+00,2.5120e+00,9*0.0000e+00/
      DATA ((ALPHA(29,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.0000e+00,
     *3*6.3100e-01,3*1.0000e+00,6*1.5850e+00,3*2.5120e+00,3*3.9810e+00,
     *0.0000e+00,2*3.9810e+00,2*0.0000e+00,3.9810e+00,12*0.0000e+00/
      DATA ((ALPHA(30,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.5850e+00,
     *3*1.0000e+00,3*1.5850e+00,6*2.5120e+00,3*3.9810e+00,3*6.3100e+00,
     *0.0000e+00,2*6.3100e+00,2*0.0000e+00,6.3100e+00,12*0.0000e+00/
      DATA ((ALPHA(31,I,J,1),I=1,NTEMP),J=1,NNE)/3*2.5120e+00,
     *3*1.5850e+00,3*2.5120e+00,6*3.9810e+00,3*6.3100e+00,0.0000e+00,
     *2*1.0000e+01,2*0.0000e+00,1.0000e+01,15*0.0000e+00/
      DATA ((ALPHA(32,I,J,1),I=1,NTEMP),J=1,NNE)/3*3.9810e+00,
     *3*2.5120e+00,3*3.9810e+00,6*6.3100e+00,3*1.0000e+01,2*0.0000e+00,
     *1.5850e+01,18*0.0000e+00/
      DATA ((ALPHA(33,I,J,1),I=1,NTEMP),J=1,NNE)/3*6.3100e+00,
     *3*3.9810e+00,3*6.3100e+00,6*1.0000e+01,3*1.5850e+01,2*0.0000e+00,
     *2.5120e+01,18*0.0000e+00/
      DATA ((ALPHA(34,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.0000e+01,
     *3*6.3100e+00,3*1.0000e+01,6*1.5850e+01,0.0000e+00,2*2.5120e+01,
     *21*0.0000e+00/
      DATA ((ALPHA(35,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.5850e+01,
     *3*1.0000e+01,3*1.5850e+01,6*2.5120e+01,2*0.0000e+00,3.9810e+01,
     *21*0.0000e+00/
      DATA ((ALPHA(36,I,J,1),I=1,NTEMP),J=1,NNE)/3*2.5120e+01,
     *3*1.5850e+01,3*2.5120e+01,3*3.9810e+01,0.0000e+00,2*3.9810e+01,
     *2*0.0000e+00,6.3100e+01,21*0.0000e+00/
      DATA ((ALPHA(37,I,J,1),I=1,NTEMP),J=1,NNE)/3*3.9810e+01,
     *3*2.5120e+01,3*3.9810e+01,3*6.3100e+01,0.0000e+00,2*6.3100e+01,
     *24*0.0000e+00/
      DATA ((ALPHA(38,I,J,1),I=1,NTEMP),J=1,NNE)/3*6.3100e+01,
     *3*3.9810e+01,3*6.3100e+01,0.0000e+00,2*1.0000e+02,2*0.0000e+00,
     *1.0000e+02,24*0.0000e+00/
      DATA ((ALPHA(39,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.0000e+02,
     *3*6.3100e+01,3*1.0000e+02,2*0.0000e+00,1.5850e+02,27*0.0000e+00/
      DATA ((ALPHA(40,I,J,1),I=1,NTEMP),J=1,NNE)/3*1.5850e+02,
     *3*1.0000e+02,3*1.5850e+02,2*0.0000e+00,2.5120e+02,27*0.0000e+00/
      DATA ((ALPHA(41,I,J,1),I=1,NTEMP),J=1,NNE)/3*2.5120e+02,
     *3*1.5850e+02,0.0000e+00,2*2.5120e+02,30*0.0000e+00/
      DATA ((ALPHA(42,I,J,1),I=1,NTEMP),J=1,NNE)/3*3.9810e+02,
     *3*2.5120e+02,2*0.0000e+00,3.9810e+02,30*0.0000e+00/
      DATA ((ALPHA(43,I,J,1),I=1,NTEMP),J=1,NNE)/3*6.3100e+02,
     *0.0000e+00,2*3.9810e+02,2*0.0000e+00,6.3100e+02,30*0.0000e+00/
      DATA ((ALPHA(44,I,J,1),I=1,NTEMP),J=1,NNE)/0.0000e+00,
     *2*1.0000e+03,0.0000e+00,2*6.3100e+02,33*0.0000e+00/
      DATA ((ALPHA(45,I,J,1),I=1,NTEMP),J=1,NNE)/2*0.0000e+00,
     *1.5850e+03,2*0.0000e+00,1.0000e+03,33*0.0000e+00/
      DATA ((ALPHA(46,I,J,1),I=1,NTEMP),J=1,NNE)/2*0.0000e+00,
     *2.5120e+03,36*0.0000e+00/
      DATA ((ALPHA(1,I,J,2),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(2,I,J,2),I=1,NTEMP),J=1,NNE)/9*2.5120e-05,
     *6*3.9810e-05,6*6.3100e-05,12*1.0000e-04,3*1.5850e-04,3*1.0000e-04/
      DATA ((ALPHA(3,I,J,2),I=1,NTEMP),J=1,NNE)/9*3.9810e-05,
     *6*6.3100e-05,6*1.0000e-04,12*1.5850e-04,3*2.5120e-04,3*1.5850e-04/
      DATA ((ALPHA(4,I,J,2),I=1,NTEMP),J=1,NNE)/9*6.3100e-05,
     *6*1.0000e-04,6*1.5850e-04,12*2.5120e-04,3*3.9810e-04,3*2.5120e-04/
      DATA ((ALPHA(5,I,J,2),I=1,NTEMP),J=1,NNE)/9*1.0000e-04,
     *6*1.5850e-04,6*2.5120e-04,12*3.9810e-04,3*6.3100e-04,3*3.9810e-04/
      DATA ((ALPHA(6,I,J,2),I=1,NTEMP),J=1,NNE)/9*1.5850e-04,
     *6*2.5120e-04,6*3.9810e-04,12*6.3100e-04,3*1.0000e-03,3*6.3100e-04/
      DATA ((ALPHA(7,I,J,2),I=1,NTEMP),J=1,NNE)/9*2.5120e-04,
     *6*3.9810e-04,6*6.3100e-04,12*1.0000e-03,3*1.5850e-03,3*1.0000e-03/
      DATA ((ALPHA(8,I,J,2),I=1,NTEMP),J=1,NNE)/9*3.9810e-04,
     *6*6.3100e-04,6*1.0000e-03,12*1.5850e-03,3*2.5120e-03,3*1.5850e-03/
      DATA ((ALPHA(9,I,J,2),I=1,NTEMP),J=1,NNE)/9*6.3100e-04,
     *6*1.0000e-03,6*1.5850e-03,12*2.5120e-03,3*3.9810e-03,3*2.5120e-03/
      DATA ((ALPHA(10,I,J,2),I=1,NTEMP),J=1,NNE)/9*1.0000e-03,
     *6*1.5850e-03,6*2.5120e-03,12*3.9810e-03,3*6.3100e-03,3*3.9810e-03/
      DATA ((ALPHA(11,I,J,2),I=1,NTEMP),J=1,NNE)/9*1.5850e-03,
     *6*2.5120e-03,6*3.9810e-03,12*6.3100e-03,3*1.0000e-02,3*6.3100e-03/
      DATA ((ALPHA(12,I,J,2),I=1,NTEMP),J=1,NNE)/9*2.5120e-03,
     *6*3.9810e-03,6*6.3100e-03,12*1.0000e-02,3*1.5850e-02,3*1.0000e-02/
      DATA ((ALPHA(13,I,J,2),I=1,NTEMP),J=1,NNE)/9*3.9810e-03,
     *6*6.3100e-03,6*1.0000e-02,12*1.5850e-02,3*2.5120e-02,3*1.5850e-02/
      DATA ((ALPHA(14,I,J,2),I=1,NTEMP),J=1,NNE)/9*6.3100e-03,
     *6*1.0000e-02,6*1.5850e-02,12*2.5120e-02,3*3.9810e-02,3*2.5120e-02/
      DATA ((ALPHA(15,I,J,2),I=1,NTEMP),J=1,NNE)/9*1.0000e-02,
     *6*1.5850e-02,6*2.5120e-02,12*3.9810e-02,3*6.3100e-02,3*3.9810e-02/
      DATA ((ALPHA(16,I,J,2),I=1,NTEMP),J=1,NNE)/9*1.5850e-02,
     *6*2.5120e-02,6*3.9810e-02,12*6.3100e-02,3*1.0000e-01,3*6.3100e-02/
      DATA ((ALPHA(17,I,J,2),I=1,NTEMP),J=1,NNE)/9*2.5120e-02,
     *6*3.9810e-02,6*6.3100e-02,12*1.0000e-01,0.0000e+00,2*1.5850e-01,
     *3*1.0000e-01/
      DATA ((ALPHA(18,I,J,2),I=1,NTEMP),J=1,NNE)/9*3.9810e-02,
     *6*6.3100e-02,6*1.0000e-01,12*1.5850e-01,2*0.0000e+00,2.5120e-01,
     *2*0.0000e+00,1.5850e-01/
      DATA ((ALPHA(19,I,J,2),I=1,NTEMP),J=1,NNE)/9*6.3100e-02,
     *6*1.0000e-01,6*1.5850e-01,6*2.5120e-01,0.0000e+00,2*2.5120e-01,
     *2*0.0000e+00,2.5120e-01,6*0.0000e+00/
      DATA ((ALPHA(20,I,J,2),I=1,NTEMP),J=1,NNE)/9*1.0000e-01,
     *6*1.5850e-01,6*2.5120e-01,6*3.9810e-01,0.0000e+00,2*3.9810e-01,
     *2*0.0000e+00,3.9810e-01,6*0.0000e+00/
      DATA ((ALPHA(21,I,J,2),I=1,NTEMP),J=1,NNE)/9*1.5850e-01,
     *6*2.5120e-01,6*3.9810e-01,6*6.3100e-01,2*0.0000e+00,6.3100e-01,
     *9*0.0000e+00/
      DATA ((ALPHA(22,I,J,2),I=1,NTEMP),J=1,NNE)/9*2.5120e-01,
     *6*3.9810e-01,6*6.3100e-01,3*1.0000e+00,0.0000e+00,2*1.0000e+00,
     *12*0.0000e+00/
      DATA ((ALPHA(23,I,J,2),I=1,NTEMP),J=1,NNE)/9*3.9810e-01,
     *6*6.3100e-01,6*1.0000e+00,0.0000e+00,2*1.5850e+00,2*0.0000e+00,
     *1.5850e+00,12*0.0000e+00/
      DATA ((ALPHA(24,I,J,2),I=1,NTEMP),J=1,NNE)/9*6.3100e-01,
     *6*1.0000e+00,6*1.5850e+00,2*0.0000e+00,2.5120e+00,15*0.0000e+00/
      DATA ((ALPHA(25,I,J,2),I=1,NTEMP),J=1,NNE)/9*1.0000e+00,
     *6*1.5850e+00,3*2.5120e+00,0.0000e+00,2*2.5120e+00,2*0.0000e+00,
     *3.9810e+00,15*0.0000e+00/
      DATA ((ALPHA(26,I,J,2),I=1,NTEMP),J=1,NNE)/9*1.5850e+00,
     *6*2.5120e+00,3*3.9810e+00,0.0000e+00,2*3.9810e+00,18*0.0000e+00/
      DATA ((ALPHA(27,I,J,2),I=1,NTEMP),J=1,NNE)/9*2.5120e+00,
     *6*3.9810e+00,0.0000e+00,2*6.3100e+00,2*0.0000e+00,6.3100e+00,
     *18*0.0000e+00/
      DATA ((ALPHA(28,I,J,2),I=1,NTEMP),J=1,NNE)/9*3.9810e+00,
     *6*6.3100e+00,0.0000e+00,2*1.0000e+01,2*0.0000e+00,1.0000e+01,
     *18*0.0000e+00/
      DATA ((ALPHA(29,I,J,2),I=1,NTEMP),J=1,NNE)/9*6.3100e+00,
     *6*1.0000e+01,2*0.0000e+00,1.5850e+01,21*0.0000e+00/
      DATA ((ALPHA(30,I,J,2),I=1,NTEMP),J=1,NNE)/9*1.0000e+01,
     *3*1.5850e+01,0.0000e+00,2*1.5850e+01,24*0.0000e+00/
      DATA ((ALPHA(31,I,J,2),I=1,NTEMP),J=1,NNE)/9*1.5850e+01,
     *0.0000e+00,2*2.5120e+01,2*0.0000e+00,2.5120e+01,24*0.0000e+00/
      DATA ((ALPHA(32,I,J,2),I=1,NTEMP),J=1,NNE)/9*2.5120e+01,
     *0.0000e+00,2*3.9810e+01,2*0.0000e+00,3.9810e+01,24*0.0000e+00/
      DATA ((ALPHA(33,I,J,2),I=1,NTEMP),J=1,NNE)/9*3.9810e+01,
     *2*0.0000e+00,6.3100e+01,27*0.0000e+00/
      DATA ((ALPHA(34,I,J,2),I=1,NTEMP),J=1,NNE)/6*6.3100e+01,
     *0.0000e+00,2*6.3100e+01,2*0.0000e+00,1.0000e+02,27*0.0000e+00/
      DATA ((ALPHA(35,I,J,2),I=1,NTEMP),J=1,NNE)/6*1.0000e+02,
     *0.0000e+00,2*1.0000e+02,30*0.0000e+00/
      DATA ((ALPHA(36,I,J,2),I=1,NTEMP),J=1,NNE)/3*1.5850e+02,
     *0.0000e+00,2*1.5850e+02,2*0.0000e+00,1.5850e+02,30*0.0000e+00/
      DATA ((ALPHA(37,I,J,2),I=1,NTEMP),J=1,NNE)/0.0000e+00,
     *2*2.5120e+02,2*0.0000e+00,2.5120e+02,33*0.0000e+00/
      DATA ((ALPHA(38,I,J,2),I=1,NTEMP),J=1,NNE)/0.0000e+00,
     *2*3.9810e+02,2*0.0000e+00,3.9810e+02,33*0.0000e+00/
      DATA ((ALPHA(39,I,J,2),I=1,NTEMP),J=1,NNE)/2*0.0000e+00,
     *6.3100e+02,36*0.0000e+00/
      DATA ((ALPHA(40,I,J,2),I=1,NTEMP),J=1,NNE)/2*0.0000e+00,
     *1.0000e+03,36*0.0000e+00/
      DATA ((ALPHA(41,I,J,2),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(42,I,J,2),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(43,I,J,2),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(44,I,J,2),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(45,I,J,2),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(46,I,J,2),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(1,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(2,I,J,3),I=1,NTEMP),J=1,NNE)/3*1.0000e-04,
     *3*6.3100e-05,3*1.5850e-04,6*1.0000e-04,6*1.5850e-04,3*2.5120e-04,
     *15*1.5850e-04/
      DATA ((ALPHA(3,I,J,3),I=1,NTEMP),J=1,NNE)/3*1.5850e-04,
     *3*1.0000e-04,3*2.5120e-04,6*1.5850e-04,6*2.5120e-04,3*3.9810e-04,
     *15*2.5120e-04/
      DATA ((ALPHA(4,I,J,3),I=1,NTEMP),J=1,NNE)/3*2.5120e-04,
     *3*1.5850e-04,3*3.9810e-04,6*2.5120e-04,6*3.9810e-04,3*6.3100e-04,
     *15*3.9810e-04/
      DATA ((ALPHA(5,I,J,3),I=1,NTEMP),J=1,NNE)/3*3.9810e-04,
     *3*2.5120e-04,3*6.3100e-04,6*3.9810e-04,6*6.3100e-04,3*1.0000e-03,
     *15*6.3100e-04/
      DATA ((ALPHA(6,I,J,3),I=1,NTEMP),J=1,NNE)/3*6.3100e-04,
     *3*3.9810e-04,3*1.0000e-03,6*6.3100e-04,6*1.0000e-03,3*1.5850e-03,
     *15*1.0000e-03/
      DATA ((ALPHA(7,I,J,3),I=1,NTEMP),J=1,NNE)/3*1.0000e-03,
     *3*6.3100e-04,3*1.5850e-03,6*1.0000e-03,6*1.5850e-03,3*2.5120e-03,
     *15*1.5850e-03/
      DATA ((ALPHA(8,I,J,3),I=1,NTEMP),J=1,NNE)/3*1.5850e-03,
     *3*1.0000e-03,3*2.5120e-03,6*1.5850e-03,6*2.5120e-03,3*3.9810e-03,
     *15*2.5120e-03/
      DATA ((ALPHA(9,I,J,3),I=1,NTEMP),J=1,NNE)/3*2.5120e-03,
     *3*1.5850e-03,3*3.9810e-03,6*2.5120e-03,6*3.9810e-03,3*6.3100e-03,
     *15*3.9810e-03/
      DATA ((ALPHA(10,I,J,3),I=1,NTEMP),J=1,NNE)/3*3.9810e-03,
     *3*2.5120e-03,3*6.3100e-03,6*3.9810e-03,6*6.3100e-03,3*1.0000e-02,
     *15*6.3100e-03/
      DATA ((ALPHA(11,I,J,3),I=1,NTEMP),J=1,NNE)/3*6.3100e-03,
     *3*3.9810e-03,3*1.0000e-02,6*6.3100e-03,6*1.0000e-02,3*1.5850e-02,
     *15*1.0000e-02/
      DATA ((ALPHA(12,I,J,3),I=1,NTEMP),J=1,NNE)/3*1.0000e-02,
     *3*6.3100e-03,3*1.5850e-02,6*1.0000e-02,6*1.5850e-02,3*2.5120e-02,
     *15*1.5850e-02/
      DATA ((ALPHA(13,I,J,3),I=1,NTEMP),J=1,NNE)/3*1.5850e-02,
     *3*1.0000e-02,3*2.5120e-02,6*1.5850e-02,6*2.5120e-02,3*3.9810e-02,
     *15*2.5120e-02/
      DATA ((ALPHA(14,I,J,3),I=1,NTEMP),J=1,NNE)/3*2.5120e-02,
     *3*1.5850e-02,3*3.9810e-02,6*2.5120e-02,6*3.9810e-02,3*6.3100e-02,
     *15*3.9810e-02/
      DATA ((ALPHA(15,I,J,3),I=1,NTEMP),J=1,NNE)/3*3.9810e-02,
     *3*2.5120e-02,3*6.3100e-02,6*3.9810e-02,6*6.3100e-02,3*1.0000e-01,
     *15*6.3100e-02/
      DATA ((ALPHA(16,I,J,3),I=1,NTEMP),J=1,NNE)/3*6.3100e-02,
     *3*3.9810e-02,3*1.0000e-01,6*6.3100e-02,6*1.0000e-01,3*1.5850e-01,
     *4*1.0000e-01,0.0000e+00,10*1.0000e-01/
      DATA ((ALPHA(17,I,J,3),I=1,NTEMP),J=1,NNE)/3*1.0000e-01,
     *3*6.3100e-02,3*1.5850e-01,6*1.0000e-01,6*1.5850e-01,3*2.5120e-01,
     *4*1.5850e-01,0.0000e+00,10*1.5850e-01/
      DATA ((ALPHA(18,I,J,3),I=1,NTEMP),J=1,NNE)/3*1.5850e-01,
     *3*1.0000e-01,3*2.5120e-01,6*1.5850e-01,6*2.5120e-01,3*3.9810e-01,
     *4*2.5120e-01,0.0000e+00,7*2.5120e-01,0.0000e+00,2*2.5120e-01/
      DATA ((ALPHA(19,I,J,3),I=1,NTEMP),J=1,NNE)/3*2.5120e-01,
     *3*1.5850e-01,3*3.9810e-01,6*2.5120e-01,6*3.9810e-01,3*6.3100e-01,
     *3*3.9810e-01,2*0.0000e+00,3.9810e-01,9*0.0000e+00/
      DATA ((ALPHA(20,I,J,3),I=1,NTEMP),J=1,NNE)/3*3.9810e-01,
     *3*2.5120e-01,3*6.3100e-01,6*3.9810e-01,6*6.3100e-01,0.0000e+00,
     *2*1.0000e+00,0.0000e+00,2*6.3100e-01,12*0.0000e+00/
      DATA ((ALPHA(21,I,J,3),I=1,NTEMP),J=1,NNE)/3*6.3100e-01,
     *3*3.9810e-01,3*1.0000e+00,6*6.3100e-01,6*1.0000e+00,2*0.0000e+00,
     *1.5850e+00,15*0.0000e+00/
      DATA ((ALPHA(22,I,J,3),I=1,NTEMP),J=1,NNE)/3*1.0000e+00,
     *3*6.3100e-01,3*1.5850e+00,6*1.0000e+00,6*1.5850e+00,18*0.0000e+00/
      DATA ((ALPHA(23,I,J,3),I=1,NTEMP),J=1,NNE)/3*1.5850e+00,
     *3*1.0000e+00,3*2.5120e+00,6*1.5850e+00,3*2.5120e+00,0.0000e+00,
     *2*2.5120e+00,18*0.0000e+00/
      DATA ((ALPHA(24,I,J,3),I=1,NTEMP),J=1,NNE)/3*2.5120e+00,
     *3*1.5850e+00,3*3.9810e+00,6*2.5120e+00,3.9810e+00,0.0000e+00,
     *3.9810e+00,21*0.0000e+00/
      DATA ((ALPHA(25,I,J,3),I=1,NTEMP),J=1,NNE)/3*3.9810e+00,
     *3*2.5120e+00,3*6.3100e+00,6*3.9810e+00,2*0.0000e+00,6.3100e+00,
     *21*0.0000e+00/
      DATA ((ALPHA(26,I,J,3),I=1,NTEMP),J=1,NNE)/3*6.3100e+00,
     *3*3.9810e+00,3*1.0000e+01,6*6.3100e+00,24*0.0000e+00/
      DATA ((ALPHA(27,I,J,3),I=1,NTEMP),J=1,NNE)/3*1.0000e+01,
     *3*6.3100e+00,3*1.5850e+01,3*1.0000e+01,2*0.0000e+00,1.0000e+01,
     *24*0.0000e+00/
      DATA ((ALPHA(28,I,J,3),I=1,NTEMP),J=1,NNE)/3*1.5850e+01,
     *3*1.0000e+01,3*2.5120e+01,3*1.5850e+01,2*0.0000e+00,1.5850e+01,
     *24*0.0000e+00/
      DATA ((ALPHA(29,I,J,3),I=1,NTEMP),J=1,NNE)/3*2.5120e+01,
     *3*1.5850e+01,3.9810e+01,0.0000e+00,3.9810e+01,0.0000e+00,
     *2*2.5120e+01,27*0.0000e+00/
      DATA ((ALPHA(30,I,J,3),I=1,NTEMP),J=1,NNE)/3*3.9810e+01,
     *3*2.5120e+01,2*0.0000e+00,6.3100e+01,30*0.0000e+00/
      DATA ((ALPHA(31,I,J,3),I=1,NTEMP),J=1,NNE)/3*6.3100e+01,
     *3*3.9810e+01,33*0.0000e+00/
      DATA ((ALPHA(32,I,J,3),I=1,NTEMP),J=1,NNE)/3*1.0000e+02,
     *3*6.3100e+01,33*0.0000e+00/
      DATA ((ALPHA(33,I,J,3),I=1,NTEMP),J=1,NNE)/3*1.5850e+02,
     *2*0.0000e+00,1.0000e+02,33*0.0000e+00/
      DATA ((ALPHA(34,I,J,3),I=1,NTEMP),J=1,NNE)/0.0000e+00,
     *2*2.5120e+02,2*0.0000e+00,1.5850e+02,33*0.0000e+00/
      DATA ((ALPHA(35,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(36,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(37,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(38,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(39,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(40,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(41,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(42,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(43,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(44,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(45,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(46,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(1,I,J,4),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(2,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.5850e-04,
     *3*2.5120e-04,9*3.9810e-04,6*6.3100e-04,6*1.5850e-03,6*1.0000e-03,
     *6*6.3100e-04/
      DATA ((ALPHA(3,I,J,4),I=1,NTEMP),J=1,NNE)/3*2.5120e-04,
     *3*3.9810e-04,9*6.3100e-04,6*1.0000e-03,6*2.5120e-03,6*1.5850e-03,
     *6*1.0000e-03/
      DATA ((ALPHA(4,I,J,4),I=1,NTEMP),J=1,NNE)/3*3.9810e-04,
     *3*6.3100e-04,9*1.0000e-03,6*1.5850e-03,6*3.9810e-03,6*2.5120e-03,
     *6*1.5850e-03/
      DATA ((ALPHA(5,I,J,4),I=1,NTEMP),J=1,NNE)/3*6.3100e-04,
     *3*1.0000e-03,9*1.5850e-03,6*2.5120e-03,6*6.3100e-03,6*3.9810e-03,
     *6*2.5120e-03/
      DATA ((ALPHA(6,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.0000e-03,
     *3*1.5850e-03,9*2.5120e-03,6*3.9810e-03,6*1.0000e-02,6*6.3100e-03,
     *6*3.9810e-03/
      DATA ((ALPHA(7,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.5850e-03,
     *3*2.5120e-03,9*3.9810e-03,6*6.3100e-03,6*1.5850e-02,6*1.0000e-02,
     *6*6.3100e-03/
      DATA ((ALPHA(8,I,J,4),I=1,NTEMP),J=1,NNE)/3*2.5120e-03,
     *3*3.9810e-03,9*6.3100e-03,6*1.0000e-02,6*2.5120e-02,6*1.5850e-02,
     *6*1.0000e-02/
      DATA ((ALPHA(9,I,J,4),I=1,NTEMP),J=1,NNE)/3*3.9810e-03,
     *3*6.3100e-03,9*1.0000e-02,6*1.5850e-02,6*3.9810e-02,6*2.5120e-02,
     *6*1.5850e-02/
      DATA ((ALPHA(10,I,J,4),I=1,NTEMP),J=1,NNE)/3*6.3100e-03,
     *3*1.0000e-02,9*1.5850e-02,6*2.5120e-02,6*6.3100e-02,6*3.9810e-02,
     *6*2.5120e-02/
      DATA ((ALPHA(11,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.0000e-02,
     *3*1.5850e-02,9*2.5120e-02,6*3.9810e-02,6*1.0000e-01,6*6.3100e-02,
     *6*3.9810e-02/
      DATA ((ALPHA(12,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.5850e-02,
     *3*2.5120e-02,9*3.9810e-02,6*6.3100e-02,6*1.5850e-01,6*1.0000e-01,
     *6*6.3100e-02/
      DATA ((ALPHA(13,I,J,4),I=1,NTEMP),J=1,NNE)/3*2.5120e-02,
     *3*3.9810e-02,9*6.3100e-02,6*1.0000e-01,6*2.5120e-01,6*1.5850e-01,
     *6*1.0000e-01/
      DATA ((ALPHA(14,I,J,4),I=1,NTEMP),J=1,NNE)/3*3.9810e-02,
     *3*6.3100e-02,9*1.0000e-01,6*1.5850e-01,6*3.9810e-01,6*2.5120e-01,
     *6*1.5850e-01/
      DATA ((ALPHA(15,I,J,4),I=1,NTEMP),J=1,NNE)/3*6.3100e-02,
     *3*1.0000e-01,9*1.5850e-01,6*2.5120e-01,6*6.3100e-01,6*3.9810e-01,
     *6*2.5120e-01/
      DATA ((ALPHA(16,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.0000e-01,
     *3*1.5850e-01,9*2.5120e-01,6*3.9810e-01,6*1.0000e+00,6*6.3100e-01,
     *6*3.9810e-01/
      DATA ((ALPHA(17,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.5850e-01,
     *3*2.5120e-01,9*3.9810e-01,6*6.3100e-01,6*1.5850e+00,6*1.0000e+00,
     *6*6.3100e-01/
      DATA ((ALPHA(18,I,J,4),I=1,NTEMP),J=1,NNE)/3*2.5120e-01,
     *3*3.9810e-01,9*6.3100e-01,6*1.0000e+00,6*2.5120e+00,6*1.5850e+00,
     *6*1.0000e+00/
      DATA ((ALPHA(19,I,J,4),I=1,NTEMP),J=1,NNE)/3*3.9810e-01,
     *3*6.3100e-01,9*1.0000e+00,6*1.5850e+00,6*3.9810e+00,6*2.5120e+00,
     *6*1.5850e+00/
      DATA ((ALPHA(20,I,J,4),I=1,NTEMP),J=1,NNE)/3*6.3100e-01,
     *3*1.0000e+00,9*1.5850e+00,6*2.5120e+00,6*6.3100e+00,6*3.9810e+00,
     *6*2.5120e+00/
      DATA ((ALPHA(21,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.0000e+00,
     *3*1.5850e+00,9*2.5120e+00,6*3.9810e+00,6*1.0000e+01,6*6.3100e+00,
     *0.0000e+00,2*3.9810e+00,2*0.0000e+00,3.9810e+00/
      DATA ((ALPHA(22,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.5850e+00,
     *3*2.5120e+00,9*3.9810e+00,6*6.3100e+00,6*1.5850e+01,3*1.0000e+01,
     *0.0000e+00,2*1.0000e+01,0.0000e+00,2*6.3100e+00,3*0.0000e+00/
      DATA ((ALPHA(23,I,J,4),I=1,NTEMP),J=1,NNE)/3*2.5120e+00,
     *3*3.9810e+00,9*6.3100e+00,6*1.0000e+01,6*2.5120e+01,0.0000e+00,
     *2*1.5850e+01,2*0.0000e+00,1.5850e+01,2*0.0000e+00,1.0000e+01,
     *3*0.0000e+00/
      DATA ((ALPHA(24,I,J,4),I=1,NTEMP),J=1,NNE)/3*3.9810e+00,
     *3*6.3100e+00,9*1.0000e+01,6*1.5850e+01,3*3.9810e+01,0.0000e+00,
     *2*3.9810e+01,0.0000e+00,2*2.5120e+01,2*0.0000e+00,2.5120e+01,
     *6*0.0000e+00/
      DATA ((ALPHA(25,I,J,4),I=1,NTEMP),J=1,NNE)/3*6.3100e+00,
     *3*1.0000e+01,9*1.5850e+01,6*2.5120e+01,3*6.3100e+01,0.0000e+00,
     *2*6.3100e+01,2*0.0000e+00,3.9810e+01,9*0.0000e+00/
      DATA ((ALPHA(26,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.0000e+01,
     *3*1.5850e+01,9*2.5120e+01,6*3.9810e+01,0.0000e+00,2*1.0000e+02,
     *2*0.0000e+00,1.0000e+02,2*0.0000e+00,6.3100e+01,9*0.0000e+00/
      DATA ((ALPHA(27,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.5850e+01,
     *3*2.5120e+01,9*3.9810e+01,6*6.3100e+01,2*0.0000e+00,1.5850e+02,
     *15*0.0000e+00/
      DATA ((ALPHA(28,I,J,4),I=1,NTEMP),J=1,NNE)/3*2.5120e+01,
     *3*3.9810e+01,9*6.3100e+01,6*1.0000e+02,2*0.0000e+00,2.5120e+02,
     *15*0.0000e+00/
      DATA ((ALPHA(29,I,J,4),I=1,NTEMP),J=1,NNE)/3*3.9810e+01,
     *3*6.3100e+01,9*1.0000e+02,6*1.5850e+02,18*0.0000e+00/
      DATA ((ALPHA(30,I,J,4),I=1,NTEMP),J=1,NNE)/3*6.3100e+01,
     *3*1.0000e+02,9*1.5850e+02,3*2.5120e+02,0.0000e+00,2*2.5120e+02,
     *18*0.0000e+00/
      DATA ((ALPHA(31,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.0000e+02,
     *3*1.5850e+02,9*2.5120e+02,0.0000e+00,2*3.9810e+02,2*0.0000e+00,
     *3.9810e+02,18*0.0000e+00/
      DATA ((ALPHA(32,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.5850e+02,
     *3*2.5120e+02,9*3.9810e+02,0.0000e+00,2*6.3100e+02,2*0.0000e+00,
     *6.3100e+02,18*0.0000e+00/
      DATA ((ALPHA(33,I,J,4),I=1,NTEMP),J=1,NNE)/3*2.5120e+02,
     *3*3.9810e+02,9*6.3100e+02,2*0.0000e+00,1.0000e+03,21*0.0000e+00/
      DATA ((ALPHA(34,I,J,4),I=1,NTEMP),J=1,NNE)/3*3.9810e+02,
     *3*6.3100e+02,6*1.0000e+03,0.0000e+00,2*1.0000e+03,24*0.0000e+00/
      DATA ((ALPHA(35,I,J,4),I=1,NTEMP),J=1,NNE)/3*6.3100e+02,
     *3*1.0000e+03,6*1.5850e+03,2*0.0000e+00,1.5850e+03,24*0.0000e+00/
      DATA ((ALPHA(36,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.0000e+03,
     *3*1.5850e+03,3*2.5120e+03,0.0000e+00,2*2.5120e+03,2*0.0000e+00,
     *2.5120e+03,24*0.0000e+00/
      DATA ((ALPHA(37,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.5850e+03,
     *3*2.5120e+03,0.0000e+00,2*3.9810e+03,2*0.0000e+00,3.9810e+03,
     *27*0.0000e+00/
      DATA ((ALPHA(38,I,J,4),I=1,NTEMP),J=1,NNE)/3*2.5120e+03,
     *3*3.9810e+03,0.0000e+00,2*6.3100e+03,2*0.0000e+00,6.3100e+03,
     *27*0.0000e+00/
      DATA ((ALPHA(39,I,J,4),I=1,NTEMP),J=1,NNE)/3*3.9810e+03,
     *3*6.3100e+03,2*0.0000e+00,1.0000e+04,30*0.0000e+00/
      DATA ((ALPHA(40,I,J,4),I=1,NTEMP),J=1,NNE)/3*6.3100e+03,
     *0.0000e+00,2*1.0000e+04,33*0.0000e+00/
      DATA ((ALPHA(41,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.0000e+04,
     *2*0.0000e+00,1.5850e+04,33*0.0000e+00/
      DATA ((ALPHA(42,I,J,4),I=1,NTEMP),J=1,NNE)/3*1.5850e+04,
     *2*0.0000e+00,2.5120e+04,33*0.0000e+00/
      DATA ((ALPHA(43,I,J,4),I=1,NTEMP),J=1,NNE)/0.0000e+00,
     *2*2.5120e+04,36*0.0000e+00/
      DATA ((ALPHA(44,I,J,4),I=1,NTEMP),J=1,NNE)/2*0.0000e+00,
     *3.9810e+04,36*0.0000e+00/
      DATA ((ALPHA(45,I,J,4),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(46,I,J,4),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(1,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(2,I,J,5),I=1,NTEMP),J=1,NNE)/3*6.3100e-04,
     *6*2.5120e-03,3*1.5850e-03,6*2.5120e-03,21*3.9810e-03/
      DATA ((ALPHA(3,I,J,5),I=1,NTEMP),J=1,NNE)/3*1.0000e-03,
     *6*3.9810e-03,3*2.5120e-03,6*3.9810e-03,21*6.3100e-03/
      DATA ((ALPHA(4,I,J,5),I=1,NTEMP),J=1,NNE)/3*1.5850e-03,
     *6*6.3100e-03,3*3.9810e-03,6*6.3100e-03,21*1.0000e-02/
      DATA ((ALPHA(5,I,J,5),I=1,NTEMP),J=1,NNE)/3*2.5120e-03,
     *6*1.0000e-02,3*6.3100e-03,6*1.0000e-02,21*1.5850e-02/
      DATA ((ALPHA(6,I,J,5),I=1,NTEMP),J=1,NNE)/3*3.9810e-03,
     *6*1.5850e-02,3*1.0000e-02,6*1.5850e-02,21*2.5120e-02/
      DATA ((ALPHA(7,I,J,5),I=1,NTEMP),J=1,NNE)/3*6.3100e-03,
     *6*2.5120e-02,3*1.5850e-02,6*2.5120e-02,21*3.9810e-02/
      DATA ((ALPHA(8,I,J,5),I=1,NTEMP),J=1,NNE)/3*1.0000e-02,
     *6*3.9810e-02,3*2.5120e-02,6*3.9810e-02,21*6.3100e-02/
      DATA ((ALPHA(9,I,J,5),I=1,NTEMP),J=1,NNE)/3*1.5850e-02,
     *6*6.3100e-02,3*3.9810e-02,6*6.3100e-02,21*1.0000e-01/
      DATA ((ALPHA(10,I,J,5),I=1,NTEMP),J=1,NNE)/3*2.5120e-02,
     *6*1.0000e-01,3*6.3100e-02,6*1.0000e-01,21*1.5850e-01/
      DATA ((ALPHA(11,I,J,5),I=1,NTEMP),J=1,NNE)/3*3.9810e-02,
     *6*1.5850e-01,3*1.0000e-01,6*1.5850e-01,21*2.5120e-01/
      DATA ((ALPHA(12,I,J,5),I=1,NTEMP),J=1,NNE)/3*6.3100e-02,
     *6*2.5120e-01,3*1.5850e-01,6*2.5120e-01,21*3.9810e-01/
      DATA ((ALPHA(13,I,J,5),I=1,NTEMP),J=1,NNE)/3*1.0000e-01,
     *6*3.9810e-01,3*2.5120e-01,6*3.9810e-01,21*6.3100e-01/
      DATA ((ALPHA(14,I,J,5),I=1,NTEMP),J=1,NNE)/3*1.5850e-01,
     *6*6.3100e-01,3*3.9810e-01,6*6.3100e-01,21*1.0000e+00/
      DATA ((ALPHA(15,I,J,5),I=1,NTEMP),J=1,NNE)/3*2.5120e-01,
     *6*1.0000e+00,3*6.3100e-01,6*1.0000e+00,21*1.5850e+00/
      DATA ((ALPHA(16,I,J,5),I=1,NTEMP),J=1,NNE)/3*3.9810e-01,
     *6*1.5850e+00,3*1.0000e+00,6*1.5850e+00,21*2.5120e+00/
      DATA ((ALPHA(17,I,J,5),I=1,NTEMP),J=1,NNE)/3*6.3100e-01,
     *6*2.5120e+00,3*1.5850e+00,6*2.5120e+00,9*3.9810e+00,0.0000e+00,
     *11*3.9810e+00/
      DATA ((ALPHA(18,I,J,5),I=1,NTEMP),J=1,NNE)/3*1.0000e+00,
     *6*3.9810e+00,3*2.5120e+00,6*3.9810e+00,9*6.3100e+00,0.0000e+00,
     *2*6.3100e+00,9*0.0000e+00/
      DATA ((ALPHA(19,I,J,5),I=1,NTEMP),J=1,NNE)/3*1.5850e+00,
     *6*6.3100e+00,3*3.9810e+00,6*6.3100e+00,9*1.0000e+01,2*0.0000e+00,
     *1.0000e+01,9*0.0000e+00/
      DATA ((ALPHA(20,I,J,5),I=1,NTEMP),J=1,NNE)/3*2.5120e+00,
     *6*1.0000e+01,3*6.3100e+00,6*1.0000e+01,6*1.5850e+01,2*0.0000e+00,
     *1.5850e+01,12*0.0000e+00/
      DATA ((ALPHA(21,I,J,5),I=1,NTEMP),J=1,NNE)/3*3.9810e+00,
     *6*1.5850e+01,3*1.0000e+01,6*1.5850e+01,3*2.5120e+01,0.0000e+00,
     *2*2.5120e+01,2*0.0000e+00,2.5120e+01,12*0.0000e+00/
      DATA ((ALPHA(22,I,J,5),I=1,NTEMP),J=1,NNE)/3*6.3100e+00,
     *6*2.5120e+01,3*1.5850e+01,6*2.5120e+01,0.0000e+00,2*3.9810e+01,
     *2*0.0000e+00,3.9810e+01,15*0.0000e+00/
      DATA ((ALPHA(23,I,J,5),I=1,NTEMP),J=1,NNE)/3*1.0000e+01,
     *6*3.9810e+01,3*2.5120e+01,6*3.9810e+01,0.0000e+00,2*6.3100e+01,
     *2*0.0000e+00,6.3100e+01,15*0.0000e+00/
      DATA ((ALPHA(24,I,J,5),I=1,NTEMP),J=1,NNE)/3*1.5850e+01,
     *6*6.3100e+01,3*3.9810e+01,6*6.3100e+01,2*0.0000e+00,1.0000e+02,
     *18*0.0000e+00/
      DATA ((ALPHA(25,I,J,5),I=1,NTEMP),J=1,NNE)/3*2.5120e+01,
     *6*1.0000e+02,3*6.3100e+01,3*1.0000e+02,0.0000e+00,2*1.0000e+02,
     *21*0.0000e+00/
      DATA ((ALPHA(26,I,J,5),I=1,NTEMP),J=1,NNE)/3*3.9810e+01,
     *6*1.5850e+02,3*1.0000e+02,3*1.5850e+02,0.0000e+00,2*1.5850e+02,
     *21*0.0000e+00/
      DATA ((ALPHA(27,I,J,5),I=1,NTEMP),J=1,NNE)/3*6.3100e+01,
     *6*2.5120e+02,3*1.5850e+02,0.0000e+00,2*2.5120e+02,2*0.0000e+00,
     *2.5120e+02,21*0.0000e+00/
      DATA ((ALPHA(28,I,J,5),I=1,NTEMP),J=1,NNE)/3*1.0000e+02,
     *6*3.9810e+02,3*2.5120e+02,2*0.0000e+00,3.9810e+02,24*0.0000e+00/
      DATA ((ALPHA(29,I,J,5),I=1,NTEMP),J=1,NNE)/3*1.5850e+02,
     *6*6.3100e+02,0.0000e+00,2*3.9810e+02,2*0.0000e+00,6.3100e+02,
     *24*0.0000e+00/
      DATA ((ALPHA(30,I,J,5),I=1,NTEMP),J=1,NNE)/3*2.5120e+02,
     *3*1.0000e+03,0.0000e+00,2*1.0000e+03,0.0000e+00,2*6.3100e+02,
     *27*0.0000e+00/
      DATA ((ALPHA(31,I,J,5),I=1,NTEMP),J=1,NNE)/3*3.9810e+02,
     *3*1.5850e+03,0.0000e+00,2*1.5850e+03,2*0.0000e+00,1.0000e+03,
     *27*0.0000e+00/
      DATA ((ALPHA(32,I,J,5),I=1,NTEMP),J=1,NNE)/3*6.3100e+02,
     *0.0000e+00,2*2.5120e+03,2*0.0000e+00,2.5120e+03,30*0.0000e+00/
      DATA ((ALPHA(33,I,J,5),I=1,NTEMP),J=1,NNE)/3*1.0000e+03,
     *2*0.0000e+00,3.9810e+03,33*0.0000e+00/
      DATA ((ALPHA(34,I,J,5),I=1,NTEMP),J=1,NNE)/3*1.5850e+03,
     *2*0.0000e+00,6.3100e+03,33*0.0000e+00/
      DATA ((ALPHA(35,I,J,5),I=1,NTEMP),J=1,NNE)/3*2.5120e+03,
     *36*0.0000e+00/
      DATA ((ALPHA(36,I,J,5),I=1,NTEMP),J=1,NNE)/0.0000e+00,
     *2*3.9810e+03,36*0.0000e+00/
      DATA ((ALPHA(37,I,J,5),I=1,NTEMP),J=1,NNE)/0.0000e+00,
     *2*6.3100e+03,36*0.0000e+00/
      DATA ((ALPHA(38,I,J,5),I=1,NTEMP),J=1,NNE)/2*0.0000e+00,
     *1.0000e+04,36*0.0000e+00/
      DATA ((ALPHA(39,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(40,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(41,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(42,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(43,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(44,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(45,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(46,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(1,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(2,I,J,6),I=1,NTEMP),J=1,NNE)/3*1.5850e-03,
     *3*2.5120e-03,3*3.9810e-03,12*6.3100e-03,9*3.9810e-03,9*6.3100e-03/
      DATA ((ALPHA(3,I,J,6),I=1,NTEMP),J=1,NNE)/3*2.5120e-03,
     *3*3.9810e-03,3*6.3100e-03,12*1.0000e-02,9*6.3100e-03,9*1.0000e-02/
      DATA ((ALPHA(4,I,J,6),I=1,NTEMP),J=1,NNE)/3*3.9810e-03,
     *3*6.3100e-03,3*1.0000e-02,12*1.5850e-02,9*1.0000e-02,9*1.5850e-02/
      DATA ((ALPHA(5,I,J,6),I=1,NTEMP),J=1,NNE)/3*6.3100e-03,
     *3*1.0000e-02,3*1.5850e-02,12*2.5120e-02,9*1.5850e-02,9*2.5120e-02/
      DATA ((ALPHA(6,I,J,6),I=1,NTEMP),J=1,NNE)/3*1.0000e-02,
     *3*1.5850e-02,3*2.5120e-02,12*3.9810e-02,9*2.5120e-02,9*3.9810e-02/
      DATA ((ALPHA(7,I,J,6),I=1,NTEMP),J=1,NNE)/3*1.5850e-02,
     *3*2.5120e-02,3*3.9810e-02,12*6.3100e-02,9*3.9810e-02,9*6.3100e-02/
      DATA ((ALPHA(8,I,J,6),I=1,NTEMP),J=1,NNE)/3*2.5120e-02,
     *3*3.9810e-02,3*6.3100e-02,12*1.0000e-01,9*6.3100e-02,9*1.0000e-01/
      DATA ((ALPHA(9,I,J,6),I=1,NTEMP),J=1,NNE)/3*3.9810e-02,
     *3*6.3100e-02,3*1.0000e-01,12*1.5850e-01,9*1.0000e-01,9*1.5850e-01/
      DATA ((ALPHA(10,I,J,6),I=1,NTEMP),J=1,NNE)/3*6.3100e-02,
     *3*1.0000e-01,3*1.5850e-01,12*2.5120e-01,9*1.5850e-01,9*2.5120e-01/
      DATA ((ALPHA(11,I,J,6),I=1,NTEMP),J=1,NNE)/3*1.0000e-01,
     *3*1.5850e-01,3*2.5120e-01,12*3.9810e-01,9*2.5120e-01,9*3.9810e-01/
      DATA ((ALPHA(12,I,J,6),I=1,NTEMP),J=1,NNE)/3*1.5850e-01,
     *3*2.5120e-01,3*3.9810e-01,12*6.3100e-01,9*3.9810e-01,9*6.3100e-01/
      DATA ((ALPHA(13,I,J,6),I=1,NTEMP),J=1,NNE)/3*2.5120e-01,
     *3*3.9810e-01,3*6.3100e-01,12*1.0000e+00,9*6.3100e-01,9*1.0000e+00/
      DATA ((ALPHA(14,I,J,6),I=1,NTEMP),J=1,NNE)/3*3.9810e-01,
     *3*6.3100e-01,3*1.0000e+00,12*1.5850e+00,4*1.0000e+00,0.0000e+00,
     *4*1.0000e+00,9*1.5850e+00/
      DATA ((ALPHA(15,I,J,6),I=1,NTEMP),J=1,NNE)/3*6.3100e-01,
     *3*1.0000e+00,3*1.5850e+00,12*2.5120e+00,4*1.5850e+00,0.0000e+00,
     *4*1.5850e+00,9*2.5120e+00/
      DATA ((ALPHA(16,I,J,6),I=1,NTEMP),J=1,NNE)/3*1.0000e+00,
     *3*1.5850e+00,3*2.5120e+00,12*3.9810e+00,4*2.5120e+00,0.0000e+00,
     *4*2.5120e+00,9*3.9810e+00/
      DATA ((ALPHA(17,I,J,6),I=1,NTEMP),J=1,NNE)/3*1.5850e+00,
     *3*2.5120e+00,3*3.9810e+00,12*6.3100e+00,4*3.9810e+00,0.0000e+00,
     *4*3.9810e+00,9*6.3100e+00/
      DATA ((ALPHA(18,I,J,6),I=1,NTEMP),J=1,NNE)/3*2.5120e+00,
     *3*3.9810e+00,3*6.3100e+00,12*1.0000e+01,0.0000e+00,3*6.3100e+00,
     *0.0000e+00,4*6.3100e+00,9*0.0000e+00/
      DATA ((ALPHA(19,I,J,6),I=1,NTEMP),J=1,NNE)/3*3.9810e+00,
     *3*6.3100e+00,3*1.0000e+01,9*1.5850e+01,0.0000e+00,2*1.5850e+01,
     *0.0000e+00,2*1.0000e+01,2*0.0000e+00,1.0000e+01,12*0.0000e+00/
      DATA ((ALPHA(20,I,J,6),I=1,NTEMP),J=1,NNE)/3*6.3100e+00,
     *3*1.0000e+01,3*1.5850e+01,9*2.5120e+01,0.0000e+00,2*2.5120e+01,
     *0.0000e+00,2*1.5850e+01,15*0.0000e+00/
      DATA ((ALPHA(21,I,J,6),I=1,NTEMP),J=1,NNE)/3*1.0000e+01,
     *3*1.5850e+01,3*2.5120e+01,6*3.9810e+01,0.0000e+00,2*3.9810e+01,
     *2*0.0000e+00,3.9810e+01,2*0.0000e+00,2.5120e+01,15*0.0000e+00/
      DATA ((ALPHA(22,I,J,6),I=1,NTEMP),J=1,NNE)/3*1.5850e+01,
     *3*2.5120e+01,3*3.9810e+01,6*6.3100e+01,2*0.0000e+00,6.3100e+01,
     *21*0.0000e+00/
      DATA ((ALPHA(23,I,J,6),I=1,NTEMP),J=1,NNE)/3*2.5120e+01,
     *3*3.9810e+01,3*6.3100e+01,3*1.0000e+02,0.0000e+00,2*1.0000e+02,
     *2*0.0000e+00,1.0000e+02,21*0.0000e+00/
      DATA ((ALPHA(24,I,J,6),I=1,NTEMP),J=1,NNE)/3*3.9810e+01,
     *3*6.3100e+01,3*1.0000e+02,0.0000e+00,2*1.5850e+02,2*0.0000e+00,
     *1.5850e+02,24*0.0000e+00/
      DATA ((ALPHA(25,I,J,6),I=1,NTEMP),J=1,NNE)/3*6.3100e+01,
     *3*1.0000e+02,3*1.5850e+02,0.0000e+00,2*2.5120e+02,2*0.0000e+00,
     *2.5120e+02,24*0.0000e+00/
      DATA ((ALPHA(26,I,J,6),I=1,NTEMP),J=1,NNE)/3*1.0000e+02,
     *3*1.5850e+02,3*2.5120e+02,2*0.0000e+00,3.9810e+02,27*0.0000e+00/
      DATA ((ALPHA(27,I,J,6),I=1,NTEMP),J=1,NNE)/3*1.5850e+02,
     *3*2.5120e+02,0.0000e+00,2*3.9810e+02,30*0.0000e+00/
      DATA ((ALPHA(28,I,J,6),I=1,NTEMP),J=1,NNE)/3*2.5120e+02,
     *3*3.9810e+02,0.0000e+00,2*6.3100e+02,30*0.0000e+00/
      DATA ((ALPHA(29,I,J,6),I=1,NTEMP),J=1,NNE)/3*3.9810e+02,
     *3*6.3100e+02,2*0.0000e+00,1.0000e+03,30*0.0000e+00/
      DATA ((ALPHA(30,I,J,6),I=1,NTEMP),J=1,NNE)/3*6.3100e+02,
     *0.0000e+00,2*1.0000e+03,33*0.0000e+00/
      DATA ((ALPHA(31,I,J,6),I=1,NTEMP),J=1,NNE)/3*1.0000e+03,
     *2*0.0000e+00,1.5850e+03,33*0.0000e+00/
      DATA ((ALPHA(32,I,J,6),I=1,NTEMP),J=1,NNE)/0.0000e+00,
     *2*1.5850e+03,2*0.0000e+00,2.5120e+03,33*0.0000e+00/
      DATA ((ALPHA(33,I,J,6),I=1,NTEMP),J=1,NNE)/0.0000e+00,
     *2*2.5120e+03,36*0.0000e+00/
      DATA ((ALPHA(34,I,J,6),I=1,NTEMP),J=1,NNE)/2*0.0000e+00,
     *3.9810e+03,36*0.0000e+00/
      DATA ((ALPHA(35,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(36,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(37,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(38,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(39,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(40,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(41,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(42,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(43,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(44,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(45,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((ALPHA(46,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/

      DATA ((SPROF(1,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8790e-01,6.2790e-01,2.7090e+00,1.9150e+00,
     *1.3510e+00,5.8310e+00,4.1270e+00,2.9130e+00,1.2570e+01,8.8880e+00,
     *6.2800e+00,2.7000e+01,1.9110e+01,1.3520e+01,5.7520e+01,4.0910e+01,
     *2.9020e+01,1.2020e+02,8.6400e+01,6.1760e+01,2.4070e+02,1.7670e+02,
     *1.2850e+02,4.4440e+02,3.3790e+02,2.5400e+02,7.3080e+02,5.8010e+02,
     *4.5540e+02/
      DATA ((SPROF(2,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8790e-01,6.2790e-01,2.7090e+00,1.9150e+00,
     *1.3510e+00,5.8310e+00,4.1270e+00,2.9130e+00,1.2570e+01,8.8880e+00,
     *6.2800e+00,2.7000e+01,1.9110e+01,1.3520e+01,5.7520e+01,4.0910e+01,
     *2.9020e+01,1.2020e+02,8.6400e+01,6.1760e+01,2.4070e+02,1.7670e+02,
     *1.2850e+02,4.4430e+02,3.3790e+02,2.5400e+02,7.3060e+02,5.8000e+02,
     *4.5540e+02/
      DATA ((SPROF(3,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8790e-01,6.2790e-01,2.7090e+00,1.9150e+00,
     *1.3510e+00,5.8310e+00,4.1270e+00,2.9130e+00,1.2570e+01,8.8880e+00,
     *6.2800e+00,2.7000e+01,1.9110e+01,1.3520e+01,5.7520e+01,4.0910e+01,
     *2.9020e+01,1.2020e+02,8.6400e+01,6.1760e+01,2.4070e+02,1.7670e+02,
     *1.2850e+02,4.4410e+02,3.3780e+02,2.5400e+02,7.3030e+02,5.7990e+02,
     *4.5530e+02/
      DATA ((SPROF(4,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8790e-01,6.2790e-01,2.7090e+00,1.9150e+00,
     *1.3510e+00,5.8310e+00,4.1270e+00,2.9130e+00,1.2570e+01,8.8880e+00,
     *6.2800e+00,2.7000e+01,1.9110e+01,1.3520e+01,5.7520e+01,4.0910e+01,
     *2.9020e+01,1.2020e+02,8.6390e+01,6.1760e+01,2.4060e+02,1.7670e+02,
     *1.2850e+02,4.4380e+02,3.3770e+02,2.5390e+02,7.2960e+02,5.7950e+02,
     *4.5520e+02/
      DATA ((SPROF(5,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8790e-01,6.2790e-01,2.7090e+00,1.9150e+00,
     *1.3510e+00,5.8310e+00,4.1270e+00,2.9130e+00,1.2570e+01,8.8880e+00,
     *6.2800e+00,2.7000e+01,1.9110e+01,1.3520e+01,5.7520e+01,4.0910e+01,
     *2.9020e+01,1.2020e+02,8.6390e+01,6.1760e+01,2.4050e+02,1.7660e+02,
     *1.2850e+02,4.4300e+02,3.3730e+02,2.5370e+02,7.2770e+02,5.7870e+02,
     *4.5480e+02/
      DATA ((SPROF(6,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8790e-01,6.2790e-01,2.7090e+00,1.9150e+00,
     *1.3510e+00,5.8310e+00,4.1270e+00,2.9130e+00,1.2570e+01,8.8880e+00,
     *6.2800e+00,2.7000e+01,1.9110e+01,1.3520e+01,5.7520e+01,4.0900e+01,
     *2.9020e+01,1.2020e+02,8.6380e+01,6.1750e+01,2.4020e+02,1.7650e+02,
     *1.2850e+02,4.4090e+02,3.3650e+02,2.5340e+02,7.2310e+02,5.7660e+02,
     *4.5390e+02/
      DATA ((SPROF(7,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8790e-01,6.2790e-01,2.7090e+00,1.9150e+00,
     *1.3510e+00,5.8310e+00,4.1270e+00,2.9130e+00,1.2570e+01,8.8880e+00,
     *6.2800e+00,2.7000e+01,1.9110e+01,1.3520e+01,5.7510e+01,4.0900e+01,
     *2.9020e+01,1.2010e+02,8.6340e+01,6.1740e+01,2.3950e+02,1.7630e+02,
     *1.2830e+02,4.3580e+02,3.3440e+02,2.5260e+02,7.1160e+02,5.7140e+02,
     *4.5160e+02/
      DATA ((SPROF(8,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8790e-01,6.2790e-01,2.7090e+00,1.9150e+00,
     *1.3510e+00,5.8310e+00,4.1270e+00,2.9130e+00,1.2570e+01,8.8880e+00,
     *6.2800e+00,2.6990e+01,1.9110e+01,1.3520e+01,5.7490e+01,4.0890e+01,
     *2.9020e+01,1.1990e+02,8.6270e+01,6.1710e+01,2.3770e+02,1.7560e+02,
     *1.2810e+02,4.2330e+02,3.2910e+02,2.5040e+02,6.8360e+02,5.5850e+02,
     *4.4590e+02/
      DATA ((SPROF(9,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8790e-01,6.2790e-01,2.7090e+00,1.9150e+00,
     *1.3510e+00,5.8310e+00,4.1270e+00,2.9130e+00,1.2560e+01,8.8880e+00,
     *6.2790e+00,2.6990e+01,1.9110e+01,1.3520e+01,5.7430e+01,4.0870e+01,
     *2.9010e+01,1.1940e+02,8.6070e+01,6.1640e+01,2.3330e+02,1.7390e+02,
     *1.2740e+02,3.9350e+02,3.1630e+02,2.4520e+02,6.1930e+02,5.2770e+02,
     *4.3190e+02/
      DATA ((SPROF(10,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8790e-01,6.2790e-01,2.7090e+00,1.9150e+00,
     *1.3510e+00,5.8310e+00,4.1270e+00,2.9130e+00,1.2560e+01,8.8870e+00,
     *6.2790e+00,2.6970e+01,1.9100e+01,1.3520e+01,5.7290e+01,4.0820e+01,
     *2.8990e+01,1.1800e+02,8.5590e+01,6.1470e+01,2.2260e+02,1.6960e+02,
     *1.2580e+02,3.2850e+02,2.8650e+02,2.3250e+02,4.8850e+02,4.5890e+02,
     *3.9900e+02/
      DATA ((SPROF(11,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8790e-01,6.2790e-01,2.7090e+00,1.9150e+00,
     *1.3510e+00,5.8310e+00,4.1270e+00,2.9130e+00,1.2560e+01,8.8860e+00,
     *6.2790e+00,2.6940e+01,1.9090e+01,1.3510e+01,5.6930e+01,4.0690e+01,
     *2.8950e+01,1.1480e+02,8.4380e+01,6.1020e+01,1.9780e+02,1.5950e+02,
     *1.2190e+02,2.1270e+02,2.2430e+02,2.0350e+02,2.8910e+02,3.2890e+02,
     *3.2820e+02/
      DATA ((SPROF(12,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8790e-01,6.2790e-01,2.7090e+00,1.9150e+00,
     *1.3510e+00,5.8310e+00,4.1270e+00,2.9130e+00,1.2550e+01,8.8820e+00,
     *6.2780e+00,2.6840e+01,1.9060e+01,1.3500e+01,5.6050e+01,4.0370e+01,
     *2.8830e+01,1.0700e+02,8.1410e+01,5.9930e+01,1.4760e+02,1.3660e+02,
     *1.1240e+02,8.2410e+01,1.2470e+02,1.4640e+02,1.1570e+02,1.6060e+02,
     *2.0650e+02/
      DATA ((SPROF(13,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8790e-01,6.2790e-01,2.7090e+00,1.9150e+00,
     *1.3510e+00,5.8300e+00,4.1270e+00,2.9130e+00,1.2530e+01,8.8740e+00,
     *6.2750e+00,2.6610e+01,1.8970e+01,1.3470e+01,5.3880e+01,3.9580e+01,
     *2.8540e+01,8.9830e+01,7.4430e+01,5.7260e+01,7.2410e+01,9.2930e+01,
     *9.1910e+01,1.9370e+01,3.5480e+01,6.6350e+01,4.0580e+01,5.1610e+01,
     *7.8190e+01/
      DATA ((SPROF(14,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8790e-01,6.2790e-01,2.7090e+00,1.9150e+00,
     *1.3510e+00,5.8280e+00,4.1260e+00,2.9120e+00,1.2470e+01,8.8530e+00,
     *6.2670e+00,2.6040e+01,1.8770e+01,1.3400e+01,4.8810e+01,3.7650e+01,
     *2.7840e+01,5.8000e+01,5.9430e+01,5.1070e+01,1.5220e+01,3.6400e+01,
     *5.5630e+01,5.3970e+00,6.5810e+00,1.2720e+01,1.4380e+01,1.6110e+01,
     *1.9650e+01/
      DATA ((SPROF(15,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8790e-01,6.2790e-01,2.7080e+00,1.9150e+00,
     *1.3510e+00,5.8220e+00,4.1240e+00,2.9120e+00,1.2320e+01,8.8010e+00,
     *6.2490e+00,2.4670e+01,1.8270e+01,1.3220e+01,3.8090e+01,3.3220e+01,
     *2.6140e+01,1.9790e+01,3.3870e+01,3.8330e+01,2.1210e+00,4.8850e+00,
     *1.6360e+01,1.7320e+00,1.7750e+00,1.9970e+00,5.0210e+00,5.2230e+00,
     *5.5210e+00/
      DATA ((SPROF(16,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8780e-01,6.2790e-01,2.7080e+00,1.9150e+00,
     *1.3510e+00,5.8070e+00,4.1180e+00,2.9100e+00,1.1960e+01,8.6720e+00,
     *6.2030e+00,2.1520e+01,1.7060e+01,1.2770e+01,2.0470e+01,2.4260e+01,
     *2.2310e+01,1.8550e+00,8.4880e+00,1.8690e+01,5.7380e-01,6.2740e-01,
     *1.3110e+00,5.6910e-01,5.5300e-01,5.4950e-01,1.7670e+00,1.7360e+00,
     *1.7110e+00/
      DATA ((SPROF(17,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2580e+00,8.8780e-01,6.2790e-01,2.7060e+00,1.9140e+00,
     *1.3510e+00,5.7700e+00,4.1050e+00,2.9050e+00,1.1110e+01,8.3560e+00,
     *6.0890e+00,1.5280e+01,1.4360e+01,1.1720e+01,4.3920e+00,1.1040e+01,
     *1.5000e+01,1.9680e-01,4.5140e-01,3.1860e+00,1.7710e-01,1.7460e-01,
     *1.8210e-01,1.8500e-01,1.7690e-01,1.6910e-01,5.9920e-01,5.7590e-01,
     *5.5070e-01/
      DATA ((SPROF(18,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2570e+00,8.8770e-01,6.2780e-01,2.7030e+00,1.9130e+00,
     *1.3510e+00,5.6780e+00,4.0730e+00,2.8940e+00,9.2180e+00,7.6110e+00,
     *5.8110e+00,6.4730e+00,9.3290e+00,9.4370e+00,1.5490e-01,1.5660e+00,
     *5.5440e+00,5.6560e-02,5.7780e-02,9.8180e-02,5.7970e-02,5.5150e-02,
     *5.3230e-02,6.1790e-02,5.8420e-02,5.5030e-02,1.9650e-01,1.8770e-01,
     *1.7830e-01/
      DATA ((SPROF(19,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8340e-01,4.1160e-01,
     *2.9120e-01,1.2560e+00,8.8730e-01,6.2770e-01,2.6930e+00,1.9090e+00,
     *1.3490e+00,5.4540e+00,3.9910e+00,2.8650e+00,5.7720e+00,6.0210e+00,
     *5.1680e+00,7.6220e-01,3.1590e+00,5.4820e+00,1.8720e-02,3.1420e-02,
     *4.7030e-01,1.8140e-02,1.7430e-02,1.7360e-02,1.9220e-02,1.8100e-02,
     *1.7080e-02,2.0460e-02,1.9390e-02,1.8200e-02,6.5060e-02,6.2610e-02,
     *5.9050e-02/
      DATA ((SPROF(20,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8330e-01,4.1150e-01,
     *2.9120e-01,1.2540e+00,8.8650e-01,6.2740e-01,2.6700e+00,1.9010e+00,
     *1.3460e+00,4.9300e+00,3.7950e+00,2.7930e+00,1.7820e+00,3.3410e+00,
     *3.8490e+00,9.9900e-03,2.1350e-01,1.4030e+00,5.7170e-03,5.6410e-03,
     *6.9240e-03,5.9590e-03,5.6180e-03,5.3480e-03,6.3510e-03,5.9850e-03,
     *5.6090e-03,6.6970e-03,6.4000e-03,6.0290e-03,2.1430e-02,2.0650e-02,
     *1.9580e-02/
      DATA ((SPROF(21,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2760e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8300e-01,4.1150e-01,
     *2.9110e-01,1.2480e+00,8.8440e-01,6.2670e-01,2.6120e+00,1.8810e+00,
     *1.3390e+00,3.8240e+00,3.3420e+00,2.6210e+00,9.4920e-02,7.6230e-01,
     *1.8360e+00,1.8430e-03,2.1710e-03,4.7580e-02,1.8490e-03,1.7560e-03,
     *1.7070e-03,1.9640e-03,1.8460e-03,1.7330e-03,2.0820e-03,1.9760e-03,
     *1.8540e-03,2.1330e-03,2.0940e-03,1.9900e-03,6.7490e-03,6.7390e-03,
     *6.4490e-03/
      DATA ((SPROF(22,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2750e-02,2.7060e-01,1.9130e-01,1.3510e-01,5.8240e-01,4.1130e-01,
     *2.9110e-01,1.2330e+00,8.7920e-01,6.2480e-01,2.4730e+00,1.8300e+00,
     *1.3210e+00,2.0210e+00,2.4290e+00,2.2350e+00,6.8370e-04,1.9240e-02,
     *2.8660e-01,5.7680e-04,5.5850e-04,5.8020e-04,6.0650e-04,5.7000e-04,
     *5.3870e-04,6.4480e-04,6.0860e-04,5.6970e-04,6.7570e-04,6.4830e-04,
     *6.1220e-04,6.7450e-04,6.7840e-04,6.5220e-04,2.1340e-03,2.1350e-03,
     *2.1060e-03/
      DATA ((SPROF(23,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8780e-02,
     *6.2750e-02,2.7050e-01,1.9130e-01,1.3510e-01,5.8100e-01,4.1070e-01,
     *2.9090e-01,1.1970e+00,8.6630e-01,6.2020e-01,2.1550e+00,1.7080e+00,
     *1.2760e+00,4.0750e-01,1.0900e+00,1.4970e+00,1.8270e-04,1.8650e-04,
     *2.8920e-03,1.8730e-04,1.7690e-04,1.6970e-04,1.9920e-04,1.8730e-04,
     *1.7550e-04,2.1020e-04,2.0020e-04,1.8820e-04,2.1320e-04,2.1110e-04,
     *2.0140e-04,0.0000e+00,2.1350e-04,2.1210e-04,2*0.0000e+00,
     *6.7600e-04/
      DATA ((SPROF(24,I,J,1),I=1,NTEMP),J=1,NNE)/1.2540e-01,8.8760e-02,
     *6.2750e-02,2.7040e-01,1.9120e-01,1.3510e-01,5.7730e-01,4.0940e-01,
     *2.9040e-01,1.1120e+00,8.3460e-01,6.0880e-01,1.5260e+00,1.4370e+00,
     *1.1710e+00,7.5070e-03,1.4580e-01,5.4700e-01,5.8120e-05,5.5600e-05,
     *5.5170e-05,6.1420e-05,5.7690e-05,5.4280e-05,6.5160e-05,6.1670e-05,
     *5.7780e-05,6.7400e-05,6.5470e-05,6.2030e-05,0.0000e+00,6.7460e-05,
     *6.5830e-05,0.0000e+00,2*6.7610e-05,2*0.0000e+00,2.1420e-04/
      DATA ((SPROF(25,I,J,1),I=1,NTEMP),J=1,NNE)/1.2530e-01,8.8730e-02,
     *6.2740e-02,2.7000e-01,1.9110e-01,1.3500e-01,5.6810e-01,4.0620e-01,
     *2.8930e-01,9.2200e-01,7.6020e-01,5.8100e-01,6.4090e-01,9.3140e-01,
     *9.4240e-01,6.0300e-05,9.9460e-04,4.3680e-02,1.8950e-05,1.7840e-05,
     *1.6960e-05,2.0150e-05,1.8980e-05,1.7770e-05,2.1170e-05,2.0240e-05,
     *1.9070e-05,2.1320e-05,2.1250e-05,2.0360e-05,0.0000e+00,2.1360e-05,
     *2.1350e-05,2*0.0000e+00,2.1430e-05,3*0.0000e+00/
      DATA ((SPROF(26,I,J,1),I=1,NTEMP),J=1,NNE)/1.2510e-01,8.8650e-02,
     *6.2710e-02,2.6910e-01,1.9080e-01,1.3490e-01,5.4570e-01,3.9810e-01,
     *2.8640e-01,5.7650e-01,6.0110e-01,5.1660e-01,7.2560e-02,3.1320e-01,
     *5.4650e-01,1.8220e-05,1.7950e-05,9.5370e-05,6.2170e-06,5.8400e-06,
     *5.4800e-06,6.5760e-06,6.2440e-06,5.8580e-06,6.7400e-06,6.6060e-06,
     *6.2820e-06,0.0000e+00,6.7480e-06,6.6400e-06,2*0.0000e+00,
     *6.7650e-06,6*0.0000e+00/
      DATA ((SPROF(27,I,J,1),I=1,NTEMP),J=1,NNE)/1.2450e-01,8.8440e-02,
     *6.2630e-02,2.6670e-01,1.8990e-01,1.3460e-01,4.9320e-01,3.7840e-01,
     *2.7920e-01,1.7720e-01,3.3330e-01,3.8470e-01,3.2560e-04,2.0300e-02,
     *1.3900e-01,5.8600e-06,5.5620e-06,5.4100e-06,2.0360e-06,1.9220e-06,
     *1.8000e-06,2.1300e-06,2.0460e-06,1.9330e-06,2.1320e-06,2.1330e-06,
     *2.0580e-06,2*0.0000e+00,2.1370e-06,9*0.0000e+00/
      DATA ((SPROF(28,I,J,1),I=1,NTEMP),J=1,NNE)/1.2300e-01,8.7920e-02,
     *6.2450e-02,2.6100e-01,1.8790e-01,1.3390e-01,3.8250e-01,3.3330e-01,
     *2.6200e-01,9.1640e-03,7.5750e-02,1.8340e-01,5.8510e-06,2.7130e-05,
     *4.4720e-03,1.9170e-06,1.8020e-06,1.7020e-06,6.6340e-07,6.3220e-07,
     *5.9420e-07,6.7410e-07,6.6600e-07,6.3570e-07,0.0000e+00,6.7500e-07,
     *6.6930e-07,2*0.0000e+00,6.7710e-07,9*0.0000e+00/
      DATA ((SPROF(29,I,J,1),I=1,NTEMP),J=1,NNE)/1.1940e-01,8.6630e-02,
     *6.1990e-02,2.4710e-01,1.8280e-01,1.3210e-01,2.0200e-01,2.4220e-01,
     *2.2340e-01,7.3550e-06,1.8350e-03,2.8530e-02,1.8260e-06,1.7670e-06,
     *2.6020e-06,6.2910e-07,5.9160e-07,5.5430e-07,2.1310e-07,2.0670e-07,
     *1.9590e-07,0.0000e+00,2.1330e-07,2.0790e-07,2*0.0000e+00,
     *2.1380e-07,12*0.0000e+00/
      DATA ((SPROF(30,I,J,1),I=1,NTEMP),J=1,NNE)/1.1090e-01,8.3460e-02,
     *6.0850e-02,2.1530e-01,1.7060e-01,1.2760e-01,4.0640e-02,1.0860e-01,
     *1.4960e-01,5.7780e-07,7.4460e-07,2.6690e-04,5.9190e-07,5.5900e-07,
     *5.3650e-07,2.0580e-07,1.9480e-07,1.8250e-07,6.7420e-08,6.7120e-08,
     *6.4300e-08,0.0000e+00,6.7530e-08,6.7450e-08,2*0.0000e+00,
     *6.7780e-08,12*0.0000e+00/
      DATA ((SPROF(31,I,J,1),I=1,NTEMP),J=1,NNE)/9.1950e-02,7.6010e-02,
     *5.8070e-02,1.5240e-01,1.4360e-01,1.1700e-01,7.2450e-04,1.4500e-02,
     *5.4650e-02,1.8360e-07,1.7560e-07,1.7640e-07,1.9400e-07,1.8220e-07,
     *1.7150e-07,6.6850e-08,6.3950e-08,6.0250e-08,0.0000e+00,2.1340e-08,
     *2.0980e-08,2*0.0000e+00,2.1390e-08,15*0.0000e+00/
      DATA ((SPROF(32,I,J,1),I=1,NTEMP),J=1,NNE)/5.7490e-02,6.0100e-02,
     *5.1630e-02,6.4000e-02,9.3030e-02,9.4210e-02,2.1890e-07,9.2300e-05,
     *4.3550e-03,5.9870e-08,5.6370e-08,5.3590e-08,6.3660e-08,5.9970e-08,
     *5.6120e-08,2.1310e-08,2.0880e-08,1.9850e-08,2*0.0000e+00,
     *6.7560e-09,18*0.0000e+00/
      DATA ((SPROF(33,I,J,1),I=1,NTEMP),J=1,NNE)/1.7670e-02,3.3320e-02,
     *3.8450e-02,7.2360e-03,3.1280e-02,5.4630e-02,5.7580e-08,5.6980e-08,
     *7.6380e-06,1.9650e-08,1.8460e-08,1.7320e-08,2.0780e-08,1.9730e-08,
     *1.8520e-08,2*6.7430e-09,6.5030e-09,2*0.0000e+00,2.1410e-09,
     *18*0.0000e+00/
      DATA ((SPROF(34,I,J,1),I=1,NTEMP),J=1,NNE)/9.1280e-04,7.5730e-03,
     *1.8330e-02,3.0370e-05,2.0240e-03,1.3900e-02,1.8520e-08,1.7580e-08,
     *1.7100e-08,6.4360e-09,6.0770e-09,5.6890e-09,6.7300e-09,6.4680e-09,
     *6.1100e-09,0.0000e+00,2.1340e-09,2.1160e-09,21*0.0000e+00/
      DATA ((SPROF(35,I,J,1),I=1,NTEMP),J=1,NNE)/5.4060e-07,1.8320e-04,
     *2.8510e-03,1.8520e-08,2.1070e-06,4.4620e-04,6.0590e-09,5.6940e-09,
     *5.3800e-09,2.0970e-09,1.9980e-09,1.8780e-09,2.1310e-09,2.1050e-09,
     *2.0100e-09,2*0.0000e+00,6.7610e-10,21*0.0000e+00/
      DATA ((SPROF(36,I,J,1),I=1,NTEMP),J=1,NNE)/1.8270e-09,1.7810e-08,
     *2.6610e-05,5.7710e-09,5.5840e-09,8.4870e-08,1.9890e-09,1.8710e-09,
     *1.7520e-09,6.7370e-10,6.5370e-10,6.1950e-10,0.0000e+00,6.7450e-10,
     *6.5720e-10,2*0.0000e+00,2.1430e-10,21*0.0000e+00/
      DATA ((SPROF(37,I,J,1),I=1,NTEMP),J=1,NNE)/5.8040e-10,5.5520e-10,
     *7.6270e-10,1.8710e-09,1.7670e-09,1.6960e-09,6.5050e-10,6.1590e-10,
     *5.7710e-10,2.1320e-10,2.1220e-10,2.0340e-10,0.0000e+00,2.1350e-10,
     *2.1330e-10,24*0.0000e+00/
      DATA ((SPROF(38,I,J,1),I=1,NTEMP),J=1,NNE)/1.8930e-10,1.7820e-10,
     *1.6940e-10,6.1350e-10,5.7620e-10,5.4210e-10,2.1140e-10,2.0220e-10,
     *1.9050e-10,0.0000e+00,6.7470e-11,6.6350e-11,2*0.0000e+00,
     *6.7650e-11,24*0.0000e+00/
      DATA ((SPROF(39,I,J,1),I=1,NTEMP),J=1,NNE)/6.2130e-11,5.8360e-11,
     *5.4760e-11,2.0120e-10,1.8960e-10,1.7750e-10,6.7380e-11,6.5970e-11,
     *6.2760e-11,2*0.0000e+00,2.1370e-11,27*0.0000e+00/
      DATA ((SPROF(40,I,J,1),I=1,NTEMP),J=1,NNE)/2.0350e-11,1.9220e-11,
     *1.7990e-11,6.5710e-11,6.2410e-11,5.8550e-11,2*2.1320e-11,
     *2.0570e-11,2*0.0000e+00,6.7710e-12,27*0.0000e+00/
      DATA ((SPROF(41,I,J,1),I=1,NTEMP),J=1,NNE)/6.6290e-12,6.3180e-12,
     *5.9390e-12,2.1280e-11,2.0450e-11,1.9320e-11,0.0000e+00,6.7500e-12,
     *6.6920e-12,30*0.0000e+00/
      DATA ((SPROF(42,I,J,1),I=1,NTEMP),J=1,NNE)/2.1310e-12,2.0670e-12,
     *1.9590e-12,6.7400e-12,6.6570e-12,6.3560e-12,2*0.0000e+00,
     *2.1380e-12,30*0.0000e+00/
      DATA ((SPROF(43,I,J,1),I=1,NTEMP),J=1,NNE)/6.7410e-13,6.7100e-13,
     *6.4330e-13,0.0000e+00,2.1330e-12,2.0780e-12,2*0.0000e+00,
     *6.7780e-13,30*0.0000e+00/
      DATA ((SPROF(44,I,J,1),I=1,NTEMP),J=1,NNE)/0.0000e+00,2.1340e-13,
     *2.0980e-13,0.0000e+00,6.7530e-13,6.7450e-13,33*0.0000e+00/
      DATA ((SPROF(45,I,J,1),I=1,NTEMP),J=1,NNE)/2*0.0000e+00,
     *6.7560e-14,2*0.0000e+00,2.1390e-13,33*0.0000e+00/
      DATA ((SPROF(46,I,J,1),I=1,NTEMP),J=1,NNE)/2*0.0000e+00,
     *2.1410e-14,36*0.0000e+00/
      DATA ((SPROF(1,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9100e-01,4.8850e-01,
     *3.4560e-01,1.4890e+00,1.0530e+00,7.4400e-01,3.2050e+00,2.2670e+00,
     *1.6030e+00,6.8840e+00,4.8730e+00,3.4500e+00,1.4700e+01,1.0450e+01,
     *7.4070e+00,3.0800e+01,2.2130e+01,1.5800e+01,6.1410e+01,4.5360e+01,
     *3.3010e+01,1.0900e+02,8.5570e+01,6.5230e+01,1.5720e+02,1.3550e+02,
     *1.1280e+02,1.7670e+02,1.6510e+02,1.5320e+02,1.7250e+02,1.6210e+02,
     *1.5840e+02/
      DATA ((SPROF(2,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9100e-01,4.8850e-01,
     *3.4560e-01,1.4890e+00,1.0530e+00,7.4400e-01,3.2050e+00,2.2670e+00,
     *1.6030e+00,6.8840e+00,4.8730e+00,3.4500e+00,1.4700e+01,1.0450e+01,
     *7.4070e+00,3.0800e+01,2.2130e+01,1.5800e+01,6.1400e+01,4.5360e+01,
     *3.3010e+01,1.0900e+02,8.5550e+01,6.5220e+01,1.5700e+02,1.3540e+02,
     *1.1270e+02,1.7640e+02,1.6470e+02,1.5280e+02,1.7310e+02,1.6240e+02,
     *1.5850e+02/
      DATA ((SPROF(3,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9100e-01,4.8850e-01,
     *3.4560e-01,1.4890e+00,1.0530e+00,7.4400e-01,3.2050e+00,2.2670e+00,
     *1.6030e+00,6.8840e+00,4.8730e+00,3.4500e+00,1.4700e+01,1.0450e+01,
     *7.4070e+00,3.0800e+01,2.2130e+01,1.5800e+01,6.1390e+01,4.5350e+01,
     *3.3010e+01,1.0890e+02,8.5510e+01,6.5210e+01,1.5680e+02,1.3520e+02,
     *1.1260e+02,1.7600e+02,1.6420e+02,1.5230e+02,1.7390e+02,1.6290e+02,
     *1.5870e+02/
      DATA ((SPROF(4,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9100e-01,4.8850e-01,
     *3.4560e-01,1.4890e+00,1.0530e+00,7.4400e-01,3.2050e+00,2.2670e+00,
     *1.6030e+00,6.8840e+00,4.8730e+00,3.4500e+00,1.4700e+01,1.0450e+01,
     *7.4070e+00,3.0800e+01,2.2130e+01,1.5800e+01,6.1360e+01,4.5340e+01,
     *3.3000e+01,1.0870e+02,8.5430e+01,6.5170e+01,1.5610e+02,1.3480e+02,
     *1.1240e+02,1.7480e+02,1.6280e+02,1.5090e+02,1.7590e+02,1.6410e+02,
     *1.5900e+02/
      DATA ((SPROF(5,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9100e-01,4.8850e-01,
     *3.4560e-01,1.4890e+00,1.0530e+00,7.4400e-01,3.2050e+00,2.2670e+00,
     *1.6030e+00,6.8840e+00,4.8730e+00,3.4500e+00,1.4700e+01,1.0440e+01,
     *7.4070e+00,3.0790e+01,2.2120e+01,1.5800e+01,6.1280e+01,4.5310e+01,
     *3.2990e+01,1.0820e+02,8.5220e+01,6.5080e+01,1.5460e+02,1.3390e+02,
     *1.1190e+02,1.7120e+02,1.5910e+02,1.4760e+02,1.7930e+02,1.6630e+02,
     *1.5960e+02/
      DATA ((SPROF(6,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9100e-01,4.8850e-01,
     *3.4560e-01,1.4890e+00,1.0530e+00,7.4400e-01,3.2050e+00,2.2670e+00,
     *1.6030e+00,6.8840e+00,4.8730e+00,3.4500e+00,1.4700e+01,1.0440e+01,
     *7.4070e+00,3.0760e+01,2.2110e+01,1.5790e+01,6.1100e+01,4.5240e+01,
     *3.2960e+01,1.0710e+02,8.4690e+01,6.4860e+01,1.5070e+02,1.3150e+02,
     *1.1060e+02,1.5940e+02,1.4940e+02,1.3950e+02,1.8160e+02,1.6860e+02,
     *1.5970e+02/
      DATA ((SPROF(7,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9100e-01,4.8850e-01,
     *3.4560e-01,1.4890e+00,1.0530e+00,7.4400e-01,3.2050e+00,2.2670e+00,
     *1.6030e+00,6.8840e+00,4.8730e+00,3.4500e+00,1.4690e+01,1.0440e+01,
     *7.4070e+00,3.0710e+01,2.2090e+01,1.5790e+01,6.0630e+01,4.5050e+01,
     *3.2890e+01,1.0430e+02,8.3380e+01,6.4300e+01,1.4150e+02,1.2580e+02,
     *1.0750e+02,1.2680e+02,1.2520e+02,1.2070e+02,1.6930e+02,1.6250e+02,
     *1.5460e+02/
      DATA ((SPROF(8,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9100e-01,4.8850e-01,
     *3.4560e-01,1.4890e+00,1.0530e+00,7.4400e-01,3.2050e+00,2.2670e+00,
     *1.6030e+00,6.8830e+00,4.8720e+00,3.4500e+00,1.4690e+01,1.0440e+01,
     *7.4060e+00,3.0570e+01,2.2040e+01,1.5770e+01,5.9490e+01,4.4600e+01,
     *3.2720e+01,9.7500e+01,8.0190e+01,6.2910e+01,1.2090e+02,1.1260e+02,
     *9.9980e+01,7.2600e+01,7.9150e+01,8.3920e+01,1.2590e+02,1.2910e+02,
     *1.2960e+02/
      DATA ((SPROF(9,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9100e-01,4.8850e-01,
     *3.4560e-01,1.4890e+00,1.0530e+00,7.4400e-01,3.2050e+00,2.2670e+00,
     *1.6030e+00,6.8810e+00,4.8720e+00,3.4500e+00,1.4670e+01,1.0440e+01,
     *7.4040e+00,3.0220e+01,2.1910e+01,1.5720e+01,5.6700e+01,4.3480e+01,
     *3.2290e+01,8.2560e+01,7.2740e+01,5.9580e+01,8.2740e+01,8.5930e+01,
     *8.3580e+01,2.9660e+01,3.2900e+01,3.7940e+01,6.8040e+01,7.2990e+01,
     *7.7690e+01/
      DATA ((SPROF(10,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9100e-01,4.8850e-01,
     *3.4560e-01,1.4890e+00,1.0530e+00,7.4400e-01,3.2040e+00,2.2670e+00,
     *1.6030e+00,6.8780e+00,4.8710e+00,3.4490e+00,1.4630e+01,1.0420e+01,
     *7.3990e+00,2.9360e+01,2.1590e+01,1.5600e+01,5.0290e+01,4.0780e+01,
     *3.1230e+01,5.5110e+01,5.7090e+01,5.1980e+01,3.7110e+01,4.5960e+01,
     *5.4180e+01,1.0400e+01,1.0810e+01,1.1580e+01,2.8510e+01,3.0260e+01,
     *3.1850e+01/
      DATA ((SPROF(11,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9100e-01,4.8850e-01,
     *3.4560e-01,1.4890e+00,1.0530e+00,7.4400e-01,3.2040e+00,2.2670e+00,
     *1.6030e+00,6.8680e+00,4.8670e+00,3.4480e+00,1.4540e+01,1.0390e+01,
     *7.3870e+00,2.7310e+01,2.0800e+01,1.5310e+01,3.7300e+01,3.4740e+01,
     *2.8720e+01,2.2130e+01,3.1740e+01,3.7050e+01,1.1430e+01,1.4210e+01,
     *2.0670e+01,3.4870e+00,3.4500e+00,3.4210e+00,1.0510e+01,1.0730e+01,
     *1.0810e+01/
      DATA ((SPROF(12,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9100e-01,4.8850e-01,
     *3.4560e-01,1.4890e+00,1.0530e+00,7.4400e-01,3.2030e+00,2.2660e+00,
     *1.6020e+00,6.8440e+00,4.8580e+00,3.4450e+00,1.4310e+01,1.0300e+01,
     *7.3570e+00,2.2770e+01,1.8950e+01,1.4600e+01,1.7980e+01,2.3300e+01,
     *2.3290e+01,4.7530e+00,8.6730e+00,1.6290e+01,3.4720e+00,3.6420e+00,
     *4.3870e+00,1.1590e+00,1.0710e+00,1.0290e+00,3.6070e+00,3.5840e+00,
     *3.4950e+00/
      DATA ((SPROF(13,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9100e-01,4.8850e-01,
     *3.4560e-01,1.4890e+00,1.0520e+00,7.4400e-01,3.2010e+00,2.2660e+00,
     *1.6020e+00,6.7830e+00,4.8370e+00,3.4370e+00,1.3730e+01,1.0090e+01,
     *7.2810e+00,1.4450e+01,1.5000e+01,1.2970e+01,3.5260e+00,8.7580e+00,
     *1.3790e+01,1.1520e+00,1.3370e+00,2.7740e+00,1.1110e+00,1.0470e+00,
     *1.0590e+00,3.5900e-01,3.4720e-01,3.3140e-01,1.2090e+00,1.1800e+00,
     *1.0800e+00/
      DATA ((SPROF(14,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9100e-01,4.8850e-01,
     *3.4560e-01,1.4880e+00,1.0520e+00,7.4390e-01,3.1940e+00,2.2630e+00,
     *1.6010e+00,6.6330e+00,4.7830e+00,3.4180e+00,1.2400e+01,9.5860e+00,
     *7.0950e+00,4.7100e+00,8.3610e+00,9.6280e+00,4.1580e-01,1.0260e+00,
     *3.8170e+00,3.3730e-01,3.3350e-01,3.6490e-01,3.4510e-01,3.3080e-01,
     *3.1770e-01,1.1570e-01,1.1290e-01,1.0830e-01,3.8490e-01,3.6260e-01,
     *3.5040e-01/
      DATA ((SPROF(15,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9090e-01,4.8840e-01,
     *3.4560e-01,1.4870e+00,1.0520e+00,7.4370e-01,3.1790e+00,2.2580e+00,
     *1.5990e+00,6.2700e+00,4.6500e+00,3.3700e+00,9.5850e+00,8.4190e+00,
     *6.6470e+00,3.8340e-01,1.9710e+00,4.5660e+00,1.0790e-01,1.1540e-01,
     *2.5440e-01,1.0750e-01,1.0360e-01,1.0110e-01,1.1210e-01,1.0770e-01,
     *1.0240e-01,3.6040e-02,3.6390e-02,3.5250e-02,1.1540e-01,1.1640e-01,
     *1.1360e-01/
      DATA ((SPROF(16,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2080e-01,2.2680e-01,1.6030e-01,6.9080e-01,4.8840e-01,
     *3.4550e-01,1.4830e+00,1.0500e+00,7.4320e-01,3.1410e+00,2.2440e+00,
     *1.5950e+00,5.4450e+00,4.3320e+00,3.2530e+00,5.0320e+00,6.0790e+00,
     *5.6430e+00,3.6990e-02,8.7810e-02,7.2170e-01,3.3920e-02,3.2950e-02,
     *3.3610e-02,3.4970e-02,3.3540e-02,3.1890e-02,3.6240e-02,3.5080e-02,
     *3.3540e-02,2*1.1380e-02,1.1440e-02,3.6040e-02,3.6080e-02,
     *3.6100e-02/
      DATA ((SPROF(17,I,J,2),I=1,NTEMP),J=1,NNE)/1.4880e-01,1.0520e-01,
     *7.4320e-02,3.2070e-01,2.2680e-01,1.6020e-01,6.9040e-01,4.8830e-01,
     *3.4550e-01,1.4730e+00,1.0470e+00,7.4200e-01,3.0460e+00,2.2100e+00,
     *1.5820e+00,3.8190e+00,3.6260e+00,2.9750e+00,1.0150e+00,2.6870e+00,
     *3.7400e+00,1.0760e-02,1.0870e-02,1.8340e-02,1.0930e-02,1.0440e-02,
     *1.0040e-02,1.1370e-02,1.0920e-02,1.0360e-02,1.1380e-02,1.1400e-02,
     *1.0960e-02,0.0000e+00,3.5980e-03,3.5990e-03,1.1390e-02,
     *2*1.1400e-02/
      DATA ((SPROF(18,I,J,2),I=1,NTEMP),J=1,NNE)/1.4870e-01,1.0520e-01,
     *7.4320e-02,3.2060e-01,2.2670e-01,1.6020e-01,6.8940e-01,4.8790e-01,
     *3.4540e-01,1.4490e+00,1.0380e+00,7.3890e-01,2.8210e+00,2.1270e+00,
     *1.5520e+00,1.5700e+00,2.3200e+00,2.3790e+00,3.0170e-02,3.5310e-01,
     *1.3340e+00,3.4110e-03,3.2850e-03,3.2480e-03,3.5400e-03,3.3840e-03,
     *3.2100e-03,3.5960e-03,3.5460e-03,3.3850e-03,3.5940e-03,3.5960e-03,
     *3.5570e-03,2*0.0000e+00,1.1380e-03,2*0.0000e+00,3.6060e-03/
      DATA ((SPROF(19,I,J,2),I=1,NTEMP),J=1,NNE)/1.4870e-01,1.0520e-01,
     *7.4320e-02,3.2040e-01,2.2660e-01,1.6020e-01,6.8700e-01,4.8700e-01,
     *3.4510e-01,1.3900e+00,1.0170e+00,7.3130e-01,2.3260e+00,1.9310e+00,
     *1.4790e+00,1.7080e-01,7.5690e-01,1.3570e+00,3.5330e-03,5.8770e-03,
     *1.0300e-01,1.1010e-03,1.0510e-03,1.0030e-03,1.1410e-03,1.1010e-03,
     *1.0460e-03,0.0000e+00,1.1370e-03,1.1020e-03,2*0.0000e+00,
     *1.1360e-03,6*0.0000e+00/
      DATA ((SPROF(20,I,J,2),I=1,NTEMP),J=1,NNE)/1.4870e-01,1.0520e-01,
     *7.4310e-02,3.1980e-01,2.2640e-01,1.6010e-01,6.8080e-01,4.8490e-01,
     *3.4430e-01,1.2520e+00,9.6510e-01,7.1240e-01,1.4340e+00,1.5160e+00,
     *1.3100e+00,1.8630e-03,4.6390e-02,3.3130e-01,1.0730e-03,1.0580e-03,
     *1.2730e-03,3.5590e-04,3.4120e-04,3.2360e-04,3.5920e-04,3.5650e-04,
     *3.4040e-04,0.0000e+00,3.5930e-04,3.5730e-04,2*0.0000e+00,
     *3.5960e-04,6*0.0000e+00/
      DATA ((SPROF(21,I,J,2),I=1,NTEMP),J=1,NNE)/1.4850e-01,1.0520e-01,
     *7.4290e-02,3.1820e-01,2.2590e-01,1.5990e-01,6.6570e-01,4.7940e-01,
     *3.4240e-01,9.6290e-01,8.4630e-01,6.6710e-01,4.2520e-01,8.2460e-01,
     *9.6630e-01,3.4480e-04,4.0050e-04,9.9510e-03,3.4280e-04,3.2840e-04,
     *3.1920e-04,1.1370e-04,1.1090e-04,1.0560e-04,2*1.1360e-04,
     *1.1100e-04,2*0.0000e+00,1.1360e-04,9*0.0000e+00/
      DATA ((SPROF(22,I,J,2),I=1,NTEMP),J=1,NNE)/1.4810e-01,1.0500e-01,
     *7.4240e-02,3.1440e-01,2.2450e-01,1.5940e-01,6.2910e-01,4.6610e-01,
     *3.3750e-01,4.9810e-01,6.0860e-01,5.6570e-01,2.0430e-02,1.7890e-01,
     *4.4970e-01,1.0730e-04,1.0430e-04,1.0740e-04,1.1080e-04,1.0580e-04,
     *1.0060e-04,3.5910e-05,3.5850e-05,3.4490e-05,0.0000e+00,3.5920e-05,
     *3.5910e-05,12*0.0000e+00/
      DATA ((SPROF(23,I,J,2),I=1,NTEMP),J=1,NNE)/1.4710e-01,1.0470e-01,
     *7.4120e-02,3.0490e-01,2.2110e-01,1.5820e-01,5.4570e-01,4.3410e-01,
     *3.2580e-01,9.5170e-02,2.6590e-01,3.7390e-01,1.2610e-04,3.9700e-03,
     *6.5920e-02,3.4490e-05,3.2950e-05,3.1680e-05,3.5780e-05,3.4410e-05,
     *3.2660e-05,0.0000e+00,1.1360e-05,1.1190e-05,2*0.0000e+00,
     *1.1360e-05,12*0.0000e+00/
      DATA ((SPROF(24,I,J,2),I=1,NTEMP),J=1,NNE)/1.4470e-01,1.0380e-01,
     *7.3820e-02,2.8230e-01,2.1270e-01,1.5520e-01,3.8190e-01,3.6310e-01,
     *2.9790e-01,1.5280e-03,3.3240e-02,1.3210e-01,3.4040e-05,3.4630e-05,
     *5.6630e-04,1.1160e-05,1.0670e-05,1.0130e-05,1.1360e-05,1.1170e-05,
     *1.0670e-05,2*0.0000e+00,3.5920e-06,15*0.0000e+00/
      DATA ((SPROF(25,I,J,2),I=1,NTEMP),J=1,NNE)/1.3880e-01,1.0170e-01,
     *7.3050e-02,2.3270e-01,1.9320e-01,1.4790e-01,1.5580e-01,2.3190e-01,
     *2.3810e-01,1.1200e-05,1.9100e-04,9.6950e-03,1.0760e-05,1.0370e-05,
     *1.0250e-05,3.5950e-06,3.4710e-06,3.3000e-06,0.0000e+00,3.5920e-06,
     *3.4810e-06,2*0.0000e+00,1.1370e-06,15*0.0000e+00/
      DATA ((SPROF(26,I,J,2),I=1,NTEMP),J=1,NNE)/1.2500e-01,9.6490e-02,
     *7.1170e-02,1.4330e-01,1.5150e-01,1.3100e-01,1.6400e-02,7.5220e-02,
     *1.3560e-01,3.3870e-06,3.3410e-06,1.7210e-05,3.4730e-06,3.3160e-06,
     *3.1660e-06,1.1360e-06,1.1250e-06,1.0750e-06,0.0000e+00,1.1360e-06,
     *1.1280e-06,18*0.0000e+00/
      DATA ((SPROF(27,I,J,2),I=1,NTEMP),J=1,NNE)/9.6160e-02,8.4620e-02,
     *6.6640e-02,4.2340e-02,8.2380e-02,9.6590e-02,6.1230e-05,4.4480e-03,
     *3.2960e-02,1.0820e-06,1.0370e-06,1.0080e-06,1.1230e-06,1.0770e-06,
     *1.0210e-06,0.0000e+00,3.5920e-07,3.5070e-07,2*0.0000e+00,
     *3.5930e-07,18*0.0000e+00/
      DATA ((SPROF(28,I,J,2),I=1,NTEMP),J=1,NNE)/4.9720e-02,6.0840e-02,
     *5.6510e-02,1.9830e-03,1.7820e-02,4.4920e-02,1.0890e-06,4.7930e-06,
     *9.4520e-04,3.4990e-07,3.3430e-07,3.1780e-07,3.5940e-07,3.5010e-07,
     *3.3360e-07,0.0000e+00,2*1.1360e-07,2*0.0000e+00,1.1370e-07,
     *18*0.0000e+00/
      DATA ((SPROF(29,I,J,2),I=1,NTEMP),J=1,NNE)/9.4820e-03,2.6570e-02,
     *3.7340e-02,1.2750e-06,3.8140e-04,6.5680e-03,3.3890e-07,3.2930e-07,
     *4.6060e-07,1.1300e-07,1.0870e-07,1.0320e-07,1.1360e-07,1.1330e-07,
     *1.0870e-07,2*0.0000e+00,3.5940e-08,21*0.0000e+00/
      DATA ((SPROF(30,I,J,2),I=1,NTEMP),J=1,NNE)/1.4780e-04,3.3160e-03,
     *1.3190e-02,1.0750e-07,1.3290e-07,5.2570e-05,1.0900e-07,1.0410e-07,
     *1.0010e-07,3.5920e-08,3.5300e-08,3.3730e-08,0.0000e+00,3.5920e-08,
     *3.5380e-08,24*0.0000e+00/
      DATA ((SPROF(31,I,J,2),I=1,NTEMP),J=1,NNE)/3.9510e-08,1.7830e-05,
     *9.6640e-04,3.4020e-08,3.2760e-08,3.2670e-08,3.5250e-08,3.3730e-08,
     *3.2000e-08,0.0000e+00,1.1360e-08,1.1000e-08,2*0.0000e+00,
     *1.1360e-08,24*0.0000e+00/
      DATA ((SPROF(32,I,J,2),I=1,NTEMP),J=1,NNE)/1.0710e-08,1.0590e-08,
     *1.3720e-06,1.0980e-08,1.0480e-08,1.0010e-08,1.1370e-08,1.0970e-08,
     *1.0430e-08,0.0000e+00,3.5900e-09,3.5660e-09,2*0.0000e+00,
     *3.5950e-09,24*0.0000e+00/
      DATA ((SPROF(33,I,J,2),I=1,NTEMP),J=1,NNE)/3.4210e-09,3.2780e-09,
     *3.1850e-09,3.5510e-09,3.4050e-09,3.2290e-09,3.5920e-09,3.5570e-09,
     *3.4100e-09,2*0.0000e+00,1.1360e-09,27*0.0000e+00/
      DATA ((SPROF(34,I,J,2),I=1,NTEMP),J=1,NNE)/1.1060e-09,1.0570e-09,
     *1.0050e-09,1.1360e-09,1.1070e-09,1.0550e-09,0.0000e+00,1.1360e-09,
     *1.1100e-09,2*0.0000e+00,3.5970e-10,27*0.0000e+00/
      DATA ((SPROF(35,I,J,2),I=1,NTEMP),J=1,NNE)/3.5740e-10,3.4380e-10,
     *3.2630e-10,3.5910e-10,3.5820e-10,3.4450e-10,0.0000e+00,
     *2*3.5910e-10,30*0.0000e+00/
      DATA ((SPROF(36,I,J,2),I=1,NTEMP),J=1,NNE)/1.1360e-10,1.1160e-10,
     *1.0670e-10,0.0000e+00,1.1360e-10,1.1200e-10,2*0.0000e+00,
     *1.1360e-10,30*0.0000e+00/
      DATA ((SPROF(37,I,J,2),I=1,NTEMP),J=1,NNE)/0.0000e+00,3.5920e-11,
     *3.4800e-11,2*0.0000e+00,3.5920e-11,33*0.0000e+00/
      DATA ((SPROF(38,I,J,2),I=1,NTEMP),J=1,NNE)/0.0000e+00,1.1350e-11,
     *1.1280e-11,2*0.0000e+00,1.1370e-11,33*0.0000e+00/
      DATA ((SPROF(39,I,J,2),I=1,NTEMP),J=1,NNE)/2*0.0000e+00,
     *3.5930e-12,36*0.0000e+00/
      DATA ((SPROF(40,I,J,2),I=1,NTEMP),J=1,NNE)/2*0.0000e+00,
     *1.1370e-12,36*0.0000e+00/
      DATA ((SPROF(41,I,J,2),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(42,I,J,2),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(43,I,J,2),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(44,I,J,2),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(45,I,J,2),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(46,I,J,2),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(1,I,J,3),I=1,NTEMP),J=1,NNE)/1.5700e-01,1.1100e-01,
     *7.8500e-02,3.3820e-01,2.3910e-01,1.6910e-01,7.2850e-01,5.1510e-01,
     *3.6430e-01,1.5680e+00,1.1090e+00,7.8430e-01,3.3710e+00,2.3850e+00,
     *1.6880e+00,7.2070e+00,5.1170e+00,3.6260e+00,1.5160e+01,1.0870e+01,
     *7.7470e+00,3.0550e+01,2.2450e+01,1.6270e+01,5.5610e+01,4.3110e+01,
     *3.2550e+01,8.4450e+01,7.1060e+01,5.7960e+01,1.0600e+02,9.4990e+01,
     *8.4270e+01,1.1910e+02,1.0940e+02,1.0110e+02,1.2600e+02,1.1760e+02,
     *1.1100e+02/
      DATA ((SPROF(2,I,J,3),I=1,NTEMP),J=1,NNE)/1.5700e-01,1.1100e-01,
     *7.8500e-02,3.3820e-01,2.3910e-01,1.6910e-01,7.2850e-01,5.1510e-01,
     *3.6430e-01,1.5680e+00,1.1090e+00,7.8430e-01,3.3710e+00,2.3850e+00,
     *1.6880e+00,7.2060e+00,5.1170e+00,3.6260e+00,1.5160e+01,1.0870e+01,
     *7.7470e+00,3.0540e+01,2.2450e+01,1.6270e+01,5.5590e+01,4.3100e+01,
     *3.2550e+01,8.4380e+01,7.1020e+01,5.7940e+01,1.0580e+02,9.4870e+01,
     *8.4200e+01,1.1850e+02,1.0910e+02,1.0090e+02,1.2500e+02,1.1680e+02,
     *1.1040e+02/
      DATA ((SPROF(3,I,J,3),I=1,NTEMP),J=1,NNE)/1.5700e-01,1.1100e-01,
     *7.8500e-02,3.3820e-01,2.3910e-01,1.6910e-01,7.2850e-01,5.1510e-01,
     *3.6430e-01,1.5680e+00,1.1090e+00,7.8430e-01,3.3710e+00,2.3850e+00,
     *1.6880e+00,7.2060e+00,5.1170e+00,3.6260e+00,1.5160e+01,1.0870e+01,
     *7.7470e+00,3.0530e+01,2.2440e+01,1.6270e+01,5.5570e+01,4.3090e+01,
     *3.2540e+01,8.4280e+01,7.0970e+01,5.7910e+01,1.0550e+02,9.4690e+01,
     *8.4100e+01,1.1770e+02,1.0860e+02,1.0060e+02,1.2360e+02,1.1570e+02,
     *1.0960e+02/
      DATA ((SPROF(4,I,J,3),I=1,NTEMP),J=1,NNE)/1.5700e-01,1.1100e-01,
     *7.8500e-02,3.3820e-01,2.3910e-01,1.6910e-01,7.2850e-01,5.1510e-01,
     *3.6430e-01,1.5680e+00,1.1090e+00,7.8430e-01,3.3710e+00,2.3850e+00,
     *1.6880e+00,7.2060e+00,5.1160e+00,3.6260e+00,1.5160e+01,1.0870e+01,
     *7.7470e+00,3.0510e+01,2.2430e+01,1.6260e+01,5.5510e+01,4.3060e+01,
     *3.2530e+01,8.4020e+01,7.0830e+01,5.7840e+01,1.0470e+02,9.4250e+01,
     *8.3840e+01,1.1570e+02,1.0730e+02,9.9850e+01,1.2020e+02,1.1290e+02,
     *1.0760e+02/
      DATA ((SPROF(5,I,J,3),I=1,NTEMP),J=1,NNE)/1.5700e-01,1.1100e-01,
     *7.8500e-02,3.3820e-01,2.3910e-01,1.6910e-01,7.2850e-01,5.1510e-01,
     *3.6430e-01,1.5680e+00,1.1090e+00,7.8430e-01,3.3710e+00,2.3850e+00,
     *1.6880e+00,7.2060e+00,5.1160e+00,3.6260e+00,1.5160e+01,1.0870e+01,
     *7.7470e+00,3.0450e+01,2.2410e+01,1.6250e+01,5.5350e+01,4.2990e+01,
     *3.2500e+01,8.3360e+01,7.0480e+01,5.7670e+01,1.0280e+02,9.3150e+01,
     *8.3180e+01,1.1130e+02,1.0440e+02,9.8090e+01,1.1340e+02,1.0720e+02,
     *1.0310e+02/
      DATA ((SPROF(6,I,J,3),I=1,NTEMP),J=1,NNE)/1.5700e-01,1.1100e-01,
     *7.8500e-02,3.3820e-01,2.3910e-01,1.6910e-01,7.2850e-01,5.1510e-01,
     *3.6430e-01,1.5680e+00,1.1090e+00,7.8430e-01,3.3710e+00,2.3850e+00,
     *1.6880e+00,7.2050e+00,5.1160e+00,3.6260e+00,1.5150e+01,1.0860e+01,
     *7.7460e+00,3.0310e+01,2.2360e+01,1.6230e+01,5.4970e+01,4.2830e+01,
     *3.2430e+01,8.1750e+01,6.9620e+01,5.7230e+01,9.8440e+01,9.0470e+01,
     *8.1560e+01,1.0240e+02,9.8130e+01,9.3990e+01,1.0200e+02,9.7180e+01,
     *9.4710e+01/
      DATA ((SPROF(7,I,J,3),I=1,NTEMP),J=1,NNE)/1.5700e-01,1.1100e-01,
     *7.8500e-02,3.3820e-01,2.3910e-01,1.6910e-01,7.2850e-01,5.1510e-01,
     *3.6430e-01,1.5680e+00,1.1090e+00,7.8430e-01,3.3710e+00,2.3850e+00,
     *1.6880e+00,7.2040e+00,5.1150e+00,3.6260e+00,1.5140e+01,1.0860e+01,
     *7.7440e+00,2.9960e+01,2.2220e+01,1.6180e+01,5.4030e+01,4.2410e+01,
     *3.2260e+01,7.7860e+01,6.7490e+01,5.6160e+01,8.8930e+01,8.4260e+01,
     *7.7670e+01,8.8360e+01,8.6630e+01,8.5390e+01,8.7590e+01,8.4270e+01,
     *8.2820e+01/
      DATA ((SPROF(8,I,J,3),I=1,NTEMP),J=1,NNE)/1.5700e-01,1.1100e-01,
     *7.8500e-02,3.3820e-01,2.3910e-01,1.6910e-01,7.2850e-01,5.1510e-01,
     *3.6430e-01,1.5680e+00,1.1090e+00,7.8430e-01,3.3700e+00,2.3850e+00,
     *1.6880e+00,7.1990e+00,5.1140e+00,3.6250e+00,1.5090e+01,1.0840e+01,
     *7.7380e+00,2.9080e+01,2.1870e+01,1.6050e+01,5.1740e+01,4.1380e+01,
     *3.1830e+01,6.9090e+01,6.2480e+01,5.3560e+01,7.1520e+01,7.1360e+01,
     *6.8930e+01,6.9480e+01,6.9600e+01,7.0260e+01,6.9280e+01,6.8810e+01,
     *6.8540e+01/
      DATA ((SPROF(9,I,J,3),I=1,NTEMP),J=1,NNE)/1.5700e-01,1.1100e-01,
     *7.8500e-02,3.3820e-01,2.3910e-01,1.6910e-01,7.2850e-01,5.1510e-01,
     *3.6430e-01,1.5680e+00,1.1090e+00,7.8430e-01,3.3700e+00,2.3850e+00,
     *1.6880e+00,7.1880e+00,5.1100e+00,3.6240e+00,1.4990e+01,1.0800e+01,
     *7.7240e+00,2.7010e+01,2.1030e+01,1.5730e+01,4.6450e+01,3.8910e+01,
     *3.0770e+01,5.2060e+01,5.1710e+01,4.7580e+01,4.6930e+01,4.9760e+01,
     *5.2020e+01,4.4800e+01,4.6680e+01,4.8390e+01,4.4120e+01,4.6010e+01,
     *4.7360e+01/
      DATA ((SPROF(10,I,J,3),I=1,NTEMP),J=1,NNE)/1.5700e-01,1.1100e-01,
     *7.8500e-02,3.3820e-01,2.3910e-01,1.6910e-01,7.2850e-01,5.1500e-01,
     *3.6430e-01,1.5680e+00,1.1090e+00,7.8430e-01,3.3690e+00,2.3850e+00,
     *1.6880e+00,7.1600e+00,5.1000e+00,3.6200e+00,1.4730e+01,1.0710e+01,
     *7.6890e+00,2.2440e+01,1.9070e+01,1.4950e+01,3.5610e+01,3.3370e+01,
     *2.8270e+01,2.8220e+01,3.3060e+01,3.5590e+01,2.2520e+01,2.4910e+01,
     *2.8350e+01,2.1450e+01,2.2720e+01,2.3970e+01,2.1200e+01,2.2510e+01,
     *2.3400e+01/
      DATA ((SPROF(11,I,J,3),I=1,NTEMP),J=1,NNE)/1.5700e-01,1.1100e-01,
     *7.8500e-02,3.3820e-01,2.3910e-01,1.6910e-01,7.2840e-01,5.1500e-01,
     *3.6420e-01,1.5670e+00,1.1090e+00,7.8430e-01,3.3660e+00,2.3840e+00,
     *1.6870e+00,7.0900e+00,5.0750e+00,3.6110e+00,1.4090e+01,1.0470e+01,
     *7.6020e+00,1.4170e+01,1.4910e+01,1.3160e+01,1.8950e+01,2.2850e+01,
     *2.2880e+01,9.7820e+00,1.2920e+01,1.7990e+01,8.1830e+00,8.7210e+00,
     *9.8080e+00,8.1060e+00,8.3860e+00,8.5590e+00,8.1110e+00,8.5040e+00,
     *8.6710e+00/
      DATA ((SPROF(12,I,J,3),I=1,NTEMP),J=1,NNE)/1.5700e-01,1.1100e-01,
     *7.8500e-02,3.3820e-01,2.3910e-01,1.6910e-01,7.2820e-01,5.1500e-01,
     *3.6420e-01,1.5670e+00,1.1090e+00,7.8420e-01,3.3590e+00,2.3810e+00,
     *1.6870e+00,6.9170e+00,5.0120e+00,3.5890e+00,1.2620e+01,9.8950e+00,
     *7.3880e+00,4.7020e+00,8.1010e+00,9.5550e+00,5.1540e+00,9.3090e+00,
     *1.3570e+01,2.8220e+00,3.2030e+00,4.6040e+00,2.7090e+00,2.7220e+00,
     *2.7690e+00,2.7590e+00,2.7690e+00,2.7280e+00,2.7830e+00,2.8480e+00,
     *2.8360e+00/
      DATA ((SPROF(13,I,J,3),I=1,NTEMP),J=1,NNE)/1.5700e-01,1.1100e-01,
     *7.8500e-02,3.3820e-01,2.3910e-01,1.6910e-01,7.2770e-01,5.1480e-01,
     *3.6420e-01,1.5650e+00,1.1080e+00,7.8400e-01,3.3410e+00,2.3750e+00,
     *1.6840e+00,6.5020e+00,4.8580e+00,3.5330e+00,9.5650e+00,8.5890e+00,
     *6.8770e+00,5.3620e-01,1.8680e+00,4.3110e+00,9.9930e-01,1.6010e+00,
     *3.9270e+00,8.6210e-01,8.6380e-01,9.0130e-01,8.7710e-01,8.5640e-01,
     *7.9550e-01,8.9840e-01,8.8730e-01,8.1440e-01,9.0920e-01,9.1190e-01,
     *8.4130e-01/
      DATA ((SPROF(14,I,J,3),I=1,NTEMP),J=1,NNE)/1.5700e-01,1.1100e-01,
     *7.8500e-02,3.3820e-01,2.3910e-01,1.6910e-01,7.2660e-01,5.1440e-01,
     *3.6400e-01,1.5600e+00,1.1060e+00,7.8340e-01,3.2960e+00,2.3590e+00,
     *1.6790e+00,5.5660e+00,4.4930e+00,3.3970e+00,4.7930e+00,6.0210e+00,
     *5.7430e+00,8.7780e-02,1.3200e-01,6.3730e-01,2.6450e-01,2.8080e-01,
     *4.1450e-01,2.5860e-01,2.5310e-01,2.4710e-01,2.7480e-01,2.6860e-01,
     *2.4920e-01,2.7850e-01,2.7620e-01,2.5850e-01,2.6120e-01,2.8000e-01,
     *2.6490e-01/
      DATA ((SPROF(15,I,J,3),I=1,NTEMP),J=1,NNE)/1.5700e-01,1.1100e-01,
     *7.8500e-02,3.3810e-01,2.3900e-01,1.6910e-01,7.2370e-01,5.1340e-01,
     *3.6370e-01,1.5490e+00,1.1020e+00,7.8200e-01,3.1860e+00,2.3190e+00,
     *1.6640e+00,3.7690e+00,3.6910e+00,3.0780e+00,8.9240e-01,2.4810e+00,
     *3.6560e+00,2.5780e-02,2.6160e-02,3.2350e-02,8.0840e-02,7.9550e-02,
     *8.1100e-02,8.2000e-02,8.0020e-02,7.7410e-02,8.1510e-02,8.4500e-02,
     *8.0000e-02,8.1320e-02,8.1530e-02,8.2340e-02,8.1380e-02,8.1480e-02,
     *8.0790e-02/
      DATA ((SPROF(16,I,J,3),I=1,NTEMP),J=1,NNE)/1.5690e-01,1.1100e-01,
     *7.8490e-02,3.3800e-01,2.3900e-01,1.6910e-01,7.1650e-01,5.1080e-01,
     *3.6280e-01,1.5210e+00,1.0920e+00,7.7840e-01,2.9260e+00,2.2220e+00,
     *1.6290e+00,1.4210e+00,2.2540e+00,2.4020e+00,4.1970e-02,2.8740e-01,
     *1.1820e+00,8.0960e-03,7.9080e-03,7.8300e-03,2.5700e-02,2.5040e-02,
     *2.4340e-02,2.5420e-02,0.0000e+00,2.4820e-02,2.5350e-02,2.5390e-02,
     *2.5650e-02,2.5350e-02,2.5370e-02,2.5390e-02,2.5370e-02,2.5400e-02,
     *2.5410e-02/
      DATA ((SPROF(17,I,J,3),I=1,NTEMP),J=1,NNE)/1.5690e-01,1.1090e-01,
     *7.8480e-02,3.3770e-01,2.3890e-01,1.6900e-01,6.9880e-01,5.0450e-01,
     *3.6050e-01,1.4520e+00,1.0670e+00,7.6940e-01,2.3630e+00,1.9970e+00,
     *1.5440e+00,1.2970e-01,6.5550e-01,1.2890e+00,8.4380e-03,1.0170e-02,
     *7.6850e-02,2.5470e-03,2.5090e-03,2.4250e-03,8.0310e-03,8.0260e-03,
     *7.7450e-03,7.9990e-03,0.0000e+00,8.0160e-03,7.9950e-03,7.9990e-03,
     *8.0050e-03,8.0000e-03,8.0050e-03,8.0080e-03,8.0170e-03,8.0230e-03,
     *8.0260e-03/
      DATA ((SPROF(18,I,J,3),I=1,NTEMP),J=1,NNE)/1.5670e-01,1.1090e-01,
     *7.8460e-02,3.3700e-01,2.3860e-01,1.6890e-01,6.5620e-01,4.8880e-01,
     *3.5490e-01,1.2930e+00,1.0070e+00,7.4740e-01,1.3820e+00,1.5260e+00,
     *1.3500e+00,3.2080e-03,3.2070e-02,2.7130e-01,2.5610e-03,2.5440e-03,
     *2.7240e-03,8.0020e-04,8.0320e-04,7.7650e-04,2.5280e-03,2.5340e-03,
     *2.4940e-03,2.5250e-03,0.0000e+00,2.5290e-03,2.5260e-03,2.5270e-03,
     *2.5260e-03,2.5300e-03,2*2.5310e-03,0.0000e+00,2.5400e-03,
     *2.5410e-03/
      DATA ((SPROF(19,I,J,3),I=1,NTEMP),J=1,NNE)/1.5620e-01,1.1070e-01,
     *7.8400e-02,3.3520e-01,2.3800e-01,1.6870e-01,5.6030e-01,4.5170e-01,
     *3.4110e-01,9.6580e-01,8.7040e-01,6.9480e-01,3.6020e-01,7.7650e-01,
     *9.6220e-01,8.2360e-04,8.7140e-04,6.2760e-03,8.0840e-04,7.8780e-04,
     *7.7030e-04,2.5240e-04,2.5280e-04,2.5040e-04,7.9810e-04,7.9880e-04,
     *8.0000e-04,2*0.0000e+00,7.9850e-04,9*0.0000e+00/
      DATA ((SPROF(20,I,J,3),I=1,NTEMP),J=1,NNE)/1.5510e-01,1.1030e-01,
     *7.8260e-02,3.3070e-01,2.3640e-01,1.6810e-01,3.7680e-01,3.7040e-01,
     *3.0890e-01,4.6440e-01,6.0330e-01,5.7840e-01,1.3180e-02,1.4280e-01,
     *4.1140e-01,2.5490e-04,2.5060e-04,2.5400e-04,2.5380e-04,2.5130e-04,
     *2.4260e-04,0.0000e+00,7.9800e-05,7.9900e-05,0.0000e+00,2.5230e-04,
     *2.5250e-04,12*0.0000e+00/
      DATA ((SPROF(21,I,J,3),I=1,NTEMP),J=1,NNE)/1.5220e-01,1.0930e-01,
     *7.7900e-02,3.1960e-01,2.3240e-01,1.6670e-01,1.3900e-01,2.2500e-01,
     *2.4070e-01,7.3970e-02,2.4030e-01,3.6490e-01,2.7930e-04,2.3160e-03,
     *4.8880e-02,8.0680e-05,7.8790e-05,7.6480e-05,7.9900e-05,8.0120e-05,
     *7.8040e-05,2*0.0000e+00,2.5210e-05,15*0.0000e+00/
      DATA ((SPROF(22,I,J,3),I=1,NTEMP),J=1,NNE)/1.4540e-01,1.0680e-01,
     *7.7000e-02,2.9340e-01,2.2270e-01,1.6320e-01,1.1390e-02,6.4310e-02,
     *1.2870e-01,8.2690e-04,2.3870e-02,1.1480e-01,8.1220e-05,8.2230e-05,
     *3.1920e-04,2.5320e-05,2.5210e-05,2.4350e-05,2.5230e-05,2.5260e-05,
     *2.5170e-05,18*0.0000e+00/
      DATA ((SPROF(23,I,J,3),I=1,NTEMP),J=1,NNE)/1.2940e-01,1.0080e-01,
     *7.4800e-02,2.3670e-01,2.0000e-01,1.5460e-01,3.0410e-05,2.7770e-03,
     *2.6690e-02,2.6590e-05,1.0060e-04,6.3080e-03,2.5460e-05,2.4890e-05,
     *2.4660e-05,7.9830e-06,8.0000e-06,7.8520e-06,0.0000e+00,7.9780e-06,
     *7.9850e-06,18*0.0000e+00/
      DATA ((SPROF(24,I,J,3),I=1,NTEMP),J=1,NNE)/9.6620e-02,8.7070e-02,
     *6.9530e-02,1.3800e-01,1.5270e-01,1.3510e-01,2.5970e-06,3.7360e-06,
     *5.1610e-04,8.0650e-06,8.0140e-06,1.2670e-05,8.0420e-06,7.8990e-06,
     *7.6380e-06,2.5220e-06,0.0000e+00,2.5280e-06,21*0.0000e+00/
      DATA ((SPROF(25,I,J,3),I=1,NTEMP),J=1,NNE)/4.6380e-02,6.0330e-02,
     *5.7880e-02,3.5560e-02,7.7510e-02,9.6270e-02,8.0440e-07,7.9090e-07,
     *8.2570e-07,2.5480e-06,2.4840e-06,2.4300e-06,2.5290e-06,2.5320e-06,
     *2.4490e-06,2*0.0000e+00,7.9710e-07,21*0.0000e+00/
      DATA ((SPROF(26,I,J,3),I=1,NTEMP),J=1,NNE)/7.3440e-03,2.4000e-02,
     *3.6510e-02,1.1840e-03,1.4120e-02,4.1090e-02,2.5510e-07,2.4880e-07,
     *2.4150e-07,8.0220e-07,7.9300e-07,7.6560e-07,7.9790e-07,7.9920e-07,
     *7.9020e-07,24*0.0000e+00/
      DATA ((SPROF(27,I,J,3),I=1,NTEMP),J=1,NNE)/7.1970e-05,2.3710e-03,
     *1.1470e-02,1.1010e-06,1.9700e-04,4.8410e-03,8.0080e-08,7.9660e-08,
     *7.6930e-08,2.5260e-07,2.5330e-07,2.4650e-07,2*0.0000e+00,
     *2.5260e-07,24*0.0000e+00/
      DATA ((SPROF(28,I,J,3),I=1,NTEMP),J=1,NNE)/8.4600e-08,7.1640e-06,
     *6.2670e-04,2.5650e-07,2.6370e-07,2.2760e-05,2.5240e-08,2.5300e-08,
     *2.4820e-08,7.9770e-08,7.9860e-08,7.9530e-08,2*0.0000e+00,
     *7.9800e-08,24*0.0000e+00/
      DATA ((SPROF(29,I,J,3),I=1,NTEMP),J=1,NNE)/2.5490e-08,2.5330e-08,
     *4.4850e-07,8.0430e-08,7.8630e-08,7.7920e-08,7.9750e-09,0.0000e+00,
     *7.9940e-09,0.0000e+00,2.5230e-08,2.5200e-08,27*0.0000e+00/
      DATA ((SPROF(30,I,J,3),I=1,NTEMP),J=1,NNE)/8.0540e-09,7.8540e-09,
     *7.6810e-09,2.5430e-08,2.4970e-08,2.4140e-08,2*0.0000e+00,
     *2.5210e-09,30*0.0000e+00/
      DATA ((SPROF(31,I,J,3),I=1,NTEMP),J=1,NNE)/2.5370e-09,2.5070e-09,
     *2.4210e-09,7.9970e-09,8.0040e-09,7.7410e-09,33*0.0000e+00/
      DATA ((SPROF(32,I,J,3),I=1,NTEMP),J=1,NNE)/7.9890e-10,8.0100e-10,
     *7.7940e-10,2.5230e-09,2.5270e-09,2.4990e-09,33*0.0000e+00/
      DATA ((SPROF(33,I,J,3),I=1,NTEMP),J=1,NNE)/2.5220e-10,2.5250e-10,
     *2.5150e-10,2*0.0000e+00,7.9880e-10,33*0.0000e+00/
      DATA ((SPROF(34,I,J,3),I=1,NTEMP),J=1,NNE)/0.0000e+00,7.9770e-11,
     *7.9840e-11,2*0.0000e+00,2.5230e-10,33*0.0000e+00/
      DATA ((SPROF(35,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(36,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(37,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(38,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(39,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(40,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(41,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(42,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(43,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(44,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(45,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(46,I,J,3),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(1,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0120e-02,3.5430e-02,2.5030e-02,1.0800e-01,7.6300e-02,
     *5.3950e-02,2.3240e-01,1.6430e-01,1.1620e-01,4.9910e-01,3.5360e-01,
     *2.5010e-01,1.0650e+00,7.5710e-01,5.3710e-01,2.2300e+00,1.6030e+00,
     *1.1450e+00,4.4540e+00,3.2870e+00,2.3910e+00,8.0830e+00,6.2550e+00,
     *4.7400e+00,1.2640e+01,1.0420e+01,8.4170e+00,1.6750e+01,1.4660e+01,
     *1.2650e+01,1.9350e+01,1.7670e+01,1.6080e+01,2.0600e+01,1.9220e+01,
     *1.8050e+01/
      DATA ((SPROF(2,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0120e-02,3.5430e-02,2.5030e-02,1.0800e-01,7.6300e-02,
     *5.3950e-02,2.3240e-01,1.6430e-01,1.1620e-01,4.9910e-01,3.5360e-01,
     *2.5010e-01,1.0650e+00,7.5710e-01,5.3710e-01,2.2300e+00,1.6030e+00,
     *1.1450e+00,4.4530e+00,3.2870e+00,2.3910e+00,8.0780e+00,6.2530e+00,
     *4.7390e+00,1.2630e+01,1.0420e+01,8.4150e+00,1.6710e+01,1.4640e+01,
     *1.2640e+01,1.9320e+01,1.7650e+01,1.6070e+01,2.0560e+01,1.9190e+01,
     *1.8030e+01/
      DATA ((SPROF(3,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0120e-02,3.5430e-02,2.5030e-02,1.0800e-01,7.6300e-02,
     *5.3950e-02,2.3240e-01,1.6430e-01,1.1620e-01,4.9910e-01,3.5360e-01,
     *2.5010e-01,1.0650e+00,7.5710e-01,5.3710e-01,2.2300e+00,1.6030e+00,
     *1.1450e+00,4.4520e+00,3.2860e+00,2.3910e+00,8.0700e+00,6.2490e+00,
     *4.7380e+00,1.2620e+01,1.0410e+01,8.4110e+00,1.6660e+01,1.4610e+01,
     *1.2620e+01,1.9280e+01,1.7620e+01,1.6050e+01,2.0500e+01,1.9140e+01,
     *1.7990e+01/
      DATA ((SPROF(4,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0120e-02,3.5430e-02,2.5030e-02,1.0800e-01,7.6300e-02,
     *5.3950e-02,2.3240e-01,1.6430e-01,1.1620e-01,4.9910e-01,3.5360e-01,
     *2.5010e-01,1.0650e+00,7.5710e-01,5.3710e-01,2.2300e+00,1.6030e+00,
     *1.1450e+00,4.4490e+00,3.2850e+00,2.3900e+00,8.0500e+00,6.2410e+00,
     *4.7340e+00,1.2580e+01,1.0390e+01,8.4020e+00,1.6540e+01,1.4530e+01,
     *1.2580e+01,1.9180e+01,1.7550e+01,1.6000e+01,2.0360e+01,1.9030e+01,
     *1.7900e+01/
      DATA ((SPROF(5,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0120e-02,3.5430e-02,2.5030e-02,1.0800e-01,7.6300e-02,
     *5.3950e-02,2.3240e-01,1.6430e-01,1.1620e-01,4.9910e-01,3.5360e-01,
     *2.5010e-01,1.0650e+00,7.5710e-01,5.3710e-01,2.2290e+00,1.6030e+00,
     *1.1450e+00,4.4420e+00,3.2820e+00,2.3890e+00,8.0010e+00,6.2190e+00,
     *4.7260e+00,1.2480e+01,1.0340e+01,8.3800e+00,1.6230e+01,1.4350e+01,
     *1.2480e+01,1.8920e+01,1.7370e+01,1.5880e+01,1.9990e+01,1.8740e+01,
     *1.7670e+01/
      DATA ((SPROF(6,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0120e-02,3.5430e-02,2.5030e-02,1.0800e-01,7.6300e-02,
     *5.3950e-02,2.3240e-01,1.6430e-01,1.1620e-01,4.9910e-01,3.5360e-01,
     *2.5010e-01,1.0650e+00,7.5710e-01,5.3710e-01,2.2290e+00,1.6030e+00,
     *1.1450e+00,4.4240e+00,3.2750e+00,2.3870e+00,7.8790e+00,6.1670e+00,
     *4.7040e+00,1.2240e+01,1.0220e+01,8.3230e+00,1.5490e+01,1.3910e+01,
     *1.2230e+01,1.8300e+01,1.6940e+01,1.5590e+01,1.9140e+01,1.8050e+01,
     *1.7130e+01/
      DATA ((SPROF(7,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0120e-02,3.5430e-02,2.5030e-02,1.0800e-01,7.6300e-02,
     *5.3950e-02,2.3240e-01,1.6430e-01,1.1620e-01,4.9910e-01,3.5360e-01,
     *2.5010e-01,1.0650e+00,7.5710e-01,5.3710e-01,2.2280e+00,1.6020e+00,
     *1.1450e+00,4.3790e+00,3.2570e+00,2.3800e+00,7.5830e+00,6.0360e+00,
     *4.6490e+00,1.1670e+01,9.9330e+00,8.1830e+00,1.3860e+01,1.2890e+01,
     *1.1630e+01,1.6900e+01,1.5930e+01,1.4890e+01,1.7310e+01,1.6550e+01,
     *1.5910e+01/
      DATA ((SPROF(8,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0120e-02,3.5430e-02,2.5030e-02,1.0800e-01,7.6300e-02,
     *5.3950e-02,2.3240e-01,1.6430e-01,1.1620e-01,4.9910e-01,3.5360e-01,
     *2.5010e-01,1.0640e+00,7.5700e-01,5.3700e-01,2.2260e+00,1.6010e+00,
     *1.1440e+00,4.2680e+00,3.2140e+00,2.3630e+00,6.8920e+00,5.7210e+00,
     *4.5150e+00,1.0380e+01,9.2440e+00,7.8430e+00,1.0790e+01,1.0750e+01,
     *1.0300e+01,1.4150e+01,1.3820e+01,1.3360e+01,1.4070e+01,1.3740e+01,
     *1.3530e+01/
      DATA ((SPROF(9,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0120e-02,3.5430e-02,2.5030e-02,1.0800e-01,7.6300e-02,
     *5.3950e-02,2.3240e-01,1.6430e-01,1.1620e-01,4.9910e-01,3.5350e-01,
     *2.5010e-01,1.0640e+00,7.5680e-01,5.3700e-01,2.2210e+00,1.5990e+00,
     *1.1440e+00,4.0030e+00,3.1070e+00,2.3220e+00,5.4480e+00,5.0060e+00,
     *4.1960e+00,7.8670e+00,7.7530e+00,7.0570e+00,6.6960e+00,7.2900e+00,
     *7.7520e+00,1.0120e+01,1.0310e+01,1.0510e+01,9.7950e+00,9.8360e+00,
     *9.9380e+00/
      DATA ((SPROF(10,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0120e-02,3.5430e-02,2.5030e-02,1.0800e-01,7.6300e-02,
     *5.3950e-02,2.3240e-01,1.6430e-01,1.1620e-01,4.9900e-01,3.5350e-01,
     *2.5010e-01,1.0620e+00,7.5630e-01,5.3680e-01,2.2070e+00,1.5950e+00,
     *1.1420e+00,3.4110e+00,2.8540e+00,2.2220e+00,3.1280e+00,3.6040e+00,
     *3.4950e+00,4.3760e+00,5.1380e+00,5.4500e+00,3.3170e+00,3.7110e+00,
     *4.3200e+00,6.0370e+00,6.3210e+00,6.7070e+00,5.8120e+00,5.9930e+00,
     *6.1680e+00/
      DATA ((SPROF(11,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0120e-02,3.5430e-02,2.5030e-02,1.0800e-01,7.6300e-02,
     *5.3950e-02,2.3240e-01,1.6430e-01,1.1620e-01,4.9890e-01,3.5350e-01,
     *2.5010e-01,1.0590e+00,7.5500e-01,5.3630e-01,2.1740e+00,1.5820e+00,
     *1.1370e+00,2.2950e+00,2.3080e+00,1.9890e+00,1.0040e+00,1.6600e+00,
     *2.2270e+00,1.6950e+00,2.2200e+00,2.9900e+00,1.4200e+00,1.5390e+00,
     *1.7320e+00,3.0840e+00,3.2680e+00,3.4770e+00,3.0060e+00,3.1610e+00,
     *3.2930e+00/
      DATA ((SPROF(12,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0120e-02,3.5430e-02,2.5030e-02,1.0800e-01,7.6290e-02,
     *5.3950e-02,2.3240e-01,1.6430e-01,1.1620e-01,4.9850e-01,3.5330e-01,
     *2.5010e-01,1.0500e+00,7.5170e-01,5.3520e-01,2.0930e+00,1.5520e+00,
     *1.1260e+00,8.8850e-01,1.3630e+00,1.5080e+00,2.2770e-01,3.6190e-01,
     *7.6940e-01,5.7650e-01,6.6650e-01,9.2460e-01,5.4290e-01,5.6740e-01,
     *5.9760e-01,1.3800e+00,1.4580e+00,1.5290e+00,1.3660e+00,1.4480e+00,
     *1.5100e+00/
      DATA ((SPROF(13,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0120e-02,3.5430e-02,2.5030e-02,1.0800e-01,7.6290e-02,
     *5.3940e-02,2.3230e-01,1.6430e-01,1.1620e-01,4.9760e-01,3.5300e-01,
     *2.4990e-01,1.0270e+00,7.4370e-01,5.3230e-01,1.9010e+00,1.4770e+00,
     *1.0990e+00,1.3320e-01,3.8600e-01,7.5710e-01,6.3590e-02,7.0060e-02,
     *1.0780e-01,1.9030e-01,1.9870e-01,2.1980e-01,1.9060e-01,1.9220e-01,
     *1.9330e-01,5.4470e-01,5.6600e-01,5.7950e-01,5.4930e-01,5.7520e-01,
     *5.9060e-01/
      DATA ((SPROF(14,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0120e-02,3.5430e-02,2.5030e-02,1.0790e-01,7.6280e-02,
     *5.3940e-02,2.3200e-01,1.6420e-01,1.1620e-01,4.9530e-01,3.5220e-01,
     *2.4960e-01,9.7350e-01,7.2370e-01,5.2500e-01,1.4960e+00,1.3060e+00,
     *1.0320e+00,2.2000e-02,3.6150e-02,1.4530e-01,1.9650e-02,1.9610e-02,
     *2.0630e-02,6.2040e-02,6.1480e-02,6.1960e-02,6.4610e-02,6.3040e-02,
     *6.1420e-02,1.9620e-01,1.9800e-01,1.9700e-01,2.0190e-01,2.0570e-01,
     *2.0550e-01/
      DATA ((SPROF(15,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0120e-02,3.5430e-02,2.5030e-02,1.0790e-01,7.6260e-02,
     *5.3930e-02,2.3150e-01,1.6400e-01,1.1610e-01,4.8950e-01,3.5010e-01,
     *2.4890e-01,8.5040e-01,6.7600e-01,5.0730e-01,8.2280e-01,9.5970e-01,
     *8.8230e-01,6.1310e-03,6.5150e-03,9.0070e-03,6.1890e-03,5.9430e-03,
     *5.8320e-03,2.0380e-02,1.9580e-02,1.8990e-02,2.1730e-02,2.0700e-02,
     *1.9730e-02,6.8060e-02,6.6550e-02,6.4440e-02,7.1470e-02,7.0540e-02,
     *6.8450e-02/
      DATA ((SPROF(16,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0110e-02,3.5430e-02,2.5030e-02,1.0770e-01,7.6210e-02,
     *5.3910e-02,2.3000e-01,1.6350e-01,1.1590e-01,4.7520e-01,3.4500e-01,
     *2.4710e-01,6.0570e-01,5.6960e-01,4.6550e-01,1.9270e-01,4.4470e-01,
     *5.9540e-01,1.9180e-03,1.8710e-03,1.8990e-03,2.0300e-03,1.9140e-03,
     *1.8220e-03,6.5720e-03,6.2300e-03,5.9110e-03,7.0580e-03,6.6720e-03,
     *6.2970e-03,2.3270e-02,2.2220e-02,2.1060e-02,2.4740e-02,2.3850e-02,
     *2.2680e-02/
      DATA ((SPROF(17,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0100e-02,3.5420e-02,2.5030e-02,1.0730e-01,7.6080e-02,
     *5.3870e-02,2.2630e-01,1.6220e-01,1.1550e-01,4.4120e-01,3.3240e-01,
     *2.4250e-01,2.5930e-01,3.7060e-01,3.7490e-01,1.1880e-02,6.8590e-02,
     *2.2300e-01,6.2730e-04,5.9320e-04,5.7040e-04,6.7420e-04,6.3110e-04,
     *5.9290e-04,2.1870e-03,2.0520e-03,1.9260e-03,2.3410e-03,2.2180e-03,
     *2.0770e-03,7.5400e-03,7.1830e-03,6.7740e-03,7.9460e-03,7.6760e-03,
     *7.3030e-03/
      DATA ((SPROF(18,I,J,4),I=1,NTEMP),J=1,NNE)/2.3260e-02,1.6430e-02,
     *1.1620e-02,5.0060e-02,3.5410e-02,2.5020e-02,1.0640e-01,7.5750e-02,
     *5.3750e-02,2.1740e-01,1.5890e-01,1.1430e-01,3.6620e-01,3.0270e-01,
     *2.3140e-01,3.2280e-02,1.2640e-01,2.1790e-01,2.0270e-03,2.7930e-03,
     *2.0590e-02,2.0760e-04,1.9450e-04,1.8350e-04,2.2410e-04,2.0930e-04,
     *1.9540e-04,7.2720e-04,6.8160e-04,6.3660e-04,7.7720e-04,7.3560e-04,
     *6.8910e-04,2.4850e-03,2.3750e-03,2.2470e-03,2.5890e-03,2.5190e-03,
     *2.4080e-03/
      DATA ((SPROF(19,I,J,4),I=1,NTEMP),J=1,NNE)/2.3250e-02,1.6430e-02,
     *1.1620e-02,4.9970e-02,3.5370e-02,2.5010e-02,1.0410e-01,7.4920e-02,
     *5.3460e-02,1.9640e-01,1.5110e-01,1.1140e-01,2.2930e-01,2.3930e-01,
     *2.0570e-01,8.6410e-04,9.0570e-03,5.5960e-02,6.0380e-04,6.0280e-04,
     *6.9890e-04,6.8880e-05,6.4260e-05,6.0100e-05,7.4160e-05,6.9450e-05,
     *6.4740e-05,2.4050e-04,2.2620e-04,2.1120e-04,2.5410e-04,2.4260e-04,
     *2.2840e-04,8.1110e-04,7.8530e-04,7.4380e-04,8.1950e-04,8.1930e-04,
     *7.9390e-04/
      DATA ((SPROF(20,I,J,4),I=1,NTEMP),J=1,NNE)/2.3240e-02,1.6420e-02,
     *1.1620e-02,4.9730e-02,3.5290e-02,2.4980e-02,9.8550e-02,7.2890e-02,
     *5.2730e-02,1.5230e-01,1.3300e-01,1.0450e-01,7.0960e-02,1.3270e-01,
     *1.5310e-01,1.9560e-04,2.1900e-04,2.0430e-03,1.9460e-04,1.8620e-04,
     *1.8340e-04,2.2810e-05,2.1300e-05,1.9850e-05,2.4380e-05,2.2980e-05,
     *2.1470e-05,7.8850e-05,7.4770e-05,7.0060e-05,8.1480e-05,7.9410e-05,
     *7.5420e-05,2.5820e-04,2.5610e-04,2.4490e-04,2.5970e-04,2.5980e-04,
     *2.5870e-04/
      DATA ((SPROF(21,I,J,4),I=1,NTEMP),J=1,NNE)/2.3220e-02,1.6420e-02,
     *1.1620e-02,4.9150e-02,3.5080e-02,2.4910e-02,8.5840e-02,6.8030e-02,
     *5.0940e-02,8.0320e-02,9.6550e-02,8.9070e-02,3.9270e-03,3.0270e-02,
     *7.2910e-02,6.0810e-05,5.9470e-05,6.2070e-05,6.3930e-05,6.0090e-05,
     *5.7220e-05,7.5180e-06,7.0540e-06,6.5770e-06,7.9550e-06,7.5720e-06,
     *7.1090e-06,2.5610e-05,2.4550e-05,2.3180e-05,2.5810e-05,2.5770e-05,
     *2.4750e-05,0.0000e+00,8.1840e-05,8.0100e-05,2*0.0000e+00,
     *8.2610e-05/
      DATA ((SPROF(22,I,J,4),I=1,NTEMP),J=1,NNE)/2.3160e-02,1.6400e-02,
     *1.1610e-02,4.7700e-02,3.4570e-02,2.4720e-02,6.0690e-02,5.7200e-02,
     *4.6710e-02,1.6150e-02,4.3210e-02,5.9570e-02,6.9250e-05,8.0600e-04,
     *1.1350e-02,1.9740e-05,1.8700e-05,1.8130e-05,2.1150e-05,1.9750e-05,
     *1.8550e-05,2.4640e-06,2.3300e-06,2.1800e-06,2.5730e-06,2.4800e-06,
     *2.3480e-06,8.1460e-06,8.0040e-06,7.6310e-06,0.0000e+00,8.1880e-06,
     *8.0720e-06,0.0000e+00,2.5990e-05,2.6010e-05,3*0.0000e+00/
      DATA ((SPROF(23,I,J,4),I=1,NTEMP),J=1,NNE)/2.3010e-02,1.6340e-02,
     *1.1590e-02,4.4270e-02,3.3300e-02,2.4270e-02,2.5410e-02,3.7000e-02,
     *3.7560e-02,3.0930e-04,5.7490e-03,2.1690e-02,1.9330e-05,1.9850e-05,
     *1.2740e-04,6.4970e-06,6.0860e-06,5.7590e-06,7.0020e-06,6.5310e-06,
     *6.0960e-06,8.0150e-07,7.6590e-07,7.2120e-07,8.1510e-07,8.0640e-07,
     *7.7190e-07,0.0000e+00,2.5820e-06,2.4990e-06,2*0.0000e+00,
     *2.6030e-06,2*0.0000e+00,8.2760e-06,3*0.0000e+00/
      DATA ((SPROF(24,I,J,4),I=1,NTEMP),J=1,NNE)/2.2640e-02,1.6210e-02,
     *1.1540e-02,3.6690e-02,3.0310e-02,2.3150e-02,2.8560e-03,1.2390e-02,
     *2.1730e-02,6.3880e-06,4.3110e-05,1.7190e-03,6.1310e-06,5.9040e-06,
     *5.9350e-06,2.1480e-06,2.0040e-06,1.8770e-06,2.3130e-06,2.1630e-06,
     *2.0150e-06,2.5740e-07,2.5030e-07,2.3780e-07,0.0000e+00,2.5850e-07,
     *2.5230e-07,0.0000e+00,8.1990e-07,8.1350e-07,2*0.0000e+00,
     *8.3010e-07,6*0.0000e+00/
      DATA ((SPROF(25,I,J,4),I=1,NTEMP),J=1,NNE)/2.1750e-02,1.5890e-02,
     *1.1430e-02,2.2900e-02,2.3950e-02,2.0580e-02,1.4010e-05,7.9550e-04,
     *5.4990e-03,1.9270e-06,1.9150e-06,5.0080e-06,2.0000e-06,1.8850e-06,
     *1.8070e-06,7.1040e-07,6.6290e-07,6.1820e-07,7.6080e-07,7.1550e-07,
     *6.6770e-07,8.1570e-08,8.1220e-08,7.8070e-08,0.0000e+00,
     *2*8.2130e-08,2*0.0000e+00,2.6090e-07,9*0.0000e+00/
      DATA ((SPROF(26,I,J,4),I=1,NTEMP),J=1,NNE)/1.9650e-02,1.5100e-02,
     *1.1140e-02,7.0050e-03,1.3240e-02,1.5300e-02,6.2110e-07,1.4620e-06,
     *1.7480e-04,6.1830e-07,5.8990e-07,5.7970e-07,6.5920e-07,6.1630e-07,
     *5.8060e-07,2.3440e-07,2.1950e-07,2.0460e-07,2.4870e-07,2.3600e-07,
     *2.2120e-07,0.0000e+00,2.5880e-08,2.5470e-08,2*0.0000e+00,
     *2.6160e-08,2*0.0000e+00,8.3320e-08,9*0.0000e+00/
      DATA ((SPROF(27,I,J,4),I=1,NTEMP),J=1,NNE)/1.5220e-02,1.3290e-02,
     *1.0450e-02,3.5820e-04,2.9920e-03,7.2720e-03,1.9290e-07,1.8820e-07,
     *2.2520e-07,2.0260e-07,1.9030e-07,1.8110e-07,2.1800e-07,2.0330e-07,
     *1.9000e-07,7.6940e-08,7.2560e-08,6.7790e-08,8.0730e-08,7.7450e-08,
     *7.3140e-08,2*0.0000e+00,8.2300e-09,15*0.0000e+00/
      DATA ((SPROF(28,I,J,4),I=1,NTEMP),J=1,NNE)/8.0170e-03,9.6460e-03,
     *8.9030e-03,4.1550e-07,7.1490e-05,1.1220e-03,6.2460e-08,5.9180e-08,
     *5.7330e-08,6.6860e-08,6.2430e-08,5.8630e-08,7.2040e-08,6.7280e-08,
     *6.2710e-08,2.5100e-08,2.3900e-08,2.2450e-08,2.5750e-08,2.5260e-08,
     *2.4090e-08,2*0.0000e+00,2.6250e-09,15*0.0000e+00/
      DATA ((SPROF(29,I,J,4),I=1,NTEMP),J=1,NNE)/1.6020e-03,4.3110e-03,
     *5.9520e-03,6.1230e-08,6.8600e-08,1.0340e-05,2.0530e-08,1.9230e-08,
     *1.8200e-08,2.2120e-08,2.0630e-08,1.9260e-08,2.3740e-08,2.2270e-08,
     *2.0770e-08,8.1350e-09,7.8290e-09,7.4160e-09,2*8.1650e-09,
     *7.8930e-09,18*0.0000e+00/
      DATA ((SPROF(30,I,J,4),I=1,NTEMP),J=1,NNE)/2.8120e-05,5.7040e-04,
     *2.1650e-03,1.9380e-08,1.8670e-08,1.8820e-08,6.7850e-09,6.3300e-09,
     *5.9290e-09,7.3050e-09,6.8310e-09,6.3650e-09,7.7780e-09,7.3560e-09,
     *6.8840e-09,2.5770e-09,2.5470e-09,2.4390e-09,0.0000e+00,2.5920e-09,
     *2.5700e-09,18*0.0000e+00/
      DATA ((SPROF(31,I,J,4),I=1,NTEMP),J=1,NNE)/2.1270e-08,3.5680e-06,
     *1.7070e-04,6.3180e-09,5.9570e-09,5.7140e-09,2.2440e-09,2.0940e-09,
     *1.9530e-09,2.4030e-09,2.2600e-09,2.1090e-09,2.5310e-09,2.4200e-09,
     *2.2780e-09,0.0000e+00,8.1740e-10,7.9740e-10,2*0.0000e+00,
     *8.2500e-10,18*0.0000e+00/
      DATA ((SPROF(32,I,J,4),I=1,NTEMP),J=1,NNE)/6.0890e-09,6.0660e-09,
     *2.9560e-07,2.0830e-09,1.9470e-09,1.8340e-09,7.4030e-10,6.9360e-10,
     *6.4630e-10,7.8590e-10,7.4570e-10,6.9900e-10,8.1390e-10,7.9090e-10,
     *7.5170e-10,0.0000e+00,2.5970e-10,2.5910e-10,2*0.0000e+00,
     *2.6350e-10,18*0.0000e+00/
      DATA ((SPROF(33,I,J,4),I=1,NTEMP),J=1,NNE)/1.9540e-09,1.8640e-09,
     *1.8320e-09,6.8880e-10,6.4230e-10,6.0050e-10,2.4320e-10,2.2930e-10,
     *2.1420e-10,2.5510e-10,2.4480e-10,2.3120e-10,2.5790e-10,2.5670e-10,
     *2.4680e-10,2*0.0000e+00,8.2730e-11,21*0.0000e+00/
      DATA ((SPROF(34,I,J,4),I=1,NTEMP),J=1,NNE)/6.4000e-10,6.0120e-10,
     *5.7220e-10,2.2770e-10,2.1260e-10,1.9820e-10,7.9340e-11,7.5550e-11,
     *7.0970e-11,8.1440e-11,7.9840e-11,7.6160e-11,0.0000e+00,8.1850e-11,
     *8.0520e-11,24*0.0000e+00/
      DATA ((SPROF(35,I,J,4),I=1,NTEMP),J=1,NNE)/2.1130e-10,1.9730e-10,
     *1.8530e-10,7.5020e-11,7.0410e-11,6.5650e-11,2.5720e-11,2.4750e-11,
     *2.3450e-11,2*2.5820e-11,2.4950e-11,2*0.0000e+00,2.6030e-11,
     *24*0.0000e+00/
      DATA ((SPROF(36,I,J,4),I=1,NTEMP),J=1,NNE)/6.9920e-11,6.5200e-11,
     *6.0860e-11,2.4590e-11,2.3260e-11,2.1760e-11,8.1500e-12,8.0540e-12,
     *7.7120e-12,0.0000e+00,8.1980e-12,8.1260e-12,2*0.0000e+00,
     *8.3000e-12,24*0.0000e+00/
      DATA ((SPROF(37,I,J,4),I=1,NTEMP),J=1,NNE)/2.3100e-11,2.1600e-11,
     *2.0120e-11,8.0040e-12,7.6500e-12,7.2040e-12,0.0000e+00,2.5850e-12,
     *2.5220e-12,2*0.0000e+00,2.6090e-12,27*0.0000e+00/
      DATA ((SPROF(38,I,J,4),I=1,NTEMP),J=1,NNE)/7.5980e-12,7.1450e-12,
     *6.6690e-12,2.5740e-12,2.5010e-12,2.3770e-12,0.0000e+00,8.2130e-13,
     *8.1930e-13,2*0.0000e+00,8.3320e-13,27*0.0000e+00/
      DATA ((SPROF(39,I,J,4),I=1,NTEMP),J=1,NNE)/2.4850e-12,2.3580e-12,
     *2.2090e-12,8.1570e-13,8.1170e-13,7.8030e-13,2*0.0000e+00,
     *2.6160e-13,30*0.0000e+00/
      DATA ((SPROF(40,I,J,4),I=1,NTEMP),J=1,NNE)/8.0670e-13,7.7420e-13,
     *7.3090e-13,0.0000e+00,2.5880e-13,2.5460e-13,33*0.0000e+00/
      DATA ((SPROF(41,I,J,4),I=1,NTEMP),J=1,NNE)/2.5750e-13,2.5250e-13,
     *2.4080e-13,2*0.0000e+00,8.2300e-14,33*0.0000e+00/
      DATA ((SPROF(42,I,J,4),I=1,NTEMP),J=1,NNE)/2*8.1650e-14,
     *7.8910e-14,2*0.0000e+00,2.6250e-14,33*0.0000e+00/
      DATA ((SPROF(43,I,J,4),I=1,NTEMP),J=1,NNE)/0.0000e+00,2.5920e-14,
     *2.5690e-14,36*0.0000e+00/
      DATA ((SPROF(44,I,J,4),I=1,NTEMP),J=1,NNE)/2*0.0000e+00,
     *8.2500e-15,36*0.0000e+00/
      DATA ((SPROF(45,I,J,4),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(46,I,J,4),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(1,I,J,5),I=1,NTEMP),J=1,NNE)/3.1390e-02,2.2180e-02,
     *1.5690e-02,6.7600e-02,4.7810e-02,3.3800e-02,1.4540e-01,1.0290e-01,
     *7.2780e-02,3.1150e-01,2.2090e-01,1.5650e-01,6.6060e-01,4.7150e-01,
     *3.3530e-01,1.3570e+00,9.8660e-01,7.0990e-01,2.5710e+00,1.9570e+00,
     *1.4530e+00,4.1020e+00,3.4110e+00,2.7210e+00,5.0690e+00,4.7030e+00,
     *4.2070e+00,5.0410e+00,4.9760e+00,4.9230e+00,4.5860e+00,4.5840e+00,
     *4.6400e+00,4.5020e+00,4.1900e+00,4.0410e+00,4.5230e+00,4.1000e+00,
     *3.8580e+00/
      DATA ((SPROF(2,I,J,5),I=1,NTEMP),J=1,NNE)/3.1390e-02,2.2180e-02,
     *1.5690e-02,6.7600e-02,4.7810e-02,3.3800e-02,1.4540e-01,1.0290e-01,
     *7.2780e-02,3.1150e-01,2.2090e-01,1.5650e-01,6.6060e-01,4.7150e-01,
     *3.3530e-01,1.3570e+00,9.8660e-01,7.0990e-01,2.5700e+00,1.9570e+00,
     *1.4530e+00,4.0980e+00,3.4080e+00,2.7200e+00,5.0610e+00,4.6960e+00,
     *4.2020e+00,5.0470e+00,4.9750e+00,4.9170e+00,4.6400e+00,4.6110e+00,
     *4.6560e+00,4.5570e+00,4.2480e+00,4.1150e+00,4.5770e+00,4.1570e+00,
     *3.9300e+00/
      DATA ((SPROF(3,I,J,5),I=1,NTEMP),J=1,NNE)/3.1390e-02,2.2180e-02,
     *1.5690e-02,6.7600e-02,4.7810e-02,3.3800e-02,1.4540e-01,1.0290e-01,
     *7.2780e-02,3.1150e-01,2.2090e-01,1.5650e-01,6.6060e-01,4.7150e-01,
     *3.3530e-01,1.3570e+00,9.8660e-01,7.0990e-01,2.5690e+00,1.9560e+00,
     *1.4530e+00,4.0910e+00,3.4050e+00,2.7180e+00,5.0490e+00,4.6860e+00,
     *4.1950e+00,5.0570e+00,4.9730e+00,4.9090e+00,4.7210e+00,4.6520e+00,
     *4.6780e+00,4.6410e+00,4.3360e+00,4.2170e+00,4.6580e+00,4.2430e+00,
     *4.0310e+00/
      DATA ((SPROF(4,I,J,5),I=1,NTEMP),J=1,NNE)/3.1390e-02,2.2180e-02,
     *1.5690e-02,6.7600e-02,4.7810e-02,3.3800e-02,1.4540e-01,1.0290e-01,
     *7.2780e-02,3.1150e-01,2.2090e-01,1.5650e-01,6.6060e-01,4.7150e-01,
     *3.3530e-01,1.3570e+00,9.8650e-01,7.0980e-01,2.5660e+00,1.9550e+00,
     *1.4520e+00,4.0740e+00,3.3960e+00,2.7140e+00,5.0190e+00,4.6600e+00,
     *4.1770e+00,5.0820e+00,4.9690e+00,4.8870e+00,4.9250e+00,4.7540e+00,
     *4.7270e+00,4.8500e+00,4.5560e+00,4.4220e+00,4.8610e+00,4.4590e+00,
     *4.2370e+00/
      DATA ((SPROF(5,I,J,5),I=1,NTEMP),J=1,NNE)/3.1390e-02,2.2180e-02,
     *1.5690e-02,6.7600e-02,4.7810e-02,3.3800e-02,1.4540e-01,1.0290e-01,
     *7.2780e-02,3.1150e-01,2.2090e-01,1.5650e-01,6.6050e-01,4.7140e-01,
     *3.3520e-01,1.3560e+00,9.8630e-01,7.0980e-01,2.5560e+00,1.9510e+00,
     *1.4510e+00,4.0320e+00,3.3730e+00,2.7030e+00,4.9440e+00,4.5960e+00,
     *4.1320e+00,5.1100e+00,4.9460e+00,4.8300e+00,5.1730e+00,4.9120e+00,
     *4.8140e+00,5.1100e+00,4.8520e+00,4.7340e+00,5.1150e+00,4.7620e+00,
     *4.5700e+00/
      DATA ((SPROF(6,I,J,5),I=1,NTEMP),J=1,NNE)/3.1390e-02,2.2180e-02,
     *1.5690e-02,6.7600e-02,4.7810e-02,3.3800e-02,1.4540e-01,1.0290e-01,
     *7.2780e-02,3.1150e-01,2.2090e-01,1.5650e-01,6.6040e-01,4.7140e-01,
     *3.3520e-01,1.3550e+00,9.8580e-01,7.0960e-01,2.5340e+00,1.9420e+00,
     *1.4470e+00,3.9290e+00,3.3170e+00,2.6760e+00,4.7540e+00,4.4370e+00,
     *4.0210e+00,5.0500e+00,4.8430e+00,4.6790e+00,5.2560e+00,5.0110e+00,
     *4.8680e+00,5.2080e+00,5.0440e+00,4.9700e+00,5.1930e+00,4.9700e+00,
     *4.8490e+00/
      DATA ((SPROF(7,I,J,5),I=1,NTEMP),J=1,NNE)/3.1390e-02,2.2180e-02,
     *1.5690e-02,6.7600e-02,4.7810e-02,3.3800e-02,1.4540e-01,1.0290e-01,
     *7.2780e-02,3.1150e-01,2.2090e-01,1.5650e-01,6.6010e-01,4.7130e-01,
     *3.3520e-01,1.3520e+00,9.8470e-01,7.0920e-01,2.4780e+00,1.9180e+00,
     *1.4370e+00,3.6830e+00,3.1810e+00,2.6100e+00,4.2870e+00,4.0630e+00,
     *3.7560e+00,4.5570e+00,4.4300e+00,4.2760e+00,4.6530e+00,4.5940e+00,
     *4.5320e+00,4.6090e+00,4.6080e+00,4.6160e+00,4.5720e+00,
     *2*4.5570e+00/
      DATA ((SPROF(8,I,J,5),I=1,NTEMP),J=1,NNE)/3.1390e-02,2.2180e-02,
     *1.5690e-02,6.7600e-02,4.7810e-02,3.3800e-02,1.4540e-01,1.0290e-01,
     *7.2780e-02,3.1150e-01,2.2090e-01,1.5650e-01,6.5920e-01,4.7090e-01,
     *3.3510e-01,1.3440e+00,9.8170e-01,7.0810e-01,2.3440e+00,1.8590e+00,
     *1.4140e+00,3.1390e+00,2.8660e+00,2.4510e+00,3.2710e+00,3.2620e+00,
     *3.1740e+00,3.2250e+00,3.2950e+00,3.3120e+00,3.1610e+00,3.2740e+00,
     *3.3270e+00,3.1400e+00,3.2400e+00,3.3080e+00,3.1120e+00,3.2250e+00,
     *3.3050e+00/
      DATA ((SPROF(9,I,J,5),I=1,NTEMP),J=1,NNE)/3.1390e-02,2.2180e-02,
     *1.5690e-02,6.7600e-02,4.7810e-02,3.3800e-02,1.4530e-01,1.0290e-01,
     *7.2780e-02,3.1140e-01,2.2090e-01,1.5650e-01,6.5700e-01,4.7020e-01,
     *3.3480e-01,1.3250e+00,9.7440e-01,7.0540e-01,2.0400e+00,1.7200e+00,
     *1.3560e+00,2.1390e+00,2.2170e+00,2.0950e+00,1.7600e+00,1.9440e+00,
     *2.1190e+00,1.6050e+00,1.6940e+00,1.7990e+00,1.5710e+00,1.6420e+00,
     *1.6950e+00,1.5840e+00,1.6500e+00,1.6900e+00,1.5860e+00,1.6690e+00,
     *1.7220e+00/
      DATA ((SPROF(10,I,J,5),I=1,NTEMP),J=1,NNE)/3.1390e-02,2.2180e-02,
     *1.5690e-02,6.7590e-02,4.7800e-02,3.3800e-02,1.4530e-01,1.0290e-01,
     *7.2770e-02,3.1110e-01,2.2080e-01,1.5650e-01,6.5160e-01,4.6820e-01,
     *3.3410e-01,1.2780e+00,9.5630e-01,6.9860e-01,1.4480e+00,1.4170e+00,
     *1.2220e+00,9.2690e-01,1.2070e+00,1.4260e+00,6.4180e-01,7.2510e-01,
     *8.8680e-01,6.0890e-01,6.2710e-01,6.5230e-01,6.1610e-01,6.2850e-01,
     *6.3380e-01,6.3420e-01,6.4930e-01,6.5160e-01,6.4530e-01,6.7100e-01,
     *6.8050e-01/
      DATA ((SPROF(11,I,J,5),I=1,NTEMP),J=1,NNE)/3.1390e-02,2.2180e-02,
     *1.5690e-02,6.7580e-02,4.7800e-02,3.3800e-02,1.4510e-01,1.0280e-01,
     *7.2750e-02,3.1060e-01,2.2060e-01,1.5640e-01,6.3810e-01,4.6320e-01,
     *3.3230e-01,1.1670e+00,9.1220e-01,6.8190e-01,6.4100e-01,8.7830e-01,
     *9.4110e-01,2.4360e-01,3.4780e-01,5.7820e-01,2.0200e-01,2.0870e-01,
     *2.3130e-01,2.0560e-01,2.0290e-01,2.0040e-01,2.1480e-01,2.1180e-01,
     *2.0650e-01,2.2450e-01,2.2370e-01,2.1850e-01,2.3080e-01,2.3440e-01,
     *2.3210e-01/
      DATA ((SPROF(12,I,J,5),I=1,NTEMP),J=1,NNE)/3.1390e-02,2.2180e-02,
     *1.5690e-02,6.7540e-02,4.7780e-02,3.3800e-02,1.4480e-01,1.0270e-01,
     *7.2710e-02,3.0910e-01,2.2010e-01,1.5620e-01,6.0550e-01,4.5110e-01,
     *3.2790e-01,9.3040e-01,8.1030e-01,6.4170e-01,1.2490e-01,2.8220e-01,
     *4.9330e-01,6.4870e-02,7.1180e-02,1.0500e-01,6.4030e-02,6.2310e-02,
     *6.2080e-02,6.7610e-02,6.4810e-02,6.2000e-02,7.1720e-02,6.9180e-02,
     *6.5890e-02,7.5270e-02,7.3610e-02,7.0660e-02,7.7440e-02,7.7280e-02,
     *7.5350e-02/
      DATA ((SPROF(13,I,J,5),I=1,NTEMP),J=1,NNE)/3.1390e-02,2.2180e-02,
     *1.5690e-02,6.7450e-02,4.7750e-02,3.3790e-02,1.4390e-01,1.0230e-01,
     *7.2590e-02,3.0560e-01,2.1880e-01,1.5580e-01,5.3080e-01,4.2200e-01,
     *3.1700e-01,5.2970e-01,6.0240e-01,5.5100e-01,2.1460e-02,3.3780e-02,
     *1.0680e-01,1.9090e-02,1.8820e-02,1.9780e-02,1.9910e-02,1.8920e-02,
     *1.8100e-02,2.1160e-02,2.0110e-02,1.8980e-02,2.2280e-02,2.1440e-02,
     *2.0340e-02,2.3070e-02,2.2570e-02,2.1700e-02,2.3450e-02,2.3330e-02,
     *2.4010e-02/
      DATA ((SPROF(14,I,J,5),I=1,NTEMP),J=1,NNE)/3.1390e-02,2.2180e-02,
     *1.5690e-02,6.7220e-02,4.7670e-02,3.3760e-02,1.4160e-01,1.0150e-01,
     *7.2310e-02,2.9690e-01,2.1560e-01,1.5460e-01,3.8160e-01,3.5680e-01,
     *2.9130e-01,1.3660e-01,2.8810e-01,3.7610e-01,5.9780e-03,6.1260e-03,
     *8.3550e-03,6.1150e-03,5.8090e-03,5.6090e-03,6.5100e-03,6.1440e-03,
     *5.7810e-03,6.9030e-03,6.5770e-03,6.1950e-03,7.1900e-03,6.9700e-03,
     *6.6430e-03,7.3460e-03,7.2500e-03,7.0330e-03,7.1900e-03,7.4010e-03,
     *7.5740e-03/
      DATA ((SPROF(15,I,J,5),I=1,NTEMP),J=1,NNE)/3.1390e-02,2.2180e-02,
     *1.5690e-02,6.6640e-02,4.7470e-02,3.3680e-02,1.3610e-01,9.9540e-02,
     *7.1590e-02,2.7600e-01,2.0790e-01,1.5180e-01,1.6750e-01,2.3440e-01,
     *2.3560e-01,1.0750e-02,4.8840e-02,1.4520e-01,1.9030e-03,1.8280e-03,
     *1.8240e-03,2.0070e-03,1.8940e-03,1.7870e-03,2.1360e-03,2.0210e-03,
     *1.9010e-03,2.2400e-03,2.1530e-03,2.0370e-03,2.2530e-03,2.2550e-03,
     *2.1700e-03,2.2570e-03,2.2590e-03,2.2700e-03,2.2670e-03,2.2700e-03,
     *2.2710e-03/
      DATA ((SPROF(16,I,J,5),I=1,NTEMP),J=1,NNE)/3.1380e-02,2.2180e-02,
     *1.5690e-02,6.5220e-02,4.6960e-02,3.3500e-02,1.2320e-01,9.4680e-02,
     *6.9820e-02,2.2980e-01,1.8960e-01,1.4500e-01,2.2530e-02,8.1950e-02,
     *1.3830e-01,1.9710e-03,2.5980e-03,1.4840e-02,6.2060e-04,5.8460e-04,
     *5.5680e-04,6.6190e-04,6.2260e-04,5.8290e-04,6.9620e-04,6.6620e-04,
     *6.2650e-04,7.0980e-04,7.0030e-04,6.7090e-04,7.1080e-04,7.1110e-04,
     *7.0730e-04,7.1320e-04,7.1360e-04,7.1380e-04,7.1850e-04,7.1910e-04,
     *7.1940e-04/
      DATA ((SPROF(17,I,J,5),I=1,NTEMP),J=1,NNE)/3.1350e-02,2.2170e-02,
     *1.5680e-02,6.1780e-02,4.5700e-02,3.3050e-02,9.5840e-02,8.3500e-02,
     *6.5560e-02,1.4510e-01,1.5050e-01,1.2910e-01,8.1520e-04,6.3930e-03,
     *3.6500e-02,5.9630e-04,5.8880e-04,6.6780e-04,2.0390e-04,1.9140e-04,
     *1.7970e-04,2.1650e-04,2.0500e-04,1.9210e-04,2.2580e-04,2.1770e-04,
     *2.0630e-04,0.0000e+00,2.2430e-04,2.1890e-04,2.2480e-04,
     *2*2.2490e-04,2.2610e-04,2*2.2620e-04,2.2880e-04,2.2890e-04,
     *2.2900e-04/
      DATA ((SPROF(18,I,J,5),I=1,NTEMP),J=1,NNE)/3.1290e-02,2.2150e-02,
     *1.5680e-02,5.3920e-02,4.2690e-02,3.1940e-02,5.1060e-02,6.0910e-02,
     *5.5990e-02,4.5940e-02,8.4280e-02,9.6510e-02,1.9160e-04,2.1070e-04,
     *1.4770e-03,1.9200e-04,1.8240e-04,1.7750e-04,6.6890e-05,6.3030e-05,
     *5.8990e-05,7.0280e-05,6.7250e-05,6.3300e-05,7.0810e-05,7.0570e-05,
     *6.7630e-05,0.0000e+00,7.0950e-05,7.0920e-05,9*0.0000e+00/
      DATA ((SPROF(19,I,J,5),I=1,NTEMP),J=1,NNE)/3.1150e-02,2.2100e-02,
     *1.5660e-02,3.8310e-02,3.5980e-02,2.9330e-02,1.0530e-02,2.7580e-02,
     *3.7660e-02,2.7420e-03,1.9720e-02,4.6490e-02,5.9810e-05,5.7960e-05,
     *5.9730e-05,6.2820e-05,5.9030e-05,5.5830e-05,2.1820e-05,2.0730e-05,
     *1.9460e-05,2.2370e-05,2.1920e-05,2.0850e-05,2.2400e-05,2.2410e-05,
     *2.2030e-05,2*0.0000e+00,2.2470e-05,9*0.0000e+00/
      DATA ((SPROF(20,I,J,5),I=1,NTEMP),J=1,NNE)/3.0790e-02,2.1970e-02,
     *1.5610e-02,1.6240e-02,2.3420e-02,2.3660e-02,2.2080e-04,3.7840e-03,
     *1.3910e-02,6.7100e-05,5.7740e-04,7.4560e-03,1.9390e-05,1.8310e-05,
     *1.7600e-05,2.0630e-05,1.9380e-05,1.8160e-05,7.0630e-06,6.7880e-06,
     *6.4190e-06,7.0780e-06,7.0790e-06,6.8250e-06,2*0.0000e+00,
     *7.0940e-06,12*0.0000e+00/
      DATA ((SPROF(21,I,J,5),I=1,NTEMP),J=1,NNE)/2.9900e-02,2.1650e-02,
     *1.5500e-02,1.8840e-03,7.9630e-03,1.3790e-02,6.2310e-06,3.2360e-05,
     *1.1450e-03,1.8930e-05,1.9260e-05,9.5440e-05,6.3570e-06,5.9680e-06,
     *5.6180e-06,6.7540e-06,6.3810e-06,5.9770e-06,2.2370e-06,2.2080e-06,
     *2.1080e-06,0.0000e+00,2.2410e-06,2.2190e-06,2*0.0000e+00,
     *2.2490e-06,12*0.0000e+00/
      DATA ((SPROF(22,I,J,5),I=1,NTEMP),J=1,NNE)/2.7770e-02,2.0870e-02,
     *1.5220e-02,1.0570e-05,5.3190e-04,3.5570e-03,1.8860e-06,1.8600e-06,
     *4.1300e-06,6.0130e-06,5.7560e-06,5.7230e-06,2.0860e-06,1.9630e-06,
     *1.8380e-06,2.1980e-06,2.0970e-06,1.9690e-06,0.0000e+00,7.0800e-07,
     *6.8880e-07,2*0.0000e+00,7.0980e-07,15*0.0000e+00/
      DATA ((SPROF(23,I,J,5),I=1,NTEMP),J=1,NNE)/2.3080e-02,1.9020e-02,
     *1.4530e-02,6.0600e-07,1.2290e-06,1.1880e-04,6.0600e-07,5.7570e-07,
     *5.6070e-07,1.9600e-06,1.8450e-06,1.7570e-06,6.8180e-07,6.4610e-07,
     *6.0570e-07,7.0710e-07,6.8520e-07,6.4950e-07,0.0000e+00,2.2420e-07,
     *2.2340e-07,2*0.0000e+00,2.2510e-07,15*0.0000e+00/
      DATA ((SPROF(24,I,J,5),I=1,NTEMP),J=1,NNE)/1.4500e-02,1.5070e-02,
     *1.2930e-02,1.8880e-07,1.8300e-07,2.1070e-07,1.9830e-07,1.8630e-07,
     *1.7620e-07,6.4320e-07,6.0390e-07,5.6690e-07,2.2140e-07,2.1200e-07,
     *1.9990e-07,2.2370e-07,2.2230e-07,2.1320e-07,2*0.0000e+00,
     *7.1040e-08,18*0.0000e+00/
      DATA ((SPROF(25,I,J,5),I=1,NTEMP),J=1,NNE)/4.5090e-03,8.4050e-03,
     *9.6560e-03,6.1200e-08,5.7830e-08,5.5570e-08,6.5100e-08,6.1170e-08,
     *5.7320e-08,2.1090e-07,1.9880e-07,1.8610e-07,7.0720e-08,6.9140e-08,
     *6.5780e-08,0.0000e+00,7.0820e-08,6.9500e-08,21*0.0000e+00/
      DATA ((SPROF(26,I,J,5),I=1,NTEMP),J=1,NNE)/2.4040e-04,1.9380e-03,
     *4.6360e-03,2.0070e-08,1.8840e-08,1.7740e-08,2.1320e-08,2.0150e-08,
     *1.8870e-08,6.8820e-08,6.5420e-08,6.1430e-08,2*2.2380e-08,
     *2.1550e-08,0.0000e+00,2*2.2430e-08,21*0.0000e+00/
      DATA ((SPROF(27,I,J,5),I=1,NTEMP),J=1,NNE)/3.5760e-07,4.8830e-05,
     *7.3420e-04,6.5890e-09,6.2000e-09,5.8040e-09,6.9430e-09,6.6230e-09,
     *6.2300e-09,2.2290e-08,2.1430e-08,2.0270e-08,0.0000e+00,7.0850e-09,
     *7.0090e-09,2*0.0000e+00,7.1110e-09,21*0.0000e+00/
      DATA ((SPROF(28,I,J,5),I=1,NTEMP),J=1,NNE)/5.9810e-08,6.5310e-08,
     *7.2340e-06,2.1550e-09,2.0420e-09,1.9140e-09,2.2360e-09,2.1640e-09,
     *2.0540e-09,7.0720e-09,6.9720e-09,6.6600e-09,2*0.0000e+00,
     *2.2440e-09,24*0.0000e+00/
      DATA ((SPROF(29,I,J,5),I=1,NTEMP),J=1,NNE)/1.8990e-08,1.8180e-08,
     *1.8140e-08,6.9950e-10,6.7010e-10,6.3190e-10,7.0740e-10,7.0260e-10,
     *6.7410e-10,0.0000e+00,2.2380e-09,2.1770e-09,2*0.0000e+00,
     *7.1190e-10,24*0.0000e+00/
      DATA ((SPROF(30,I,J,5),I=1,NTEMP),J=1,NNE)/6.1900e-09,5.8280e-09,
     *5.5490e-09,2.2360e-10,2.1850e-10,2.0810e-10,0.0000e+00,2.2390e-10,
     *2.1970e-10,0.0000e+00,7.0880e-10,7.0630e-10,27*0.0000e+00/
      DATA ((SPROF(31,I,J,5),I=1,NTEMP),J=1,NNE)/2.0320e-09,1.9080e-09,
     *1.7910e-09,2*7.0760e-11,6.8150e-11,0.0000e+00,2*7.0930e-11,
     *2*0.0000e+00,2.2460e-10,27*0.0000e+00/
      DATA ((SPROF(32,I,J,5),I=1,NTEMP),J=1,NNE)/6.6670e-10,6.2840e-10,
     *5.8820e-10,0.0000e+00,2.2400e-11,2.2170e-11,2*0.0000e+00,
     *2.2490e-11,30*0.0000e+00/
      DATA ((SPROF(33,I,J,5),I=1,NTEMP),J=1,NNE)/2.1760e-10,2.0680e-10,
     *1.9420e-10,2*0.0000e+00,7.0980e-12,33*0.0000e+00/
      DATA ((SPROF(34,I,J,5),I=1,NTEMP),J=1,NNE)/7.0460e-11,6.7760e-11,
     *6.4080e-11,2*0.0000e+00,2.2510e-12,33*0.0000e+00/
      DATA ((SPROF(35,I,J,5),I=1,NTEMP),J=1,NNE)/2.2360e-11,2.2040e-11,
     *2.1070e-11,36*0.0000e+00/
      DATA ((SPROF(36,I,J,5),I=1,NTEMP),J=1,NNE)/0.0000e+00,7.0790e-12,
     *6.8840e-12,36*0.0000e+00/
      DATA ((SPROF(37,I,J,5),I=1,NTEMP),J=1,NNE)/0.0000e+00,2.2420e-12,
     *2.2340e-12,36*0.0000e+00/
      DATA ((SPROF(38,I,J,5),I=1,NTEMP),J=1,NNE)/2*0.0000e+00,
     *7.1040e-13,36*0.0000e+00/
      DATA ((SPROF(39,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(40,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(41,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(42,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(43,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(44,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(45,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(46,I,J,5),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(1,I,J,6),I=1,NTEMP),J=1,NNE)/3.5120e-02,2.4840e-02,
     *1.7560e-02,7.5590e-02,5.3470e-02,3.7820e-02,1.6230e-01,1.1500e-01,
     *8.1390e-02,3.4570e-01,2.4600e-01,1.7460e-01,7.2000e-01,5.1940e-01,
     *3.7170e-01,1.4110e+00,1.0540e+00,7.7210e-01,2.4100e+00,1.9370e+00,
     *1.5010e+00,3.3220e+00,2.9350e+00,2.5040e+00,3.8170e+00,3.5700e+00,
     *3.3150e+00,4.0450e+00,3.8590e+00,3.6910e+00,4.1510e+00,4.0500e+00,
     *3.8590e+00,4.1060e+00,3.9570e+00,3.9030e+00,4.1020e+00,3.8840e+00,
     *3.7860e+00/
      DATA ((SPROF(2,I,J,6),I=1,NTEMP),J=1,NNE)/3.5120e-02,2.4840e-02,
     *1.7560e-02,7.5590e-02,5.3470e-02,3.7820e-02,1.6230e-01,1.1500e-01,
     *8.1390e-02,3.4570e-01,2.4600e-01,1.7460e-01,7.2000e-01,5.1940e-01,
     *3.7170e-01,1.4110e+00,1.0540e+00,7.7200e-01,2.4080e+00,1.9360e+00,
     *1.5000e+00,3.3190e+00,2.9330e+00,2.5030e+00,3.8110e+00,3.5660e+00,
     *3.3120e+00,4.0300e+00,3.8480e+00,3.6840e+00,4.0870e+00,3.9840e+00,
     *3.8190e+00,4.0470e+00,3.8970e+00,3.8330e+00,4.0480e+00,3.8310e+00,
     *3.7260e+00/
      DATA ((SPROF(3,I,J,6),I=1,NTEMP),J=1,NNE)/3.5120e-02,2.4840e-02,
     *1.7560e-02,7.5590e-02,5.3470e-02,3.7820e-02,1.6230e-01,1.1500e-01,
     *8.1390e-02,3.4570e-01,2.4600e-01,1.7460e-01,7.1990e-01,5.1930e-01,
     *3.7170e-01,1.4100e+00,1.0540e+00,7.7200e-01,2.4050e+00,1.9340e+00,
     *1.5000e+00,3.3150e+00,2.9310e+00,2.5020e+00,3.8010e+00,3.5590e+00,
     *3.3080e+00,4.0070e+00,3.8320e+00,3.6740e+00,3.9910e+00,3.8830e+00,
     *3.7610e+00,3.9580e+00,3.8070e+00,3.7410e+00,3.9660e+00,3.7510e+00,
     *3.6460e+00/
      DATA ((SPROF(4,I,J,6),I=1,NTEMP),J=1,NNE)/3.5120e-02,2.4840e-02,
     *1.7560e-02,7.5590e-02,5.3470e-02,3.7820e-02,1.6230e-01,1.1500e-01,
     *8.1390e-02,3.4560e-01,2.4600e-01,1.7460e-01,7.1970e-01,5.1930e-01,
     *3.7160e-01,1.4090e+00,1.0530e+00,7.7170e-01,2.3970e+00,1.9300e+00,
     *1.4980e+00,3.3040e+00,2.9240e+00,2.4980e+00,3.7760e+00,3.5430e+00,
     *3.2980e+00,3.9520e+00,3.7940e+00,3.6500e+00,3.8100e+00,3.6980e+00,
     *3.6350e+00,3.7880e+00,3.6350e+00,3.5610e+00,3.8070e+00,3.5950e+00,
     *3.4840e+00/
      DATA ((SPROF(5,I,J,6),I=1,NTEMP),J=1,NNE)/3.5120e-02,2.4840e-02,
     *1.7560e-02,7.5590e-02,5.3470e-02,3.7820e-02,1.6230e-01,1.1500e-01,
     *8.1390e-02,3.4560e-01,2.4600e-01,1.7460e-01,7.1920e-01,5.1910e-01,
     *3.7160e-01,1.4050e+00,1.0520e+00,7.7120e-01,2.3760e+00,1.9210e+00,
     *1.4930e+00,3.2770e+00,2.9080e+00,2.4890e+00,3.7150e+00,3.5030e+00,
     *3.2710e+00,3.8290e+00,3.7040e+00,3.5900e+00,3.5180e+00,3.4120e+00,
     *3.3990e+00,3.5080e+00,3.3600e+00,3.2820e+00,3.5370e+00,3.3370e+00,
     *3.2230e+00/
      DATA ((SPROF(6,I,J,6),I=1,NTEMP),J=1,NNE)/3.5120e-02,2.4840e-02,
     *1.7560e-02,7.5590e-02,5.3470e-02,3.7820e-02,1.6230e-01,1.1500e-01,
     *8.1390e-02,3.4550e-01,2.4590e-01,1.7460e-01,7.1810e-01,5.1870e-01,
     *3.7140e-01,1.3960e+00,1.0480e+00,7.6970e-01,2.3250e+00,1.8960e+00,
     *1.4820e+00,3.2110e+00,2.8670e+00,2.4660e+00,3.5740e+00,3.4070e+00,
     *3.2060e+00,3.5890e+00,3.5140e+00,3.4550e+00,3.1510e+00,3.0740e+00,
     *3.0640e+00,3.1380e+00,3.0220e+00,2.9620e+00,3.1650e+00,3.0060e+00,
     *2.9090e+00/
      DATA ((SPROF(7,I,J,6),I=1,NTEMP),J=1,NNE)/3.5120e-02,2.4840e-02,
     *1.7560e-02,7.5590e-02,5.3470e-02,3.7820e-02,1.6220e-01,1.1500e-01,
     *8.1380e-02,3.4510e-01,2.4580e-01,1.7460e-01,7.1520e-01,5.1760e-01,
     *3.7100e-01,1.3740e+00,1.0390e+00,7.6620e-01,2.2030e+00,1.8350e+00,
     *1.4550e+00,3.0530e+00,2.7680e+00,2.4090e+00,3.2750e+00,3.1870e+00,
     *3.0500e+00,3.2120e+00,3.1810e+00,3.1800e+00,2.6530e+00,2.6400e+00,
     *2.6390e+00,2.6380e+00,2.6000e+00,2.5870e+00,2.6490e+00,2.5870e+00,
     *2.5480e+00/
      DATA ((SPROF(8,I,J,6),I=1,NTEMP),J=1,NNE)/3.5120e-02,2.4840e-02,
     *1.7560e-02,7.5590e-02,5.3470e-02,3.7820e-02,1.6220e-01,1.1490e-01,
     *8.1380e-02,3.4440e-01,2.4560e-01,1.7450e-01,7.0810e-01,5.1490e-01,
     *3.7000e-01,1.3190e+00,1.0160e+00,7.5740e-01,1.9270e+00,1.6920e+00,
     *1.3890e+00,2.6990e+00,2.5360e+00,2.2710e+00,2.7330e+00,2.7360e+00,
     *2.7020e+00,2*2.6800e+00,2.7050e+00,1.8640e+00,1.9030e+00,
     *1.9370e+00,1.8620e+00,1.9000e+00,1.9290e+00,1.8590e+00,1.9010e+00,
     *1.9270e+00/
      DATA ((SPROF(9,I,J,6),I=1,NTEMP),J=1,NNE)/3.5120e-02,2.4840e-02,
     *1.7560e-02,7.5590e-02,5.3470e-02,3.7820e-02,1.6210e-01,1.1490e-01,
     *8.1370e-02,3.4240e-01,2.4480e-01,1.7420e-01,6.9040e-01,5.0830e-01,
     *3.6760e-01,1.1900e+00,9.6150e-01,7.3560e-01,1.3920e+00,1.3830e+00,
     *1.2370e+00,2.0290e+00,2.0510e+00,1.9640e+00,1.9130e+00,1.9760e+00,
     *2.0400e+00,1.8750e+00,1.9190e+00,1.9610e+00,9.7230e-01,1.0010e+00,
     *1.0250e+00,9.8180e-01,1.0220e+00,1.0450e+00,9.7830e-01,1.0340e+00,
     *1.0680e+00/
      DATA ((SPROF(10,I,J,6),I=1,NTEMP),J=1,NNE)/3.5120e-02,2.4840e-02,
     *1.7560e-02,7.5580e-02,5.3470e-02,3.7820e-02,1.6190e-01,1.1480e-01,
     *8.1340e-02,3.3750e-01,2.4310e-01,1.7360e-01,6.4800e-01,4.9200e-01,
     *3.6160e-01,9.2290e-01,8.3700e-01,6.8380e-01,6.6270e-01,8.4730e-01,
     *9.2640e-01,1.1130e+00,1.2580e+00,1.3820e+00,9.8140e-01,1.0410e+00,
     *1.1220e+00,9.6610e-01,9.9750e-01,1.0250e+00,3.8530e-01,3.9070e-01,
     *3.9120e-01,3.9240e-01,4.0620e-01,4.0930e-01,3.9120e-01,4.1430e-01,
     *4.2550e-01/
      DATA ((SPROF(11,I,J,6),I=1,NTEMP),J=1,NNE)/3.5120e-02,2.4840e-02,
     *1.7560e-02,7.5560e-02,5.3460e-02,3.7820e-02,1.6140e-01,1.1470e-01,
     *8.1280e-02,3.2560e-01,2.3870e-01,1.7200e-01,5.5290e-01,4.5330e-01,
     *3.4680e-01,4.9530e-01,5.9240e-01,5.6930e-01,1.7070e-01,2.8240e-01,
     *4.5930e-01,4.0440e-01,4.7950e-01,6.2660e-01,3.6900e-01,3.8030e-01,
     *4.0190e-01,3.7540e-01,3.7880e-01,3.7790e-01,1.3140e-01,1.3080e-01,
     *1.2770e-01,1.3390e-01,1.3620e-01,1.3500e-01,1.3350e-01,1.3890e-01,
     *1.4060e-01/
      DATA ((SPROF(12,I,J,6),I=1,NTEMP),J=1,NNE)/3.5120e-02,2.4840e-02,
     *1.7560e-02,7.5510e-02,5.3440e-02,3.7810e-02,1.6020e-01,1.1420e-01,
     *8.1120e-02,2.9750e-01,2.2810e-01,1.6810e-01,3.7170e-01,3.6920e-01,
     *3.1240e-01,1.2080e-01,2.5390e-01,3.6030e-01,3.9550e-02,4.8890e-02,
     *9.8270e-02,1.2130e-01,1.2830e-01,1.5620e-01,1.2150e-01,1.1940e-01,
     *1.1810e-01,1.2690e-01,1.2440e-01,1.2030e-01,4.2620e-02,4.1830e-02,
     *4.0360e-02,4.3300e-02,4.3330e-02,4.2520e-02,4.3330e-02,4.4050e-02,
     *4.4000e-02/
      DATA ((SPROF(13,I,J,6),I=1,NTEMP),J=1,NNE)/3.5120e-02,2.4830e-02,
     *1.7560e-02,7.5380e-02,5.3400e-02,3.7800e-02,1.5700e-01,1.1310e-01,
     *8.0720e-02,2.3720e-01,2.0340e-01,1.5870e-01,1.3950e-01,2.2090e-01,
     *2.4020e-01,1.4810e-02,3.8150e-02,1.1720e-01,1.1150e-02,1.1230e-02,
     *1.2890e-02,3.7370e-02,3.6470e-02,3.7060e-02,3.9120e-02,3.7440e-02,
     *3.5740e-02,4.1170e-02,3.9730e-02,3.7820e-02,1.3260e-02,1.3070e-02,
     *1.2670e-02,1.3370e-02,1.3360e-02,1.3180e-02,1.2450e-02,1.3480e-02,
     *1.3460e-02/
      DATA ((SPROF(14,I,J,6),I=1,NTEMP),J=1,NNE)/3.5100e-02,2.4830e-02,
     *1.7560e-02,7.5060e-02,5.3280e-02,3.7750e-02,1.4940e-01,1.1030e-01,
     *7.9720e-02,1.3440e-01,1.5260e-01,1.3730e-01,1.4860e-02,6.1970e-02,
     *1.2450e-01,3.5480e-03,3.9340e-03,1.0090e-02,3.5060e-03,3.3590e-03,
     *3.2930e-03,1.1390e-02,1.0890e-02,1.0490e-02,1.1940e-02,0.0000e+00,
     *1.0850e-02,1.2970e-02,1.2560e-02,1.1980e-02,3.8620e-03,3.8650e-03,
     *3.9870e-03,3.8670e-03,3.8750e-03,3.8780e-03,3.8840e-03,3.8940e-03,
     *3.9000e-03/
      DATA ((SPROF(15,I,J,6),I=1,NTEMP),J=1,NNE)/3.5070e-02,2.4820e-02,
     *1.7560e-02,7.4250e-02,5.3000e-02,3.7650e-02,1.3190e-01,1.0360e-01,
     *7.7260e-02,3.2750e-02,7.4210e-02,9.5490e-02,1.2420e-03,3.6400e-03,
     *2.4440e-02,1.0980e-03,1.0710e-03,1.1140e-03,1.1370e-03,1.0820e-03,
     *1.0280e-03,3.6760e-03,3.4990e-03,3.3090e-03,3.8300e-03,0.0000e+00,
     *3.5130e-03,4.0490e-03,3.9660e-03,3.8210e-03,1.2090e-03,
     *2*1.2020e-03,1.2140e-03,1.2150e-03,1.2070e-03,1.2150e-03,
     *1.2250e-03,1.2260e-03/
      DATA ((SPROF(16,I,J,6),I=1,NTEMP),J=1,NNE)/3.4990e-02,2.4790e-02,
     *1.7550e-02,7.2270e-02,5.2280e-02,3.7400e-02,9.6430e-02,8.8540e-02,
     *7.1410e-02,1.3190e-03,1.2350e-02,3.8400e-02,3.4810e-04,3.5680e-04,
     *7.7090e-04,3.5280e-04,3.3560e-04,3.2310e-04,3.7010e-04,3.5230e-04,
     *3.3220e-04,1.1890e-03,1.1400e-03,1.0800e-03,1.1990e-03,0.0000e+00,
     *1.1450e-03,1.1990e-03,2*1.2000e-03,3.7920e-04,3.7940e-04,
     *3.7950e-04,3.8160e-04,3.8180e-04,3.8190e-04,3.8670e-04,3.8700e-04,
     *3.8720e-04/
      DATA ((SPROF(17,I,J,6),I=1,NTEMP),J=1,NNE)/3.4790e-02,2.4720e-02,
     *1.7520e-02,6.7530e-02,5.0540e-02,3.6770e-02,4.3950e-02,5.9670e-02,
     *5.8590e-02,1.1530e-04,2.5630e-04,3.9800e-03,1.0980e-04,1.0570e-04,
     *1.0550e-04,1.1460e-04,1.0880e-04,1.0280e-04,1.1950e-04,1.1490e-04,
     *1.0880e-04,3.7770e-04,3.7130e-04,3.5340e-04,3.7770e-04,0.0000e+00,
     *3.7300e-04,3.7820e-04,3.7830e-04,3.7840e-04,2*1.2010e-04,
     *1.2020e-04,1.2130e-04,2*1.2140e-04,1.2390e-04,2*1.2400e-04/
      DATA ((SPROF(18,I,J,6),I=1,NTEMP),J=1,NNE)/3.4280e-02,2.4540e-02,
     *1.7460e-02,5.6950e-02,4.6400e-02,3.5230e-02,6.1830e-03,2.2160e-02,
     *3.5660e-02,3.4540e-05,3.4260e-05,4.9580e-05,3.5510e-05,3.3700e-05,
     *3.2160e-05,3.7170e-05,3.5500e-05,3.3510e-05,3.7670e-05,3.7280e-05,
     *3.5550e-05,0.0000e+00,1.1980e-04,1.1540e-04,1.1930e-04,0.0000e+00,
     *1.1940e-04,3*1.1960e-04,9*0.0000e+00/
      DATA ((SPROF(19,I,J,6),I=1,NTEMP),J=1,NNE)/3.3040e-02,2.4090e-02,
     *1.7300e-02,3.7110e-02,3.7450e-02,3.1650e-02,8.4140e-05,1.8710e-03,
     *1.0250e-02,1.1030e-05,1.0540e-05,1.0290e-05,1.1540e-05,1.0970e-05,
     *1.0350e-05,1.1910e-05,1.1570e-05,1.0990e-05,0.0000e+00,1.1910e-05,
     *1.1610e-05,0.0000e+00,3.7690e-05,3.7400e-05,2*0.0000e+00,
     *3.7740e-05,12*0.0000e+00/
      DATA ((SPROF(20,I,J,6),I=1,NTEMP),J=1,NNE)/3.0120e-02,2.3000e-02,
     *1.6900e-02,1.2670e-02,2.1860e-02,2.4180e-02,1.1150e-05,1.5490e-05,
     *4.5820e-04,3.5790e-06,3.3950e-06,3.2220e-06,3.7370e-06,3.5820e-06,
     *3.3870e-06,3.7640e-06,3.7470e-06,3.5960e-06,0.0000e+00,3.7670e-06,
     *3.7600e-06,0.0000e+00,2*1.1920e-05,15*0.0000e+00/
      DATA ((SPROF(21,I,J,6),I=1,NTEMP),J=1,NNE)/2.3880e-02,2.0480e-02,
     *1.5950e-02,8.6230e-04,5.6600e-03,1.2290e-02,3.4500e-06,3.3600e-06,
     *3.6410e-06,1.1630e-06,1.1080e-06,1.0450e-06,1.1900e-06,1.1660e-06,
     *1.1080e-06,0.0000e+00,1.1910e-06,1.1700e-06,2*0.0000e+00,
     *1.1920e-06,2*0.0000e+00,3.7730e-06,15*0.0000e+00/
      DATA ((SPROF(22,I,J,6),I=1,NTEMP),J=1,NNE)/1.3330e-02,1.5300e-02,
     *1.3780e-02,4.8070e-06,1.9350e-04,2.2490e-03,1.1100e-06,1.0570e-06,
     *1.0180e-06,3.7590e-07,3.6160e-07,3.4260e-07,3.7640e-07,3.7650e-07,
     *3.6270e-07,2*0.0000e+00,3.7660e-07,21*0.0000e+00/
      DATA ((SPROF(23,I,J,6),I=1,NTEMP),J=1,NNE)/3.0840e-03,7.3550e-03,
     *9.5560e-03,1.0970e-06,1.1590e-06,3.2720e-05,3.6090e-07,3.4260e-07,
     *3.2400e-07,1.1900e-07,1.1750e-07,1.1230e-07,0.0000e+00,1.1910e-07,
     *1.1790e-07,2*0.0000e+00,1.1920e-07,21*0.0000e+00/
      DATA ((SPROF(24,I,J,6),I=1,NTEMP),J=1,NNE)/7.9200e-05,1.1690e-03,
     *3.8080e-03,3.4600e-07,2*3.3300e-07,1.1710e-07,1.1190e-07,
     *1.0570e-07,0.0000e+00,3.7640e-08,3.6630e-08,2*0.0000e+00,
     *3.7670e-08,24*0.0000e+00/
      DATA ((SPROF(25,I,J,6),I=1,NTEMP),J=1,NNE)/3.7090e-07,1.1890e-05,
     *3.7770e-04,1.1200e-07,1.0630e-07,1.0150e-07,3.7640e-08,3.6510e-08,
     *3.4670e-08,0.0000e+00,1.1910e-08,1.1870e-08,2*0.0000e+00,
     *1.1930e-08,24*0.0000e+00/
      DATA ((SPROF(26,I,J,6),I=1,NTEMP),J=1,NNE)/1.0900e-07,1.0820e-07,
     *1.2530e-06,3.6410e-08,3.4620e-08,3.2680e-08,1.1900e-08,1.1830e-08,
     *1.1360e-08,2*0.0000e+00,3.7690e-09,27*0.0000e+00/
      DATA ((SPROF(27,I,J,6),I=1,NTEMP),J=1,NNE)/3.4800e-08,3.3260e-08,
     *3.2480e-08,1.1800e-08,1.1310e-08,1.0690e-08,0.0000e+00,3.7630e-09,
     *3.6960e-09,30*0.0000e+00/
      DATA ((SPROF(28,I,J,6),I=1,NTEMP),J=1,NNE)/1.1300e-08,1.0720e-08,
     *1.0170e-08,3.7640e-09,3.6830e-09,3.5090e-09,0.0000e+00,
     *2*1.1910e-09,30*0.0000e+00/
      DATA ((SPROF(29,I,J,6),I=1,NTEMP),J=1,NNE)/3.6730e-09,3.5000e-09,
     *3.3020e-09,2*1.1900e-09,1.1480e-09,2*0.0000e+00,3.7700e-10,
     *30*0.0000e+00/
      DATA ((SPROF(30,I,J,6),I=1,NTEMP),J=1,NNE)/1.1880e-09,1.1430e-09,
     *1.0830e-09,0.0000e+00,3.7640e-10,3.7290e-10,33*0.0000e+00/
      DATA ((SPROF(31,I,J,6),I=1,NTEMP),J=1,NNE)/3.7630e-10,3.7140e-10,
     *3.5510e-10,2*0.0000e+00,1.1920e-10,33*0.0000e+00/
      DATA ((SPROF(32,I,J,6),I=1,NTEMP),J=1,NNE)/0.0000e+00,1.1900e-10,
     *1.1590e-10,2*0.0000e+00,3.7730e-11,33*0.0000e+00/
      DATA ((SPROF(33,I,J,6),I=1,NTEMP),J=1,NNE)/0.0000e+00,3.7650e-11,
     *3.7550e-11,36*0.0000e+00/
      DATA ((SPROF(34,I,J,6),I=1,NTEMP),J=1,NNE)/2*0.0000e+00,
     *1.1920e-11,36*0.0000e+00/
      DATA ((SPROF(35,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(36,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(37,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(38,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(39,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(40,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(41,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(42,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(43,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(44,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(45,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/
      DATA ((SPROF(46,I,J,6),I=1,NTEMP),J=1,NNE)/39*0.0000e+00/

      DATA FIRST/.TRUE./
      EXP10(X)=EXP(2.30258509299405D0*X)
C
C  Read in the table (this is done only once)
C
      IF(FIRST) THEN
        FIRST=.FALSE.
cC
cC  Open file and process errors
cC
c        OPEN(1,file=HFILE,IOSTAT=IERR,FORM='UNFORMATTED',STATUS='OLD')
c        IF(IERR.NE.0) THEN
c          WRITE(*,*) 'ERROR: STEHLE did not find Hydrogen line file'
c          WRITE(*,*) HFILE
c          STOP
c        ENDIF
cC
cC  Read table dimensions and process errors
cC
c        READ(1,ERR=9,END=10) I,I,I,I
cC
cC  Read line ID and construct line index
cC
c        READ(1,ERR=9,END=10) (I,I,LINE=1,NLINE)
cC
cC  Read temperatures and electron densities
cC
c        READ(1,ERR=9,END=10) (DT,I=1,NTEMP)
c        READ(1,ERR=9,END=10) (DT,J=1,NNE)
cC
cC  Read wavelength conversion factor, wing extrapolation parameter
cC  and line widths
cC
c        READ(1,ERR=9,END=10) ((DT,J=1,NNE),L=1,NLINE)
c        READ(1,ERR=9,END=10) ((DT,J=1,NNE),L=1,NLINE)
c        READ(1,ERR=9,END=10) (((DT,I=1,NTEMP),J=1,NNE),
c     *                       L=1,NLINE)
cC
cC  Read the number of detuning points and the two big arrays
cC  for detuning (ALPHA) and line profiles (SPROF)
cC
c        READ(1,ERR=9,END=10) (((IT,I=1,NTEMP),J=1,NNE),L=1,NLINE)
c        DO 2 L=1,NLINE
c        READ(1,ERR=9,END=10) (((TALPHA(K,I,J,L),K=1,NWL),
c     *                       I=1,NTEMP),J=1,NNE)
c        READ(1,ERR=9,END=10) (((TPROF(K,I,J,L),K=1,NWL),
c     *                       I=1,NTEMP),J=1,NNE)
c   2    continue
c        CLOSE(1)
c        do L=1,NLINE
c          do J=1,NNE
c            do i=1,NTEMP
c              do k=1,NWL
c                if(abs(TALPHA(k,i,j,l)-alpha(k,i,j,l)).gt.1.d-5) then
c                  write(*,*) k,i,j,l,TALPHA(k,i,j,l),alpha(k,i,j,l)
c                  stop
c                endif
c                if(abs(TALPHA(k,i,j,l)-alpha(k,i,j,l)).gt.1.d-5) then
c                  write(*,*) k,i,j,l,TPROF(k,i,j,l),sprof(k,i,j,l)
c                  stop
c                endif
c              enddo
c            enddo
c          enddo
c        enddo
        IF(LINDEX(NLOW,NUP).GT.0) WRITE(*,*) 
     *    'Will use Stehle''s tables for Hydrogen line profiles'
      END IF
C
C  Interpolation section
C
      LINE=LINDEX(NLOW,NUP)
C
C  Check if line is in the table
C
      IF(LINE.EQ.0) THEN
        HLIN=-1.
        RETURN
      END IF
C
C  Temperature interpolation (no extrapolation)
C
      IF(TEMP.LT.T(1).OR.TEMP.GT.T(NTEMP)) THEN
        HLIN=-1.
        RETURN
      END IF
      ITEMP=2
   3  IF(T(ITEMP).LT.TEMP.AND.ITEMP.LT.NTEMP) THEN
        ITEMP=ITEMP+1
        GO TO 3
      END IF
      DT=LOG10(T(ITEMP))-LOG10(T(ITEMP-1))
      DT1=(LOG10(TEMP)-LOG10(T(ITEMP-1)))/DT
      DT2=(LOG10(T(ITEMP))-LOG10(TEMP))/DT
C
C   Electron density interpolation (no extrapolation)
C
      ANE=LOG10(XXNE)
      IF(ANE.LT.XNE(1).OR.ANE.GT.XNE(NNE)) THEN
        HLIN=-1.
        RETURN
      END IF
      INE=2
   4  IF(XNE(INE).LT.ANE.AND.INE.LT.NNE) THEN
        INE=INE+1
        GO TO 4
      END IF
      DNE=XNE(INE)-XNE(INE-1)
      DNE1=(ANE-XNE(INE-1))/DNE
      DNE2=(XNE(INE)-ANE)/DNE
C
C  Interpolate over alpha. For each IT and IN we consider 3 cases:
C  1. alpha is in the first interval - simple linear interpolation
C  2. alpha is outside the table     - extrapolate using wing parameter
C  3. alpha is in between            - log interpolation
C
      IT=ITEMP-1
      IN=INE-1
      ALP=ABS(WAVE-WAVE0)/F0(IN,LINE)
      IF(ALP.LT.ALPHA(2,IT,IN,LINE)) THEN
        DAL=ALPHA(2,IT,IN,LINE)-ALPHA(1,IT,IN,LINE)
        DAL1=(ALP-ALPHA(1,IT,IN,LINE))/DAL
        DAL2=(ALPHA(2,IT,IN,LINE)-ALP)/DAL
        S11=DAL2*SPROF(1,IT,IN,LINE)+DAL1*SPROF(2,IT,IN,LINE)
      ELSE IF(ALP.GT.ALPHA(NALPHA(IT,IN,LINE),IT,IN,LINE)) THEN
        S11=WING(IN,LINE)/(ALP*ALP*SQRT(ALP))*SQRT(WAVE/WAVE0)
      ELSE
        ALP=LOG10(ALP)
        K=2
   5    IF(LOG10(ALPHA(K,IT,IN,LINE)).LT.ALP.AND.
     *     K.LT.NALPHA(IT,IN,LINE)) THEN
          K=K+1
          GO TO 5
        END IF
        DAL=LOG10(ALPHA(K,IT,IN,LINE))-LOG10(ALPHA(K-1,IT,IN,LINE))
        DAL1=(ALP-LOG10(ALPHA(K-1,IT,IN,LINE)))/DAL
        DAL2=(LOG10(ALPHA(K,IT,IN,LINE))-ALP)/DAL
        S11=DAL2*LOG10(SPROF(K-1,IT,IN,LINE))+
     +      DAL1*LOG10(SPROF(K,  IT,IN,LINE))
        S11=EXP10(S11)
      END IF
      S11=LOG10(S11/F0(IN,LINE))
C
      IT=ITEMP
      IN=INE-1
      ALP=ABS(WAVE-WAVE0)/F0(IN,LINE)
      IF(ALP.LT.ALPHA(2,IT,IN,LINE)) THEN
        DAL=ALPHA(2,IT,IN,LINE)-ALPHA(1,IT,IN,LINE)
        DAL1=(ALP-ALPHA(1,IT,IN,LINE))/DAL
        DAL2=(ALPHA(2,IT,IN,LINE)-ALP)/DAL
        S21=DAL2*SPROF(1,IT,IN,LINE)+DAL1*SPROF(2,IT,IN,LINE)
      ELSE IF(ALP.GT.ALPHA(NALPHA(IT,IN,LINE),IT,IN,LINE)) THEN
        S21=WING(IN,LINE)/(ALP*ALP*SQRT(ALP))*SQRT(WAVE/WAVE0)
      ELSE
        ALP=LOG10(ALP)
        K=2
   6    IF(LOG10(ALPHA(K,IT,IN,LINE)).LT.ALP.AND.
     *     K.LT.NALPHA(IT,IN,LINE)) THEN
          K=K+1
          GO TO 6
        END IF
        DAL=LOG10(ALPHA(K,IT,IN,LINE))-LOG10(ALPHA(K-1,IT,IN,LINE))
        DAL1=(ALP-LOG10(ALPHA(K-1,IT,IN,LINE)))/DAL
        DAL2=(LOG10(ALPHA(K,IT,IN,LINE))-ALP)/DAL
        S21=DAL2*LOG10(SPROF(K-1,IT,IN,LINE))+
     +      DAL1*LOG10(SPROF(K,  IT,IN,LINE))
        S21=EXP10(S21)
      END IF
      S21=LOG10(S21/F0(IN,LINE))
C
      IT=ITEMP-1
      IN=INE
      ALP=ABS(WAVE-WAVE0)/F0(IN,LINE)
      IF(ALP.LT.ALPHA(2,IT,IN,LINE)) THEN
        DAL=ALPHA(2,IT,IN,LINE)-ALPHA(1,IT,IN,LINE)
        DAL1=(ALP-ALPHA(1,IT,IN,LINE))/DAL
        DAL2=(ALPHA(2,IT,IN,LINE)-ALP)/DAL
        S12=DAL2*SPROF(1,IT,IN,LINE)+DAL1*SPROF(2,IT,IN,LINE)
      ELSE IF(ALP.GT.ALPHA(NALPHA(IT,IN,LINE),IT,IN,LINE)) THEN
        S12=WING(IN,LINE)/(ALP*ALP*SQRT(ALP))*SQRT(WAVE/WAVE0)
      ELSE
        ALP=LOG10(ALP)
        K=2
   7    IF(LOG10(ALPHA(K,IT,IN,LINE)).LT.ALP.AND.
     *     K.LT.NALPHA(IT,IN,LINE)) THEN
          K=K+1
          GO TO 7
        END IF
        DAL=LOG10(ALPHA(K,IT,IN,LINE))-LOG10(ALPHA(K-1,IT,IN,LINE))
        DAL1=(ALP-LOG10(ALPHA(K-1,IT,IN,LINE)))/DAL
        DAL2=(LOG10(ALPHA(K,IT,IN,LINE))-ALP)/DAL
        S12=DAL2*LOG10(SPROF(K-1,IT,IN,LINE))+
     +      DAL1*LOG10(SPROF(K,  IT,IN,LINE))
        S12=EXP10(S12)
      END IF
      S12=LOG10(S12/F0(IN,LINE))
C
      IT=ITEMP
      IN=INE
      ALP=ABS(WAVE-WAVE0)/F0(IN,LINE)
      IF(ALP.LT.ALPHA(2,IT,IN,LINE)) THEN
        DAL=ALPHA(2,IT,IN,LINE)-ALPHA(1,IT,IN,LINE)
        DAL1=(ALP-ALPHA(1,IT,IN,LINE))/DAL
        DAL2=(ALPHA(2,IT,IN,LINE)-ALP)/DAL
        S22=DAL2*SPROF(1,IT,IN,LINE)+DAL1*SPROF(2,IT,IN,LINE)
      ELSE IF(ALP.GT.ALPHA(NALPHA(IT,IN,LINE),IT,IN,LINE)) THEN
        S22=WING(IN,LINE)/(ALP*ALP*SQRT(ALP))*SQRT(WAVE/WAVE0)
      ELSE
        ALP=LOG10(ALP)
        K=2
   8    IF(LOG10(ALPHA(K,IT,IN,LINE)).LT.ALP.AND.
     *     K.LT.NALPHA(IT,IN,LINE)) THEN
          K=K+1
          GO TO 8
        END IF
        DAL=LOG10(ALPHA(K,IT,IN,LINE))-LOG10(ALPHA(K-1,IT,IN,LINE))
        DAL1=(ALP-LOG10(ALPHA(K-1,IT,IN,LINE)))/DAL
        DAL2=(LOG10(ALPHA(K,IT,IN,LINE))-ALP)/DAL
        S22=DAL2*LOG10(SPROF(K-1,IT,IN,LINE))+
     +      DAL1*LOG10(SPROF(K,  IT,IN,LINE))
        S22=EXP10(S22)
      END IF
      S22=LOG10(S22/F0(IN,LINE))
C
C  Interpolate over T and Ne
C
      HLIN=EXP10(DT2*DNE2*S11+DT1*DNE2*S21+DT2*DNE1*S12+DT1*DNE1*S22)
      RETURN
C
C  I/O error processing
C
   9  I=INDEX(HFILE,' ')-1
      IF(I.LE.0) I=LEN(HFILE)
      WRITE(*,*) 'ERROR reading binary file '//HFILE(1:I)//' in HTABLE'
      STOP
  10  I=INDEX(HFILE,' ')-1
      IF(I.LE.0) I=LEN(HFILE)
      WRITE(*,*) 'EOF found while reading binary file '//HFILE(1:I)//
     *           ' in HTABLE'
      STOP
      END

      SUBROUTINE HGRID(HFILE,WAVE,WAVE0,TEMP,NHYD,NLOW,NUP,HLIN)
C
C  N Piskunov's code to return Stehle Stark profiles
C  ADAPTED BY PAUL BARKLEM FOR MY GRIDS OF HYDROGEN PROFILES
C  ALL DOUBLE-PRECISION NOW ALSO
C  MAIN THING IS PROFILES ARE NOT ASSUMED SYMMETRIC
C
C  THERE IS NO WING APPROX YET, JUST SET TO A VERY SMALL NUMBER
C
      IMPLICIT NONE
      REAL*8 WAVE,WAVE0
      INTEGER MNPROFS,MNTEMP,MNNH,MNLINE
      PARAMETER (MNTEMP=10,MNNH=30,MNLINE=10,MNPROFS=200)
      REAL*8 T(MNTEMP),NH(MNNH),NHYD,TEMP,HLIN
      REAL*8 F0(MNNH,MNLINE),
     *     MALPHA(MNPROFS,MNTEMP,MNNH,MNLINE),
     *     PALPHA(MNPROFS,MNTEMP,MNNH,MNLINE),
     *     FALPHA(MNPROFS,MNTEMP,MNNH,MNLINE),
     *     MSPROF(MNPROFS,MNTEMP,MNNH,MNLINE),
     *     PSPROF(MNPROFS,MNTEMP,MNNH,MNLINE),
     *     FSPROF(MNPROFS,MNTEMP,MNNH,MNLINE)
      INTEGER MNALPHA(MNTEMP,MNNH,MNLINE),
     *     PNALPHA(MNTEMP,MNNH,MNLINE),
     *     FNALPHA(MNTEMP,MNNH,MNLINE)
      REAL*8 ALP,DAL,DAL1,DAL2,S11,S12,S21,S22
      REAL*8 DT,DT1,DT2,DNH,DNH1,DNH2
      REAL*8 SQRTPI
      INTEGER NPROFS,NTEMP,NNH,NLINE
      INTEGER NU(MNLINE),NL(MNLINE),NLOW,NUP
      INTEGER IERR,I,K,LINE,IN,IT,ITEMP,INH
      INTEGER MIDDLE
      INTEGER*2 LINDEX(100,100)
      LOGICAL FIRST
      CHARACTER HFILE*(*)
      SAVE T,NH,F0,MALPHA,MSPROF,PALPHA,PSPROF,PNALPHA,MNALPHA,
     *     NPROFS,NTEMP,NNH,NLINE,FIRST,LINDEX
      DATA FIRST/.TRUE./,LINDEX/10000*0/
      PARAMETER (SQRTPI=1.77245385)
      REAL*8 DEXP10,X
      DEXP10(X)=EXP(2.30258509299405D0*X)
C
C  Read in the table (this is done only once)
C
      IF(FIRST) THEN
        FIRST=.FALSE.
C
C  Open file and process errors
C
        OPEN(1,file=HFILE,IOSTAT=IERR,FORM='UNFORMATTED',STATUS='OLD')
        IF(IERR.NE.0) THEN
          WRITE(*,*) 'ERROR: HGRID did not find Hydrogen line file'
          WRITE(*,*) HFILE
          STOP
        ENDIF
C
C  Read table dimensions and process errors
C
        READ(1,ERR=9,END=10) NLINE
C
C  Read line ID and construct line index, read grid parameters
C
        READ(1,ERR=9,END=10) (NL(LINE),NU(LINE),LINE=1,NLINE)
        DO 401 I=1,NLINE
 401       LINDEX(NL(I),NU(I))=I
        READ(1,ERR=9,END=10) NNH
        READ(1,ERR=9,END=10) (NH(I),I=1,NNH)
        READ(1,ERR=9,END=10) NTEMP
        READ(1,ERR=9,END=10) (T(I),I=1,NTEMP)
C
C  Read the number of detuning points and the two big arrays
C  for detuning (ALPHA) and line profiles (SPROF)
C  Right Now ALPHA=LAMBDA, SPROF=INTENSITY (normalised)
C  hence F0=1 everywhere 
C
        READ(1,ERR=9,END=10) NPROFS
        MIDDLE=(NPROFS-1)/2+1
        DO LINE=1,NLINE
           DO IN=1,NNH
              DO IT=1,NTEMP
                 DO I=1,NPROFS
          READ(1,ERR=9,END=10) FALPHA(I,IT,IN,LINE),FSPROF(I,IT,IN,LINE)
                 END DO
              END DO
           END DO
        END DO
        DO LINE=1,NLINE
           DO IN=1,NNH
              F0(IN,LINE)=1.d0
           END DO
        END DO
        DO LINE=1,NLINE
           DO IN=1,NNH
              DO ITEMP=1,NTEMP
                 FNALPHA(ITEMP,IN,LINE)=NPROFS
                 PNALPHA(ITEMP,IN,LINE)=MIDDLE
                 MNALPHA(ITEMP,IN,LINE)=MIDDLE
              END DO
           END DO
        END DO
C
C  SET UP SEPARATE ARRAYS OF EACH SIDE OF THE PROFILE
C
        DO LINE=1,NLINE
           DO IN=1,NNH
              DO IT=1,NTEMP
                 DO K=1,MIDDLE
                    PALPHA(K,IT,IN,LINE)=FALPHA(MIDDLE+K-1,IT,IN,LINE)
              MALPHA(K,IT,IN,LINE)=DABS(FALPHA(MIDDLE-K+1,IT,IN,LINE))
                    PSPROF(K,IT,IN,LINE)=FSPROF(MIDDLE+K-1,IT,IN,LINE)
                    MSPROF(K,IT,IN,LINE)=FSPROF(MIDDLE-K+1,IT,IN,LINE)
                 END DO
              END DO
           END DO
        END DO                    
C
C  IF OUR LINE EXIST THEN WRITE OUT FIRST TIME
C
        IF(LINDEX(NLOW,NUP).GT.0) WRITE(*,*) 
     *    'Will use PB''s tables for Hydrogen line profiles'
C
      END IF  ! ENDS PART ONLY DONE FIRST TIME
C
C  Interpolation section
C
      LINE=LINDEX(NLOW,NUP)
C
C  Check if line is in the table
C
      IF(LINE.EQ.0) THEN
        HLIN=-1.
        RETURN
      END IF
C
C  Temperature and perturber density interpolation (no extrapolation)
C
      IF(TEMP.LT.T(1).OR.TEMP.GT.T(NTEMP)) THEN
        HLIN=-1.
        RETURN
      END IF
      ITEMP=2
   3  IF(T(ITEMP).LT.TEMP.AND.ITEMP.LT.NTEMP) THEN
        ITEMP=ITEMP+1
        GO TO 3
      END IF
      DT=DLOG10(T(ITEMP))-DLOG10(T(ITEMP-1))
      DT1=(DLOG10(TEMP)-DLOG10(T(ITEMP-1)))/DT
      DT2=(DLOG10(T(ITEMP))-DLOG10(TEMP))/DT
C
      IF(NHYD.LT.NH(1).OR.NHYD.GT.NH(NNH)) THEN
        HLIN=-1.
        RETURN
      END IF
      INH=2
   4  IF(NH(INH).LT.NHYD.AND.INH.LT.NNH) THEN
        INH=INH+1
        GO TO 4
      END IF
      DNH=DLOG10(NH(INH))-DLOG10(NH(INH-1))
      DNH1=(DLOG10(NHYD)-DLOG10(NH(INH-1)))/DNH
      DNH2=(DLOG10(NH(INH))-DLOG10(NHYD))/DNH
C
C  WE MUST SET UP TO ALLOW FOR THE FACT THAT THE PROFILE IS NOT ASSUMED 
C  TO BE SYMMETRIC
C
C  Interpolate over alpha. For each IT and IN we consider 3 cases:
C  1. alpha is in the first interval - simple linear interpolation
C  2. alpha is outside the table     - extrapolate using wing parameter
C  3. alpha is in between            - log interpolation
C
c  no wing extrapolation (impact approx invalid), we just take it as zero.
c
      IF ((WAVE-WAVE0).GE.0.d0) THEN
C
C  CASE OF POSITIVE DETUNING
C
      IT=ITEMP-1
      IN=INH-1
      ALP=DABS(WAVE-WAVE0)/F0(IN,LINE)
      IF(ALP.LT.PALPHA(2,IT,IN,LINE)) THEN
        DAL=PALPHA(2,IT,IN,LINE)-PALPHA(1,IT,IN,LINE)
        DAL1=(ALP-PALPHA(1,IT,IN,LINE))/DAL
        DAL2=(PALPHA(2,IT,IN,LINE)-ALP)/DAL
        S11=DAL2*PSPROF(1,IT,IN,LINE)+DAL1*PSPROF(2,IT,IN,LINE)
      ELSE IF(ALP.GT.PALPHA(PNALPHA(IT,IN,LINE),IT,IN,LINE)) THEN
         S11=0.d0
      ELSE
        ALP=DLOG10(ALP)
        K=2
   5    IF(DLOG10(PALPHA(K,IT,IN,LINE)).LT.ALP.AND.
     *     K.LT.PNALPHA(IT,IN,LINE)) THEN
          K=K+1
          GO TO 5
        END IF
        DAL=DLOG10(PALPHA(K,IT,IN,LINE))-DLOG10(PALPHA(K-1,IT,IN,LINE))
        DAL1=(ALP-DLOG10(PALPHA(K-1,IT,IN,LINE)))/DAL
        DAL2=(DLOG10(PALPHA(K,IT,IN,LINE))-ALP)/DAL
        S11=DAL2*DLOG10(PSPROF(K-1,IT,IN,LINE))+
     +      DAL1*DLOG10(PSPROF(K,  IT,IN,LINE))
        S11=DEXP10(S11)
      END IF
      S11=DLOG10(S11/F0(IN,LINE))
C
      IT=ITEMP
      IN=INH-1
      ALP=DABS(WAVE-WAVE0)/F0(IN,LINE)
      IF(ALP.LT.PALPHA(2,IT,IN,LINE)) THEN
        DAL=PALPHA(2,IT,IN,LINE)-PALPHA(1,IT,IN,LINE)
        DAL1=(ALP-PALPHA(1,IT,IN,LINE))/DAL
        DAL2=(PALPHA(2,IT,IN,LINE)-ALP)/DAL
        S21=DAL2*PSPROF(1,IT,IN,LINE)+DAL1*PSPROF(2,IT,IN,LINE)
      ELSE IF(ALP.GT.PALPHA(PNALPHA(IT,IN,LINE),IT,IN,LINE)) THEN
         S21=0.d0
      ELSE
        ALP=DLOG10(ALP)
        K=2
   6    IF(DLOG10(PALPHA(K,IT,IN,LINE)).LT.ALP.AND.
     *     K.LT.PNALPHA(IT,IN,LINE)) THEN
          K=K+1
          GO TO 6
        END IF
        DAL=DLOG10(PALPHA(K,IT,IN,LINE))-DLOG10(PALPHA(K-1,IT,IN,LINE))
        DAL1=(ALP-DLOG10(PALPHA(K-1,IT,IN,LINE)))/DAL
        DAL2=(DLOG10(PALPHA(K,IT,IN,LINE))-ALP)/DAL
        S21=DAL2*DLOG10(PSPROF(K-1,IT,IN,LINE))+
     +      DAL1*DLOG10(PSPROF(K,  IT,IN,LINE))
        S21=DEXP10(S21)
      END IF
      S21=DLOG10(S21/F0(IN,LINE))
C
      IT=ITEMP-1
      IN=INH
      ALP=DABS(WAVE-WAVE0)/F0(IN,LINE)
      IF(ALP.LT.PALPHA(2,IT,IN,LINE)) THEN
        DAL=PALPHA(2,IT,IN,LINE)-PALPHA(1,IT,IN,LINE)
        DAL1=(ALP-PALPHA(1,IT,IN,LINE))/DAL
        DAL2=(PALPHA(2,IT,IN,LINE)-ALP)/DAL
        S12=DAL2*PSPROF(1,IT,IN,LINE)+DAL1*PSPROF(2,IT,IN,LINE)
      ELSE IF(ALP.GT.PALPHA(PNALPHA(IT,IN,LINE),IT,IN,LINE)) THEN
         S12=0.d0
      ELSE
        ALP=DLOG10(ALP)
        K=2
   7    IF(DLOG10(PALPHA(K,IT,IN,LINE)).LT.ALP.AND.
     *     K.LT.PNALPHA(IT,IN,LINE)) THEN
          K=K+1
          GO TO 7
        END IF
        DAL=DLOG10(PALPHA(K,IT,IN,LINE))-DLOG10(PALPHA(K-1,IT,IN,LINE))
        DAL1=(ALP-DLOG10(PALPHA(K-1,IT,IN,LINE)))/DAL
        DAL2=(DLOG10(PALPHA(K,IT,IN,LINE))-ALP)/DAL
        S12=DAL2*DLOG10(PSPROF(K-1,IT,IN,LINE))+
     +      DAL1*DLOG10(PSPROF(K,  IT,IN,LINE))
        S12=DEXP10(S12)
      END IF
      S12=DLOG10(S12/F0(IN,LINE))
C
      IT=ITEMP
      IN=INH
      ALP=DABS(WAVE-WAVE0)/F0(IN,LINE)
      IF(ALP.LT.PALPHA(2,IT,IN,LINE)) THEN
        DAL=PALPHA(2,IT,IN,LINE)-PALPHA(1,IT,IN,LINE)
        DAL1=(ALP-PALPHA(1,IT,IN,LINE))/DAL
        DAL2=(PALPHA(2,IT,IN,LINE)-ALP)/DAL
        S22=DAL2*PSPROF(1,IT,IN,LINE)+DAL1*PSPROF(2,IT,IN,LINE)
      ELSE IF(ALP.GT.PALPHA(PNALPHA(IT,IN,LINE),IT,IN,LINE)) THEN
         S22=0.d0
      ELSE
        ALP=DLOG10(ALP)
        K=2
   8    IF(DLOG10(PALPHA(K,IT,IN,LINE)).LT.ALP.AND.
     *     K.LT.PNALPHA(IT,IN,LINE)) THEN
          K=K+1
          GO TO 8
        END IF
        DAL=DLOG10(PALPHA(K,IT,IN,LINE))-DLOG10(PALPHA(K-1,IT,IN,LINE))
        DAL1=(ALP-DLOG10(PALPHA(K-1,IT,IN,LINE)))/DAL
        DAL2=(DLOG10(PALPHA(K,IT,IN,LINE))-ALP)/DAL
        S22=DAL2*DLOG10(PSPROF(K-1,IT,IN,LINE))+
     +      DAL1*DLOG10(PSPROF(K,  IT,IN,LINE))
        S22=DEXP10(S22)
      END IF
      S22=DLOG10(S22/F0(IN,LINE))
C
      ELSE
C
C  NOW THE CASE OF NEGATIVE ALPHA
C
      IT=ITEMP-1
      IN=INH-1
      ALP=DABS(WAVE-WAVE0)/F0(IN,LINE)
      IF(ALP.LT.MALPHA(2,IT,IN,LINE)) THEN
        DAL=MALPHA(2,IT,IN,LINE)-MALPHA(1,IT,IN,LINE)
        DAL1=(ALP-MALPHA(1,IT,IN,LINE))/DAL
        DAL2=(MALPHA(2,IT,IN,LINE)-ALP)/DAL
        S11=DAL2*MSPROF(1,IT,IN,LINE)+DAL1*MSPROF(2,IT,IN,LINE)
      ELSE IF(ALP.GT.MALPHA(MNALPHA(IT,IN,LINE),IT,IN,LINE)) THEN
         S11=0.d0
      ELSE
        ALP=DLOG10(ALP)
        K=2
  15    IF(DLOG10(MALPHA(K,IT,IN,LINE)).LT.ALP.AND.
     *     K.LT.MNALPHA(IT,IN,LINE)) THEN
          K=K+1
          GO TO 15
        END IF
        DAL=DLOG10(MALPHA(K,IT,IN,LINE))-DLOG10(MALPHA(K-1,IT,IN,LINE))
        DAL1=(ALP-DLOG10(MALPHA(K-1,IT,IN,LINE)))/DAL
        DAL2=(DLOG10(MALPHA(K,IT,IN,LINE))-ALP)/DAL
        S11=DAL2*DLOG10(MSPROF(K-1,IT,IN,LINE))+
     +      DAL1*DLOG10(MSPROF(K,  IT,IN,LINE))
        S11=DEXP10(S11)
      END IF
      S11=DLOG10(S11/F0(IN,LINE))
C
      IT=ITEMP
      IN=INH-1
      ALP=DABS(WAVE-WAVE0)/F0(IN,LINE)
      IF(ALP.LT.MALPHA(2,IT,IN,LINE)) THEN
        DAL=MALPHA(2,IT,IN,LINE)-MALPHA(1,IT,IN,LINE)
        DAL1=(ALP-MALPHA(1,IT,IN,LINE))/DAL
        DAL2=(MALPHA(2,IT,IN,LINE)-ALP)/DAL
        S21=DAL2*MSPROF(1,IT,IN,LINE)+DAL1*MSPROF(2,IT,IN,LINE)
      ELSE IF(ALP.GT.MALPHA(MNALPHA(IT,IN,LINE),IT,IN,LINE)) THEN
         S21=0.d0
      ELSE
        ALP=DLOG10(ALP)
        K=2
  16    IF(DLOG10(MALPHA(K,IT,IN,LINE)).LT.ALP.AND.
     *     K.LT.MNALPHA(IT,IN,LINE)) THEN
          K=K+1
          GO TO 16
        END IF
        DAL=DLOG10(MALPHA(K,IT,IN,LINE))-DLOG10(MALPHA(K-1,IT,IN,LINE))
        DAL1=(ALP-DLOG10(MALPHA(K-1,IT,IN,LINE)))/DAL
        DAL2=(DLOG10(MALPHA(K,IT,IN,LINE))-ALP)/DAL
        S21=DAL2*DLOG10(MSPROF(K-1,IT,IN,LINE))+
     +      DAL1*DLOG10(MSPROF(K,  IT,IN,LINE))
        S21=DEXP10(S21)
      END IF
      S21=DLOG10(S21/F0(IN,LINE))
C
      IT=ITEMP-1
      IN=INH
      ALP=DABS(WAVE-WAVE0)/F0(IN,LINE)
      IF(ALP.LT.MALPHA(2,IT,IN,LINE)) THEN
        DAL=MALPHA(2,IT,IN,LINE)-MALPHA(1,IT,IN,LINE)
        DAL1=(ALP-MALPHA(1,IT,IN,LINE))/DAL
        DAL2=(MALPHA(2,IT,IN,LINE)-ALP)/DAL
        S12=DAL2*MSPROF(1,IT,IN,LINE)+DAL1*MSPROF(2,IT,IN,LINE)
      ELSE IF(ALP.GT.MALPHA(MNALPHA(IT,IN,LINE),IT,IN,LINE)) THEN
         S12=0.d0
      ELSE
        ALP=DLOG10(ALP)
        K=2
  17    IF(DLOG10(MALPHA(K,IT,IN,LINE)).LT.ALP.AND.
     *     K.LT.MNALPHA(IT,IN,LINE)) THEN
          K=K+1
          GO TO 17
        END IF
        DAL=DLOG10(MALPHA(K,IT,IN,LINE))-DLOG10(MALPHA(K-1,IT,IN,LINE))
        DAL1=(ALP-DLOG10(MALPHA(K-1,IT,IN,LINE)))/DAL
        DAL2=(DLOG10(MALPHA(K,IT,IN,LINE))-ALP)/DAL
        S12=DAL2*DLOG10(MSPROF(K-1,IT,IN,LINE))+
     +      DAL1*DLOG10(MSPROF(K,  IT,IN,LINE))
        S12=DEXP10(S12)
      END IF
      S12=DLOG10(S12/F0(IN,LINE))
C
      IT=ITEMP
      IN=INH
      ALP=DABS(WAVE-WAVE0)/F0(IN,LINE)
      IF(ALP.LT.MALPHA(2,IT,IN,LINE)) THEN
        DAL=MALPHA(2,IT,IN,LINE)-MALPHA(1,IT,IN,LINE)
        DAL1=(ALP-MALPHA(1,IT,IN,LINE))/DAL
        DAL2=(MALPHA(2,IT,IN,LINE)-ALP)/DAL
        S22=DAL2*MSPROF(1,IT,IN,LINE)+DAL1*MSPROF(2,IT,IN,LINE)
      ELSE IF(ALP.GT.MALPHA(MNALPHA(IT,IN,LINE),IT,IN,LINE)) THEN
         S22=0.d0
      ELSE
        ALP=DLOG10(ALP)
        K=2
  18    IF(DLOG10(MALPHA(K,IT,IN,LINE)).LT.ALP.AND.
     *     K.LT.MNALPHA(IT,IN,LINE)) THEN
          K=K+1
          GO TO 18
        END IF
        DAL=DLOG10(MALPHA(K,IT,IN,LINE))-DLOG10(MALPHA(K-1,IT,IN,LINE))
        DAL1=(ALP-DLOG10(MALPHA(K-1,IT,IN,LINE)))/DAL
        DAL2=(DLOG10(MALPHA(K,IT,IN,LINE))-ALP)/DAL
        S22=DAL2*DLOG10(MSPROF(K-1,IT,IN,LINE))+
     +      DAL1*DLOG10(MSPROF(K,  IT,IN,LINE))
        S22=DEXP10(S22)
      END IF
      S22=DLOG10(S22/F0(IN,LINE))
C
      END IF
C
C  Interpolate over T and NH
C
      HLIN=DEXP10(DT2*DNH2*S11+DT1*DNH2*S21+DT2*DNH1*S12+DT1*DNH1*S22)
      RETURN
C
C  I/O error processing
C
   9  I=INDEX(HFILE,' ')-1
      IF(I.LE.0) I=LEN(HFILE)
      WRITE(*,*) 'ERROR reading binary file '//HFILE(1:I)//' in HGRID'
      STOP
  10  I=INDEX(HFILE,' ')-1
      IF(I.LE.0) I=LEN(HFILE)
      WRITE(*,*) 'EOF found while reading binary file '//HFILE(1:I)//
     *           ' in HGRID'
      STOP
      END


      SUBROUTINE HSELF_PDWIDTH(LINE,TEMP,H1FRC,WIDTH)
C
C   Computes the Lorentzian WIDTH self broadening 
C   of the Hydrogen Balmer Lines in the p-d approximation
C   Output in usual rad /s cm^3
C   
C   Here LINE = 1,2,3... for Halpha, Hbeta, Hgamma ...
C
      IMPLICIT NONE
      REAL*4 H1FRC
      REAL*4 C,PI,KB,M0,A0,SIGMAH(4),ALPHAH(4),
     *       X,GAMMAF,GX,VBAR,SIGMA,TEMP
      REAL*8 GVW, WIDTH
      INTEGER LINE
C
      PARAMETER (C=2.997925E+18,PI=3.14159265)
      PARAMETER (KB=1.380658E-23,M0=1.660540E-27)
      PARAMETER (A0=5.29177249E-11)  
      DATA SIGMAH/ 1180., 2320., 4208., 0.0/
      DATA ALPHAH/ 0.677, 0.455, 0.380, 0.0/
C  
      X = 2. - ALPHAH(LINE) * .5
      GX = X - 1.0
      GAMMAF = 1+(-.5748646+(.9512363+(-.6998588+(.4245549-.1010678*GX
     ;           )*GX)*GX)*GX)*GX
      SIGMA = SIGMAH(LINE) * A0 * A0
      GVW = (4./PI)**(ALPHAH(LINE)*0.5)*GAMMAF*1.E4*SIGMA
      VBAR = SQRT(8.*KB*TEMP/PI/M0 * 2./1.008)
      GVW = GVW * ((VBAR/1.E4)**(1.-ALPHAH(LINE)))
      GVW = GVW * H1FRC * 1.E6
C 
      WIDTH=GVW
      RETURN
C
      END

      SUBROUTINE HE_PDWIDTH(LINE,TEMP,HE1FRC,HEWID)
C 
C  Compute approximate broadening by He collisions (p-d approx)
C
C  Output in usual rad /s cm^3
C   
C  27/4/00 PB
C
      IMPLICIT NONE
      REAL*4 PI,C,HE1FRC,GAMMAF
      REAL*4 SIGMAH(4),ALPHAH(4),X,GX,A0,SIGMA,VBAR,KB,M0,TEMP
      REAL*4 HEWID,GVW
      INTEGER LINE
      PARAMETER (C=2.997925E+18,PI=3.14159265,A0=5.29177249E-11)    
      PARAMETER (KB=1.380658E-23,M0=1.660540E-27)
      DATA SIGMAH/ 834., 1998., 3837., 5700. /
      DATA ALPHAH/ .280, .330, .260, .250 /
C
      X = 2. - ALPHAH(LINE) * .5
      GX = X - 1.0
      GAMMAF = 1+(-.5748646+(.9512363+(-.6998588+(.4245549-.1010678*GX
     ;           )*GX)*GX)*GX)*GX
      SIGMA = SIGMAH(LINE) * A0 * A0
      GVW = (4./PI)**(ALPHAH(LINE)*0.5)*GAMMAF*1.E4*SIGMA
      VBAR = SQRT(8.*KB*TEMP/PI/M0 * (1./1.008 + 1./4.003))
      HEWID = GVW * ((VBAR/1.E4)**(1.-ALPHAH(LINE))) 
      HEWID = HEWID * 0.625            !polarisability difference term
      HEWID = HEWID * HE1FRC * 1.E6    !in cgs
C
      RETURN
      END

      SUBROUTINE RAD_WIDTH(LINE,WIDTH)
C
C  Returns the radiation damping width (hwhm) in rad/s cm^3
C
C  27/4/00 PB
C
      INTEGER LINE
      REAL*4 WIDTH,GAMRAD(4)
      DATA GAMRAD/ 5.698, 5.001, 4.815, 4.740 / 
C
      WIDTH = GAMRAD(LINE) * 1.e8 / 2.
C
      RETURN
      END


      SUBROUTINE ALI_GRIEM(LINE,WAVE0,H1FRC,WIDTH)
C
C   Computes the Resonance Width (AA) of the Hydrogen Balmer Lines
C   Halfwidth given H1FRC perturbers per cm^3 
C
C   NB: assumes 2p level only! more correct than adding the 3p or 4p etc.
C   since 2p-nd dominates
C   so LINE is not used actually
C
C  GAMMA = lambda^2/(2 pi c) * 1.15 * e^2/(mc) f12 lambda12 N(H)   fuhrmann
C  GAMMA = lambda^2/(2 pi c) * 1.108 * e^2/(mc) f12 lambda12 N(H)   ali and griem
C
      IMPLICIT NONE
      REAL*4 H1FRC
      REAL*8 WAVE0,WIDTH,FW
      INTEGER LINE
C
C     WAVE0 in AA, WIDTH in AA  
C
C   the two different leading numbers makes only a minute difference
C     FW=2.6037D-27*WAVE0*WAVE0   !balmer lower level (fuhrmann 2 pi)
C
      FW=2.516D-27*WAVE0*WAVE0   !balmer lower level (ali and griem 1.92 pi)
      FW=FW*H1FRC
C      
      WIDTH=FW/2.d0
C
      RETURN
C
      END

C
C  THE FOLLOWING IS KURUCZ'S SUBROUTINE
C
C  SLIGHTLY ALTERED BY PB & REGNER TRAMPEDACH
C
      SUBROUTINE VCS(PR,XNE,T,DEL,II,N,M)
C     CALCULATES VIDAL COOPER AND SMITH PROFILES FOR FIRST FOUR BALMER 
C     LINES.  THEY ARE RETURNED IN ANGSTROM UNITS.                     
C     ASSUMES RAW PROFILES ARE IN ALPHA UNITS.                      
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION PR(40),DEL(40)
C     DIMENSION ALPHA(40),PRALPH(40)
      DIMENSION PRALPH(40)
      DIMENSION SVCS(6,17,40,4),ALPHA0(4)
      DATA ((SVCS(I,J, 1,1),J=1,17),I=1,6)/
     & -1.6331, -1.2998,  -.9664,  -.6331,  -.3022,   .0257,   .3454,
     &   .6432,   .9052,  1.1242,  1.2920,  1.3856,  1.4101,  1.3851,
     &  1.3699,  1.3350,  1.3368, -1.7836, -1.4503, -1.1170,  -.7836,
     &  -.4517,  -.1215,   .2028,   .5119,   .7899,  1.0257,  1.2180,
     &  1.3456,  1.3899,  1.3851,  1.3699,  1.3350,  1.3368, -1.9341,
     & -1.6008, -1.2675,  -.9341,  -.6008,  -.2704,   .0573,   .3746,
     &   .6680,   .9217,  1.1332,  1.2936,  1.3738,  1.3793,  1.3526,
     &  1.3350,  1.3368, -2.0846, -1.7513, -1.4180, -1.0846,  -.7513,
     &  -.4196,  -.0899,   .2335,   .5400,   .8112,  1.0402,  1.2191,
     &  1.3489,  1.3817,  1.3565,  1.3273,  1.3043, -2.2352, -1.9018,
     & -1.5685, -1.2352,  -.9018,  -.5685,  -.2383,   .0888,   .4045,
     &   .6939,   .9402,  1.1477,  1.3058,  1.3811,  1.3708,  1.3389,
     &  1.3052, -2.3857, -2.0523, -1.7190, -1.3857, -1.0523,  -.7190,
     &  -.3857,  -.0523,   .2810,   .6143,   .9402,  1.1477,  1.3058,
     &  1.3811,  1.3708,  1.3389,  1.3052/
      DATA ((SVCS(I,J, 2,1),J=1,17),I=1,6)/
     & -1.6331, -1.2998,  -.9664,  -.6331,  -.3022,   .0257,   .3454,
     &   .6432,   .9050,  1.1235,  1.2907,  1.3804,  1.4029,  1.3784,
     &  1.3692,  1.3345,  1.3363, -1.7836, -1.4503, -1.1170,  -.7836,
     &  -.4517,  -.1215,   .2028,   .5119,   .7898,  1.0253,  1.2167,
     &  1.3418,  1.3835,  1.3784,  1.3692,  1.3345,  1.3363, -1.9341,
     & -1.6008, -1.2675,  -.9341,  -.6008,  -.2704,   .0573,   .3746,
     &   .6679,   .9215,  1.1323,  1.2920,  1.3683,  1.3725,  1.3520,
     &  1.3345,  1.3363, -2.0846, -1.7513, -1.4180, -1.0846,  -.7513,
     &  -.4196,  -.0899,   .2335,   .5400,   .8111,  1.0398,  1.2225,
     &  1.3446,  1.3749,  1.3558,  1.3267,  1.3038, -2.2352, -1.9018,
     & -1.5685, -1.2352,  -.9018,  -.5685,  -.2383,   .0888,   .4045,
     &   .6938,   .9399,  1.1467,  1.3041,  1.3747,  1.3700,  1.3383,
     &  1.3047, -2.3857, -2.0523, -1.7190, -1.3857, -1.0523,  -.7190,
     &  -.3857,  -.0523,   .2810,   .6143,   .9399,  1.1467,  1.3041,
     &  1.3747,  1.3700,  1.3383,  1.3047/
      DATA ((SVCS(I,J, 3,1),J=1,17),I=1,6)/
     & -1.6331, -1.2998,  -.9664,  -.6331,  -.3022,   .0257,   .3454,
     &   .6430,   .9046,  1.1219,  1.2862,  1.3677,  1.3858,  1.3621,
     &  1.3541,  1.3218,  1.3243, -1.7836, -1.4503, -1.1170,  -.7836,
     &  -.4517,  -.1215,   .2028,   .5119,   .7895,  1.0245,  1.2133,
     &  1.3322,  1.3679,  1.3621,  1.3541,  1.3218,  1.3243, -1.9341,
     & -1.6008, -1.2675,  -.9341,  -.6008,  -.2704,   .0573,   .3746,
     &   .6678,   .9210,  1.1303,  1.2865,  1.3549,  1.3562,  1.3375,
     &  1.3218,  1.3243, -2.0846, -1.7513, -1.4180, -1.0846,  -.7513,
     &  -.4196,  -.0899,   .2335,   .5400,   .8108,  1.0386,  1.2281,
     &  1.3341,  1.3585,  1.3400,  1.3141,  1.2934, -2.2352, -1.9018,
     & -1.5685, -1.2352,  -.9018,  -.5685,  -.2383,   .0888,   .4045,
     &   .6937,   .9394,  1.1446,  1.2975,  1.3593,  1.3524,  1.3237,
     &  1.2934, -2.3857, -2.0523, -1.7190, -1.3857, -1.0523,  -.7190,
     &  -.3857,  -.0524,   .2810,   .6142,   .9394,  1.1446,  1.2975,
     &  1.3593,  1.3524,  1.3237,  1.2934/
      DATA ((SVCS(I,J, 4,1),J=1,17),I=1,6)/
     & -1.6331, -1.2998,  -.9664,  -.6331,  -.3022,   .0257,   .3454,
     &   .6428,   .9035,  1.1176,  1.2716,  1.3369,  1.3456,  1.3245,
     &  1.3193,  1.2923,  1.2956, -1.7836, -1.4503, -1.1170,  -.7836,
     &  -.4517,  -.1215,   .2028,   .5116,   .7889,  1.0220,  1.2049,
     &  1.3090,  1.3314,  1.3245,  1.3193,  1.2923,  1.2956, -1.9341,
     & -1.6008, -1.2675,  -.9341,  -.6008,  -.2704,   .0573,   .3746,
     &   .6675,   .9198,  1.1258,  1.2697,  1.3224,  1.3183,  1.3036,
     &  1.2923,  1.2956, -2.0846, -1.7513, -1.4180, -1.0846,  -.7513,
     &  -.4196,  -.0899,   .2335,   .5397,   .8102,  1.0362,  1.2248,
     &  1.3077,  1.3199,  1.3038,  1.2840,  1.2679, -2.2352, -1.9018,
     & -1.5685, -1.2352,  -.9018,  -.5685,  -.2383,   .0888,   .4043,
     &   .6934,   .9380,  1.1389,  1.2774,  1.3224,  1.3118,  1.2898,
     &  1.2662, -2.3857, -2.0523, -1.7190, -1.3857, -1.0523,  -.7190,
     &  -.3857,  -.0524,   .2809,   .6140,   .9380,  1.1389,  1.2774,
     &  1.3224,  1.3118,  1.2898,  1.2662/
      DATA ((SVCS(I,J, 5,1),J=1,17),I=1,6)/
     & -1.6331, -1.2998,  -.9664,  -.6331,  -.3022,   .0257,   .3452,
     &   .6421,   .9007,  1.1072,  1.2353,  1.2669,  1.2620,  1.2465,
     &  1.2467,  1.2292,  1.2338, -1.7836, -1.4503, -1.1170,  -.7836,
     &  -.4517,  -.1215,   .2025,   .5113,   .7875,  1.0162,  1.1838,
     &  1.2543,  1.2524,  1.2465,  1.2467,  1.2292,  1.2338, -1.9341,
     & -1.6008, -1.2675,  -.9341,  -.6008,  -.2704,   .0573,   .3744,
     &   .6667,   .9167,  1.1136,  1.2304,  1.2494,  1.2390,  1.2325,
     &  1.2292,  1.2338, -2.0846, -1.7513, -1.4180, -1.0846,  -.7513,
     &  -.4196,  -.0899,   .2335,   .5393,   .8085,  1.0294,  1.1830,
     &  1.2467,  1.2380,  1.2281,  1.2196,  1.2125, -2.2352, -1.9018,
     & -1.5685, -1.2352,  -.9018,  -.5685,  -.2384,   .0888,   .4041,
     &   .6924,   .9344,  1.1252,  1.2333,  1.2413,  1.2284,  1.2191,
     &  1.2076, -2.3857, -2.0523, -1.7190, -1.3857, -1.0523,  -.7190,
     &  -.3857,  -.0524,   .2808,   .6134,   .9344,  1.1252,  1.2333,
     &  1.2413,  1.2284,  1.2191,  1.2076/
      DATA ((SVCS(I,J, 6,1),J=1,17),I=1,6)/
     & -1.6331, -1.2998,  -.9664,  -.6331,  -.3022,   .0257,   .3446,
     &   .6403,   .8938,  1.0810,  1.1569,  1.1297,  1.1173,  1.1109,
     &  1.1183,  1.1129,  1.1193, -1.7836, -1.4503, -1.1170,  -.7836,
     &  -.4517,  -.1216,   .2025,   .5104,   .7838,  1.0022,  1.1332,
     &  1.1367,  1.1099,  1.1109,  1.1183,  1.1129,  1.1193, -1.9341,
     & -1.6008, -1.2675,  -.9341,  -.6008,  -.2705,   .0573,   .3738,
     &   .6647,   .9090,  1.0842,  1.1427,  1.1096,  1.1007,  1.1065,
     &  1.1129,  1.1193, -2.0846, -1.7513, -1.4180, -1.0846,  -.7513,
     &  -.4196,  -.0900,   .2333,   .5383,   .8043,  1.0133,  1.1345,
     &  1.1196,  1.0934,  1.0962,  1.1024,  1.1069, -2.2352, -1.9018,
     & -1.5685, -1.2352,  -.9018,  -.5685,  -.2384,   .0888,   .4036,
     &   .6902,   .9257,  1.0917,  1.1326,  1.0920,  1.0864,  1.0934,
     &  1.0983, -2.3857, -2.0523, -1.7190, -1.3857, -1.0523,  -.7190,
     &  -.3857,  -.0525,   .2805,   .6120,   .9257,  1.0917,  1.1326,
     &  1.0920,  1.0864,  1.0934,  1.0983/
      DATA ((SVCS(I,J, 7,1),J=1,17),I=1,6)/
     & -1.6331, -1.2998,  -.9664,  -.6331,  -.3023,   .0253,   .3436,
     &   .6359,   .8766,  1.0179,   .9914,   .9261,   .9226,   .9242,
     &   .9353,   .9397,   .9463, -1.7836, -1.4503, -1.1170,  -.7836,
     &  -.4519,  -.1217,   .2019,   .5081,   .7744,   .9667,  1.0175,
     &   .9372,   .9151,   .9242,   .9353,   .9397,   .9463, -1.9341,
     & -1.6008, -1.2675,  -.9341,  -.6008,  -.2705,   .0569,   .3725,
     &   .6598,   .8896,  1.0137,   .9674,   .9098,   .9128,   .9270,
     &   .9397,   .9463, -2.0846, -1.7513, -1.4180, -1.0847,  -.7513,
     &  -.4198,  -.0901,   .2325,   .5357,   .7938,   .9734,   .9959,
     &   .9160,   .9007,   .9137,   .9296,   .9428, -2.2352, -1.9018,
     & -1.5685, -1.2352,  -.9018,  -.5685,  -.2384,   .0885,   .4023,
     &   .6845,   .9038,  1.0111,   .9445,   .8922,   .8985,   .9153,
     &   .9317, -2.3857, -2.0523, -1.7190, -1.3857, -1.0523,  -.7190,
     &  -.3857,  -.0526,   .2797,   .6085,   .9038,  1.0111,   .9445,
     &   .8922,   .8985,   .9153,   .9317/
      DATA ((SVCS(I,J, 8,1),J=1,17),I=1,6)/
     & -1.6331, -1.2998,  -.9664,  -.6332,  -.3024,   .0249,   .3410,
     &   .6249,   .8337,   .8734,   .7408,   .7038,   .7034,   .7106,
     &   .7169,   .7269,   .7297, -1.7836, -1.4503, -1.1170,  -.7836,
     &  -.4519,  -.1220,   .2006,   .5023,   .7512,   .8810,   .7963,
     &   .7087,   .7036,   .7106,   .7169,   .7269,   .7297, -1.9341,
     & -1.6008, -1.2675,  -.9341,  -.6009,  -.2707,   .0565,   .3696,
     &   .6473,   .8418,   .8565,   .7251,   .6995,   .7058,   .7171,
     &   .7269,   .7297, -2.0846, -1.7513, -1.4180, -1.0847,  -.7513,
     &  -.4198,  -.0904,   .2310,   .5290,   .7676,   .8780,   .7728,
     &   .6982,   .6972,   .7087,   .7226,   .7349, -2.2352, -1.9018,
     & -1.5685, -1.2352,  -.9018,  -.5686,  -.2386,   .0878,   .3988,
     &   .6703,   .8498,   .8392,   .7088,   .6897,   .6974,   .7116,
     &   .7272, -2.3857, -2.0523, -1.7190, -1.3857, -1.0524,  -.7190,
     &  -.3858,  -.0530,   .2778,   .5997,   .8498,   .8392,   .7088,
     &   .6897,   .6974,   .7116,   .7272/
      DATA ((SVCS(I,J, 9,1),J=1,17),I=1,6)/
     & -1.6331, -1.2998,  -.9665,  -.6332,  -.3028,   .0233,   .3345,
     &   .5971,   .7289,   .5966,   .4799,   .4603,   .4515,   .4679,
     &   .4634,   .4810,   .4748, -1.7836, -1.4503, -1.1170,  -.7837,
     &  -.4520,  -.1227,   .1970,   .4877,   .6933,   .6881,   .5126,
     &   .4777,   .4709,   .4679,   .4634,   .4810,   .4748, -1.9341,
     & -1.6008, -1.2675,  -.9342,  -.6009,  -.2711,   .0546,   .3621,
     &   .6160,   .7262,   .5790,   .4912,   .4812,   .4802,   .4812,
     &   .4810,   .4748, -2.0846, -1.7513, -1.4180, -1.0847,  -.7514,
     &  -.4200,  -.0913,   .2271,   .5124,   .7027,   .6706,   .5101,
     &   .4874,   .4856,   .4878,   .4916,   .4950, -2.2352, -1.9018,
     & -1.5685, -1.2352,  -.9019,  -.5687,  -.2391,   .0856,   .3901,
     &   .6349,   .7213,   .5610,   .4936,   .4878,   .4897,   .4942,
     &   .5001, -2.3857, -2.0523, -1.7190, -1.3857, -1.0524,  -.7191,
     &  -.3860,  -.0540,   .2731,   .5777,   .7213,   .5610,   .4936,
     &   .4878,   .4897,   .4942,   .5001/
      DATA ((SVCS(I,J,10,1),J=1,17),I=1,6)/
     & -1.6331, -1.2998,  -.9665,  -.6334,  -.3036,   .0195,   .3179,
     &   .5281,   .4866,   .2274,   .1647,   .1443,   .1313,   .1578,
     &   .1474,   .1746,   .1608, -1.7836, -1.4503, -1.1170,  -.7838,
     &  -.4525,  -.1246,   .1886,   .4512,   .5517,   .3316,   .2028,
     &   .1761,   .1652,   .1578,   .1474,   .1746,   .1608, -1.9341,
     & -1.6008, -1.2675,  -.9342,  -.6011,  -.2720,   .0504,   .3430,
     &   .5381,   .4667,   .2443,   .2028,   .1898,   .1841,   .1801,
     &   .1746,   .1608, -2.0846, -1.7513, -1.4180, -1.0847,  -.7515,
     &  -.4204,  -.0935,   .2172,   .4707,   .5453,   .3204,   .2266,
     &   .2095,   .2033,   .2006,   .1995,   .1973, -2.2352, -1.9018,
     & -1.5685, -1.2352,  -.9019,  -.5689,  -.2402,   .0806,   .3681,
     &   .5468,   .4426,   .2550,   .2256,   .2177,   .2151,   .2148,
     &   .2156, -2.3857, -2.0523, -1.7190, -1.3857, -1.0524,  -.7192,
     &  -.3866,  -.0566,   .2612,   .5224,   .4426,   .2550,   .2256,
     &   .2177,   .2151,   .2148,   .2156/
      DATA ((SVCS(I,J,11,1),J=1,17),I=1,6)/
     & -1.6331, -1.2998,  -.9666,  -.6338,  -.3057,   .0103,   .2762,
     &   .3577,   .0116,  -.2101,  -.2469,  -.2576,  -.2631,  -.2340,
     &  -.2397,  -.2087,  -.2214, -1.7836, -1.4503, -1.1170,  -.7840,
     &  -.4535,  -.1294,   .1673,   .3600,   .2227,  -.1399,  -.2147,
     &  -.2297,  -.2330,  -.2340,  -.2397,  -.2087,  -.2214, -1.9341,
     & -1.6008, -1.2675,  -.9343,  -.6017,  -.2744,   .0394,   .2953,
     &   .3465,  -.0067,  -.1801,  -.2061,  -.2118,  -.2098,  -.2073,
     &  -.2087,  -.2214, -2.0846, -1.7513, -1.4180, -1.0847,  -.7517,
     &  -.4216,  -.0990,   .1928,   .3666,   .1895,  -.1223,  -.1874,
     &  -.1946,  -.1942,  -.1888,  -.1836,  -.1817, -2.2352, -1.9018,
     & -1.5685, -1.2352,  -.9020,  -.5695,  -.2429,   .0682,   .3132,
     &   .3320,  -.0272,  -.1575,  -.1826,  -.1827,  -.1780,  -.1704,
     &  -.1629, -2.3857, -2.0523, -1.7190, -1.3857, -1.0525,  -.7195,
     &  -.3880,  -.0631,   .2312,   .3833,  -.0272,  -.1575,  -.1826,
     &  -.1827,  -.1780,  -.1704,  -.1629/
      DATA ((SVCS(I,J,12,1),J=1,17),I=1,6)/
     & -1.6331, -1.2999,  -.9668,  -.6350,  -.3108,  -.0134,   .1723,
     &  -.0484,  -.6133,  -.7097,  -.7169,  -.7151,  -.7080,  -.6792,
     &  -.6774,  -.6474,  -.6550, -1.7836, -1.4503, -1.1172,  -.7845,
     &  -.4561,  -.1412,   .1139,   .1339,  -.4113,  -.6780,  -.7044,
     &  -.6994,  -.6908,  -.6792,  -.6774,  -.6474,  -.6550, -1.9341,
     & -1.6008, -1.2676,  -.9346,  -.6030,  -.2803,   .0124,   .1758,
     &  -.1009,  -.6101,  -.6912,  -.6938,  -.6861,  -.6672,  -.6533,
     &  -.6474,  -.6550, -2.0847, -1.7513, -1.4180, -1.0849,  -.7524,
     &  -.4246,  -.1127,   .1310,   .1099,  -.4511,  -.6680,  -.6908,
     &  -.6817,  -.6680,  -.6467,  -.6313,  -.6223, -2.2352, -1.9018,
     & -1.5685, -1.2353,  -.9024,  -.5710,  -.2498,   .0366,   .1764,
     &  -.1580,  -.6153,  -.6857,  -.6844,  -.6698,  -.6513,  -.6295,
     &  -.6126, -2.3857, -2.0523, -1.7190, -1.3857, -1.0526,  -.7203,
     &  -.3915,  -.0793,   .1560,   .1036,  -.6153,  -.6857,  -.6844,
     &  -.6698,  -.6513,  -.6295,  -.6126/
      DATA ((SVCS(I,J,13,1),J=1,17),I=1,6)/
     & -1.6332, -1.3000,  -.9674,  -.6378,  -.3238,  -.0724,  -.0868,
     &  -.8589, -1.1952, -1.2115, -1.1980, -1.1797, -1.1654, -1.1434,
     & -1.1351, -1.1120, -1.1149, -1.7836, -1.4504, -1.1175,  -.7860,
     &  -.4626,  -.1710,  -.0201,  -.4066, -1.1383, -1.2099, -1.2026,
     & -1.1829, -1.1606, -1.1434, -1.1351, -1.1120, -1.1149, -1.9341,
     & -1.6009, -1.2677,  -.9353,  -.6062,  -.2953,  -.0560,  -.1206,
     &  -.9314, -1.1987, -1.2101, -1.1917, -1.1667, -1.1429, -1.1231,
     & -1.1120, -1.1149, -2.0847, -1.7513, -1.4181, -1.0852,  -.7540,
     &  -.4322,  -.1472,  -.0233,  -.4919, -1.1559, -1.2149, -1.2043,
     & -1.1807, -1.1533, -1.1270, -1.1056, -1.0925, -2.2352, -1.9018,
     & -1.5686, -1.2355,  -.9032,  -.5748,  -.2672,  -.0422,  -.1626,
     &  -.9961, -1.2093, -1.2175, -1.1976, -1.1711, -1.1411, -1.1133,
     & -1.0937, -2.3857, -2.0524, -1.7190, -1.3858, -1.0530,  -.7222,
     &  -.4003,  -.1200,  -.0330,  -.7060, -1.2093, -1.2175, -1.1976,
     & -1.1711, -1.1411, -1.1133, -1.0937/
      DATA ((SVCS(I,J,14,1),J=1,17),I=1,6)/
     & -1.6332, -1.3003,  -.9690,  -.6448,  -.3563,  -.2204,  -.7158,
     & -1.6609, -1.7151, -1.7009, -1.6761, -1.6517, -1.6336, -1.6158,
     & -1.6057, -1.5889, -1.5889, -1.7837, -1.4506, -1.1182,  -.7895,
     &  -.4789,  -.2458,  -.3542, -1.4291, -1.7129, -1.7144, -1.6906,
     & -1.6621, -1.6358, -1.6158, -1.6057, -1.5889, -1.5889, -1.9342,
     & -1.6009, -1.2681,  -.9371,  -.6144,  -.3330,  -.2272,  -.8303,
     & -1.6842, -1.7258, -1.7086, -1.6799, -1.6490, -1.6212, -1.6003,
     & -1.5889, -1.5889, -2.0847, -1.7514, -1.4183, -1.0861,  -.7581,
     &  -.4510,  -.2339,  -.4075, -1.5206, -1.7286, -1.7268, -1.7014,
     & -1.6696, -1.6370, -1.6085, -1.5877, -1.5754, -2.2352, -1.9019,
     & -1.5687, -1.2359,  -.9052,  -.5843,  -.3108,  -.2400,  -.9578,
     & -1.7080, -1.7414, -1.7235, -1.6940, -1.6603, -1.6267, -1.5977,
     & -1.5763, -2.3857, -2.0524, -1.7191, -1.3860, -1.0540,  -.7269,
     &  -.4223,  -.2223,  -.4866, -1.7709, -1.7414, -1.7235, -1.6940,
     & -1.6603, -1.6267, -1.5977, -1.5763/
      DATA ((SVCS(I,J,15,1),J=1,17),I=1,6)/
     & -1.6334, -1.3011,  -.9728,  -.6625,  -.4378,  -.5906, -1.9278,
     & -2.2113, -2.2050, -2.1778, -2.1482, -2.1230, -2.1050, -2.0925,
     & -2.0835, -2.0734, -2.0722, -1.7838, -1.4510, -1.1201,  -.7983,
     &  -.5197,  -.4336, -1.1658, -2.1940, -2.2215, -2.1990, -2.1673,
     & -2.1360, -2.1103, -2.0925, -2.0835, -2.0734, -2.0722, -1.9342,
     & -1.6011, -1.2691,  -.9415,  -.6349,  -.4276,  -.6546, -2.0414,
     & -2.2306, -2.2213, -2.1914, -2.1572, -2.1249, -2.0990, -2.0818,
     & -2.0734, -2.0722, -2.0847, -1.7515, -1.4188, -1.0883,  -.7684,
     &  -.4985,  -.4512, -1.3260, -2.2196, -2.2403, -2.2172, -2.1836,
     & -2.1478, -2.1152, -2.0895, -2.0729, -2.0652, -2.2352, -1.9019,
     & -1.5689, -1.2370,  -.9104,  -.6081,  -.4204,  -.7328, -2.1237,
     & -2.2510, -2.2413, -2.2118, -2.1761, -2.1395, -2.1070, -2.0818,
     & -2.0662, -2.3857, -2.0524, -1.7192, -1.3866, -1.0566,  -.7388,
     &  -.4776,  -.4792, -1.6060, -2.2971, -2.2413, -2.2118, -2.1761,
     & -2.1395, -2.1070, -2.0818, -2.0662/
      DATA ((SVCS(I,J,16,1),J=1,17),I=1,6)/
     & -1.6338, -1.3032,  -.9824,  -.7070,  -.6424, -1.4991, -2.6916,
     & -2.7091, -2.6824, -2.6490, -2.6182, -2.5943, -2.5791, -2.5727,
     & -2.5662, -2.5627, -2.5619, -1.7840, -1.4520, -1.1249,  -.8206,
     &  -.6225,  -.9034, -2.5550, -2.7219, -2.7069, -2.6749, -2.6400,
     & -2.6094, -2.5867, -2.5727, -2.5662, -2.5627, -2.5619, -1.9343,
     & -1.6017, -1.2714,  -.9526,  -.6866,  -.6647, -1.6904, -2.7185,
     & -2.7296, -2.7018, -2.6659, -2.6313, -2.6014, -2.5802, -2.5680,
     & -2.5627, -2.5619, -2.0847, -1.7517, -1.4200, -1.0939,  -.7942,
     &  -.6176,  -.9944, -2.6415, -2.7445, -2.7286, -2.6955, -2.6580,
     & -2.6223, -2.5945, -2.5750, -2.5648, -2.5608, -2.2352, -1.9020,
     & -1.5695, -1.2398,  -.9233,  -.6680,  -.6955, -1.9027, -2.7440,
     & -2.7515, -2.7251, -2.6891, -2.6509, -2.6155, -2.5887, -2.5711,
     & -2.5626, -2.3857, -2.0525, -1.7195, -1.3880, -1.0631,  -.7688,
     &  -.6167, -1.1160, -2.8056, -2.7876, -2.7251, -2.6891, -2.6509,
     & -2.6155, -2.5887, -2.5711, -2.5626/
      DATA ((SVCS(I,J,17,1),J=1,17),I=1,6)/
     & -1.6350, -1.3084, -1.0064,  -.8188, -1.1556, -3.0630, -3.2092,
     & -3.1898, -3.1558, -3.1226, -3.0962, -3.0788, -3.0698, -3.0672,
     & -3.0650, -3.0649, -3.0653, -1.7845, -1.4546, -1.1370,  -.8764,
     &  -.8804, -2.0537, -3.2115, -3.2146, -3.1840, -3.1477, -3.1146,
     & -3.0896, -3.0743, -3.0672, -3.0650, -3.0649, -3.0653, -1.9346,
     & -1.6030, -1.2775,  -.9805,  -.8162, -1.2588, -3.1513, -3.2333,
     & -3.2125, -3.1769, -3.1396, -3.1072, -3.0840, -3.0709, -3.0657,
     & -3.0649, -3.0653, -2.0849, -1.7524, -1.4230, -1.1079,  -.8590,
     &  -.9169, -2.3021, -3.2380, -3.2382, -3.2077, -3.1696, -3.1321,
     & -3.1008, -3.0795, -3.0685, -3.0651, -3.0655, -2.2353, -1.9024,
     & -1.5710, -1.2468,  -.9557,  -.8185, -1.3830, -3.2034, -3.2571,
     & -3.2371, -3.2020, -3.1627, -3.1254, -3.0954, -3.0760, -3.0669,
     & -3.0650, -2.3857, -2.0526, -1.7203, -1.3915, -1.0793,  -.8440,
     &  -.9640, -2.6436, -3.3007, -3.2717, -3.2020, -3.1627, -3.1254,
     & -3.0954, -3.0760, -3.0669, -3.0650/
      DATA ((SVCS(I,J,18,1),J=1,17),I=1,6)/
     & -1.6378, -1.3214, -1.0669, -1.0986, -2.4203, -3.6990, -3.6951,
     & -3.6625, -3.6275, -3.5988, -3.5797, -3.5702, -3.5674, -3.5674,
     & -3.5685, -3.5699, -3.5711, -1.7860, -1.4611, -1.1672, -1.0168,
     & -1.5272, -3.6540, -3.7169, -3.6919, -3.6552, -3.6200, -3.5928,
     & -3.5759, -3.5686, -3.5674, -3.5685, -3.5699, -3.5711, -1.9353,
     & -1.6062, -1.2926, -1.0507, -1.1410, -2.7033, -3.7265, -3.7196,
     & -3.6859, -3.6474, -3.6130, -3.5874, -3.5730, -3.5678, -3.5680,
     & -3.5699, -3.5711, -2.0852, -1.7540, -1.4305, -1.1429, -1.0219,
     & -1.6661, -3.7031, -3.7421, -3.7169, -3.6792, -3.6402, -3.6066,
     & -3.5830, -3.5709, -3.5677, -3.5690, -3.5715, -2.2355, -1.9032,
     & -1.5748, -1.2643, -1.0371, -1.1955, -3.0084, -3.7528, -3.7450,
     & -3.7124, -3.6728, -3.6338, -3.6012, -3.5795, -3.5694, -3.5677,
     & -3.5699, -2.3858, -2.0530, -1.7222, -1.4003, -1.1200, -1.0330,
     & -1.8386, -3.8097, -3.7821, -3.7471, -3.6728, -3.6338, -3.6012,
     & -3.5795, -3.5694, -3.5677, -3.5699/
      DATA ((SVCS(I,J,19,1),J=1,17),I=1,6)/
     & -1.6448, -1.3541, -1.2188, -1.8031, -4.1433, -4.1969, -4.1690,
     & -4.1332, -4.1022, -4.0813, -4.0710, -4.0685, -4.0698, -4.0712,
     & -4.0737, -4.0757, -4.0812, -1.7895, -1.4775, -1.2431, -1.3688,
     & -3.1110, -4.2108, -4.1982, -4.1628, -4.1260, -4.0963, -4.0776,
     & -4.0697, -4.0689, -4.0712, -4.0737, -4.0757, -4.0812, -1.9371,
     & -1.6144, -1.3306, -1.2270, -1.9582, -4.1969, -4.2233, -4.1944,
     & -4.1557, -4.1190, -4.0910, -4.0748, -4.0691, -4.0698, -4.0728,
     & -4.0757, -4.0812, -2.0861, -1.7581, -1.4495, -1.2311, -1.4304,
     & -3.4566, -4.2383, -4.2246, -4.1888, -4.1487, -4.1127, -4.0867,
     & -4.0727, -4.0689, -4.0707, -4.0743, -4.0774, -2.2359, -1.9052,
     & -1.5843, -1.3084, -1.2417, -2.1430, -4.2305, -4.2496, -4.2218,
     & -4.1828, -4.1422, -4.1072, -4.0831, -4.0712, -4.0691, -4.0718,
     & -4.0756, -2.3860, -2.0540, -1.7269, -1.4223, -1.2223, -1.5071,
     & -3.8603, -4.2920, -4.2584, -4.2174, -4.1422, -4.1072, -4.0831,
     & -4.0712, -4.0691, -4.0718, -4.0756/
      DATA ((SVCS(I,J,20,1),J=1,17),I=1,6)/
     & -1.6625, -1.4364, -1.6001, -3.5472, -4.6834, -4.6751, -4.6398,
     & -4.6068, -4.5835, -4.5719, -4.5693, -4.5714, -4.5746, -4.5764,
     & -4.5825, -4.5836, -4.5864, -1.7983, -1.5186, -1.4339, -2.2536,
     & -4.6794, -4.7025, -4.6708, -4.6330, -4.6008, -4.5799, -4.5706,
     & -4.5698, -4.5728, -4.5764, -4.5825, -4.5836, -4.5864, -1.9415,
     & -1.6349, -1.4260, -1.6695, -3.9461, -4.7212, -4.7022, -4.6645,
     & -4.6262, -4.5955, -4.5769, -4.5699, -4.5706, -4.5743, -4.5781,
     & -4.5836, -4.5864, -2.0883, -1.7684, -1.4972, -1.4525, -2.4567,
     & -4.7238, -4.7300, -4.6981, -4.6580, -4.6198, -4.5907, -4.5746,
     & -4.5696, -4.5715, -4.5756, -4.5794, -4.5844, -2.2370, -1.9104,
     & -1.6081, -1.4191, -1.7551, -4.3479, -4.7486, -4.7303, -4.6929,
     & -4.6515, -4.6140, -4.5870, -4.5728, -4.5698, -4.5724, -4.5768,
     & -4.5805, -2.3866, -2.0566, -1.7388, -1.4776, -1.4792, -2.6970,
     & -4.8008, -4.7691, -4.7292, -4.6834, -4.6140, -4.5870, -4.5728,
     & -4.5698, -4.5724, -4.5768, -4.5805/
      DATA ((SVCS(I,J,21,1),J=1,17),I=1,6)/
     & -1.7070, -1.6428, -2.5580, -5.2123, -5.1702, -5.1470, -5.1123,
     & -5.0865, -5.0730, -5.0697, -5.0721, -5.0761, -5.0795, -5.0810,
     & -5.0871, -5.0876, -5.0890, -1.8206, -1.6218, -1.9130, -4.4113,
     & -5.1935, -5.1786, -5.1409, -5.1063, -5.0827, -5.0716, -5.0701,
     & -5.0734, -5.0776, -5.0810, -5.0871, -5.0876, -5.0890, -1.9526,
     & -1.6866, -1.6656, -2.7806, -5.2517, -5.2082, -5.1737, -5.1342,
     & -5.1008, -5.0794, -5.0706, -5.0708, -5.0747, -5.0790, -5.0823,
     & -5.0876, -5.0890, -2.0939, -1.7942, -1.6170, -2.0084, -4.8450,
     & -5.2352, -5.2071, -5.1677, -5.1279, -5.0959, -5.0768, -5.0702,
     & -5.0716, -5.0760, -5.0803, -5.0834, -5.0880, -2.2398, -1.9233,
     & -1.6680, -1.6970, -3.0438, -5.2926, -5.2368, -5.2031, -5.1617,
     & -5.1221, -5.0918, -5.0748, -5.0700, -5.0724, -5.0771, -5.0813,
     & -5.0843, -2.3880, -2.0631, -1.7688, -1.6167, -2.1242, -5.1743,
     & -5.2790, -5.2412, -5.1959, -5.1484, -5.0918, -5.0748, -5.0700,
     & -5.0724, -5.0771, -5.0813, -5.0843/
      DATA ((SVCS(I,J,22,1),J=1,17),I=1,6)/
     & -1.8188, -2.1614, -4.8941, -5.6788, -5.6451, -5.6189, -5.5904,
     & -5.5745, -5.5699, -5.5722, -5.5766, -5.5807, -5.5855, -5.5863,
     & -5.5894, -5.5896, -5.5917, -1.8764, -1.8812, -3.1159, -5.7193,
     & -5.6765, -5.6496, -5.6131, -5.5864, -5.5728, -5.5702, -5.5735,
     & -5.5781, -5.5819, -5.5863, -5.5894, -5.5896, -5.5917, -1.9805,
     & -1.8162, -2.2673, -5.3555, -5.7212, -5.6830, -5.6434, -5.6075,
     & -5.5827, -5.5715, -5.5707, -5.5746, -5.5794, -5.5830, -5.5870,
     & -5.5896, -5.5917, -2.1079, -1.8590, -1.9179, -3.4036, -5.7571,
     & -5.7201, -5.6780, -5.6371, -5.6022, -5.5797, -5.5709, -5.5714,
     & -5.5758, -5.5805, -5.5840, -5.5875, -5.5917, -2.2468, -1.9557,
     & -1.8185, -2.3954, -5.6724, -5.7688, -5.7131, -5.6724, -5.6312,
     & -5.5977, -5.5774, -5.5704, -5.5720, -5.5769, -5.5814, -5.5847,
     & -5.5917, -2.3915, -2.0793, -1.8440, -1.9658, -3.7418, -5.8099,
     & -5.7528, -5.7091, -5.6604, -5.6157, -5.5774, -5.5704, -5.5720,
     & -5.5769, -5.5814, -5.5847, -5.5917/
      DATA ((SVCS(I,J,23,1),J=1,17),I=1,6)/
     & -2.0994, -3.4636, -6.1682, -6.1441, -6.1183, -6.0956, -6.0766,
     & -6.0703, -6.0720, -6.0766, -6.0810, -6.0842, -6.0886, -6.0889,
     & -6.0905, -6.0917, -6.0917, -2.0168, -2.5325, -5.8759, -6.1849,
     & -6.1498, -6.1210, -6.0911, -6.0745, -6.0703, -6.0730, -6.0779,
     & -6.0822, -6.0852, -6.0889, -6.0905, -6.0917, -6.0917, -2.0507,
     & -2.1419, -3.7779, -6.2185, -6.1866, -6.1532, -6.1151, -6.0871,
     & -6.0730, -6.0705, -6.0742, -6.0791, -6.0832, -6.0860, -6.0893,
     & -6.0917, -6.0917, -2.1429, -2.0219, -2.6738, -6.1586, -6.2199,
     & -6.1944, -6.1473, -6.1097, -6.0837, -6.0719, -6.0710, -6.0753,
     & -6.0802, -6.0841, -6.0878, -6.0917, -6.0917, -2.2643, -2.0371,
     & -2.1964, -4.1466, -6.2532, -6.2401, -6.1837, -6.1414, -6.1049,
     & -6.0809, -6.0711, -6.0716, -6.0762, -6.0811, -6.0848, -6.0917,
     & -6.0917, -2.4003, -2.1200, -2.0330, -2.8429, -6.2739, -6.2828,
     & -6.2221, -6.1735, -6.1261, -6.0899, -6.0711, -6.0716, -6.0762,
     & -6.0811, -6.0848, -6.0917, -6.0917/
      DATA ((SVCS(I,J,24,1),J=1,17),I=1,6)/
     & -2.8045, -6.3876, -6.6286, -6.6123, -6.5953, -6.5795, -6.5707,
     & -6.5715, -6.5763, -6.5810, -6.5845, -6.5879, -6.5901, -6.5903,
     & -6.5911, -6.5917, -6.5917, -2.3693, -4.1677, -6.6754, -6.6494,
     & -6.6220, -6.5971, -6.5771, -6.5706, -6.5725, -6.5774, -6.5820,
     & -6.5854, -6.5884, -6.5903, -6.5911, -6.5917, -6.5917, -2.2270,
     & -2.9601, -6.6485, -6.6769, -6.6508, -6.6242, -6.5926, -6.5750,
     & -6.5704, -6.5735, -6.5786, -6.5830, -6.5860, -6.5888, -6.5905,
     & -6.5917, -6.5917, -2.2311, -2.4310, -4.5696, -6.6903, -6.6782,
     & -6.6649, -6.6185, -6.5887, -6.5735, -6.5707, -6.5745, -6.5795,
     & -6.5839, -6.5867, -6.5897, -6.5917, -6.5917, -2.3084, -2.2417,
     & -3.1458, -6.6977, -6.7056, -6.7075, -6.6525, -6.6133, -6.5854,
     & -6.5724, -6.5711, -6.5754, -6.5805, -6.5845, -6.5872, -6.5917,
     & -6.5917, -2.4223, -2.2223, -2.5077, -5.0373, -6.7330, -6.7489,
     & -6.6875, -6.6379, -6.5975, -6.5741, -6.5711, -6.5754, -6.5805,
     & -6.5845, -6.5872, -6.5917, -6.5917/
      DATA ((SVCS(I,J,25,1),J=1,17),I=1,6)/
     & -4.5744, -7.1039, -7.0959, -7.0879, -7.0797, -7.0718, -7.0711,
     & -7.0755, -7.0806, -7.0844, -7.0870, -7.0898, -7.0909, -7.0910,
     & -7.0917, -7.0917, -7.0917, -3.2548, -7.1214, -7.1347, -7.1167,
     & -7.0982, -7.0805, -7.0711, -7.0718, -7.0767, -7.0816, -7.0852,
     & -7.0885, -7.0900, -7.0910, -7.0917, -7.0917, -7.0917, -2.6698,
     & -5.0123, -7.1533, -7.1354, -7.1175, -7.0994, -7.0780, -7.0707,
     & -7.0726, -7.0777, -7.0825, -7.0859, -7.0888, -7.0902, -7.0917,
     & -7.0917, -7.0917, -2.4525, -3.4585, -7.1340, -7.1365, -7.1363,
     & -7.1357, -7.0951, -7.0759, -7.0706, -7.0735, -7.0788, -7.0834,
     & -7.0865, -7.0892, -7.0917, -7.0917, -7.0917, -2.4191, -2.7554,
     & -5.5197, -7.1376, -7.1551, -7.1726, -7.1230, -7.0913, -7.0743,
     & -7.0707, -7.0743, -7.0797, -7.0841, -7.0870, -7.0917, -7.0917,
     & -7.0917, -2.4776, -2.4792, -3.7000, -7.1385, -7.1739, -7.2091,
     & -7.1512, -7.1068, -7.0780, -7.0679, -7.0743, -7.0797, -7.0841,
     & -7.0870, -7.0917, -7.0917, -7.0917/
      DATA ((SVCS(I,J,26,1),J=1,17),I=1,6)/
     & -7.5620, -7.5759, -7.5746, -7.5733, -7.5719, -7.5707, -7.5746,
     & -7.5799, -7.5840, -7.5869, -7.5893, -7.5907, -7.5913, -7.5913,
     & -7.5917, -7.5917, -7.5917, -5.4762, -7.6088, -7.5997, -7.5906,
     & -7.5814, -7.5722, -7.5712, -7.5756, -7.5809, -7.5849, -7.5874,
     & -7.5901, -7.5908, -7.5913, -7.5917, -7.5917, -7.5917, -3.7819,
     & -7.6162, -7.6086, -7.5998, -7.5910, -7.5820, -7.5714, -7.5719,
     & -7.5768, -7.5819, -7.5855, -7.5879, -7.5902, -7.5909, -7.5917,
     & -7.5917, -7.5917, -3.0086, -6.0272, -7.5801, -7.5903, -7.6005,
     & -7.6105, -7.5794, -7.5709, -7.5725, -7.5777, -7.5827, -7.5862,
     & -7.5884, -7.5904, -7.5917, -7.5917, -7.5917, -2.6971, -4.0459,
     & -7.5516, -7.5808, -7.6100, -7.6392, -7.5986, -7.5772, -7.5707,
     & -7.5733, -7.5787, -7.5835, -7.5867, -7.5887, -7.5917, -7.5917,
     & -7.5917, -2.6167, -3.1244, -6.6351, -7.5713, -7.6195, -7.6677,
     & -7.6177, -7.5837, -7.5689, -7.5689, -7.5787, -7.5835, -7.5867,
     & -7.5887, -7.5917, -7.5917, -7.5917/
      DATA ((SVCS(I,J,27,1),J=1,17),I=1,6)/
     & -8.0596, -8.0624, -8.0652, -8.0680, -8.0708, -8.0736, -8.0789,
     & -8.0835, -8.0867, -8.0893, -8.0905, -8.0912, -8.0915, -8.0917,
     & -8.0917, -8.0917, -8.0917, -8.0803, -8.0787, -8.0767, -8.0747,
     & -8.0727, -8.0707, -8.0745, -8.0799, -8.0843, -8.0872, -8.0895,
     & -8.0909, -8.0912, -8.0917, -8.0917, -8.0917, -8.0917, -6.5624,
     & -8.0797, -8.0780, -8.0763, -8.0746, -8.0729, -8.0711, -8.0756,
     & -8.0809, -8.0851, -8.0877, -8.0898, -8.0909, -8.0917, -8.0917,
     & -8.0917, -8.0917, -4.4053, -8.0294, -8.0451, -8.0608, -8.0765,
     & -8.0921, -8.0720, -8.0717, -8.0765, -8.0818, -8.0857, -8.0881,
     & -8.0900, -8.0917, -8.0917, -8.0917, -8.0917, -3.3955, -7.2074,
     & -8.0122, -8.0453, -8.0784, -8.1115, -8.0816, -8.0713, -8.0722,
     & -8.0774, -8.0825, -8.0862, -8.0885, -8.0917, -8.0917, -8.0917,
     & -8.0917, -2.9659, -4.7451, -7.9793, -8.0298, -8.0803, -8.1308,
     & -8.0912, -8.0709, -8.0679, -8.0730, -8.0825, -8.0862, -8.0885,
     & -8.0917, -8.0917, -8.0917, -8.0917/
      DATA ((SVCS(I,J,28,1),J=1,17),I=1,6)/
     & -8.5577, -8.5617, -8.5657, -8.5697, -8.5737, -8.5777, -8.5829,
     & -8.5862, -8.5885, -8.5905, -8.5911, -8.5914, -8.5917, -8.5917,
     & -8.5917, -8.5917, -8.5917, -8.5613, -8.5637, -8.5661, -8.5685,
     & -8.5707, -8.5733, -8.5789, -8.5837, -8.5869, -8.5889, -8.5906,
     & -8.5913, -8.5917, -8.5917, -8.5917, -8.5917, -8.5917, -8.5577,
     & -8.5603, -8.5629, -8.5655, -8.5681, -8.5707, -8.5743, -8.5799,
     & -8.5844, -8.5872, -8.5892, -8.5907, -8.5917, -8.5917, -8.5917,
     & -8.5917, -8.5917, -7.8138, -8.5155, -8.5321, -8.5487, -8.5653,
     & -8.5817, -8.5711, -8.5751, -8.5807, -8.5850, -8.5877, -8.5899,
     & -8.5917, -8.5917, -8.5917, -8.5917, -8.5917, -5.1497, -8.4707,
     & -8.5013, -8.5319, -8.5625, -8.5931, -8.5728, -8.5714, -8.5761,
     & -8.5815, -8.5855, -8.5880, -8.5917, -8.5917, -8.5917, -8.5917,
     & -8.5917, -3.8431, -8.2778, -8.4705, -8.5151, -8.5597, -8.6043,
     & -8.5745, -8.5677, -8.5715, -8.5780, -8.5855, -8.5880, -8.5917,
     & -8.5917, -8.5917, -8.5917, -8.5917/
      DATA ((SVCS(I,J,29,1),J=1,17),I=1,6)/
     & -9.0614, -9.0655, -9.0696, -9.0737, -9.0778, -9.0819, -9.0858,
     & -9.0882, -9.0901, -9.0911, -9.0914, -9.0915, -9.0917, -9.0917,
     & -9.0917, -9.0917, -9.0917, -9.0560, -9.0603, -9.0646, -9.0689,
     & -9.0732, -9.0775, -9.0827, -9.0863, -9.0886, -9.0903, -9.0911,
     & -9.0915, -9.0917, -9.0917, -9.0917, -9.0917, -9.0917, -9.0510,
     & -9.0554, -9.0598, -9.0642, -9.0686, -9.0730, -9.0785, -9.0835,
     & -9.0868, -9.0889, -9.0904, -9.0912, -9.0917, -9.0917, -9.0917,
     & -9.0917, -9.0917, -9.0052, -9.0199, -9.0346, -9.0493, -9.0640,
     & -9.0786, -9.0739, -9.0795, -9.0842, -9.0872, -9.0892, -9.0908,
     & -9.0917, -9.0917, -9.0917, -9.0917, -9.0917, -8.8615, -8.9844,
     & -9.0094, -9.0344, -9.0594, -9.0844, -9.0709, -9.0746, -9.0803,
     & -9.0848, -9.0877, -9.0895, -9.0917, -9.0917, -9.0917, -9.0917,
     & -9.0917, -6.0459, -8.9489, -8.9842, -9.0195, -9.0548, -9.0901,
     & -9.0679, -9.0697, -9.0764, -9.0824, -9.0877, -9.0895, -9.0917,
     & -9.0917, -9.0917, -9.0917, -9.0917/
      DATA ((SVCS(I,J,30,1),J=1,17),I=1,6)/
     & -9.5695, -9.5726, -9.5757, -9.5788, -9.5819, -9.5850, -9.5879,
     & -9.5896, -9.5909, -9.5914, -9.5915, -9.5917, -9.5917, -9.5917,
     & -9.5917, -9.5917, -9.5917, -9.5592, -9.5637, -9.5682, -9.5727,
     & -9.5772, -9.5817, -9.5857, -9.5882, -9.5901, -9.5910, -9.5914,
     & -9.5917, -9.5917, -9.5917, -9.5917, -9.5917, -9.5917, -9.5541,
     & -9.5587, -9.5633, -9.5679, -9.5725, -9.5771, -9.5824, -9.5862,
     & -9.5885, -9.5903, -9.5910, -9.5917, -9.5917, -9.5917, -9.5917,
     & -9.5917, -9.5917, -9.5174, -9.5300, -9.5426, -9.5552, -9.5678,
     & -9.5802, -9.5781, -9.5832, -9.5867, -9.5889, -9.5904, -9.5917,
     & -9.5917, -9.5917, -9.5917, -9.5917, -9.5917, -9.4807, -9.5013,
     & -9.5219, -9.5425, -9.5631, -9.5837, -9.5733, -9.5789, -9.5839,
     & -9.5870, -9.5892, -9.5917, -9.5917, -9.5917, -9.5917, -9.5917,
     & -9.5917, -9.4408, -9.4726, -9.5012, -9.5298, -9.5584, -9.5870,
     & -9.5685, -9.5746, -9.5811, -9.5851, -9.5892, -9.5917, -9.5917,
     & -9.5917, -9.5917, -9.5917, -9.5917/
      DATA ((SVCS(I,J,31,1),J=1,17),I=1,6)/
     &-10.0758,-10.0781,-10.0804,-10.0827,-10.0850,-10.0873,-10.0893,
     &-10.0906,-10.0913,-10.0915,-10.0916,-10.0917,-10.0917,-10.0917,
     &-10.0917,-10.0917,-10.0917,-10.0674,-10.0709,-10.0744,-10.0779,
     &-10.0814,-10.0849,-10.0878,-10.0895,-10.0909,-10.0913,-10.0915,
     &-10.0917,-10.0917,-10.0917,-10.0917,-10.0917,-10.0917,-10.0642,
     &-10.0676,-10.0710,-10.0744,-10.0778,-10.0812,-10.0854,-10.0882,
     &-10.0897,-10.0910,-10.0914,-10.0917,-10.0917,-10.0917,-10.0917,
     &-10.0917,-10.0917,-10.0346,-10.0445,-10.0544,-10.0643,-10.0742,
     &-10.0841,-10.0820,-10.0860,-10.0885,-10.0900,-10.0910,-10.0917,
     &-10.0917,-10.0917,-10.0917,-10.0917,-10.0917,-10.0050,-10.0214,
     &-10.0378,-10.0542,-10.0706,-10.0870,-10.0773,-10.0827,-10.0865,
     &-10.0888,-10.0901,-10.0917,-10.0917,-10.0917,-10.0917,-10.0917,
     &-10.0917, -9.9754, -9.9983,-10.0212,-10.0441,-10.0670,-10.0899,
     &-10.0726,-10.0794,-10.0845,-10.0876,-10.0901,-10.0917,-10.0917,
     &-10.0917,-10.0917,-10.0917,-10.0917/
      DATA ((SVCS(I,J,32,1),J=1,17),I=1,6)/
     &-10.5804,-10.5821,-10.5838,-10.5855,-10.5872,-10.5889,-10.5905,
     &-10.5911,-10.5915,-10.5916,-10.5917,-10.5917,-10.5917,-10.5917,
     &-10.5917,-10.5917,-10.5917,-10.5747,-10.5772,-10.5797,-10.5822,
     &-10.5847,-10.5872,-10.5892,-10.5906,-10.5913,-10.5915,-10.5917,
     &-10.5917,-10.5917,-10.5917,-10.5917,-10.5917,-10.5917,-10.5730,
     &-10.5753,-10.5776,-10.5799,-10.5822,-10.5845,-10.5877,-10.5896,
     &-10.5907,-10.5913,-10.5917,-10.5917,-10.5917,-10.5917,-10.5917,
     &-10.5917,-10.5917,-10.5457,-10.5542,-10.5627,-10.5712,-10.5797,
     &-10.5882,-10.5852,-10.5880,-10.5897,-10.5908,-10.5917,-10.5917,
     &-10.5917,-10.5917,-10.5917,-10.5917,-10.5917,-10.5184,-10.5331,
     &-10.5478,-10.5625,-10.5772,-10.5919,-10.5814,-10.5857,-10.5884,
     &-10.5899,-10.5917,-10.5917,-10.5917,-10.5917,-10.5917,-10.5917,
     &-10.5917,-10.4911,-10.5120,-10.5329,-10.5538,-10.5747,-10.5956,
     &-10.5776,-10.5834,-10.5871,-10.5890,-10.5917,-10.5917,-10.5917,
     &-10.5917,-10.5917,-10.5917,-10.5917/
      DATA ((SVCS(I,J,33,1),J=1,17),I=1,6)/
     &-11.0845,-11.0856,-11.0867,-11.0878,-11.0889,-11.0900,-11.0911,
     &-11.0914,-11.0916,-11.0917,-11.0917,-11.0917,-11.0917,-11.0917,
     &-11.0917,-11.0917,-11.0917,-11.0798,-11.0816,-11.0834,-11.0852,
     &-11.0870,-11.0888,-11.0901,-11.0911,-11.0915,-11.0917,-11.0917,
     &-11.0917,-11.0917,-11.0917,-11.0917,-11.0917,-11.0917,-11.0779,
     &-11.0797,-11.0815,-11.0833,-11.0851,-11.0869,-11.0891,-11.0903,
     &-11.0912,-11.0917,-11.0917,-11.0917,-11.0917,-11.0917,-11.0917,
     &-11.0917,-11.0917,-11.0496,-11.0580,-11.0664,-11.0748,-11.0832,
     &-11.0916,-11.0873,-11.0894,-11.0907,-11.0917,-11.0917,-11.0917,
     &-11.0917,-11.0917,-11.0917,-11.0917,-11.0917,-11.0213,-11.0363,
     &-11.0513,-11.0663,-11.0813,-11.0963,-11.0847,-11.0877,-11.0896,
     &-11.0917,-11.0917,-11.0917,-11.0917,-11.0917,-11.0917,-11.0917,
     &-11.0917,-10.9930,-11.0146,-11.0362,-11.0578,-11.0794,-11.1010,
     &-11.0821,-11.0860,-11.0885,-11.0917,-11.0917,-11.0917,-11.0917,
     &-11.0917,-11.0917,-11.0917,-11.0917/
      DATA ((SVCS(I,J,34,1),J=1,17),I=1,6)/
     &-11.5863,-11.5872,-11.5881,-11.5890,-11.5899,-11.5908,-11.5914,
     &-11.5915,-11.5916,-11.5917,-11.5917,-11.5917,-11.5917,-11.5917,
     &-11.5917,-11.5917,-11.5917,-11.5839,-11.5851,-11.5863,-11.5875,
     &-11.5887,-11.5899,-11.5909,-11.5914,-11.5916,-11.5917,-11.5917,
     &-11.5917,-11.5917,-11.5917,-11.5917,-11.5917,-11.5917,-11.5827,
     &-11.5839,-11.5851,-11.5863,-11.5875,-11.5887,-11.5901,-11.5910,
     &-11.5914,-11.5917,-11.5917,-11.5917,-11.5917,-11.5917,-11.5917,
     &-11.5917,-11.5917,-11.5551,-11.5629,-11.5707,-11.5785,-11.5863,
     &-11.5941,-11.5891,-11.5902,-11.5912,-11.5917,-11.5917,-11.5917,
     &-11.5917,-11.5917,-11.5917,-11.5917,-11.5917,-11.5275,-11.5419,
     &-11.5563,-11.5707,-11.5851,-11.5995,-11.5870,-11.5892,-11.5904,
     &-11.5917,-11.5917,-11.5917,-11.5917,-11.5917,-11.5917,-11.5917,
     &-11.5917,-11.4999,-11.5209,-11.5419,-11.5629,-11.5839,-11.6049,
     &-11.5849,-11.5882,-11.5896,-11.5917,-11.5917,-11.5917,-11.5917,
     &-11.5917,-11.5917,-11.5917,-11.5917/
      DATA ((SVCS(I,J,35,1),J=1,17),I=1,6)/
     &-12.0882,-12.0888,-12.0894,-12.0900,-12.0906,-12.0912,-12.0915,
     &-12.0916,-12.0917,-12.0917,-12.0917,-12.0917,-12.0917,-12.0917,
     &-12.0917,-12.0917,-12.0917,-12.0863,-12.0872,-12.0881,-12.0890,
     &-12.0899,-12.0908,-12.0913,-12.0915,-12.0917,-12.0917,-12.0917,
     &-12.0917,-12.0917,-12.0917,-12.0917,-12.0917,-12.0917,-12.0868,
     &-12.0874,-12.0880,-12.0886,-12.0892,-12.0898,-12.0909,-12.0913,
     &-12.0917,-12.0917,-12.0917,-12.0917,-12.0917,-12.0917,-12.0917,
     &-12.0917,-12.0917,-12.0589,-12.0663,-12.0737,-12.0811,-12.0885,
     &-12.0959,-12.0900,-12.0909,-12.0917,-12.0917,-12.0917,-12.0917,
     &-12.0917,-12.0917,-12.0917,-12.0917,-12.0917,-12.0310,-12.0452,
     &-12.0594,-12.0736,-12.0878,-12.1020,-12.0887,-12.0901,-12.0917,
     &-12.0917,-12.0917,-12.0917,-12.0917,-12.0917,-12.0917,-12.0917,
     &-12.0917,-12.0031,-12.0241,-12.0451,-12.0661,-12.0871,-12.1081,
     &-12.0874,-12.0893,-12.0917,-12.0917,-12.0917,-12.0917,-12.0917,
     &-12.0917,-12.0917,-12.0917,-12.0917/
      DATA ((SVCS(I,J,36,1),J=1,17),I=1,6)/
     &-12.5899,-12.5902,-12.5905,-12.5908,-12.5911,-12.5914,-12.5916,
     &-12.5916,-12.5917,-12.5917,-12.5917,-12.5917,-12.5917,-12.5917,
     &-12.5917,-12.5917,-12.5917,-12.5882,-12.5888,-12.5894,-12.5900,
     &-12.5906,-12.5912,-12.5915,-12.5916,-12.5917,-12.5917,-12.5917,
     &-12.5917,-12.5917,-12.5917,-12.5917,-12.5917,-12.5917,-12.5881,
     &-12.5886,-12.5891,-12.5896,-12.5901,-12.5906,-12.5913,-12.5915,
     &-12.5917,-12.5917,-12.5917,-12.5917,-12.5917,-12.5917,-12.5917,
     &-12.5917,-12.5917,-12.5596,-12.5671,-12.5746,-12.5821,-12.5896,
     &-12.5971,-12.5906,-12.5913,-12.5917,-12.5917,-12.5917,-12.5917,
     &-12.5917,-12.5917,-12.5917,-12.5917,-12.5917,-12.5311,-12.5456,
     &-12.5601,-12.5746,-12.5891,-12.6036,-12.5899,-12.5907,-12.5917,
     &-12.5917,-12.5917,-12.5917,-12.5917,-12.5917,-12.5917,-12.5917,
     &-12.5917,-12.5026,-12.5241,-12.5456,-12.5671,-12.5886,-12.6101,
     &-12.5892,-12.5901,-12.5917,-12.5917,-12.5917,-12.5917,-12.5917,
     &-12.5917,-12.5917,-12.5917,-12.5917/
      DATA ((SVCS(I,J,37,1),J=1,17),I=1,6)/
     &-13.0919,-13.0918,-13.0918,-13.0917,-13.0917,-13.0916,-13.0916,
     &-13.0917,-13.0917,-13.0917,-13.0917,-13.0917,-13.0917,-13.0917,
     &-13.0917,-13.0917,-13.0917,-13.0927,-13.0924,-13.0922,-13.0919,
     &-13.0917,-13.0914,-13.0916,-13.0917,-13.0917,-13.0917,-13.0917,
     &-13.0917,-13.0917,-13.0917,-13.0917,-13.0917,-13.0917,-13.0939,
     &-13.0933,-13.0928,-13.0922,-13.0917,-13.0911,-13.0915,-13.0917,
     &-13.0917,-13.0917,-13.0917,-13.0917,-13.0917,-13.0917,-13.0917,
     &-13.0917,-13.0917,-13.0671,-13.0732,-13.0794,-13.0855,-13.0917,
     &-13.0978,-13.0911,-13.0917,-13.0917,-13.0917,-13.0917,-13.0917,
     &-13.0917,-13.0917,-13.0917,-13.0917,-13.0917,-13.0403,-13.0531,
     &-13.0660,-13.0788,-13.0917,-13.1045,-13.0906,-13.0917,-13.0917,
     &-13.0917,-13.0917,-13.0917,-13.0917,-13.0917,-13.0917,-13.0917,
     &-13.0917,-13.0135,-13.0330,-13.0526,-13.0721,-13.0917,-13.1112,
     &-13.0901,-13.0917,-13.0917,-13.0917,-13.0917,-13.0917,-13.0917,
     &-13.0917,-13.0917,-13.0917,-13.0917/
      DATA ((SVCS(I,J,38,1),J=1,17),I=1,6)/
     &-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,
     &-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,
     &-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,
     &-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,
     &-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,
     &-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,
     &-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,
     &-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,
     &-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,
     &-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,
     &-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,
     &-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,
     &-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,
     &-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,-13.5917,
     &-13.5917,-13.5917,-13.5917,-13.5917/
      DATA ((SVCS(I,J,39,1),J=1,17),I=1,6)/
     &-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,
     &-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,
     &-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,
     &-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,
     &-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,
     &-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,
     &-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,
     &-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,
     &-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,
     &-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,
     &-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,
     &-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,
     &-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,
     &-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,-14.0917,
     &-14.0917,-14.0917,-14.0917,-14.0917/
      DATA ((SVCS(I,J,40,1),J=1,17),I=1,6)/
     &-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,
     &-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,
     &-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,
     &-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,
     &-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,
     &-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,
     &-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,
     &-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,
     &-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,
     &-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,
     &-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,
     &-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,
     &-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,
     &-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,-14.5917,
     &-14.5917,-14.5917,-14.5917,-14.5917/
      DATA ((SVCS(I,J, 1,2),J=1,17),I=1,6)/
     & -1.5028, -1.1694,  -.8368,  -.5065,  -.1810,   .1307,   .4033,
     &   .5934,   .6508,   .5878,   .5260,   .5363,   .5810,   .5490,
     &   .6001,   .6001,   .6001, -1.6533, -1.3200,  -.9872,  -.6550,
     &  -.3261,  -.0071,   .2871,   .5185,   .6321,   .5977,   .4998,
     &   .4719,   .5030,   .5490,   .6001,   .6001,   .6001, -1.8038,
     & -1.4705, -1.1375,  -.8055,  -.4747,  -.1495,   .1587,   .4242,
     &   .5942,   .6144,   .5092,   .4296,   .4392,   .4814,   .5228,
     &   .5228,   .5228, -1.9543, -1.6210, -1.2876,  -.9543,  -.6234,
     &  -.2949,   .0241,   .3124,   .5316,   .6184,   .5447,   .4185,
     &   .3863,   .4216,   .4637,   .4637,   .4637, -2.1048, -1.7715,
     & -1.4382, -1.1052,  -.7726,  -.4424,  -.1182,   .1881,   .4458,
     &   .5967,   .5852,   .4456,   .3518,   .3654,   .4082,   .4082,
     &   .4082, -2.2553, -1.9220, -1.5887, -1.2553,  -.9220,  -.5887,
     &  -.2553,   .0780,   .4113,   .5967,   .5852,   .4456,   .3518,
     &   .3654,   .4082,   .4082,   .4082/
      DATA ((SVCS(I,J, 2,2),J=1,17),I=1,6)/
     & -1.5028, -1.1694,  -.8368,  -.5065,  -.1810,   .1307,   .4033,
     &   .5934,   .6509,   .5885,   .5280,   .5387,   .5833,   .5492,
     &   .6003,   .6003,   .6003, -1.6533, -1.3200,  -.9872,  -.6550,
     &  -.3261,  -.0071,   .2871,   .5184,   .6320,   .5980,   .5015,
     &   .4744,   .5051,   .5492,   .6003,   .6003,   .6003, -1.8038,
     & -1.4705, -1.1375,  -.8055,  -.4747,  -.1495,   .1587,   .4242,
     &   .5942,   .6145,   .5101,   .4320,   .4417,   .4816,   .5230,
     &   .5230,   .5230, -1.9543, -1.6210, -1.2876,  -.9543,  -.6234,
     &  -.2949,   .0241,   .3124,   .5316,   .6184,   .5452,   .4205,
     &   .3892,   .4219,   .4639,   .4639,   .4639, -2.1048, -1.7715,
     & -1.4382, -1.1052,  -.7726,  -.4424,  -.1182,   .1881,   .4458,
     &   .5966,   .5855,   .4470,   .3549,   .3657,   .4084,   .4084,
     &   .4084, -2.2553, -1.9220, -1.5887, -1.2553,  -.9220,  -.5887,
     &  -.2553,   .0780,   .4113,   .5966,   .5855,   .4470,   .3549,
     &   .3657,   .4084,   .4084,   .4084/
      DATA ((SVCS(I,J, 3,2),J=1,17),I=1,6)/
     & -1.5028, -1.1694,  -.8368,  -.5065,  -.1810,   .1307,   .4033,
     &   .5933,   .6509,   .5902,   .5328,   .5446,   .5887,   .5538,
     &   .6046,   .6046,   .6046, -1.6533, -1.3200,  -.9872,  -.6550,
     &  -.3261,  -.0071,   .2871,   .5184,   .6320,   .5988,   .5053,
     &   .4803,   .5108,   .5538,   .6046,   .6046,   .6046, -1.8038,
     & -1.4705, -1.1375,  -.8055,  -.4747,  -.1495,   .1587,   .4242,
     &   .5941,   .6147,   .5127,   .4379,   .4481,   .4866,   .5267,
     &   .5267,   .5267, -1.9543, -1.6210, -1.2876,  -.9543,  -.6234,
     &  -.2949,   .0241,   .3124,   .5315,   .6184,   .5464,   .4255,
     &   .3962,   .4278,   .4682,   .4682,   .4682, -2.1048, -1.7715,
     & -1.4382, -1.1052,  -.7726,  -.4424,  -.1182,   .1878,   .4456,
     &   .5966,   .5859,   .4502,   .3621,   .3731,   .4138,   .4138,
     &   .4138, -2.2553, -1.9220, -1.5887, -1.2553,  -.9220,  -.5887,
     &  -.2553,   .0780,   .4113,   .5966,   .5859,   .4502,   .3621,
     &   .3731,   .4138,   .4138,   .4138/
      DATA ((SVCS(I,J, 4,2),J=1,17),I=1,6)/
     & -1.5028, -1.1694,  -.8368,  -.5065,  -.1810,   .1307,   .4033,
     &   .5931,   .6511,   .5943,   .5442,   .5585,   .6013,   .5647,
     &   .6146,   .6146,   .6146, -1.6533, -1.3200,  -.9872,  -.6550,
     &  -.3261,  -.0071,   .2871,   .5183,   .6317,   .6008,   .5145,
     &   .4947,   .5240,   .5647,   .6146,   .6146,   .6146, -1.8038,
     & -1.4705, -1.1375,  -.8055,  -.4747,  -.1495,   .1587,   .4241,
     &   .5938,   .6152,   .5188,   .4519,   .4628,   .4983,   .5358,
     &   .5358,   .5358, -1.9543, -1.6210, -1.2876,  -.9543,  -.6234,
     &  -.2949,   .0241,   .3124,   .5314,   .6184,   .5495,   .4374,
     &   .4131,   .4419,   .4784,   .4784,   .4784, -2.1048, -1.7715,
     & -1.4382, -1.1052,  -.7726,  -.4424,  -.1182,   .1878,   .4456,
     &   .5964,   .5869,   .4586,   .3797,   .3904,   .4267,   .4267,
     &   .4267, -2.2553, -1.9220, -1.5887, -1.2553,  -.9220,  -.5887,
     &  -.2554,   .0780,   .4112,   .5964,   .5869,   .4586,   .3797,
     &   .3904,   .4267,   .4267,   .4267/
      DATA ((SVCS(I,J, 5,2),J=1,17),I=1,6)/
     & -1.5028, -1.1694,  -.8368,  -.5065,  -.1810,   .1307,   .4029,
     &   .5924,   .6514,   .6040,   .5700,   .5884,   .6281,   .5882,
     &   .6362,   .6362,   .6362, -1.6533, -1.3200,  -.9872,  -.6550,
     &  -.3261,  -.0072,   .2869,   .5179,   .6313,   .6054,   .5359,
     &   .5265,   .5527,   .5882,   .6362,   .6362,   .6362, -1.8038,
     & -1.4705, -1.1375,  -.8055,  -.4747,  -.1495,   .1587,   .4239,
     &   .5933,   .6167,   .5331,   .4837,   .4955,   .5243,   .5558,
     &   .5558,   .5558, -1.9543, -1.6210, -1.2876,  -.9543,  -.6234,
     &  -.2949,   .0237,   .3122,   .5308,   .6183,   .5568,   .4652,
     &   .4499,   .4729,   .5013,   .5013,   .5013, -2.1048, -1.7715,
     & -1.4382, -1.1052,  -.7726,  -.4424,  -.1183,   .1878,   .4453,
     &   .5959,   .5895,   .4781,   .4186,   .4283,   .4555,   .4555,
     &   .4555, -2.2553, -1.9220, -1.5887, -1.2553,  -.9220,  -.5887,
     &  -.2554,   .0779,   .4110,   .5959,   .5895,   .4781,   .4186,
     &   .4283,   .4555,   .4555,   .4555/
      DATA ((SVCS(I,J, 6,2),J=1,17),I=1,6)/
     & -1.5028, -1.1694,  -.8368,  -.5065,  -.1810,   .1307,   .4024,
     &   .5910,   .6521,   .6254,   .6200,   .6422,   .6748,   .6313,
     &   .6748,   .6748,   .6748, -1.6533, -1.3200,  -.9872,  -.6550,
     &  -.3261,  -.0073,   .2867,   .5168,   .6303,   .6162,   .5799,
     &   .5859,   .6053,   .6313,   .6748,   .6748,   .6748, -1.8038,
     & -1.4705, -1.1375,  -.8055,  -.4747,  -.1495,   .1584,   .4232,
     &   .5920,   .6199,   .5644,   .5453,   .5564,   .5731,   .5944,
     &   .5944,   .5944, -1.9543, -1.6210, -1.2876,  -.9543,  -.6234,
     &  -.2949,   .0237,   .3120,   .5298,   .6179,   .5736,   .5214,
     &   .5193,   .5312,   .5459,   .5459,   .5459, -2.1048, -1.7715,
     & -1.4382, -1.1052,  -.7726,  -.4424,  -.1183,   .1875,   .4445,
     &   .5946,   .5955,   .5200,   .4930,   .4989,   .5105,   .5105,
     &   .5105, -2.2553, -1.9220, -1.5887, -1.2553,  -.9220,  -.5887,
     &  -.2554,   .0778,   .4104,   .5946,   .5955,   .5200,   .4930,
     &   .4989,   .5105,   .5105,   .5105/
      DATA ((SVCS(I,J, 7,2),J=1,17),I=1,6)/
     & -1.5028, -1.1694,  -.8368,  -.5067,  -.1812,   .1300,   .4009,
     &   .5871,   .6534,   .6645,   .6915,   .7099,   .7303,   .6884,
     &   .7226,   .7226,   .7226, -1.6533, -1.3200,  -.9872,  -.6550,
     &  -.3262,  -.0074,   .2858,   .5141,   .6275,   .6378,   .6507,
     &   .6675,   .6753,   .6884,   .7226,   .7226,   .7226, -1.8038,
     & -1.4705, -1.1375,  -.8055,  -.4747,  -.1497,   .1581,   .4216,
     &   .5885,   .6268,   .6211,   .6358,   .6408,   .6423,   .6500,
     &   .6500,   .6500, -1.9543, -1.6210, -1.2876,  -.9543,  -.6234,
     &  -.2950,   .0237,   .3109,   .5271,   .6171,   .6072,   .6108,
     &   .6174,   .6148,   .6125,   .6125,   .6125, -2.1048, -1.7715,
     & -1.4382, -1.1052,  -.7726,  -.4425,  -.1184,   .1872,   .4428,
     &   .5915,   .6085,   .5935,   .6012,   .5988,   .5922,   .5922,
     &   .5922, -2.2553, -1.9220, -1.5887, -1.2553,  -.9220,  -.5887,
     &  -.2555,   .0775,   .4090,   .5915,   .6085,   .5935,   .6012,
     &   .5988,   .5922,   .5922,   .5922/
      DATA ((SVCS(I,J, 8,2),J=1,17),I=1,6)/
     & -1.5028, -1.1694,  -.8368,  -.5067,  -.1814,   .1290,   .3972,
     &   .5777,   .6529,   .7085,   .7426,   .7472,   .7524,   .7233,
     &   .7423,   .7423,   .7423, -1.6533, -1.3200,  -.9872,  -.6552,
     &  -.3262,  -.0080,   .2838,   .5076,   .6198,   .6688,   .7175,
     &   .7275,   .7228,   .7233,   .7423,   .7423,   .7423, -1.8038,
     & -1.4705, -1.1375,  -.8055,  -.4748,  -.1499,   .1569,   .4175,
     &   .5797,   .6373,   .6882,   .7132,   .7077,   .6972,   .6934,
     &   .6934,   .6934, -1.9543, -1.6210, -1.2877,  -.9543,  -.6236,
     &  -.2951,   .0228,   .3086,   .5201,   .6137,   .6562,   .6989,
     &   .7010,   .6869,   .6724,   .6724,   .6724, -2.1048, -1.7715,
     & -1.4382, -1.1052,  -.7726,  -.4425,  -.1187,   .1858,   .4382,
     &   .5835,   .6302,   .6794,   .6982,   .6866,   .6678,   .6678,
     &   .6678, -2.2553, -1.9220, -1.5887, -1.2553,  -.9220,  -.5887,
     &  -.2556,   .0768,   .4056,   .5835,   .6302,   .6794,   .6982,
     &   .6866,   .6678,   .6678,   .6678/
      DATA ((SVCS(I,J, 9,2),J=1,17),I=1,6)/
     & -1.5028, -1.1694,  -.8368,  -.5068,  -.1820,   .1268,   .3879,
     &   .5539,   .6351,   .6945,   .6994,   .6903,   .6832,   .6764,
     &   .6773,   .6773,   .6773, -1.6533, -1.3200,  -.9872,  -.6552,
     &  -.3265,  -.0093,   .2785,   .4909,   .5973,   .6721,   .7018,
     &   .6950,   .6839,   .6764,   .6773,   .6773,   .6773, -1.8038,
     & -1.4705, -1.1375,  -.8055,  -.4750,  -.1506,   .1541,   .4072,
     &   .5571,   .6358,   .6966,   .7004,   .6879,   .6747,   .6659,
     &   .6659,   .6659, -1.9543, -1.6210, -1.2877,  -.9543,  -.6236,
     &  -.2954,   .0216,   .3025,   .5028,   .5988,   .6768,   .7042,
     &   .6952,   .6792,   .6639,   .6639,   .6639, -2.1048, -1.7715,
     & -1.4382, -1.1053,  -.7726,  -.4427,  -.1195,   .1827,   .4268,
     &   .5626,   .6422,   .7020,   .7040,   .6890,   .6706,   .6706,
     &   .6706, -2.2553, -1.9220, -1.5887, -1.2554,  -.9220,  -.5888,
     &  -.2560,   .0749,   .3969,   .5626,   .6422,   .7020,   .7040,
     &   .6890,   .6706,   .6706,   .6706/
      DATA ((SVCS(I,J,10,2),J=1,17),I=1,6)/
     & -1.5028, -1.1695,  -.8371,  -.5071,  -.1834,   .1206,   .3644,
     &   .4936,   .5417,   .5336,   .5149,   .5024,   .4894,   .5047,
     &   .4900,   .4900,   .4900, -1.6533, -1.3200,  -.9872,  -.6554,
     &  -.3273,  -.0125,   .2653,   .4495,   .5261,   .5533,   .5363,
     &   .5237,   .5143,   .5047,   .4900,   .4900,   .4900, -1.8038,
     & -1.4705, -1.1375,  -.8055,  -.4753,  -.1522,   .1467,   .3813,
     &   .4987,   .5553,   .5545,   .5407,   .5311,   .5232,   .5159,
     &   .5159,   .5159, -1.9543, -1.6210, -1.2877,  -.9543,  -.6238,
     &  -.2963,   .0175,   .2878,   .4592,   .5358,   .5677,   .5556,
     &   .5451,   .5368,   .5296,   .5296,   .5296, -2.1048, -1.7715,
     & -1.4382, -1.1053,  -.7726,  -.4431,  -.1214,   .1744,   .3985,
     &   .5057,   .5680,   .5688,   .5576,   .5488,   .5408,   .5408,
     &   .5408, -2.2553, -1.9220, -1.5887, -1.2554,  -.9221,  -.5890,
     &  -.2570,   .0702,   .3752,   .5057,   .5680,   .5688,   .5576,
     &   .5488,   .5408,   .5408,   .5408/
      DATA ((SVCS(I,J,11,2),J=1,17),I=1,6)/
     & -1.5028, -1.1695,  -.8371,  -.5079,  -.1870,   .1048,   .3058,
     &   .3414,   .2667,   .2103,   .1956,   .1886,   .1761,   .2055,
     &   .1816,   .1816,   .1816, -1.6533, -1.3200,  -.9872,  -.6558,
     &  -.3291,  -.0207,   .2322,   .3462,   .3179,   .2440,   .2196,
     &   .2148,   .2127,   .2055,   .1816,   .1816,   .1816, -1.8038,
     & -1.4705, -1.1375,  -.8058,  -.4763,  -.1565,   .1290,   .3168,
     &   .3456,   .2844,   .2408,   .2325,   .2333,   .2345,   .2315,
     &   .2315,   .2315, -1.9543, -1.6210, -1.2877,  -.9543,  -.6242,
     &  -.2984,   .0082,   .2504,   .3499,   .3263,   .2632,   .2460,
     &   .2458,   .2499,   .2538,   .2538,   .2538, -2.1048, -1.7715,
     & -1.4382, -1.1054,  -.7729,  -.4442,  -.1263,   .1538,   .3274,
     &   .3489,   .2945,   .2579,   .2538,   .2577,   .2639,   .2639,
     &   .2639, -2.2553, -1.9220, -1.5887, -1.2554,  -.9222,  -.5896,
     &  -.2596,   .0584,   .3206,   .3489,   .2945,   .2579,   .2538,
     &   .2577,   .2639,   .2639,   .2639/
      DATA ((SVCS(I,J,12,2),J=1,17),I=1,6)/
     & -1.5028, -1.1696,  -.8377,  -.5099,  -.1961,   .0656,   .1608,
     &  -.0145,  -.1919,  -.2218,  -.2206,  -.2177,  -.2248,  -.1927,
     &  -.2169,  -.2169,  -.2169, -1.6533, -1.3200,  -.9876,  -.6568,
     &  -.3338,  -.0412,   .1492,   .0955,  -.1259,  -.2038,  -.2069,
     &  -.1983,  -.1905,  -.1927,  -.2169,  -.2169,  -.2169, -1.8038,
     & -1.4705, -1.1376,  -.8063,  -.4786,  -.1670,   .0842,   .1572,
     &  -.0221,  -.1771,  -.1991,  -.1901,  -.1761,  -.1647,  -.1623,
     &  -.1623,  -.1623, -1.9543, -1.6210, -1.2877,  -.9547,  -.6254,
     &  -.3037,  -.0154,   .1572,   .0828,  -.1253,  -.1906,  -.1881,
     &  -.1733,  -.1553,  -.1411,  -.1411,  -.1411, -2.1048, -1.7715,
     & -1.4382, -1.1055,  -.7737,  -.4468,  -.1385,   .1028,   .1517,
     &  -.0352,  -.1733,  -.1899,  -.1767,  -.1578,  -.1376,  -.1376,
     &  -.1376, -2.2553, -1.9220, -1.5887, -1.2555,  -.9225,  -.5910,
     &  -.2659,   .0289,   .1835,  -.0352,  -.1733,  -.1899,  -.1767,
     &  -.1578,  -.1376,  -.1376,  -.1376/
      DATA ((SVCS(I,J,13,2),J=1,17),I=1,6)/
     & -1.5029, -1.1698,  -.8386,  -.5150,  -.2190,  -.0327,  -.1872,
     &  -.6146,  -.7036,  -.6990,  -.6834,  -.6724,  -.6745,  -.6467,
     &  -.6668,  -.6668,  -.6668, -1.6533, -1.3201,  -.9880,  -.6594,
     &  -.3453,  -.0926,  -.0554,  -.4429,  -.6853,  -.7007,  -.6834,
     &  -.6631,  -.6490,  -.6467,  -.6668,  -.6668,  -.6668, -1.8038,
     & -1.4706, -1.1379,  -.8074,  -.4844,  -.1934,  -.0282,  -.2229,
     &  -.6297,  -.7022,  -.6910,  -.6670,  -.6436,  -.6269,  -.6211,
     &  -.6211,  -.6211, -1.9543, -1.6210, -1.2879,  -.9555,  -.6284,
     &  -.3172,  -.0746,  -.0731,  -.4814,  -.6931,  -.7022,  -.6790,
     &  -.6511,  -.6256,  -.6066,  -.6066,  -.6066, -2.1048, -1.7715,
     & -1.4383, -1.1058,  -.7749,  -.4537,  -.1690,  -.0255,  -.2635,
     &  -.6505,  -.7106,  -.6962,  -.6678,  -.6371,  -.6098,  -.6098,
     &  -.6098, -2.2553, -1.9220, -1.5887, -1.2556,  -.9232,  -.5944,
     &  -.2819,  -.0453,  -.0693,  -.6505,  -.7106,  -.6962,  -.6678,
     &  -.6371,  -.6098,  -.6098,  -.6098/
      DATA ((SVCS(I,J,14,2),J=1,17),I=1,6)/
     & -1.5030, -1.1704,  -.8416,  -.5277,  -.2765,  -.2769,  -.9003,
     & -1.1978, -1.2036, -1.1823, -1.1607, -1.1475, -1.1468, -1.1265,
     & -1.1412, -1.1412, -1.1412, -1.6534, -1.3204,  -.9897,  -.6657,
     &  -.3745,  -.2215,  -.5435, -1.1532, -1.2130, -1.1952, -1.1681,
     & -1.1448, -1.1299, -1.1265, -1.1412, -1.1412, -1.1412, -1.8039,
     & -1.4707, -1.1386,  -.8108,  -.4991,  -.2598,  -.3063,  -.9682,
     & -1.2136, -1.2123, -1.1844, -1.1543, -1.1298, -1.1142, -1.1092,
     & -1.1092, -1.1092, -1.9543, -1.6211, -1.2882,  -.9570,  -.6358,
     &  -.3510,  -.2228,  -.6144, -1.1822, -1.2278, -1.2056, -1.1730,
     & -1.1415, -1.1163, -1.1007, -1.1007, -1.1007, -2.1048, -1.7716,
     & -1.4385, -1.1066,  -.7788,  -.4707,  -.2457,  -.3419, -1.0345,
     & -1.2333, -1.2288, -1.1977, -1.1625, -1.1304, -1.1055, -1.1055,
     & -1.1055, -2.2554, -1.9220, -1.5888, -1.2560,  -.9251,  -.6031,
     &  -.3221,  -.2317,  -.8464, -1.2333, -1.2288, -1.1977, -1.1625,
     & -1.1304, -1.1055, -1.1055, -1.1055/
      DATA ((SVCS(I,J,15,2),J=1,17),I=1,6)/
     & -1.5033, -1.1719,  -.8486,  -.5596,  -.4206,  -.8645, -1.6664,
     & -1.7077, -1.6874, -1.6621, -1.6419, -1.6306, -1.6295, -1.6176,
     & -1.6258, -1.6258, -1.6258, -1.6536, -1.3212,  -.9931,  -.6817,
     &  -.4478,  -.5420, -1.4661, -1.7144, -1.7064, -1.6786, -1.6509,
     & -1.6308, -1.6198, -1.6176, -1.6258, -1.6258, -1.6258, -1.8039,
     & -1.4711, -1.1403,  -.8187,  -.5361,  -.4262,  -.9662, -1.6962,
     & -1.7256, -1.7001, -1.6686, -1.6404, -1.6207, -1.6107, -1.6084,
     & -1.6084, -1.6084, -1.9544, -1.6213, -1.2891,  -.9610,  -.6544,
     &  -.4358,  -.5904, -1.5528, -1.7371, -1.7247, -1.6919, -1.6588,
     & -1.6310, -1.6126, -1.6036, -1.6036, -1.6036, -2.1049, -1.7717,
     & -1.4389, -1.1086,  -.7881,  -.5136,  -.4377, -1.0781, -1.7272,
     & -1.7476, -1.7201, -1.6840, -1.6505, -1.6234, -1.6066, -1.6066,
     & -1.6066, -2.2554, -1.9221, -1.5890, -1.2570,  -.9298,  -.6248,
     &  -.4229,  -.6697, -1.8016, -1.7476, -1.7201, -1.6840, -1.6505,
     & -1.6234, -1.6066, -1.6066, -1.6066/
      DATA ((SVCS(I,J,16,2),J=1,17),I=1,6)/
     & -1.5041, -1.1757,  -.8658,  -.6398,  -.7802, -1.9638, -2.2068,
     & -2.1944, -2.1672, -2.1443, -2.1304, -2.1248, -2.1238, -2.1202,
     & -2.1230, -2.1230, -2.1230, -1.6540, -1.3231, -1.0017,  -.7219,
     &  -.6315, -1.3125, -2.1978, -2.2157, -2.1886, -2.1590, -2.1366,
     & -2.1240, -2.1196, -2.1202, -2.1230, -2.1230, -2.1230, -1.8041,
     & -1.4720, -1.1447,  -.8389,  -.6289,  -.8404, -2.0665, -2.2291,
     & -2.2142, -2.1811, -2.1510, -2.1299, -2.1189, -2.1158, -2.1173,
     & -2.1173, -2.1173, -1.9545, -1.6218, -1.2913,  -.9710,  -.7011,
     &  -.6482, -1.4582, -2.2269, -2.2366, -2.2087, -2.1733, -2.1439,
     & -2.1245, -2.1154, -2.1137, -2.1137, -2.1137, -2.1049, -1.7719,
     & -1.4400, -1.1137,  -.8116,  -.6212,  -.9147, -2.1460, -2.2556,
     & -2.2358, -2.2000, -2.1665, -2.1382, -2.1207, -2.1133, -2.1133,
     & -2.1133, -2.2554, -1.9222, -1.5896, -1.2596,  -.9416,  -.6794,
     &  -.6763, -1.7474, -2.3090, -2.2358, -2.2000, -2.1665, -2.1382,
     & -2.1207, -2.1133, -2.1133, -2.1133/
      DATA ((SVCS(I,J,17,2),J=1,17),I=1,6)/
     & -1.5062, -1.1851,  -.9097,  -.8407, -1.6546, -2.6902, -2.7001,
     & -2.6741, -2.6488, -2.6321, -2.6240, -2.6218, -2.6220, -2.6225,
     & -2.6229, -2.6229, -2.6229, -1.6550, -1.3278, -1.0236,  -.8230,
     & -1.0904, -2.5753, -2.7185, -2.6983, -2.6678, -2.6430, -2.6282,
     & -2.6223, -2.6216, -2.6225, -2.6229, -2.6229, -2.6229, -1.8046,
     & -1.4744, -1.1557,  -.8897,  -.8620, -1.8330, -2.7215, -2.7238,
     & -2.6927, -2.6609, -2.6377, -2.6253, -2.6214, -2.6221, -2.6238,
     & -2.6238, -2.6238, -1.9547, -1.6229, -1.2968,  -.9965,  -.8184,
     & -1.1782, -2.6576, -2.7452, -2.7212, -2.6859, -2.6544, -2.6334,
     & -2.6234, -2.6214, -2.6231, -2.6231, -2.6231, -2.1050, -1.7725,
     & -1.4427, -1.1264,  -.8703,  -.8911, -2.0283, -2.7533, -2.7503,
     & -2.7165, -2.6794, -2.6492, -2.6302, -2.6221, -2.6216, -2.6216,
     & -2.6216, -2.2555, -1.9225, -1.5910, -1.2659,  -.9711,  -.8165,
     & -1.3006, -2.8253, -2.7928, -2.7165, -2.6794, -2.6492, -2.6302,
     & -2.6221, -2.6216, -2.6216, -2.6216/
      DATA ((SVCS(I,J,18,2),J=1,17),I=1,6)/
     & -1.5113, -1.2089, -1.0195, -1.3439, -3.0787, -3.1998, -3.1795,
     & -3.1534, -3.1354, -3.1271, -3.1256, -3.1270, -3.1282, -3.1302,
     & -3.1296, -3.1296, -3.1296, -1.6575, -1.3397, -1.0786, -1.0769,
     & -2.2028, -3.2081, -3.2041, -3.1745, -3.1482, -3.1322, -3.1260,
     & -3.1262, -3.1284, -3.1302, -3.1296, -3.1296, -3.1296, -1.8059,
     & -1.4803, -1.1832, -1.0170, -1.4442, -3.1609, -3.2273, -3.2011,
     & -3.1685, -3.1433, -3.1296, -3.1256, -3.1271, -3.1302, -3.1324,
     & -3.1324, -3.1324, -1.9554, -1.6259, -1.3105, -1.0603, -1.1125,
     & -2.4351, -3.2398, -3.2300, -3.1955, -3.1623, -3.1391, -3.1278,
     & -3.1258, -3.1284, -3.1319, -3.1319, -3.1319, -2.1054, -1.7740,
     & -1.4496, -1.1583, -1.0181, -1.5642, -3.2160, -3.2563, -3.2272,
     & -3.1896, -3.1570, -3.1358, -3.1267, -3.1262, -3.1295, -3.1295,
     & -3.1295, -2.2556, -1.9232, -1.5944, -1.2819, -1.0453, -1.1582,
     & -2.7743, -3.3063, -3.2637, -3.1896, -3.1570, -3.1358, -3.1267,
     & -3.1262, -3.1295, -3.1295, -3.1295/
      DATA ((SVCS(I,J,19,2),J=1,17),I=1,6)/
     & -1.5241, -1.2685, -1.2952, -2.5745, -3.6893, -3.6830, -3.6574,
     & -3.6379, -3.6289, -3.6278, -3.6302, -3.6332, -3.6347, -3.6370,
     & -3.6405, -3.6405, -3.6405, -1.6640, -1.3695, -1.2169, -1.7124,
     & -3.6556, -3.7060, -3.6805, -3.6527, -3.6349, -3.6280, -3.6286,
     & -3.6319, -3.6351, -3.6370, -3.6405, -3.6405, -3.6405, -1.8091,
     & -1.4952, -1.2524, -1.3368, -2.8422, -3.7212, -3.7077, -3.6753,
     & -3.6480, -3.6325, -3.6276, -3.6295, -3.6334, -3.6370, -3.6390,
     & -3.6390, -3.6390, -1.9570, -1.6334, -1.3451, -1.2205, -1.8483,
     & -3.7073, -3.7345, -3.7040, -3.6694, -3.6436, -3.6304, -3.6278,
     & -3.6308, -3.6351, -3.6385, -3.6385, -3.6385, -2.1062, -1.7777,
     & -1.4669, -1.2385, -1.3889, -3.1245, -3.7533, -3.7357, -3.6990,
     & -3.6641, -3.6402, -3.6293, -3.6282, -3.6319, -3.6364, -3.6364,
     & -3.6364, -2.2560, -1.9251, -1.6031, -1.3221, -1.2317, -2.0196,
     & -3.8181, -3.7765, -3.7300, -3.6641, -3.6402, -3.6293, -3.6282,
     & -3.6319, -3.6364, -3.6364, -3.6364/
      DATA ((SVCS(I,J,20,2),J=1,17),I=1,6)/
     & -1.5564, -1.4184, -1.9867, -4.1456, -4.1840, -4.1615, -4.1406,
     & -4.1302, -4.1288, -4.1319, -4.1359, -4.1388, -4.1401, -4.1419,
     & -4.1460, -4.1460, -4.1460, -1.6801, -1.4444, -1.5638, -3.2527,
     & -4.2007, -4.1856, -4.1575, -4.1375, -4.1292, -4.1295, -4.1334,
     & -4.1375, -4.1404, -4.1419, -4.1460, -4.1460, -4.1460, -1.8172,
     & -1.5327, -1.4261, -2.1377, -4.1944, -4.2118, -4.1821, -4.1529,
     & -4.1348, -4.1288, -4.1305, -4.1349, -4.1390, -4.1419, -4.1435,
     & -4.1435, -4.1435, -1.9610, -1.6521, -1.4321, -1.6229, -3.5750,
     & -4.2326, -4.2118, -4.1769, -4.1484, -4.1327, -4.1288, -4.1316,
     & -4.1363, -4.1404, -4.1432, -4.1432, -4.1432, -2.1082, -1.7871,
     & -1.5104, -1.4401, -2.3166, -4.2343, -4.2420, -4.2081, -4.1717,
     & -4.1447, -4.1312, -4.1291, -4.1327, -4.1375, -4.1415, -4.1415,
     & -4.1415, -2.2570, -1.9298, -1.6248, -1.4229, -1.6989, -3.9584,
     & -4.2893, -4.2430, -4.1957, -4.1447, -4.1312, -4.1291, -4.1327,
     & -4.1375, -4.1415, -4.1415, -4.1415/
      DATA ((SVCS(I,J,21,2),J=1,17),I=1,6)/
     & -1.6375, -1.7943, -3.6735, -4.6792, -4.6651, -4.6434, -4.6313,
     & -4.6293, -4.6326, -4.6371, -4.6407, -4.6432, -4.6458, -4.6468,
     & -4.6488, -4.6488, -4.6488, -1.7206, -1.6326, -2.4337, -4.6782,
     & -4.6891, -4.6625, -4.6405, -4.6302, -4.6299, -4.6340, -4.6385,
     & -4.6421, -4.6444, -4.6468, -4.6488, -4.6488, -4.6488, -1.8375,
     & -1.6268, -1.8623, -4.0337, -4.7106, -4.6887, -4.6584, -4.6375,
     & -4.6297, -4.6306, -4.6353, -4.6400, -4.6432, -4.6453, -4.6475,
     & -4.6475, -4.6475, -1.9712, -1.6991, -1.6504, -2.6308, -4.7172,
     & -4.7178, -4.6847, -4.6538, -4.6351, -4.6293, -4.6317, -4.6366,
     & -4.6411, -4.6444, -4.6463, -4.6463, -4.6463, -2.1132, -1.8106,
     & -1.6196, -1.9462, -4.3696, -4.7435, -4.7172, -4.6799, -4.6499,
     & -4.6334, -4.6293, -4.6326, -4.6377, -4.6421, -4.6451, -4.6451,
     & -4.6451, -2.2596, -1.9416, -1.6794, -1.6763, -2.8712, -4.8018,
     & -4.7560, -4.7074, -4.6649, -4.6334, -4.6293, -4.6326, -4.6377,
     & -4.6421, -4.6451, -4.6451, -4.6451/
      DATA ((SVCS(I,J,22,2),J=1,17),I=1,6)/
     & -1.8411, -2.7387, -5.1600, -5.1677, -5.1468, -5.1327, -5.1294,
     & -5.1325, -5.1374, -5.1416, -5.1445, -5.1461, -5.1487, -5.1492,
     & -5.1502, -5.1502, -5.1502, -1.8225, -2.1050, -4.5051, -5.1884,
     & -5.1679, -5.1441, -5.1314, -5.1298, -5.1337, -5.1388, -5.1427,
     & -5.1454, -5.1480, -5.1492, -5.1502, -5.1502, -5.1502, -1.8884,
     & -1.8631, -2.9559, -5.1980, -5.1944, -5.1646, -5.1410, -5.1305,
     & -5.1304, -5.1350, -5.1400, -5.1438, -5.1462, -5.1485, -5.1496,
     & -5.1496, -5.1496, -1.9966, -1.8173, -2.1986, -4.8700, -5.2202,
     & -5.1928, -5.1602, -5.1382, -5.1299, -5.1312, -5.1363, -5.1412,
     & -5.1447, -5.1469, -5.1490, -5.1490, -5.1490, -2.1260, -1.8697,
     & -1.8936, -3.2129, -5.2349, -5.2249, -5.1889, -5.1560, -5.1360,
     & -5.1297, -5.1321, -5.1374, -5.1421, -5.1454, -5.1474, -5.1474,
     & -5.1474, -2.2659, -1.9711, -1.8165, -2.3124, -5.1983, -5.2693,
     & -5.2201, -5.1745, -5.1421, -5.1297, -5.1321, -5.1374, -5.1421,
     & -5.1454, -5.1474, -5.1474, -5.1474/
      DATA ((SVCS(I,J,23,2),J=1,17),I=1,6)/
     & -2.3525, -5.0186, -5.6668, -5.6503, -5.6345, -5.6295, -5.6321,
     & -5.6371, -5.6419, -5.6449, -5.6470, -5.6489, -5.6502, -5.6504,
     & -5.6510, -5.6510, -5.6510, -2.0783, -3.2910, -5.6776, -5.6724,
     & -5.6484, -5.6330, -5.6297, -5.6332, -5.6385, -5.6428, -5.6459,
     & -5.6476, -5.6498, -5.6504, -5.6510, -5.6510, -5.6510, -2.0163,
     & -2.4565, -5.3803, -5.6970, -5.6712, -5.6453, -5.6317, -5.6300,
     & -5.6343, -5.6396, -5.6438, -5.6465, -5.6482, -5.6501, -5.6506,
     & -5.6506, -5.6506, -2.0606, -2.1140, -3.5740, -5.7140, -5.7003,
     & -5.6676, -5.6421, -5.6308, -5.6306, -5.6356, -5.6407, -5.6447,
     & -5.6472, -5.6488, -5.6503, -5.6503, -5.6503, -2.1580, -2.0181,
     & -2.5824, -5.6240, -5.7300, -5.6983, -5.6633, -5.6394, -5.6302,
     & -5.6313, -5.6366, -5.6417, -5.6453, -5.6476, -5.6490, -5.6490,
     & -5.6490, -2.2819, -2.0453, -2.1608, -3.9062, -5.7830, -5.7340,
     & -5.6854, -5.6484, -5.6296, -5.6313, -5.6366, -5.6417, -5.6453,
     & -5.6476, -5.6490, -5.6490, -5.6490/
      DATA ((SVCS(I,J,24,2),J=1,17),I=1,6)/
     & -3.6366, -6.1849, -6.1534, -6.1369, -6.1297, -6.1314, -6.1366,
     & -6.1415, -6.1451, -6.1473, -6.1493, -6.1503, -6.1509, -6.1511,
     & -6.1517, -6.1517, -6.1517, -2.7206, -5.9486, -6.1746, -6.1531,
     & -6.1353, -6.1296, -6.1324, -6.1378, -6.1425, -6.1459, -6.1478,
     & -6.1496, -6.1507, -6.1511, -6.1517, -6.1517, -6.1517, -2.3375,
     & -3.9458, -6.1923, -6.1776, -6.1505, -6.1337, -6.1298, -6.1334,
     & -6.1390, -6.1435, -6.1465, -6.1484, -6.1499, -6.1509, -6.1517,
     & -6.1517, -6.1517, -2.2212, -2.8595, -6.1951, -6.2056, -6.1754,
     & -6.1472, -6.1323, -6.1301, -6.1345, -6.1401, -6.1444, -6.1471,
     & -6.1488, -6.1502, -6.1517, -6.1517, -6.1517, -2.2383, -2.3908,
     & -4.3085, -6.2284, -6.2073, -6.1717, -6.1441, -6.1313, -6.1307,
     & -6.1356, -6.1410, -6.1450, -6.1475, -6.1492, -6.1517, -6.1517,
     & -6.1517, -2.3221, -2.2317, -3.0259, -6.2864, -6.2482, -6.1982,
     & -6.1564, -6.1326, -6.1268, -6.1356, -6.1410, -6.1450, -6.1475,
     & -6.1492, -6.1517, -6.1517, -6.1517/
      DATA ((SVCS(I,J,25,2),J=1,17),I=1,6)/
     & -6.4540, -6.6550, -6.6396, -6.6302, -6.6306, -6.6356, -6.6409,
     & -6.6449, -6.6474, -6.6490, -6.6505, -6.6510, -6.6513, -6.6517,
     & -6.6517, -6.6517, -6.6517, -4.3329, -6.6892, -6.6580, -6.6383,
     & -6.6299, -6.6315, -6.6370, -6.6421, -6.6457, -6.6480, -6.6494,
     & -6.6507, -6.6512, -6.6517, -6.6517, -6.6517, -6.6517, -3.1444,
     & -6.6870, -6.6824, -6.6564, -6.6366, -6.6299, -6.6325, -6.6381,
     & -6.6430, -6.6463, -6.6484, -6.6498, -6.6508, -6.6517, -6.6517,
     & -6.6517, -6.6517, -2.6246, -4.7285, -6.7269, -6.6834, -6.6533,
     & -6.6347, -6.6299, -6.6334, -6.6392, -6.6438, -6.6469, -6.6488,
     & -6.6502, -6.6517, -6.6517, -6.6517, -6.6517, -2.4400, -3.3271,
     & -6.7527, -6.7147, -6.6809, -6.6501, -6.6332, -6.6300, -6.6343,
     & -6.6402, -6.6445, -6.6472, -6.6492, -6.6517, -6.6517, -6.6517,
     & -6.6517, -2.4229, -2.6999, -5.1882, -6.7631, -6.7118, -6.6661,
     & -6.6369, -6.6266, -6.6294, -6.6402, -6.6445, -6.6472, -6.6492,
     & -6.6517, -6.6517, -6.6517, -6.6517/
      DATA ((SVCS(I,J,26,2),J=1,17),I=1,6)/
     & -7.1377, -7.1352, -7.1312, -7.1299, -7.1345, -7.1402, -7.1445,
     & -7.1472, -7.1490, -7.1503, -7.1511, -7.1513, -7.1515, -7.1517,
     & -7.1517, -7.1517, -7.1517, -7.1457, -7.1580, -7.1421, -7.1308,
     & -7.1306, -7.1357, -7.1412, -7.1453, -7.1478, -7.1494, -7.1505,
     & -7.1512, -7.1514, -7.1517, -7.1517, -7.1517, -7.1517, -5.1676,
     & -7.1946, -7.1628, -7.1403, -7.1303, -7.1314, -7.1369, -7.1422,
     & -7.1459, -7.1483, -7.1497, -7.1507, -7.1512, -7.1517, -7.1517,
     & -7.1517, -7.1517, -3.6381, -7.2172, -7.1913, -7.1605, -7.1381,
     & -7.1300, -7.1322, -7.1379, -7.1431, -7.1465, -7.1486, -7.1499,
     & -7.1510, -7.1517, -7.1517, -7.1517, -7.1517, -2.9467, -5.6674,
     & -7.2146, -7.1903, -7.1573, -7.1362, -7.1299, -7.1331, -7.1389,
     & -7.1438, -7.1470, -7.1490, -7.1501, -7.1517, -7.1517, -7.1517,
     & -7.1517, -2.6763, -3.8758, -7.2377, -7.2268, -7.1778, -7.1427,
     & -7.1278, -7.1283, -7.1347, -7.1438, -7.1470, -7.1490, -7.1501,
     & -7.1517, -7.1517, -7.1517, -7.1517/
      DATA ((SVCS(I,J,27,2),J=1,17),I=1,6)/
     & -7.6229, -7.6264, -7.6293, -7.6332, -7.6390, -7.6440, -7.6470,
     & -7.6490, -7.6501, -7.6510, -7.6514, -7.6515, -7.6517, -7.6517,
     & -7.6517, -7.6517, -7.6517, -7.6408, -7.6372, -7.6325, -7.6299,
     & -7.6343, -7.6402, -7.6447, -7.6474, -7.6492, -7.6505, -7.6511,
     & -7.6514, -7.6517, -7.6517, -7.6517, -7.6517, -7.6517, -7.6763,
     & -7.6623, -7.6451, -7.6315, -7.6304, -7.6354, -7.6411, -7.6453,
     & -7.6480, -7.6496, -7.6507, -7.6512, -7.6517, -7.6517, -7.6517,
     & -7.6517, -7.6517, -6.1708, -7.6780, -7.6608, -7.6428, -7.6310,
     & -7.6312, -7.6366, -7.6421, -7.6461, -7.6484, -7.6498, -7.6508,
     & -7.6517, -7.6517, -7.6517, -7.6517, -7.6517, -4.2195, -7.6813,
     & -7.6744, -7.6655, -7.6404, -7.6304, -7.6319, -7.6375, -7.6428,
     & -7.6465, -7.6486, -7.6499, -7.6517, -7.6517, -7.6517, -7.6517,
     & -7.6517, -3.3127, -6.7733, -7.6880, -7.6912, -7.6506, -7.6296,
     & -7.6272, -7.6329, -7.6395, -7.6465, -7.6486, -7.6499, -7.6517,
     & -7.6517, -7.6517, -7.6517, -7.6517/
      DATA ((SVCS(I,J,28,2),J=1,17),I=1,6)/
     & -8.1208, -8.1265, -8.1319, -8.1379, -8.1430, -8.1465, -8.1487,
     & -8.1500, -8.1509, -8.1513, -8.1515, -8.1516, -8.1517, -8.1517,
     & -8.1517, -8.1517, -8.1517, -8.1238, -8.1269, -8.1295, -8.1329,
     & -8.1389, -8.1439, -8.1470, -8.1490, -8.1502, -8.1511, -8.1514,
     & -8.1515, -8.1517, -8.1517, -8.1517, -8.1517, -8.1517, -8.1439,
     & -8.1393, -8.1338, -8.1298, -8.1340, -8.1400, -8.1446, -8.1475,
     & -8.1494, -8.1504, -8.1512, -8.1514, -8.1517, -8.1517, -8.1517,
     & -8.1517, -8.1517, -8.1520, -8.1457, -8.1394, -8.1327, -8.1303,
     & -8.1351, -8.1409, -8.1453, -8.1479, -8.1497, -8.1505, -8.1512,
     & -8.1517, -8.1517, -8.1517, -8.1517, -8.1517, -7.3416, -8.1415,
     & -8.1441, -8.1460, -8.1318, -8.1308, -8.1360, -8.1418, -8.1459,
     & -8.1484, -8.1498, -8.1507, -8.1517, -8.1517, -8.1517, -8.1517,
     & -8.1517, -4.9112, -8.1373, -8.1488, -8.1603, -8.1334, -8.1265,
     & -8.1311, -8.1383, -8.1439, -8.1484, -8.1498, -8.1507, -8.1517,
     & -8.1517, -8.1517, -8.1517, -8.1517/
      DATA ((SVCS(I,J,29,2),J=1,17),I=1,6)/
     & -8.6256, -8.6311, -8.6364, -8.6421, -8.6459, -8.6484, -8.6498,
     & -8.6508, -8.6513, -8.6515, -8.6516, -8.6517, -8.6517, -8.6517,
     & -8.6517, -8.6517, -8.6517, -8.6201, -8.6259, -8.6315, -8.6375,
     & -8.6428, -8.6465, -8.6488, -8.6501, -8.6509, -8.6514, -8.6515,
     & -8.6517, -8.6517, -8.6517, -8.6517, -8.6517, -8.6517, -8.6254,
     & -8.6278, -8.6299, -8.6325, -8.6386, -8.6438, -8.6470, -8.6492,
     & -8.6503, -8.6510, -8.6514, -8.6517, -8.6517, -8.6517, -8.6517,
     & -8.6517, -8.6517, -8.6261, -8.6274, -8.6287, -8.6299, -8.6336,
     & -8.6396, -8.6444, -8.6474, -8.6494, -8.6505, -8.6511, -8.6517,
     & -8.6517, -8.6517, -8.6517, -8.6517, -8.6517, -8.6126, -8.6199,
     & -8.6272, -8.6341, -8.6300, -8.6345, -8.6404, -8.6451, -8.6478,
     & -8.6496, -8.6505, -8.6517, -8.6517, -8.6517, -8.6517, -8.6517,
     & -8.6517, -8.4318, -8.6124, -8.6257, -8.6390, -8.6268, -8.6294,
     & -8.6364, -8.6428, -8.6462, -8.6496, -8.6505, -8.6517, -8.6517,
     & -8.6517, -8.6517, -8.6517, -8.6517/
      DATA ((SVCS(I,J,30,2),J=1,17),I=1,6)/
     & -9.1321, -9.1365, -9.1408, -9.1453, -9.1479, -9.1497, -9.1506,
     & -9.1512, -9.1515, -9.1516, -9.1517, -9.1517, -9.1517, -9.1517,
     & -9.1517, -9.1517, -9.1517, -9.1246, -9.1303, -9.1359, -9.1417,
     & -9.1459, -9.1484, -9.1499, -9.1507, -9.1513, -9.1515, -9.1517,
     & -9.1517, -9.1517, -9.1517, -9.1517, -9.1517, -9.1517, -9.1202,
     & -9.1258, -9.1312, -9.1370, -9.1426, -9.1464, -9.1487, -9.1501,
     & -9.1508, -9.1514, -9.1517, -9.1517, -9.1517, -9.1517, -9.1517,
     & -9.1517, -9.1517, -9.1162, -9.1215, -9.1268, -9.1321, -9.1380,
     & -9.1434, -9.1469, -9.1490, -9.1503, -9.1511, -9.1517, -9.1517,
     & -9.1517, -9.1517, -9.1517, -9.1517, -9.1517, -9.1062, -9.1142,
     & -9.1222, -9.1301, -9.1329, -9.1389, -9.1441, -9.1473, -9.1493,
     & -9.1504, -9.1517, -9.1517, -9.1517, -9.1517, -9.1517, -9.1517,
     & -9.1517, -9.0962, -9.1069, -9.1176, -9.1283, -9.1278, -9.1344,
     & -9.1413, -9.1456, -9.1483, -9.1504, -9.1517, -9.1517, -9.1517,
     & -9.1517, -9.1517, -9.1517, -9.1517/
      DATA ((SVCS(I,J,31,2),J=1,17),I=1,6)/
     & -9.6384, -9.6414, -9.6444, -9.6474, -9.6494, -9.6505, -9.6511,
     & -9.6515, -9.6516, -9.6517, -9.6517, -9.6517, -9.6517, -9.6517,
     & -9.6517, -9.6517, -9.6517, -9.6310, -9.6357, -9.6404, -9.6451,
     & -9.6478, -9.6498, -9.6505, -9.6512, -9.6515, -9.6517, -9.6517,
     & -9.6517, -9.6517, -9.6517, -9.6517, -9.6517, -9.6517, -9.6233,
     & -9.6293, -9.6353, -9.6413, -9.6457, -9.6484, -9.6499, -9.6507,
     & -9.6512, -9.6517, -9.6517, -9.6517, -9.6517, -9.6517, -9.6517,
     & -9.6517, -9.6517, -9.6178, -9.6240, -9.6302, -9.6364, -9.6421,
     & -9.6463, -9.6486, -9.6501, -9.6509, -9.6517, -9.6517, -9.6517,
     & -9.6517, -9.6517, -9.6517, -9.6517, -9.6517, -9.6123, -9.6187,
     & -9.6251, -9.6313, -9.6373, -9.6428, -9.6467, -9.6490, -9.6503,
     & -9.6517, -9.6517, -9.6517, -9.6517, -9.6517, -9.6517, -9.6517,
     & -9.6517, -9.6068, -9.6134, -9.6200, -9.6266, -9.6325, -9.6393,
     & -9.6448, -9.6479, -9.6497, -9.6517, -9.6517, -9.6517, -9.6517,
     & -9.6517, -9.6517, -9.6517, -9.6517/
      DATA ((SVCS(I,J,32,2),J=1,17),I=1,6)/
     &-10.1424,-10.1446,-10.1468,-10.1490,-10.1503,-10.1509,-10.1514,
     &-10.1516,-10.1517,-10.1517,-10.1517,-10.1517,-10.1517,-10.1517,
     &-10.1517,-10.1517,-10.1517,-10.1374,-10.1407,-10.1440,-10.1473,
     &-10.1493,-10.1505,-10.1511,-10.1514,-10.1517,-10.1517,-10.1517,
     &-10.1517,-10.1517,-10.1517,-10.1517,-10.1517,-10.1517,-10.1300,
     &-10.1349,-10.1397,-10.1447,-10.1478,-10.1496,-10.1506,-10.1512,
     &-10.1517,-10.1517,-10.1517,-10.1517,-10.1517,-10.1517,-10.1517,
     &-10.1517,-10.1517,-10.1254,-10.1305,-10.1356,-10.1407,-10.1453,
     &-10.1481,-10.1498,-10.1507,-10.1517,-10.1517,-10.1517,-10.1517,
     &-10.1517,-10.1517,-10.1517,-10.1517,-10.1517,-10.1232,-10.1273,
     &-10.1314,-10.1354,-10.1415,-10.1458,-10.1484,-10.1500,-10.1517,
     &-10.1517,-10.1517,-10.1517,-10.1517,-10.1517,-10.1517,-10.1517,
     &-10.1517,-10.1210,-10.1241,-10.1272,-10.1303,-10.1377,-10.1435,
     &-10.1470,-10.1493,-10.1517,-10.1517,-10.1517,-10.1517,-10.1517,
     &-10.1517,-10.1517,-10.1517,-10.1517/
      DATA ((SVCS(I,J,33,2),J=1,17),I=1,6)/
     &-10.6456,-10.6471,-10.6486,-10.6501,-10.6509,-10.6513,-10.6515,
     &-10.6517,-10.6517,-10.6517,-10.6517,-10.6517,-10.6517,-10.6517,
     &-10.6517,-10.6517,-10.6517,-10.6421,-10.6444,-10.6467,-10.6490,
     &-10.6503,-10.6509,-10.6514,-10.6517,-10.6517,-10.6517,-10.6517,
     &-10.6517,-10.6517,-10.6517,-10.6517,-10.6517,-10.6517,-10.6368,
     &-10.6402,-10.6436,-10.6470,-10.6492,-10.6503,-10.6511,-10.6517,
     &-10.6517,-10.6517,-10.6517,-10.6517,-10.6517,-10.6517,-10.6517,
     &-10.6517,-10.6517,-10.6331,-10.6368,-10.6405,-10.6442,-10.6476,
     &-10.6494,-10.6505,-10.6517,-10.6517,-10.6517,-10.6517,-10.6517,
     &-10.6517,-10.6517,-10.6517,-10.6517,-10.6517,-10.6322,-10.6348,
     &-10.6374,-10.6398,-10.6447,-10.6478,-10.6498,-10.6517,-10.6517,
     &-10.6517,-10.6517,-10.6517,-10.6517,-10.6517,-10.6517,-10.6517,
     &-10.6517,-10.6313,-10.6328,-10.6343,-10.6358,-10.6418,-10.6462,
     &-10.6491,-10.6517,-10.6517,-10.6517,-10.6517,-10.6517,-10.6517,
     &-10.6517,-10.6517,-10.6517,-10.6517/
      DATA ((SVCS(I,J,34,2),J=1,17),I=1,6)/
     &-11.1480,-11.1489,-11.1498,-11.1507,-11.1513,-11.1515,-11.1516,
     &-11.1517,-11.1517,-11.1517,-11.1517,-11.1517,-11.1517,-11.1517,
     &-11.1517,-11.1517,-11.1517,-11.1452,-11.1468,-11.1484,-11.1500,
     &-11.1508,-11.1513,-11.1515,-11.1517,-11.1517,-11.1517,-11.1517,
     &-11.1517,-11.1517,-11.1517,-11.1517,-11.1517,-11.1517,-11.1413,
     &-11.1438,-11.1463,-11.1488,-11.1502,-11.1509,-11.1514,-11.1517,
     &-11.1517,-11.1517,-11.1517,-11.1517,-11.1517,-11.1517,-11.1517,
     &-11.1517,-11.1517,-11.1392,-11.1417,-11.1442,-11.1467,-11.1490,
     &-11.1503,-11.1510,-11.1517,-11.1517,-11.1517,-11.1517,-11.1517,
     &-11.1517,-11.1517,-11.1517,-11.1517,-11.1517,-11.1391,-11.1406,
     &-11.1421,-11.1436,-11.1472,-11.1493,-11.1505,-11.1517,-11.1517,
     &-11.1517,-11.1517,-11.1517,-11.1517,-11.1517,-11.1517,-11.1517,
     &-11.1517,-11.1390,-11.1395,-11.1400,-11.1405,-11.1454,-11.1483,
     &-11.1500,-11.1517,-11.1517,-11.1517,-11.1517,-11.1517,-11.1517,
     &-11.1517,-11.1517,-11.1517,-11.1517/
      DATA ((SVCS(I,J,35,2),J=1,17),I=1,6)/
     &-11.6493,-11.6499,-11.6505,-11.6511,-11.6515,-11.6516,-11.6517,
     &-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,
     &-11.6517,-11.6517,-11.6517,-11.6480,-11.6489,-11.6498,-11.6507,
     &-11.6511,-11.6515,-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,
     &-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,-11.6448,
     &-11.6465,-11.6482,-11.6499,-11.6507,-11.6513,-11.6517,-11.6517,
     &-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,
     &-11.6517,-11.6517,-11.6426,-11.6446,-11.6466,-11.6486,-11.6501,
     &-11.6509,-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,
     &-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,-11.6424,-11.6437,
     &-11.6450,-11.6463,-11.6488,-11.6503,-11.6517,-11.6517,-11.6517,
     &-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,
     &-11.6517,-11.6422,-11.6428,-11.6434,-11.6440,-11.6475,-11.6497,
     &-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,-11.6517,
     &-11.6517,-11.6517,-11.6517,-11.6517/
      DATA ((SVCS(I,J,36,2),J=1,17),I=1,6)/
     &-12.1502,-12.1506,-12.1510,-12.1514,-12.1516,-12.1516,-12.1517,
     &-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,
     &-12.1517,-12.1517,-12.1517,-12.1493,-12.1499,-12.1505,-12.1511,
     &-12.1514,-12.1516,-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,
     &-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,-12.1473,
     &-12.1484,-12.1495,-12.1506,-12.1512,-12.1515,-12.1517,-12.1517,
     &-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,
     &-12.1517,-12.1517,-12.1459,-12.1472,-12.1485,-12.1498,-12.1507,
     &-12.1513,-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,
     &-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,-12.1459,-12.1467,
     &-12.1475,-12.1483,-12.1499,-12.1508,-12.1517,-12.1517,-12.1517,
     &-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,
     &-12.1517,-12.1459,-12.1462,-12.1465,-12.1468,-12.1491,-12.1503,
     &-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,-12.1517,
     &-12.1517,-12.1517,-12.1517,-12.1517/
      DATA ((SVCS(I,J,37,2),J=1,17),I=1,6)/
     &-12.6509,-12.6511,-12.6513,-12.6515,-12.6516,-12.6517,-12.6517,
     &-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,
     &-12.6517,-12.6517,-12.6517,-12.6499,-12.6504,-12.6509,-12.6514,
     &-12.6515,-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,
     &-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,-12.6487,
     &-12.6495,-12.6503,-12.6511,-12.6514,-12.6517,-12.6517,-12.6517,
     &-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,
     &-12.6517,-12.6517,-12.6481,-12.6489,-12.6497,-12.6505,-12.6511,
     &-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,
     &-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,-12.6481,-12.6486,
     &-12.6491,-12.6496,-12.6505,-12.6517,-12.6517,-12.6517,-12.6517,
     &-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,
     &-12.6517,-12.6481,-12.6483,-12.6485,-12.6487,-12.6499,-12.6517,
     &-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,-12.6517,
     &-12.6517,-12.6517,-12.6517,-12.6517/
      DATA ((SVCS(I,J,38,2),J=1,17),I=1,6)/
     &-13.1513,-13.1514,-13.1515,-13.1516,-13.1517,-13.1517,-13.1517,
     &-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,
     &-13.1517,-13.1517,-13.1517,-13.1509,-13.1511,-13.1513,-13.1515,
     &-13.1516,-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,
     &-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,-13.1501,
     &-13.1505,-13.1509,-13.1513,-13.1516,-13.1517,-13.1517,-13.1517,
     &-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,
     &-13.1517,-13.1517,-13.1495,-13.1500,-13.1505,-13.1510,-13.1514,
     &-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,
     &-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,-13.1495,-13.1498,
     &-13.1501,-13.1504,-13.1511,-13.1517,-13.1517,-13.1517,-13.1517,
     &-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,
     &-13.1517,-13.1495,-13.1496,-13.1497,-13.1498,-13.1508,-13.1517,
     &-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,-13.1517,
     &-13.1517,-13.1517,-13.1517,-13.1517/
      DATA ((SVCS(I,J,39,2),J=1,17),I=1,6)/
     &-13.6516,-13.6516,-13.6516,-13.6516,-13.6517,-13.6517,-13.6517,
     &-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,
     &-13.6517,-13.6517,-13.6517,-13.6513,-13.6514,-13.6515,-13.6516,
     &-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,
     &-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,-13.6509,
     &-13.6511,-13.6513,-13.6515,-13.6517,-13.6517,-13.6517,-13.6517,
     &-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,
     &-13.6517,-13.6517,-13.6507,-13.6509,-13.6511,-13.6513,-13.6517,
     &-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,
     &-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,-13.6509,-13.6509,
     &-13.6509,-13.6509,-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,
     &-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,
     &-13.6517,-13.6511,-13.6509,-13.6507,-13.6505,-13.6517,-13.6517,
     &-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,-13.6517,
     &-13.6517,-13.6517,-13.6517,-13.6517/
      DATA ((SVCS(I,J,40,2),J=1,17),I=1,6)/
     &-14.1514,-14.1515,-14.1516,-14.1517,-14.1517,-14.1517,-14.1517,
     &-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,
     &-14.1517,-14.1517,-14.1517,-14.1516,-14.1516,-14.1516,-14.1516,
     &-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,
     &-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,-14.1510,
     &-14.1512,-14.1514,-14.1516,-14.1517,-14.1517,-14.1517,-14.1517,
     &-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,
     &-14.1517,-14.1517,-14.1506,-14.1509,-14.1512,-14.1515,-14.1517,
     &-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,
     &-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,-14.1504,-14.1507,
     &-14.1510,-14.1513,-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,
     &-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,
     &-14.1517,-14.1502,-14.1505,-14.1508,-14.1511,-14.1517,-14.1517,
     &-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,-14.1517,
     &-14.1517,-14.1517,-14.1517,-14.1517/
      DATA ((SVCS(I,J, 1,3),J=1,17),I=1,6)/
     & -1.4536, -1.1202,  -.7897,  -.4614,  -.1433,   .1471,   .3771,
     &   .5193,   .6006,   .6500,   .6624,   .6672,   .7018,   .6551,
     &   .6551,   .6551,   .6551, -1.6041, -1.2707,  -.9382,  -.6091,
     &  -.2842,   .0208,   .2826,   .4618,   .5607,   .6248,   .6490,
     &   .6399,   .6314,   .6551,   .6551,   .6551,   .6551, -1.7546,
     & -1.4212, -1.0884,  -.7575,  -.4299,  -.1131,   .1729,   .3913,
     &   .5186,   .5954,   .6411,   .6378,   .6131,   .6000,   .6000,
     &   .6000,   .6000, -1.9051, -1.5718, -1.2384,  -.9073,  -.5764,
     &  -.2537,   .0492,   .3041,   .4697,   .5610,   .6267,   .6445,
     &   .6175,   .5883,   .5883,   .5883,   .5883, -2.0556, -1.7223,
     & -1.3889, -1.0564,  -.7254,  -.3973,  -.0833,   .1978,   .4072,
     &   .5240,   .6024,   .6496,   .6355,   .6002,   .6002,   .6002,
     &   .6002, -2.2061, -1.8728, -1.5395, -1.2061,  -.8728,  -.5395,
     &  -.2061,   .1272,   .4072,   .5240,   .6024,   .6496,   .6355,
     &   .6002,   .6002,   .6002,   .6002/
      DATA ((SVCS(I,J, 2,3),J=1,17),I=1,6)/
     & -1.4536, -1.1202,  -.7897,  -.4614,  -.1433,   .1471,   .3771,
     &   .5193,   .6006,   .6499,   .6622,   .6671,   .7015,   .6548,
     &   .6548,   .6548,   .6548, -1.6041, -1.2707,  -.9382,  -.6091,
     &  -.2842,   .0208,   .2826,   .4618,   .5607,   .6248,   .6489,
     &   .6398,   .6312,   .6548,   .6548,   .6548,   .6548, -1.7546,
     & -1.4212, -1.0884,  -.7575,  -.4299,  -.1131,   .1729,   .3913,
     &   .5186,   .5953,   .6410,   .6376,   .6128,   .5998,   .5998,
     &   .5998,   .5998, -1.9051, -1.5718, -1.2384,  -.9073,  -.5764,
     &  -.2537,   .0492,   .3041,   .4697,   .5610,   .6266,   .6443,
     &   .6172,   .5881,   .5881,   .5881,   .5881, -2.0556, -1.7223,
     & -1.3889, -1.0564,  -.7254,  -.3973,  -.0833,   .1978,   .4072,
     &   .5240,   .6023,   .6494,   .6352,   .6000,   .6000,   .6000,
     &   .6000, -2.2061, -1.8728, -1.5395, -1.2061,  -.8728,  -.5395,
     &  -.2061,   .1272,   .4072,   .5240,   .6023,   .6494,   .6352,
     &   .6000,   .6000,   .6000,   .6000/
      DATA ((SVCS(I,J, 3,3),J=1,17),I=1,6)/
     & -1.4536, -1.1202,  -.7897,  -.4614,  -.1433,   .1471,   .3771,
     &   .5192,   .6003,   .6498,   .6621,   .6670,   .7011,   .6546,
     &   .6546,   .6546,   .6546, -1.6041, -1.2707,  -.9382,  -.6091,
     &  -.2842,   .0208,   .2824,   .4616,   .5605,   .6247,   .6488,
     &   .6397,   .6309,   .6546,   .6546,   .6546,   .6546, -1.7546,
     & -1.4212, -1.0884,  -.7575,  -.4299,  -.1131,   .1729,   .3913,
     &   .5185,   .5953,   .6409,   .6375,   .6126,   .5996,   .5996,
     &   .5996,   .5996, -1.9051, -1.5718, -1.2384,  -.9073,  -.5764,
     &  -.2537,   .0492,   .3041,   .4697,   .5610,   .6265,   .6442,
     &   .6169,   .5879,   .5879,   .5879,   .5879, -2.0556, -1.7223,
     & -1.3889, -1.0564,  -.7254,  -.3973,  -.0833,   .1978,   .4072,
     &   .5240,   .6023,   .6493,   .6348,   .5997,   .5997,   .5997,
     &   .5997, -2.2061, -1.8728, -1.5395, -1.2061,  -.8728,  -.5395,
     &  -.2061,   .1272,   .4072,   .5240,   .6023,   .6493,   .6348,
     &   .5997,   .5997,   .5997,   .5997/
      DATA ((SVCS(I,J, 4,3),J=1,17),I=1,6)/
     & -1.4536, -1.1202,  -.7897,  -.4614,  -.1433,   .1471,   .3769,
     &   .5189,   .5996,   .6475,   .6589,   .6637,   .7008,   .6543,
     &   .6543,   .6543,   .6543, -1.6041, -1.2707,  -.9382,  -.6091,
     &  -.2843,   .0208,   .2824,   .4615,   .5600,   .6232,   .6455,
     &   .6365,   .6307,   .6543,   .6543,   .6543,   .6543, -1.7546,
     & -1.4212, -1.0884,  -.7575,  -.4299,  -.1131,   .1729,   .3913,
     &   .5183,   .5943,   .6381,   .6340,   .6123,   .5994,   .5994,
     &   .5994,   .5994, -1.9051, -1.5718, -1.2384,  -.9073,  -.5764,
     &  -.2537,   .0492,   .3041,   .4695,   .5605,   .6246,   .6405,
     &   .6166,   .5877,   .5877,   .5877,   .5877, -2.0556, -1.7223,
     & -1.3889, -1.0564,  -.7254,  -.3973,  -.0833,   .1978,   .4071,
     &   .5237,   .6010,   .6458,   .6344,   .5994,   .5994,   .5994,
     &   .5994, -2.2061, -1.8728, -1.5395, -1.2061,  -.8728,  -.5395,
     &  -.2061,   .1272,   .4071,   .5237,   .6010,   .6458,   .6344,
     &   .5994,   .5994,   .5994,   .5994/
      DATA ((SVCS(I,J, 5,3),J=1,17),I=1,6)/
     & -1.4536, -1.1202,  -.7897,  -.4614,  -.1433,   .1471,   .3768,
     &   .5184,   .5976,   .6419,   .6508,   .6558,   .6918,   .6477,
     &   .6477,   .6477,   .6477, -1.6041, -1.2707,  -.9382,  -.6091,
     &  -.2843,   .0208,   .2824,   .4612,   .5589,   .6192,   .6378,
     &   .6288,   .6242,   .6477,   .6477,   .6477,   .6477, -1.7546,
     & -1.4212, -1.0884,  -.7575,  -.4299,  -.1131,   .1729,   .3909,
     &   .5176,   .5917,   .6312,   .6255,   .6055,   .5944,   .5944,
     &   .5944,   .5944, -1.9051, -1.5718, -1.2384,  -.9073,  -.5764,
     &  -.2538,   .0492,   .3038,   .4691,   .5591,   .6195,   .6313,
     &   .6085,   .5821,   .5821,   .5821,   .5821, -2.0556, -1.7223,
     & -1.3889, -1.0564,  -.7254,  -.3973,  -.0833,   .1978,   .4069,
     &   .5230,   .5978,   .6372,   .6243,   .5922,   .5922,   .5922,
     &   .5922, -2.2061, -1.8728, -1.5395, -1.2061,  -.8728,  -.5395,
     &  -.2061,   .1271,   .4069,   .5230,   .5978,   .6372,   .6243,
     &   .5922,   .5922,   .5922,   .5922/
      DATA ((SVCS(I,J, 6,3),J=1,17),I=1,6)/
     & -1.4536, -1.1202,  -.7897,  -.4614,  -.1433,   .1471,   .3762,
     &   .5168,   .5926,   .6284,   .6326,   .6380,   .6720,   .6327,
     &   .6327,   .6327,   .6327, -1.6041, -1.2707,  -.9382,  -.6091,
     &  -.2843,   .0208,   .2819,   .4601,   .5561,   .6092,   .6198,
     &   .6112,   .6094,   .6327,   .6327,   .6327,   .6327, -1.7546,
     & -1.4212, -1.0884,  -.7575,  -.4299,  -.1132,   .1726,   .3904,
     &   .5160,   .5856,   .6150,   .6061,   .5901,   .5824,   .5824,
     &   .5824,   .5824, -1.9051, -1.5718, -1.2384,  -.9073,  -.5764,
     &  -.2538,   .0492,   .3034,   .4681,   .5557,   .6069,   .6102,
     &   .5899,   .5690,   .5690,   .5690,   .5690, -2.0556, -1.7223,
     & -1.3889, -1.0564,  -.7254,  -.3973,  -.0834,   .1976,   .4062,
     &   .5211,   .5899,   .6169,   .6012,   .5752,   .5752,   .5752,
     &   .5752, -2.2061, -1.8728, -1.5395, -1.2061,  -.8728,  -.5395,
     &  -.2062,   .1270,   .4062,   .5211,   .5899,   .6169,   .6012,
     &   .5752,   .5752,   .5752,   .5752/
      DATA ((SVCS(I,J, 7,3),J=1,17),I=1,6)/
     & -1.4536, -1.1202,  -.7897,  -.4614,  -.1435,   .1464,   .3747,
     &   .5130,   .5807,   .5988,   .5957,   .6034,   .6351,   .6025,
     &   .6025,   .6025,   .6025, -1.6041, -1.2707,  -.9382,  -.6091,
     &  -.2844,   .0208,   .2810,   .4577,   .5491,   .5865,   .5824,
     &   .5759,   .5793,   .6025,   .6025,   .6025,   .6025, -1.7546,
     & -1.4212, -1.0884,  -.7575,  -.4300,  -.1133,   .1723,   .3888,
     &   .5119,   .5710,   .5796,   .5664,   .5582,   .5570,   .5570,
     &   .5570,   .5570, -1.9051, -1.5718, -1.2384,  -.9073,  -.5764,
     &  -.2538,   .0488,   .3025,   .4654,   .5475,   .5788,   .5666,
     &   .5523,   .5413,   .5413,   .5413,   .5413, -2.0556, -1.7223,
     & -1.3889, -1.0564,  -.7254,  -.3973,  -.0835,   .1970,   .4045,
     &   .5167,   .5716,   .5733,   .5551,   .5405,   .5405,   .5405,
     &   .5405, -2.2061, -1.8728, -1.5395, -1.2061,  -.8728,  -.5395,
     &  -.2063,   .1266,   .4045,   .5167,   .5716,   .5733,   .5551,
     &   .5405,   .5405,   .5405,   .5405/
      DATA ((SVCS(I,J, 8,3),J=1,17),I=1,6)/
     & -1.4536, -1.1202,  -.7897,  -.4616,  -.1438,   .1452,   .3711,
     &   .5035,   .5536,   .5451,   .5397,   .5517,   .5821,   .5551,
     &   .5551,   .5551,   .5551, -1.6041, -1.2707,  -.9382,  -.6091,
     &  -.2845,   .0199,   .2788,   .4516,   .5325,   .5406,   .5221,
     &   .5218,   .5316,   .5551,   .5551,   .5551,   .5551, -1.7546,
     & -1.4212, -1.0885,  -.7575,  -.4300,  -.1136,   .1708,   .3847,
     &   .5019,   .5387,   .5173,   .5056,   .5073,   .5140,   .5140,
     &   .5140,   .5140, -1.9051, -1.5718, -1.2384,  -.9073,  -.5764,
     &  -.2540,   .0484,   .2999,   .4589,   .5281,   .5234,   .4979,
     &   .4936,   .4946,   .4946,   .4946,   .4946, -2.0556, -1.7223,
     & -1.3889, -1.0564,  -.7254,  -.3974,  -.0839,   .1956,   .4002,
     &   .5054,   .5320,   .5000,   .4857,   .4844,   .4844,   .4844,
     &   .4844, -2.2061, -1.8728, -1.5395, -1.2061,  -.8728,  -.5395,
     &  -.2065,   .1257,   .4002,   .5054,   .5320,   .5000,   .4857,
     &   .4844,   .4844,   .4844,   .4844/
      DATA ((SVCS(I,J, 9,3),J=1,17),I=1,6)/
     & -1.4536, -1.1202,  -.7897,  -.4617,  -.1444,   .1427,   .3617,
     &   .4802,   .5005,   .4803,   .4829,   .4954,   .5206,   .4982,
     &   .4982,   .4982,   .4982, -1.6041, -1.2707,  -.9382,  -.6092,
     &  -.2849,   .0183,   .2732,   .4363,   .4951,   .4735,   .4616,
     &   .4664,   .4773,   .4982,   .4982,   .4982,   .4982, -1.7546,
     & -1.4213, -1.0885,  -.7575,  -.4303,  -.1144,   .1679,   .3747,
     &   .4774,   .4802,   .4499,   .4473,   .4524,   .4613,   .4613,
     &   .4613,   .4613, -1.9051, -1.5718, -1.2384,  -.9073,  -.5766,
     &  -.2544,   .0465,   .2938,   .4428,   .4867,   .4506,   .4346,
     &   .4355,   .4403,   .4403,   .4403,   .4403, -2.0556, -1.7223,
     & -1.3889, -1.0564,  -.7254,  -.3976,  -.0848,   .1920,   .3895,
     &   .4790,   .4657,   .4291,   .4241,   .4263,   .4263,   .4263,
     &   .4263, -2.2061, -1.8728, -1.5395, -1.2061,  -.8728,  -.5396,
     &  -.2070,   .1233,   .3895,   .4790,   .4657,   .4291,   .4241,
     &   .4263,   .4263,   .4263,   .4263/
      DATA ((SVCS(I,J,10,3),J=1,17),I=1,6)/
     & -1.4536, -1.1202,  -.7899,  -.4621,  -.1462,   .1355,   .3387,
     &   .4249,   .4181,   .4153,   .4170,   .4201,   .4265,   .4190,
     &   .4190,   .4190,   .4190, -1.6041, -1.2707,  -.9385,  -.6094,
     &  -.2858,   .0145,   .2594,   .3985,   .4216,   .4077,   .4079,
     &   .4081,   .4111,   .4190,   .4190,   .4190,   .4190, -1.7546,
     & -1.4213, -1.0885,  -.7577,  -.4307,  -.1165,   .1599,   .3501,
     &   .4213,   .4055,   .4012,   .3995,   .3986,   .4011,   .4011,
     &   .4011,   .4011, -1.9051, -1.5718, -1.2384,  -.9073,  -.5768,
     &  -.2554,   .0422,   .2785,   .4031,   .4133,   .3969,   .3945,
     &   .3908,   .3890,   .3890,   .3890,   .3890, -2.0556, -1.7223,
     & -1.3890, -1.0565,  -.7256,  -.3982,  -.0871,   .1830,   .3627,
     &   .4203,   .3972,   .3922,   .3876,   .3827,   .3827,   .3827,
     &   .3827, -2.2061, -1.8728, -1.5395, -1.2061,  -.8729,  -.5399,
     &  -.2082,   .1174,   .3627,   .4203,   .3972,   .3922,   .3876,
     &   .3827,   .3827,   .3827,   .3827/
      DATA ((SVCS(I,J,11,3),J=1,17),I=1,6)/
     & -1.4536, -1.1203,  -.7899,  -.4631,  -.1506,   .1179,   .2813,
     &   .3021,   .2810,   .2728,   .2660,   .2579,   .2385,   .2567,
     &   .2567,   .2567,   .2567, -1.6041, -1.2708,  -.9385,  -.6099,
     &  -.2880,   .0052,   .2245,   .3075,   .2903,   .2840,   .2783,
     &   .2739,   .2686,   .2567,   .2567,   .2567,   .2567, -1.7546,
     & -1.4213, -1.0885,  -.7580,  -.4319,  -.1215,   .1399,   .2885,
     &   .3019,   .2909,   .2871,   .2824,   .2792,   .2760,   .2760,
     &   .2760,   .2760, -1.9051, -1.5718, -1.2385,  -.9073,  -.5774,
     &  -.2581,   .0310,   .2400,   .3090,   .2949,   .2938,   .2894,
     &   .2853,   .2824,   .2824,   .2824,   .2824, -2.0556, -1.7223,
     & -1.3890, -1.0565,  -.7258,  -.3995,  -.0930,   .1605,   .2964,
     &   .3023,   .2984,   .2960,   .2909,   .2867,   .2867,   .2867,
     &   .2867, -2.2061, -1.8728, -1.5395, -1.2062,  -.8730,  -.5406,
     &  -.2114,   .1027,   .2964,   .3023,   .2984,   .2960,   .2909,
     &   .2867,   .2867,   .2867,   .2867/
      DATA ((SVCS(I,J,12,3),J=1,17),I=1,6)/
     & -1.4536, -1.1204,  -.7905,  -.4656,  -.1616,   .0737,   .1418,
     &   .0504,  -.0052,  -.0195,  -.0243,  -.0359,  -.0711,  -.0342,
     &  -.0342,  -.0342,  -.0342, -1.6041, -1.2708,  -.9389,  -.6112,
     &  -.2937,  -.0190,   .1383,   .1014,   .0249,   .0000,  -.0024,
     &  -.0024,  -.0088,  -.0342,  -.0342,  -.0342,  -.0342, -1.7546,
     & -1.4213, -1.0887,  -.7585,  -.4347,  -.1342,   .0899,   .1402,
     &   .0588,   .0179,   .0107,   .0141,   .0175,   .0149,   .0149,
     &   .0149,   .0149, -1.9051, -1.5718, -1.2385,  -.9076,  -.5789,
     &  -.2646,   .0039,   .1446,   .0986,   .0382,   .0208,   .0228,
     &   .0294,   .0354,   .0354,   .0354,   .0354, -2.0556, -1.7223,
     & -1.3890, -1.0567,  -.7268,  -.4029,  -.1075,   .1041,   .1380,
     &   .0622,   .0306,   .0273,   .0338,   .0426,   .0426,   .0426,
     &   .0426, -2.2061, -1.8728, -1.5395, -1.2063,  -.8734,  -.5423,
     &  -.2194,   .0656,   .1380,   .0622,   .0306,   .0273,   .0338,
     &   .0426,   .0426,   .0426,   .0426/
      DATA ((SVCS(I,J,13,3),J=1,17),I=1,6)/
     & -1.4537, -1.1207,  -.7921,  -.4719,  -.1892,  -.0367,  -.1764,
     &  -.3898,  -.4363,  -.4343,  -.4304,  -.4382,  -.4756,  -.4352,
     &  -.4352,  -.4352,  -.4352, -1.6041, -1.2710,  -.9393,  -.6142,
     &  -.3079,  -.0792,  -.0718,  -.3093,  -.4179,  -.4250,  -.4135,
     &  -.4054,  -.4082,  -.4352,  -.4352,  -.4352,  -.4352, -1.7546,
     & -1.4214, -1.0890,  -.7602,  -.4420,  -.1660,  -.0342,  -.1949,
     &  -.3838,  -.4202,  -.4094,  -.3935,  -.3820,  -.3806,  -.3806,
     &  -.3806,  -.3806, -1.9051, -1.5718, -1.2387,  -.9083,  -.5825,
     &  -.2810,  -.0651,  -.0855,  -.3203,  -.4113,  -.4124,  -.3932,
     &  -.3741,  -.3596,  -.3596,  -.3596,  -.3596, -2.0556, -1.7223,
     & -1.3891, -1.0570,  -.7284,  -.4113,  -.1441,  -.0358,  -.2154,
     &  -.3901,  -.4169,  -.4015,  -.3786,  -.3571,  -.3571,  -.3571,
     &  -.3571, -2.2061, -1.8728, -1.5395, -1.2065,  -.8743,  -.5466,
     &  -.2394,  -.0274,  -.2154,  -.3901,  -.4169,  -.4015,  -.3786,
     &  -.3571,  -.3571,  -.3571,  -.3571/
      DATA ((SVCS(I,J,14,3),J=1,17),I=1,6)/
     & -1.4538, -1.1215,  -.7953,  -.4876,  -.2585,  -.3064,  -.7655,
     &  -.9197,  -.9215,  -.9059,  -.8969,  -.9003,  -.9307,  -.8972,
     &  -.8972,  -.8972,  -.8972, -1.6042, -1.2714,  -.9412,  -.6223,
     &  -.3437,  -.2292,  -.5449,  -.8897,  -.9248,  -.9080,  -.8887,
     &  -.8771,  -.8768,  -.8972,  -.8972,  -.8972,  -.8972, -1.7546,
     & -1.4216, -1.0899,  -.7642,  -.4602,  -.2457,  -.3360,  -.7972,
     &  -.9248,  -.9183,  -.8928,  -.8722,  -.8586,  -.8560,  -.8560,
     &  -.8560,  -.8560, -1.9051, -1.5719, -1.2392,  -.9104,  -.5918,
     &  -.3223,  -.2363,  -.5935,  -.9073,  -.9311,  -.9076,  -.8788,
     &  -.8560,  -.8413,  -.8413,  -.8413,  -.8413, -2.0556, -1.7224,
     & -1.3893, -1.0581,  -.7331,  -.4323,  -.2358,  -.3732,  -.8327,
     &  -.9401,  -.9274,  -.8959,  -.8665,  -.8453,  -.8453,  -.8453,
     &  -.8453, -2.2061, -1.8728, -1.5396, -1.2070,  -.8767,  -.5575,
     &  -.2898,  -.2613,  -.8327,  -.9401,  -.9274,  -.8959,  -.8665,
     &  -.8453,  -.8453,  -.8453,  -.8453/
      DATA ((SVCS(I,J,15,3),J=1,17),I=1,6)/
     & -1.4542, -1.1233,  -.8041,  -.5271,  -.4317,  -.9187, -1.4044,
     & -1.4304, -1.4126, -1.3952, -1.3862, -1.3879, -1.4090, -1.3864,
     & -1.3864, -1.3864, -1.3864, -1.6044, -1.2723,  -.9454,  -.6423,
     &  -.4333,  -.5969, -1.3107, -1.4359, -1.4254, -1.4013, -1.3829,
     & -1.3734, -1.3732, -1.3864, -1.3864, -1.3864, -1.3864, -1.7547,
     & -1.4220, -1.0921,  -.7742,  -.5058,  -.4447, -1.0020, -1.4309,
     & -1.4415, -1.4166, -1.3903, -1.3719, -1.3625, -1.3614, -1.3614,
     & -1.3614, -1.3614, -1.9052, -1.5722, -1.2402,  -.9154,  -.6148,
     &  -.4257,  -.6531, -1.3586, -1.4564, -1.4378, -1.4065, -1.3799,
     & -1.3620, -1.3542, -1.3542, -1.3542, -1.3542, -2.0557, -1.7225,
     & -1.3899, -1.0606,  -.7447,  -.4852,  -.4642, -1.0921, -1.4574,
     & -1.4599, -1.4310, -1.3974, -1.3711, -1.3442, -1.3442, -1.3442,
     & -1.3442, -2.2061, -1.8729, -1.5399, -1.2082,  -.8826,  -.5848,
     &  -.4163,  -.7740, -1.4574, -1.4599, -1.4310, -1.3974, -1.3711,
     & -1.3442, -1.3442, -1.3442, -1.3442/
      DATA ((SVCS(I,J,16,3),J=1,17),I=1,6)/
     & -1.4552, -1.1281,  -.8259,  -.6262,  -.8589, -1.8239, -1.9337,
     & -1.9223, -1.9024, -1.8897, -1.8857, -1.8874, -1.8993, -1.8877,
     & -1.8877, -1.8877, -1.8877, -1.6049, -1.2747,  -.9562,  -.6923,
     &  -.6576, -1.4177, -1.9355, -1.9374, -1.9169, -1.8938, -1.8831,
     & -1.8788, -1.8804, -1.8877, -1.8877, -1.8877, -1.8877, -1.7550,
     & -1.4232, -1.0975,  -.7994,  -.6207,  -.9329, -1.8778, -1.9578,
     & -1.9344, -1.9087, -1.8874, -1.8771, -1.8739, -1.8755, -1.8755,
     & -1.8755, -1.8755, -1.9053, -1.5727, -1.2430,  -.9281,  -.6728,
     &  -.6842, -1.5444, -1.9622, -1.9586, -1.9285, -1.8996, -1.8817,
     & -1.8726, -1.8706, -1.8706, -1.8706, -1.8706, -2.0557, -1.7228,
     & -1.3912, -1.0669,  -.7739,  -.6180, -1.0207, -1.9270, -1.9821,
     & -1.9539, -1.9219, -1.8928, -1.8771, -1.8697, -1.8697, -1.8697,
     & -1.8697, -2.2062, -1.8730, -1.5406, -1.2114,  -.8973,  -.6532,
     &  -.7160, -1.8711, -1.9821, -1.9539, -1.9219, -1.8928, -1.8771,
     & -1.8697, -1.8697, -1.8697, -1.8697/
      DATA ((SVCS(I,J,17,3),J=1,17),I=1,6)/
     & -1.4578, -1.1399,  -.8804,  -.8745, -1.8303, -2.4257, -2.4285,
     & -2.4105, -2.3973, -2.3919, -2.3918, -2.3937, -2.3971, -2.3946,
     & -2.3946, -2.3946, -2.3946, -1.6062, -1.2806,  -.9838,  -.8182,
     & -1.2120, -2.3842, -2.4445, -2.4259, -2.4055, -2.3934, -2.3894,
     & -2.3904, -2.3928, -2.3946, -2.3946, -2.3946, -2.3946, -1.7556,
     & -1.4262, -1.1113,  -.8626,  -.9076, -1.9935, -2.4537, -2.4470,
     & -2.4209, -2.4007, -2.3904, -2.3881, -2.3899, -2.3928, -2.3928,
     & -2.3928, -2.3928, -1.9056, -1.5742, -1.2499,  -.9598,  -.8187,
     & -1.3198, -2.4314, -2.4689, -2.4431, -2.4151, -2.3963, -2.3883,
     & -2.3875, -2.3903, -2.3903, -2.3903, -2.3903, -2.0559, -1.7235,
     & -1.3947, -1.0829,  -.8474,  -.9496, -2.1505, -2.4835, -2.4704,
     & -2.4378, -2.4101, -2.3932, -2.3870, -2.3876, -2.3876, -2.3876,
     & -2.3876, -2.2063, -1.8734, -1.5423, -1.2194,  -.9344,  -.8252,
     & -1.4961, -2.5517, -2.4704, -2.4378, -2.4101, -2.3932, -2.3870,
     & -2.3876, -2.3876, -2.3876, -2.3876/
      DATA ((SVCS(I,J,18,3),J=1,17),I=1,6)/
     & -1.4642, -1.1697, -1.0175, -1.4913, -2.8827, -2.9307, -2.9165,
     & -2.9027, -2.8965, -2.8962, -2.8979, -2.8989, -2.8976, -2.8996,
     & -2.8996, -2.8996, -2.8996, -1.6094, -1.2955, -1.0526, -1.1333,
     & -2.4395, -2.9408, -2.9337, -2.9133, -2.9003, -2.8959, -2.8969,
     & -2.8993, -2.9007, -2.8996, -2.8996, -2.8996, -2.8996, -1.7572,
     & -1.4336, -1.1458, -1.0216, -1.6183, -2.9274, -2.9539, -2.9307,
     & -2.9097, -2.8983, -2.8959, -2.8979, -2.9010, -2.9027, -2.9027,
     & -2.9027, -2.9027, -1.9064, -1.5779, -1.2671, -1.0398, -1.1837,
     & -2.6295, -2.9702, -2.9543, -2.9263, -2.9059, -2.8969, -2.8962,
     & -2.8993, -2.9027, -2.9027, -2.9027, -2.9027, -2.0563, -1.7254,
     & -1.4033, -1.1229, -1.0317, -1.7660, -2.9674, -2.9805, -2.9508,
     & -2.9215, -2.9031, -2.8962, -2.8969, -2.9007, -2.9007, -2.9007,
     & -2.9007, -2.2065, -1.8743, -1.5466, -1.2394, -1.0274, -1.2512,
     & -2.9422, -3.0256, -2.9508, -2.9215, -2.9031, -2.8962, -2.8969,
     & -2.9007, -2.9007, -2.9007, -2.9007/
      DATA ((SVCS(I,J,19,3),J=1,17),I=1,6)/
     & -1.4803, -1.2445, -1.3615, -2.8834, -3.4242, -3.4196, -3.4064,
     & -3.4001, -3.4003, -3.4032, -3.4059, -3.4072, -3.4047, -3.4079,
     & -3.4079, -3.4079, -3.4079, -1.6175, -1.3329, -1.2254, -1.9172,
     & -3.4154, -3.4362, -3.4182, -3.4044, -3.3998, -3.4012, -3.4047,
     & -3.4078, -3.4093, -3.4079, -3.4079, -3.4079, -3.4079, -1.7613,
     & -1.4523, -1.2324, -1.4202, -3.1078, -3.4520, -3.4370, -3.4151,
     & -3.4026, -3.3998, -3.4024, -3.4063, -3.4096, -3.4113, -3.4113,
     & -3.4113, -3.4113, -1.9084, -1.5873, -1.3106, -1.2402, -2.0872,
     & -3.4535, -3.4602, -3.4337, -3.4116, -3.4012, -3.4002, -3.4037,
     & -3.4079, -3.4112, -3.4112, -3.4112, -3.4112, -2.0573, -1.7300,
     & -1.4250, -1.2234, -1.4936, -3.2880, -3.4825, -3.4598, -3.4296,
     & -3.4087, -3.4004, -3.4009, -3.4051, -3.4093, -3.4093, -3.4093,
     & -3.4093, -2.2070, -1.8767, -1.5575, -1.2898, -1.2613, -2.3183,
     & -3.5400, -3.4930, -3.4296, -3.4087, -3.4004, -3.4009, -3.4051,
     & -3.4093, -3.4093, -3.4093, -3.4093/
      DATA ((SVCS(I,J,20,3),J=1,17),I=1,6)/
     & -1.5208, -1.4325, -2.2195, -3.9003, -3.9197, -3.9083, -3.9017,
     & -3.9020, -3.9059, -3.9097, -3.9122, -3.9133, -3.9150, -3.9166,
     & -3.9166, -3.9166, -3.9166, -1.6377, -1.4269, -1.6590, -3.5860,
     & -3.9326, -3.9212, -3.9069, -3.9017, -3.9031, -3.9073, -3.9112,
     & -3.9140, -3.9151, -3.9166, -3.9166, -3.9166, -3.9166, -1.7714,
     & -1.4993, -1.4500, -2.4112, -3.9363, -3.9404, -3.9194, -3.9052,
     & -3.9017, -3.9041, -3.9087, -3.9126, -3.9154, -3.9169, -3.9169,
     & -3.9169, -3.9169, -1.9135, -1.6108, -1.4196, -1.7428, -3.7844,
     & -3.9610, -3.9397, -3.9161, -3.9038, -3.9020, -3.9055, -3.9104,
     & -3.9140, -3.9165, -3.9165, -3.9165, -3.9165, -2.0598, -1.7418,
     & -1.4795, -1.4757, -2.6353, -3.9727, -3.9662, -3.9367, -3.9133,
     & -3.9027, -3.9024, -3.9069, -3.9115, -3.9151, -3.9151, -3.9151,
     & -3.9151, -2.2082, -1.8826, -1.5848, -1.4163, -1.8461, -4.0135,
     & -4.0060, -3.9598, -3.9133, -3.9027, -3.9024, -3.9069, -3.9115,
     & -3.9151, -3.9151, -3.9151, -3.9151/
      DATA ((SVCS(I,J,21,3),J=1,17),I=1,6)/
     & -1.6225, -1.9035, -4.0691, -4.4149, -4.4094, -4.4027, -4.4030,
     & -4.4069, -4.4114, -4.4149, -4.4170, -4.4177, -4.4202, -4.4211,
     & -4.4211, -4.4211, -4.4211, -1.6886, -1.6629, -2.7406, -4.4177,
     & -4.4229, -4.4090, -4.4025, -4.4038, -4.4084, -4.4129, -4.4161,
     & -4.4182, -4.4191, -4.4211, -4.4211, -4.4211, -4.4211, -1.7968,
     & -1.6173, -1.9961, -4.2812, -4.4401, -4.4232, -4.4076, -4.4024,
     & -4.4048, -4.4097, -4.4141, -4.4173, -4.4192, -4.4201, -4.4201,
     & -4.4201, -4.4201, -1.9262, -1.6698, -1.6929, -2.9910, -4.4520,
     & -4.4447, -4.4208, -4.4059, -4.4026, -4.4060, -4.4111, -4.4153,
     & -4.4183, -4.4200, -4.4200, -4.4200, -4.4200, -2.0662, -1.7713,
     & -1.6165, -2.1089, -4.4046, -4.4695, -4.4433, -4.4178, -4.4047,
     & -4.4031, -4.4072, -4.4123, -4.4163, -4.4191, -4.4191, -4.4191,
     & -4.4191, -2.2114, -1.8973, -1.6532, -1.7336, -3.2997, -4.5198,
     & -4.4714, -4.4308, -4.4047, -4.4031, -4.4072, -4.4123, -4.4163,
     & -4.4191, -4.4191, -4.4191, -4.4191/
      DATA ((SVCS(I,J,22,3),J=1,17),I=1,6)/
     & -1.8780, -3.0847, -4.9003, -4.9090, -4.9031, -4.9031, -4.9073,
     & -4.9122, -4.9161, -4.9187, -4.9201, -4.9216, -4.9229, -4.9233,
     & -4.9233, -4.9233, -4.9233, -1.8163, -2.2550, -4.7799, -4.9212,
     & -4.9108, -4.9031, -4.9038, -4.9083, -4.9133, -4.9172, -4.9194,
     & -4.9208, -4.9223, -4.9233, -4.9233, -4.9233, -4.9233, -1.8607,
     & -1.9135, -3.3569, -4.9318, -4.9263, -4.9097, -4.9031, -4.9048,
     & -4.9097, -4.9147, -4.9179, -4.9201, -4.9215, -4.9228, -4.9228,
     & -4.9228, -4.9228, -1.9582, -1.8180, -2.3806, -4.8962, -4.9473,
     & -4.9255, -4.9083, -4.9031, -4.9059, -4.9112, -4.9158, -4.9190,
     & -4.9208, -4.9223, -4.9223, -4.9223, -4.9223, -2.0821, -1.8454,
     & -1.9602, -3.6755, -4.9658, -4.9496, -4.9230, -4.9069, -4.9031,
     & -4.9069, -4.9122, -4.9165, -4.9197, -4.9215, -4.9215, -4.9215,
     & -4.9215, -2.2194, -1.9344, -1.8252, -2.5313, -5.0296, -4.9835,
     & -4.9400, -4.9107, -4.9031, -4.9069, -4.9122, -4.9165, -4.9197,
     & -4.9215, -4.9215, -4.9215, -4.9215/
      DATA ((SVCS(I,J,23,3),J=1,17),I=1,6)/
     & -2.5192, -5.3481, -5.4060, -5.4032, -5.4025, -5.4067, -5.4120,
     & -5.4163, -5.4193, -5.4210, -5.4219, -5.4236, -5.4242, -5.4244,
     & -5.4244, -5.4244, -5.4244, -2.1370, -3.7371, -5.4127, -5.4120,
     & -5.4037, -5.4034, -5.4079, -5.4132, -5.4174, -5.4200, -5.4216,
     & -5.4226, -5.4239, -5.4244, -5.4244, -5.4244, -5.4244, -2.0211,
     & -2.6578, -5.3867, -5.4274, -5.4125, -5.4036, -5.4042, -5.4092,
     & -5.4144, -5.4183, -5.4207, -5.4222, -5.4235, -5.4242, -5.4242,
     & -5.4242, -5.4242, -2.0384, -2.1902, -4.0929, -5.4437, -5.4300,
     & -5.4111, -5.4034, -5.4052, -5.4105, -5.4156, -5.4191, -5.4212,
     & -5.4227, -5.4239, -5.4239, -5.4239, -5.4239, -2.1222, -2.0316,
     & -2.8238, -5.4432, -5.4546, -5.4286, -5.4094, -5.4033, -5.4060,
     & -5.4116, -5.4165, -5.4198, -5.4217, -5.4231, -5.4231, -5.4231,
     & -5.4231, -2.2394, -2.0274, -2.2571, -4.4974, -5.4976, -5.4502,
     & -5.4162, -5.4016, -5.4060, -5.4116, -5.4165, -5.4198, -5.4217,
     & -5.4231, -5.4231, -5.4231, -5.4231/
      DATA ((SVCS(I,J,24,3),J=1,17),I=1,6)/
     & -4.1247, -5.9154, -5.9024, -5.9020, -5.9059, -5.9115, -5.9161,
     & -5.9194, -5.9215, -5.9226, -5.9237, -5.9246, -5.9249, -5.9250,
     & -5.9250, -5.9250, -5.9250, -2.9427, -5.9301, -5.9112, -5.9045,
     & -5.9027, -5.9073, -5.9126, -5.9172, -5.9201, -5.9219, -5.9234,
     & -5.9241, -5.9247, -5.9250, -5.9250, -5.9250, -5.9250, -2.4240,
     & -4.5139, -5.9237, -5.9151, -5.9045, -5.9034, -5.9083, -5.9136,
     & -5.9179, -5.9208, -5.9226, -5.9237, -5.9245, -5.9249, -5.9249,
     & -5.9249, -5.9249, -2.2399, -3.1250, -5.9761, -5.9333, -5.9144,
     & -5.9041, -5.9045, -5.9094, -5.9151, -5.9190, -5.9212, -5.9230,
     & -5.9237, -5.9247, -5.9247, -5.9247, -5.9247, -2.2230, -2.4991,
     & -4.9545, -5.9551, -5.9348, -5.9129, -5.9038, -5.9052, -5.9108,
     & -5.9158, -5.9197, -5.9219, -5.9234, -5.9241, -5.9241, -5.9241,
     & -5.9241, -2.2898, -2.2613, -3.3416, -6.0117, -5.9625, -5.9232,
     & -5.9037, -5.9007, -5.9108, -5.9158, -5.9197, -5.9219, -5.9234,
     & -5.9241, -5.9241, -5.9241, -5.9241/
      DATA ((SVCS(I,J,25,3),J=1,17),I=1,6)/
     & -6.3935, -6.4022, -6.4010, -6.4047, -6.4106, -6.4158, -6.4194,
     & -6.4216, -6.4231, -6.4241, -6.4246, -6.4251, -6.4252, -6.4255,
     & -6.4255, -6.4255, -6.4255, -4.9527, -6.4171, -6.4047, -6.4023,
     & -6.4060, -6.4118, -6.4167, -6.4200, -6.4220, -6.4234, -6.4245,
     & -6.4248, -6.4251, -6.4255, -6.4255, -6.4255, -6.4255, -3.4360,
     & -6.4455, -6.4165, -6.4057, -6.4031, -6.4072, -6.4130, -6.4176,
     & -6.4207, -6.4225, -6.4237, -6.4246, -6.4250, -6.4255, -6.4255,
     & -6.4255, -6.4255, -2.7459, -5.4326, -6.4441, -6.4183, -6.4054,
     & -6.4036, -6.4084, -6.4141, -6.4184, -6.4211, -6.4230, -6.4239,
     & -6.4246, -6.4255, -6.4255, -6.4255, -6.4255, -2.4761, -3.6731,
     & -6.4610, -6.4402, -6.4171, -6.4047, -6.4043, -6.4095, -6.4150,
     & -6.4191, -6.4216, -6.4232, -6.4241, -6.4255, -6.4255, -6.4255,
     & -6.4255, -2.4163, -2.8485, -5.9257, -6.4755, -6.4320, -6.4066,
     & -6.4004, -6.4049, -6.4150, -6.4191, -6.4216, -6.4232, -6.4241,
     & -6.4255, -6.4255, -6.4255, -6.4255/
      DATA ((SVCS(I,J,26,3),J=1,17),I=1,6)/
     & -6.8941, -6.8993, -6.9034, -6.9094, -6.9151, -6.9190, -6.9215,
     & -6.9230, -6.9241, -6.9248, -6.9251, -6.9253, -6.9255, -6.9255,
     & -6.9255, -6.9255, -6.9255, -6.9016, -6.9030, -6.9017, -6.9048,
     & -6.9108, -6.9161, -6.9197, -6.9219, -6.9234, -6.9241, -6.9250,
     & -6.9252, -6.9255, -6.9255, -6.9255, -6.9255, -6.9255, -5.9326,
     & -6.9192, -6.9073, -6.9027, -6.9059, -6.9119, -6.9169, -6.9205,
     & -6.9223, -6.9237, -6.9245, -6.9251, -6.9255, -6.9255, -6.9255,
     & -6.9255, -6.9255, -4.0168, -6.9288, -6.9189, -6.9073, -6.9031,
     & -6.9069, -6.9129, -6.9179, -6.9208, -6.9230, -6.9241, -6.9245,
     & -6.9255, -6.9255, -6.9255, -6.9255, -6.9255, -3.1117, -6.4487,
     & -6.9263, -6.9223, -6.9066, -6.9038, -6.9080, -6.9140, -6.9187,
     & -6.9212, -6.9230, -6.9241, -6.9255, -6.9255, -6.9255, -6.9255,
     & -6.9255, -2.7342, -4.3227, -6.9337, -6.9428, -6.9112, -6.9003,
     & -6.9031, -6.9101, -6.9187, -6.9212, -6.9230, -6.9241, -6.9255,
     & -6.9255, -6.9255, -6.9255, -6.9255/
      DATA ((SVCS(I,J,27,3),J=1,17),I=1,6)/
     & -7.3970, -7.4027, -7.4079, -7.4140, -7.4185, -7.4212, -7.4231,
     & -7.4240, -7.4248, -7.4252, -7.4253, -7.4254, -7.4255, -7.4255,
     & -7.4255, -7.4255, -7.4255, -7.3942, -7.3993, -7.4035, -7.4093,
     & -7.4151, -7.4192, -7.4217, -7.4233, -7.4242, -7.4248, -7.4253,
     & -7.4254, -7.4255, -7.4255, -7.4255, -7.4255, -7.4255, -7.4033,
     & -7.4039, -7.4026, -7.4046, -7.4105, -7.4160, -7.4199, -7.4222,
     & -7.4237, -7.4245, -7.4250, -7.4253, -7.4255, -7.4255, -7.4255,
     & -7.4255, -7.4255, -6.9964, -7.4052, -7.4046, -7.4032, -7.4057,
     & -7.4116, -7.4169, -7.4204, -7.4226, -7.4239, -7.4246, -7.4250,
     & -7.4255, -7.4255, -7.4255, -7.4255, -7.4255, -4.7075, -7.3986,
     & -7.4047, -7.4094, -7.4034, -7.4067, -7.4126, -7.4177, -7.4209,
     & -7.4230, -7.4241, -7.4247, -7.4255, -7.4255, -7.4255, -7.4255,
     & -7.4255, -3.5325, -7.3019, -7.4048, -7.4176, -7.4015, -7.4017,
     & -7.4085, -7.4150, -7.4209, -7.4230, -7.4241, -7.4247, -7.4255,
     & -7.4255, -7.4255, -7.4255, -7.4255/
      DATA ((SVCS(I,J,28,3),J=1,17),I=1,6)/
     & -7.9029, -7.9079, -7.9129, -7.9176, -7.9208, -7.9230, -7.9241,
     & -7.9248, -7.9252, -7.9254, -7.9254, -7.9255, -7.9255, -7.9255,
     & -7.9255, -7.9255, -7.9255, -7.8960, -7.9020, -7.9076, -7.9140,
     & -7.9187, -7.9215, -7.9234, -7.9241, -7.9248, -7.9252, -7.9254,
     & -7.9255, -7.9255, -7.9255, -7.9255, -7.9255, -7.9255, -7.8943,
     & -7.8992, -7.9034, -7.9090, -7.9147, -7.9194, -7.9219, -7.9234,
     & -7.9245, -7.9250, -7.9253, -7.9255, -7.9255, -7.9255, -7.9255,
     & -7.9255, -7.9255, -7.8910, -7.8956, -7.9002, -7.9045, -7.9101,
     & -7.9158, -7.9197, -7.9223, -7.9237, -7.9245, -7.9251, -7.9255,
     & -7.9255, -7.9255, -7.9255, -7.9255, -7.9255, -7.8212, -7.8881,
     & -7.8963, -7.9038, -7.9052, -7.9112, -7.9165, -7.9205, -7.9226,
     & -7.9241, -7.9248, -7.9255, -7.9255, -7.9255, -7.9255, -7.9255,
     & -7.9255, -5.5359, -7.8806, -7.8924, -7.9042, -7.9009, -7.9066,
     & -7.9133, -7.9187, -7.9226, -7.9241, -7.9248, -7.9255, -7.9255,
     & -7.9255, -7.9255, -7.9255, -7.9255/
      DATA ((SVCS(I,J,29,3),J=1,17),I=1,6)/
     & -8.4099, -8.4134, -8.4168, -8.4204, -8.4226, -8.4239, -8.4246,
     & -8.4252, -8.4254, -8.4255, -8.4255, -8.4255, -8.4255, -8.4255,
     & -8.4255, -8.4255, -8.4255, -8.4024, -8.4075, -8.4125, -8.4176,
     & -8.4209, -8.4230, -8.4241, -8.4247, -8.4252, -8.4254, -8.4255,
     & -8.4255, -8.4255, -8.4255, -8.4255, -8.4255, -8.4255, -8.3956,
     & -8.4016, -8.4073, -8.4135, -8.4184, -8.4214, -8.4232, -8.4242,
     & -8.4248, -8.4253, -8.4254, -8.4255, -8.4255, -8.4255, -8.4255,
     & -8.4255, -8.4255, -8.3906, -8.3966, -8.4026, -8.4085, -8.4145,
     & -8.4191, -8.4218, -8.4235, -8.4245, -8.4249, -8.4253, -8.4255,
     & -8.4255, -8.4255, -8.4255, -8.4255, -8.4255, -8.3840, -8.3908,
     & -8.3976, -8.4042, -8.4095, -8.4154, -8.4196, -8.4222, -8.4238,
     & -8.4246, -8.4250, -8.4255, -8.4255, -8.4255, -8.4255, -8.4255,
     & -8.4255, -8.3747, -8.3850, -8.3926, -8.4002, -8.4047, -8.4117,
     & -8.4174, -8.4209, -8.4238, -8.4246, -8.4250, -8.4255, -8.4255,
     & -8.4255, -8.4255, -8.4255, -8.4255/
      DATA ((SVCS(I,J,30,3),J=1,17),I=1,6)/
     & -8.9145, -8.9171, -8.9197, -8.9223, -8.9237, -8.9245, -8.9251,
     & -8.9254, -8.9255, -8.9255, -8.9255, -8.9255, -8.9255, -8.9255,
     & -8.9255, -8.9255, -8.9255, -8.9085, -8.9125, -8.9165, -8.9205,
     & -8.9226, -8.9241, -8.9248, -8.9251, -8.9254, -8.9255, -8.9255,
     & -8.9255, -8.9255, -8.9255, -8.9255, -8.9255, -8.9255, -8.9014,
     & -8.9068, -8.9119, -8.9172, -8.9208, -8.9230, -8.9241, -8.9248,
     & -8.9252, -8.9254, -8.9255, -8.9255, -8.9255, -8.9255, -8.9255,
     & -8.9255, -8.9255, -8.8971, -8.9025, -8.9079, -8.9129, -8.9183,
     & -8.9212, -8.9234, -8.9245, -8.9248, -8.9252, -8.9255, -8.9255,
     & -8.9255, -8.9255, -8.9255, -8.9255, -8.9255, -8.8948, -8.8992,
     & -8.9036, -8.9080, -8.9140, -8.9187, -8.9215, -8.9234, -8.9245,
     & -8.9252, -8.9255, -8.9255, -8.9255, -8.9255, -8.9255, -8.9255,
     & -8.9255, -8.8925, -8.8959, -8.8993, -8.9027, -8.9097, -8.9162,
     & -8.9196, -8.9223, -8.9245, -8.9252, -8.9255, -8.9255, -8.9255,
     & -8.9255, -8.9255, -8.9255, -8.9255/
      DATA ((SVCS(I,J,31,3),J=1,17),I=1,6)/
     & -9.4184, -9.4201, -9.4218, -9.4235, -9.4245, -9.4250, -9.4253,
     & -9.4255, -9.4255, -9.4255, -9.4255, -9.4255, -9.4255, -9.4255,
     & -9.4255, -9.4255, -9.4255, -9.4144, -9.4170, -9.4195, -9.4222,
     & -9.4238, -9.4246, -9.4250, -9.4253, -9.4255, -9.4255, -9.4255,
     & -9.4255, -9.4255, -9.4255, -9.4255, -9.4255, -9.4255, -9.4084,
     & -9.4123, -9.4161, -9.4201, -9.4226, -9.4240, -9.4247, -9.4252,
     & -9.4254, -9.4255, -9.4255, -9.4255, -9.4255, -9.4255, -9.4255,
     & -9.4255, -9.4255, -9.4044, -9.4086, -9.4128, -9.4170, -9.4207,
     & -9.4229, -9.4241, -9.4248, -9.4252, -9.4255, -9.4255, -9.4255,
     & -9.4255, -9.4255, -9.4255, -9.4255, -9.4255, -9.4034, -9.4064,
     & -9.4094, -9.4124, -9.4177, -9.4211, -9.4232, -9.4244, -9.4249,
     & -9.4255, -9.4255, -9.4255, -9.4255, -9.4255, -9.4255, -9.4255,
     & -9.4255, -9.4024, -9.4042, -9.4060, -9.4078, -9.4147, -9.4193,
     & -9.4223, -9.4240, -9.4249, -9.4255, -9.4255, -9.4255, -9.4255,
     & -9.4255, -9.4255, -9.4255, -9.4255/
      DATA ((SVCS(I,J,32,3),J=1,17),I=1,6)/
     & -9.9212, -9.9223, -9.9234, -9.9245, -9.9248, -9.9253, -9.9254,
     & -9.9255, -9.9255, -9.9255, -9.9255, -9.9255, -9.9255, -9.9255,
     & -9.9255, -9.9255, -9.9255, -9.9177, -9.9196, -9.9215, -9.9234,
     & -9.9245, -9.9248, -9.9253, -9.9255, -9.9255, -9.9255, -9.9255,
     & -9.9255, -9.9255, -9.9255, -9.9255, -9.9255, -9.9255, -9.9144,
     & -9.9169, -9.9194, -9.9219, -9.9237, -9.9245, -9.9252, -9.9255,
     & -9.9255, -9.9255, -9.9255, -9.9255, -9.9255, -9.9255, -9.9255,
     & -9.9255, -9.9255, -9.9125, -9.9149, -9.9173, -9.9197, -9.9223,
     & -9.9241, -9.9248, -9.9255, -9.9255, -9.9255, -9.9255, -9.9255,
     & -9.9255, -9.9255, -9.9255, -9.9255, -9.9255, -9.9126, -9.9139,
     & -9.9152, -9.9165, -9.9205, -9.9226, -9.9241, -9.9255, -9.9255,
     & -9.9255, -9.9255, -9.9255, -9.9255, -9.9255, -9.9255, -9.9255,
     & -9.9255, -9.9127, -9.9129, -9.9131, -9.9133, -9.9187, -9.9211,
     & -9.9234, -9.9255, -9.9255, -9.9255, -9.9255, -9.9255, -9.9255,
     & -9.9255, -9.9255, -9.9255, -9.9255/
      DATA ((SVCS(I,J,33,3),J=1,17),I=1,6)/
     &-10.4227,-10.4234,-10.4241,-10.4248,-10.4252,-10.4254,-10.4255,
     &-10.4255,-10.4255,-10.4255,-10.4255,-10.4255,-10.4255,-10.4255,
     &-10.4255,-10.4255,-10.4255,-10.4212,-10.4222,-10.4232,-10.4242,
     &-10.4249,-10.4253,-10.4254,-10.4255,-10.4255,-10.4255,-10.4255,
     &-10.4255,-10.4255,-10.4255,-10.4255,-10.4255,-10.4255,-10.4177,
     &-10.4196,-10.4215,-10.4234,-10.4245,-10.4250,-10.4254,-10.4255,
     &-10.4255,-10.4255,-10.4255,-10.4255,-10.4255,-10.4255,-10.4255,
     &-10.4255,-10.4255,-10.4158,-10.4178,-10.4198,-10.4218,-10.4237,
     &-10.4246,-10.4250,-10.4255,-10.4255,-10.4255,-10.4255,-10.4255,
     &-10.4255,-10.4255,-10.4255,-10.4255,-10.4255,-10.4155,-10.4168,
     &-10.4181,-10.4194,-10.4222,-10.4238,-10.4247,-10.4255,-10.4255,
     &-10.4255,-10.4255,-10.4255,-10.4255,-10.4255,-10.4255,-10.4255,
     &-10.4255,-10.4152,-10.4158,-10.4164,-10.4170,-10.4207,-10.4230,
     &-10.4244,-10.4255,-10.4255,-10.4255,-10.4255,-10.4255,-10.4255,
     &-10.4255,-10.4255,-10.4255,-10.4255/
      DATA ((SVCS(I,J,34,3),J=1,17),I=1,6)/
     &-10.9240,-10.9244,-10.9248,-10.9252,-10.9254,-10.9255,-10.9255,
     &-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,
     &-10.9255,-10.9255,-10.9255,-10.9227,-10.9234,-10.9241,-10.9248,
     &-10.9252,-10.9254,-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,
     &-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,-10.9208,
     &-10.9219,-10.9230,-10.9241,-10.9248,-10.9252,-10.9255,-10.9255,
     &-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,
     &-10.9255,-10.9255,-10.9189,-10.9204,-10.9219,-10.9234,-10.9245,
     &-10.9252,-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,
     &-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,-10.9194,-10.9201,
     &-10.9208,-10.9215,-10.9234,-10.9245,-10.9255,-10.9255,-10.9255,
     &-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,
     &-10.9255,-10.9199,-10.9198,-10.9197,-10.9196,-10.9223,-10.9238,
     &-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,-10.9255,
     &-10.9255,-10.9255,-10.9255,-10.9255/
      DATA ((SVCS(I,J,35,3),J=1,17),I=1,6)/
     &-11.4244,-11.4247,-11.4250,-11.4253,-11.4255,-11.4255,-11.4255,
     &-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,
     &-11.4255,-11.4255,-11.4255,-11.4237,-11.4242,-11.4247,-11.4252,
     &-11.4254,-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,
     &-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,-11.4224,
     &-11.4232,-11.4240,-11.4248,-11.4252,-11.4255,-11.4255,-11.4255,
     &-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,
     &-11.4255,-11.4255,-11.4217,-11.4225,-11.4233,-11.4241,-11.4248,
     &-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,
     &-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,-11.4216,-11.4221,
     &-11.4226,-11.4231,-11.4242,-11.4255,-11.4255,-11.4255,-11.4255,
     &-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,
     &-11.4255,-11.4215,-11.4217,-11.4219,-11.4221,-11.4236,-11.4255,
     &-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,-11.4255,
     &-11.4255,-11.4255,-11.4255,-11.4255/
      DATA ((SVCS(I,J,36,3),J=1,17),I=1,6)/
     &-11.9250,-11.9251,-11.9252,-11.9253,-11.9255,-11.9255,-11.9255,
     &-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,
     &-11.9255,-11.9255,-11.9255,-11.9252,-11.9252,-11.9252,-11.9252,
     &-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,
     &-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,-11.9231,
     &-11.9238,-11.9245,-11.9252,-11.9255,-11.9255,-11.9255,-11.9255,
     &-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,
     &-11.9255,-11.9255,-11.9218,-11.9228,-11.9238,-11.9248,-11.9252,
     &-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,
     &-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,-11.9211,-11.9221,
     &-11.9231,-11.9241,-11.9248,-11.9255,-11.9255,-11.9255,-11.9255,
     &-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,
     &-11.9255,-11.9204,-11.9214,-11.9224,-11.9234,-11.9244,-11.9255,
     &-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,-11.9255,
     &-11.9255,-11.9255,-11.9255,-11.9255/
      DATA ((SVCS(I,J,37,3),J=1,17),I=1,6)/
     &-12.4254,-12.4254,-12.4254,-12.4254,-12.4255,-12.4255,-12.4255,
     &-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,
     &-12.4255,-12.4255,-12.4255,-12.4251,-12.4252,-12.4253,-12.4254,
     &-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,
     &-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,-12.4244,
     &-12.4247,-12.4250,-12.4253,-12.4255,-12.4255,-12.4255,-12.4255,
     &-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,
     &-12.4255,-12.4255,-12.4241,-12.4244,-12.4247,-12.4250,-12.4255,
     &-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,
     &-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,-12.4238,-12.4241,
     &-12.4244,-12.4247,-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,
     &-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,
     &-12.4255,-12.4235,-12.4238,-12.4241,-12.4244,-12.4255,-12.4255,
     &-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,-12.4255,
     &-12.4255,-12.4255,-12.4255,-12.4255/
      DATA ((SVCS(I,J,38,3),J=1,17),I=1,6)/
     &-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,
     &-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,
     &-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,
     &-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,
     &-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9248,
     &-12.9250,-12.9252,-12.9254,-12.9255,-12.9255,-12.9255,-12.9255,
     &-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,
     &-12.9255,-12.9255,-12.9243,-12.9246,-12.9249,-12.9252,-12.9255,
     &-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,
     &-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9234,-12.9240,
     &-12.9246,-12.9252,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,
     &-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,
     &-12.9255,-12.9225,-12.9234,-12.9243,-12.9252,-12.9255,-12.9255,
     &-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,-12.9255,
     &-12.9255,-12.9255,-12.9255,-12.9255/
      DATA ((SVCS(I,J,39,3),J=1,17),I=1,6)/
     &-13.4254,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,
     &-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,
     &-13.4255,-13.4255,-13.4255,-13.4254,-13.4255,-13.4255,-13.4255,
     &-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,
     &-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4251,
     &-13.4253,-13.4254,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,
     &-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,
     &-13.4255,-13.4255,-13.4248,-13.4251,-13.4253,-13.4255,-13.4255,
     &-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,
     &-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4245,-13.4249,
     &-13.4252,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,
     &-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,
     &-13.4255,-13.4242,-13.4247,-13.4251,-13.4255,-13.4255,-13.4255,
     &-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,-13.4255,
     &-13.4255,-13.4255,-13.4255,-13.4255/
      DATA ((SVCS(I,J,40,3),J=1,17),I=1,6)/
     &-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,
     &-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,
     &-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,
     &-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,
     &-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,
     &-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,
     &-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,
     &-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,
     &-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,
     &-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,
     &-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,
     &-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,
     &-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,
     &-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,-13.9255,
     &-13.9255,-13.9255,-13.9255,-13.9255/
      DATA ((SVCS(I,J, 1,4),J=1,17),I=1,6)/
     & -1.4290, -1.0973,  -.7673,  -.4432,  -.1388,   .1216,   .2953,
     &   .3653,   .3659,   .3578,   .3681,   .3994,   .4932,   .4932,
     &   .4932,   .4932,   .4932, -1.5795, -1.2471,  -.9154,  -.5884,
     &  -.2720,   .0120,   .2287,   .3400,   .3585,   .3403,   .3360,
     &   .3439,   .3683,   .3683,   .3683,   .3683,   .3683, -1.7300,
     & -1.3964, -1.0644,  -.7342,  -.4124,  -.1096,   .1427,   .3019,
     &   .3533,   .3370,   .3193,   .3181,   .3250,   .3250,   .3250,
     &   .3250,   .3250, -1.8805, -1.5472, -1.2156,  -.8834,  -.5558,
     &  -.2427,   .0382,   .2443,   .3402,   .3426,   .3137,   .3030,
     &   .3048,   .3048,   .3048,   .3048,   .3048, -2.0310, -1.6977,
     & -1.3644, -1.0323,  -.7033,  -.3806,  -.0810,   .1647,   .3115,
     &   .3487,   .3199,   .2954,   .2941,   .2941,   .2941,   .2941,
     &   .2941, -2.1816, -1.8482, -1.5149, -1.1816,  -.8482,  -.5149,
     &  -.1816,   .1518,   .3115,   .3487,   .3199,   .2954,   .2941,
     &   .2941,   .2941,   .2941,   .2941/
      DATA ((SVCS(I,J, 2,4),J=1,17),I=1,6)/
     & -1.4290, -1.0973,  -.7673,  -.4432,  -.1388,   .1216,   .2953,
     &   .3653,   .3659,   .3578,   .3682,   .3995,   .4935,   .4935,
     &   .4935,   .4935,   .4935, -1.5795, -1.2471,  -.9154,  -.5884,
     &  -.2720,   .0120,   .2287,   .3400,   .3585,   .3403,   .3361,
     &   .3439,   .3684,   .3684,   .3684,   .3684,   .3684, -1.7300,
     & -1.3964, -1.0644,  -.7342,  -.4124,  -.1096,   .1427,   .3019,
     &   .3533,   .3370,   .3195,   .3182,   .3251,   .3251,   .3251,
     &   .3251,   .3251, -1.8805, -1.5472, -1.2156,  -.8834,  -.5558,
     &  -.2427,   .0382,   .2443,   .3402,   .3426,   .3139,   .3031,
     &   .3050,   .3050,   .3050,   .3050,   .3050, -2.0310, -1.6977,
     & -1.3644, -1.0323,  -.7033,  -.3806,  -.0810,   .1647,   .3115,
     &   .3487,   .3200,   .2956,   .2944,   .2944,   .2944,   .2944,
     &   .2944, -2.1816, -1.8482, -1.5149, -1.1816,  -.8482,  -.5149,
     &  -.1816,   .1518,   .3115,   .3487,   .3200,   .2956,   .2944,
     &   .2944,   .2944,   .2944,   .2944/
      DATA ((SVCS(I,J, 3,4),J=1,17),I=1,6)/
     & -1.4290, -1.0973,  -.7673,  -.4432,  -.1388,   .1216,   .2953,
     &   .3653,   .3659,   .3579,   .3683,   .3996,   .4939,   .4939,
     &   .4939,   .4939,   .4939, -1.5795, -1.2471,  -.9154,  -.5884,
     &  -.2720,   .0120,   .2287,   .3400,   .3585,   .3404,   .3362,
     &   .3440,   .3686,   .3686,   .3686,   .3686,   .3686, -1.7300,
     & -1.3964, -1.0644,  -.7342,  -.4124,  -.1096,   .1427,   .3019,
     &   .3533,   .3371,   .3196,   .3183,   .3253,   .3253,   .3253,
     &   .3253,   .3253, -1.8805, -1.5472, -1.2156,  -.8834,  -.5558,
     &  -.2427,   .0382,   .2443,   .3402,   .3426,   .3140,   .3033,
     &   .3052,   .3052,   .3052,   .3052,   .3052, -2.0310, -1.6977,
     & -1.3644, -1.0323,  -.7033,  -.3806,  -.0810,   .1647,   .3115,
     &   .3487,   .3202,   .2958,   .2948,   .2948,   .2948,   .2948,
     &   .2948, -2.1816, -1.8482, -1.5149, -1.1816,  -.8482,  -.5149,
     &  -.1816,   .1518,   .3115,   .3487,   .3202,   .2958,   .2948,
     &   .2948,   .2948,   .2948,   .2948/
      DATA ((SVCS(I,J, 4,4),J=1,17),I=1,6)/
     & -1.4290, -1.0973,  -.7673,  -.4432,  -.1388,   .1216,   .2951,
     &   .3651,   .3660,   .3579,   .3684,   .3997,   .4942,   .4942,
     &   .4942,   .4942,   .4942, -1.5795, -1.2471,  -.9154,  -.5884,
     &  -.2720,   .0120,   .2284,   .3400,   .3585,   .3404,   .3364,
     &   .3441,   .3687,   .3687,   .3687,   .3687,   .3687, -1.7300,
     & -1.3964, -1.0644,  -.7342,  -.4124,  -.1096,   .1427,   .3017,
     &   .3533,   .3371,   .3198,   .3184,   .3254,   .3254,   .3254,
     &   .3254,   .3254, -1.8805, -1.5472, -1.2156,  -.8834,  -.5558,
     &  -.2428,   .0382,   .2443,   .3400,   .3426,   .3142,   .3034,
     &   .3054,   .3054,   .3054,   .3054,   .3054, -2.0310, -1.6977,
     & -1.3644, -1.0323,  -.7033,  -.3806,  -.0810,   .1647,   .3113,
     &   .3487,   .3203,   .2960,   .2951,   .2951,   .2951,   .2951,
     &   .2951, -2.1816, -1.8482, -1.5149, -1.1816,  -.8482,  -.5149,
     &  -.1816,   .1517,   .3113,   .3487,   .3203,   .2960,   .2951,
     &   .2951,   .2951,   .2951,   .2951/
      DATA ((SVCS(I,J, 5,4),J=1,17),I=1,6)/
     & -1.4290, -1.0973,  -.7673,  -.4432,  -.1388,   .1216,   .2951,
     &   .3649,   .3664,   .3591,   .3685,   .3998,   .4946,   .4946,
     &   .4946,   .4946,   .4946, -1.5795, -1.2471,  -.9154,  -.5884,
     &  -.2720,   .0120,   .2284,   .3398,   .3585,   .3414,   .3365,
     &   .3442,   .3689,   .3689,   .3689,   .3689,   .3689, -1.7300,
     & -1.3964, -1.0644,  -.7342,  -.4124,  -.1096,   .1427,   .3017,
     &   .3531,   .3377,   .3199,   .3185,   .3256,   .3256,   .3256,
     &   .3256,   .3256, -1.8805, -1.5472, -1.2156,  -.8834,  -.5558,
     &  -.2428,   .0382,   .2440,   .3400,   .3428,   .3143,   .3036,
     &   .3056,   .3056,   .3056,   .3056,   .3056, -2.0310, -1.6977,
     & -1.3644, -1.0323,  -.7033,  -.3806,  -.0810,   .1644,   .3111,
     &   .3487,   .3204,   .2962,   .2955,   .2955,   .2955,   .2955,
     &   .2955, -2.1816, -1.8482, -1.5149, -1.1816,  -.8482,  -.5149,
     &  -.1816,   .1517,   .3111,   .3487,   .3204,   .2962,   .2955,
     &   .2955,   .2955,   .2955,   .2955/
      DATA ((SVCS(I,J, 6,4),J=1,17),I=1,6)/
     & -1.4290, -1.0973,  -.7673,  -.4432,  -.1388,   .1212,   .2947,
     &   .3644,   .3672,   .3619,   .3713,   .4023,   .4949,   .4949,
     &   .4949,   .4949,   .4949, -1.5795, -1.2471,  -.9154,  -.5884,
     &  -.2720,   .0120,   .2281,   .3393,   .3587,   .3440,   .3397,
     &   .3465,   .3690,   .3690,   .3690,   .3690,   .3690, -1.7300,
     & -1.3964, -1.0644,  -.7342,  -.4124,  -.1096,   .1424,   .3012,
     &   .3528,   .3395,   .3241,   .3216,   .3257,   .3257,   .3257,
     &   .3257,   .3257, -1.8805, -1.5472, -1.2156,  -.8834,  -.5558,
     &  -.2428,   .0382,   .2438,   .3395,   .3436,   .3183,   .3079,
     &   .3058,   .3058,   .3058,   .3058,   .3058, -2.0310, -1.6977,
     & -1.3644, -1.0323,  -.7033,  -.3806,  -.0811,   .1644,   .3107,
     &   .3485,   .3233,   .3021,   .2958,   .2958,   .2958,   .2958,
     &   .2958, -2.1816, -1.8482, -1.5149, -1.1816,  -.8482,  -.5149,
     &  -.1816,   .1515,   .3107,   .3485,   .3233,   .3021,   .2958,
     &   .2958,   .2958,   .2958,   .2958/
      DATA ((SVCS(I,J, 7,4),J=1,17),I=1,6)/
     & -1.4290, -1.0973,  -.7673,  -.4432,  -.1389,   .1209,   .2936,
     &   .3630,   .3690,   .3675,   .3768,   .4069,   .5013,   .5013,
     &   .5013,   .5013,   .5013, -1.5795, -1.2471,  -.9154,  -.5884,
     &  -.2721,   .0116,   .2274,   .3379,   .3589,   .3497,   .3464,
     &   .3512,   .3725,   .3725,   .3725,   .3725,   .3725, -1.7300,
     & -1.3964, -1.0645,  -.7342,  -.4125,  -.1098,   .1421,   .2999,
     &   .3520,   .3436,   .3324,   .3280,   .3294,   .3294,   .3294,
     &   .3294,   .3294, -1.8805, -1.5472, -1.2156,  -.8834,  -.5558,
     &  -.2428,   .0378,   .2430,   .3383,   .3450,   .3272,   .3170,
     &   .3111,   .3111,   .3111,   .3111,   .3111, -2.0310, -1.6977,
     & -1.3644, -1.0323,  -.7033,  -.3807,  -.0812,   .1638,   .3096,
     &   .3483,   .3300,   .3141,   .3045,   .3045,   .3045,   .3045,
     &   .3045, -2.1816, -1.8482, -1.5149, -1.1816,  -.8482,  -.5149,
     &  -.1817,   .1511,   .3096,   .3483,   .3300,   .3141,   .3045,
     &   .3045,   .3045,   .3045,   .3045/
      DATA ((SVCS(I,J, 8,4),J=1,17),I=1,6)/
     & -1.4290, -1.0973,  -.7673,  -.4433,  -.1392,   .1199,   .2907,
     &   .3596,   .3716,   .3762,   .3838,   .4116,   .5042,   .5042,
     &   .5042,   .5042,   .5042, -1.5795, -1.2471,  -.9154,  -.5884,
     &  -.2722,   .0111,   .2256,   .3345,   .3587,   .3591,   .3566,
     &   .3579,   .3768,   .3768,   .3768,   .3768,   .3768, -1.7300,
     & -1.3964, -1.0645,  -.7345,  -.4125,  -.1101,   .1408,   .2973,
     &   .3495,   .3506,   .3460,   .3379,   .3351,   .3351,   .3351,
     &   .3351,   .3351, -1.8805, -1.5472, -1.2156,  -.8834,  -.5558,
     &  -.2430,   .0374,   .2410,   .3351,   .3477,   .3422,   .3316,
     &   .3199,   .3199,   .3199,   .3199,   .3199, -2.0310, -1.6977,
     & -1.3644, -1.0324,  -.7033,  -.3808,  -.0816,   .1626,   .3066,
     &   .3469,   .3424,   .3336,   .3187,   .3187,   .3187,   .3187,
     &   .3187, -2.1816, -1.8482, -1.5149, -1.1816,  -.8482,  -.5150,
     &  -.1819,   .1500,   .3066,   .3469,   .3424,   .3336,   .3187,
     &   .3187,   .3187,   .3187,   .3187/
      DATA ((SVCS(I,J, 9,4),J=1,17),I=1,6)/
     & -1.4290, -1.0973,  -.7673,  -.4435,  -.1400,   .1173,   .2838,
     &   .3504,   .3707,   .3779,   .3820,   .4050,   .4839,   .4839,
     &   .4839,   .4839,   .4839, -1.5795, -1.2471,  -.9154,  -.5885,
     &  -.2726,   .0095,   .2209,   .3257,   .3555,   .3647,   .3606,
     &   .3583,   .3735,   .3735,   .3735,   .3735,   .3735, -1.7300,
     & -1.3964, -1.0645,  -.7345,  -.4127,  -.1110,   .1380,   .2903,
     &   .3428,   .3568,   .3549,   .3430,   .3367,   .3367,   .3367,
     &   .3367,   .3367, -1.8805, -1.5472, -1.2156,  -.8834,  -.5560,
     &  -.2434,   .0354,   .2358,   .3269,   .3493,   .3549,   .3418,
     &   .3255,   .3255,   .3255,   .3255,   .3255, -2.0310, -1.6977,
     & -1.3644, -1.0324,  -.7033,  -.3810,  -.0825,   .1593,   .2995,
     &   .3426,   .3553,   .3491,   .3292,   .3292,   .3292,   .3292,
     &   .3292, -2.1816, -1.8482, -1.5149, -1.1816,  -.8483,  -.5151,
     &  -.1825,   .1474,   .2995,   .3426,   .3553,   .3491,   .3292,
     &   .3292,   .3292,   .3292,   .3292/
      DATA ((SVCS(I,J,10,4),J=1,17),I=1,6)/
     & -1.4290, -1.0973,  -.7673,  -.4439,  -.1418,   .1106,   .2665,
     &   .3259,   .3458,   .3462,   .3471,   .3623,   .4113,   .4113,
     &   .4113,   .4113,   .4113, -1.5795, -1.2471,  -.9154,  -.5887,
     &  -.2736,   .0056,   .2090,   .3038,   .3349,   .3398,   .3330,
     &   .3306,   .3412,   .3412,   .3412,   .3412,   .3412, -1.7300,
     & -1.3964, -1.0645,  -.7345,  -.4133,  -.1131,   .1303,   .2725,
     &   .3220,   .3381,   .3308,   .3201,   .3147,   .3147,   .3147,
     &   .3147,   .3147, -1.8805, -1.5472, -1.2156,  -.8834,  -.5563,
     &  -.2446,   .0310,   .2232,   .3060,   .3336,   .3343,   .3204,
     &   .3071,   .3071,   .3071,   .3071,   .3071, -2.0310, -1.6977,
     & -1.3644, -1.0324,  -.7033,  -.3815,  -.0850,   .1511,   .2813,
     &   .3249,   .3395,   .3276,   .3107,   .3107,   .3107,   .3107,
     &   .3107, -2.1816, -1.8482, -1.5149, -1.1816,  -.8483,  -.5154,
     &  -.1839,   .1408,   .2813,   .3249,   .3395,   .3276,   .3107,
     &   .3107,   .3107,   .3107,   .3107/
      DATA ((SVCS(I,J,11,4),J=1,17),I=1,6)/
     & -1.4290, -1.0974,  -.7675,  -.4450,  -.1464,   .0938,   .2235,
     &   .2591,   .2591,   .2545,   .2536,   .2565,   .2591,   .2591,
     &   .2591,   .2591,   .2591, -1.5795, -1.2471,  -.9154,  -.5894,
     &  -.2760,  -.0038,   .1796,   .2485,   .2608,   .2541,   .2497,
     &   .2487,   .2521,   .2521,   .2521,   .2521,   .2521, -1.7300,
     & -1.3965, -1.0646,  -.7347,  -.4145,  -.1183,   .1119,   .2287,
     &   .2596,   .2570,   .2492,   .2443,   .2428,   .2428,   .2428,
     &   .2428,   .2428, -1.8805, -1.5472, -1.2156,  -.8837,  -.5569,
     &  -.2473,   .0204,   .1915,   .2519,   .2615,   .2519,   .2440,
     &   .2388,   .2388,   .2388,   .2388,   .2388, -2.0310, -1.6977,
     & -1.3644, -1.0325,  -.7038,  -.3830,  -.0910,   .1303,   .2360,
     &   .2634,   .2574,   .2472,   .2393,   .2393,   .2393,   .2393,
     &   .2393, -2.1816, -1.8482, -1.5149, -1.1816,  -.8485,  -.5162,
     &  -.1875,   .1243,   .2360,   .2634,   .2574,   .2472,   .2393,
     &   .2393,   .2393,   .2393,   .2393/
      DATA ((SVCS(I,J,12,4),J=1,17),I=1,6)/
     & -1.4290, -1.0976,  -.7683,  -.4477,  -.1580,   .0519,   .1186,
     &   .0973,   .0788,   .0741,   .0697,   .0550,   .0022,   .0022,
     &   .0022,   .0022,   .0022, -1.5795, -1.2472,  -.9158,  -.5907,
     &  -.2822,  -.0280,   .1065,   .1129,   .0903,   .0831,   .0824,
     &   .0810,   .0726,   .0726,   .0726,   .0726,   .0726, -1.7300,
     & -1.3965, -1.0647,  -.7354,  -.4177,  -.1316,   .0656,   .1209,
     &   .1024,   .0896,   .0881,   .0892,   .0899,   .0899,   .0899,
     &   .0899,   .0899, -1.8805, -1.5472, -1.2157,  -.8841,  -.5585,
     &  -.2544,  -.0070,   .1133,   .1156,   .0966,   .0917,   .0924,
     &   .0948,   .0948,   .0948,   .0948,   .0948, -2.0310, -1.6977,
     & -1.3644, -1.0326,  -.7047,  -.3867,  -.1061,   .0792,   .1242,
     &   .1059,   .0955,   .0945,   .0959,   .0959,   .0959,   .0959,
     &   .0959, -2.1816, -1.8482, -1.5149, -1.1817,  -.8489,  -.5181,
     &  -.1964,   .0828,   .1242,   .1059,   .0955,   .0945,   .0959,
     &   .0959,   .0959,   .0959,   .0959/
      DATA ((SVCS(I,J,13,4),J=1,17),I=1,6)/
     & -1.4291, -1.0979,  -.7696,  -.4546,  -.1871,  -.0517,  -.1219,
     &  -.2142,  -.2345,  -.2349,  -.2400,  -.2669,  -.3607,  -.3607,
     &  -.3607,  -.3607,  -.3607, -1.5796, -1.2473,  -.9165,  -.5941,
     &  -.2975,  -.0883,  -.0692,  -.1769,  -.2198,  -.2205,  -.2145,
     &  -.2147,  -.2328,  -.2328,  -.2328,  -.2328,  -.2328, -1.7300,
     & -1.3965, -1.0651,  -.7373,  -.4256,  -.1648,  -.0490,  -.1259,
     &  -.2034,  -.2143,  -.2057,  -.1960,  -.1925,  -.1925,  -.1925,
     &  -.1925,  -.1925, -1.8805, -1.5473, -1.2159,  -.8847,  -.5624,
     &  -.2721,  -.0751,  -.0742,  -.1763,  -.2092,  -.2043,  -.1908,
     &  -.1786,  -.1786,  -.1786,  -.1786,  -.1786, -2.0311, -1.6977,
     & -1.3645, -1.0331,  -.7066,  -.3958,  -.1442,  -.0474,  -.1324,
     &  -.2011,  -.2063,  -.1938,  -.1779,  -.1779,  -.1779,  -.1779,
     &  -.1779, -2.1816, -1.8482, -1.5150, -1.1819,  -.8500,  -.5229,
     &  -.2189,  -.0214,  -.1324,  -.2011,  -.2063,  -.1938,  -.1779,
     &  -.1779,  -.1779,  -.1779,  -.1779/
      DATA ((SVCS(I,J,14,4),J=1,17),I=1,6)/
     & -1.4293, -1.0987,  -.7734,  -.4719,  -.2599,  -.3012,  -.5771,
     &  -.6651,  -.6661,  -.6584,  -.6602,  -.6868,  -.7937,  -.7937,
     &  -.7937,  -.7937,  -.7937, -1.5796, -1.2478,  -.9187,  -.6029,
     &  -.3359,  -.2377,  -.4563,  -.6430,  -.6623,  -.6494,  -.6364,
     &  -.6340,  -.6521,  -.6521,  -.6521,  -.6521,  -.6521, -1.7301,
     & -1.3967, -1.0661,  -.7416,  -.4455,  -.2480,  -.3225,  -.5896,
     &  -.6607,  -.6519,  -.6319,  -.6160,  -.6107,  -.6107,  -.6107,
     &  -.6107,  -.6107, -1.8806, -1.5474, -1.2164,  -.8871,  -.5727,
     &  -.3163,  -.2434,  -.4816,  -.6492,  -.6596,  -.6386,  -.6144,
     &  -.5969,  -.5969,  -.5969,  -.5969,  -.5969, -2.0311, -1.6978,
     & -1.3648, -1.0342,  -.7118,  -.4188,  -.2391,  -.3472,  -.6080,
     &  -.6684,  -.6525,  -.6251,  -.5993,  -.5993,  -.5993,  -.5993,
     &  -.5993, -2.1816, -1.8483, -1.5151, -1.1825,  -.8526,  -.5351,
     &  -.2753,  -.1600,  -.6080,  -.6684,  -.6525,  -.6251,  -.5993,
     &  -.5993,  -.5993,  -.5993,  -.5993/
      DATA ((SVCS(I,J,15,4),J=1,17),I=1,6)/
     & -1.4297, -1.1008,  -.7696,  -.5151,  -.4407,  -.8410, -1.1454,
     & -1.1637, -1.1504, -1.1397, -1.1394, -1.1589, -1.2585, -1.2585,
     & -1.2585, -1.2585, -1.2585, -1.5799, -1.2488,  -.9165,  -.6251,
     &  -.4324,  -.5969, -1.0817, -1.1674, -1.1558, -1.1366, -1.1237,
     & -1.1212, -1.1339, -1.1339, -1.1339, -1.1339, -1.1339, -1.7302,
     & -1.3973, -1.0651,  -.7530,  -.4955,  -.4542,  -.8972, -1.1615,
     & -1.1679, -1.1450, -1.1233, -1.1097, -1.1064, -1.1064, -1.1064,
     & -1.1064, -1.1064, -1.8806, -1.5476, -1.2159,  -.8928,  -.5983,
     &  -.4272,  -.6436, -1.1146, -1.1816, -1.1610, -1.1331, -1.1105,
     & -1.0973, -1.0973, -1.0973, -1.0973, -1.0973, -2.0311, -1.6979,
     & -1.3654, -1.0370,  -.7247,  -.4765,  -.4740,  -.9562, -1.1852,
     & -1.1833, -1.1524, -1.1227, -1.1001, -1.1001, -1.1001, -1.1001,
     & -1.1001, -2.1816, -1.8483, -1.5154, -1.1839,  -.8592,  -.5656,
     &  -.4170,  -.7830, -1.1852, -1.1833, -1.1524, -1.1227, -1.1001,
     & -1.1001, -1.1001, -1.1001, -1.1001/
      DATA ((SVCS(I,J,16,4),J=1,17),I=1,6)/
     & -1.4309, -1.1061,  -.7734,  -.6236,  -.8798, -1.5929, -1.6731,
     & -1.6627, -1.6484, -1.6409, -1.6411, -1.6531, -1.7268, -1.7268,
     & -1.7268, -1.7268, -1.7268, -1.5804, -1.2514,  -.9187,  -.6805,
     &  -.6724, -1.3426, -1.6747, -1.6745, -1.6544, -1.6400, -1.6325,
     & -1.6325, -1.6392, -1.6392, -1.6392, -1.6392, -1.6392, -1.7305,
     & -1.3985, -1.0661,  -.7809,  -.6209,  -.9496, -1.6341, -1.6897,
     & -1.6700, -1.6470, -1.6325, -1.6260, -1.6229, -1.6229, -1.6229,
     & -1.6229, -1.6229, -1.8808, -1.5483, -1.2164,  -.9066,  -.6623,
     &  -.7027, -1.4322, -1.6979, -1.6902, -1.6625, -1.6396, -1.6258,
     & -1.6209, -1.6209, -1.6209, -1.6209, -1.6209, -2.0312, -1.6983,
     & -1.3669, -1.0441,  -.7572,  -.6212, -1.0311, -1.6763, -1.7131,
     & -1.6855, -1.6544, -1.6334, -1.6212, -1.6212, -1.6212, -1.6212,
     & -1.6212, -2.1816, -1.8485, -1.5162, -1.1875,  -.8757,  -.6423,
     &  -.7376, -1.7375, -1.7131, -1.6855, -1.6544, -1.6334, -1.6212,
     & -1.6212, -1.6212, -1.6212, -1.6212/
      DATA ((SVCS(I,J,17,4),J=1,17),I=1,6)/
     & -1.4337, -1.1193,  -.7830,  -.8945, -1.7959, -2.1699, -2.1728,
     & -2.1605, -2.1530, -2.1515, -2.1537, -2.1591, -2.2182, -2.2182,
     & -2.2182, -2.2182, -2.2182, -1.5819, -1.2580,  -.9234,  -.8193,
     & -1.2555, -2.1424, -2.1885, -2.1704, -2.1559, -2.1494, -2.1490,
     & -2.1514, -2.1547, -2.1547, -2.1547, -2.1547, -2.1547, -1.7312,
     & -1.4018, -1.0685,  -.8514,  -.9333, -1.9154, -2.1955, -2.1862,
     & -2.1651, -2.1511, -2.1470, -2.1476, -2.1504, -2.1504, -2.1504,
     & -2.1504, -2.1504, -1.8811, -1.5500, -1.2176,  -.9420,  -.8230,
     & -1.3641, -2.1806, -2.2064, -2.1825, -2.1606, -2.1473, -2.1430,
     & -2.1472, -2.1472, -2.1472, -2.1472, -2.1472, -2.0313, -1.6991,
     & -1.3708, -1.0619,  -.8386,  -.9809, -2.0250, -2.2241, -2.2068,
     & -2.1779, -2.1560, -2.1445, -2.1419, -2.1419, -2.1419, -2.1419,
     & -2.1419, -2.1817, -1.8489, -1.5181, -1.1964,  -.9172,  -.8349,
     & -1.5809, -2.2874, -2.2068, -2.1779, -2.1560, -2.1445, -2.1419,
     & -2.1419, -2.1419, -2.1419, -2.1419/
      DATA ((SVCS(I,J,18,4),J=1,17),I=1,6)/
     & -1.4409, -1.1525,  -.8074, -1.5586, -2.6402, -2.6782, -2.6696,
     & -2.6621, -2.6603, -2.6621, -2.6647, -2.6657, -2.6992, -2.6992,
     & -2.6992, -2.6992, -2.6992, -1.5855, -1.2746,  -.9355, -1.1664,
     & -2.3987, -2.6857, -2.6811, -2.6676, -2.6609, -2.6603, -2.6627,
     & -2.6653, -2.6664, -2.6664, -2.6664, -2.6664, -2.6664, -1.7330,
     & -1.4101, -1.0746, -1.0277, -1.6923, -2.6770, -2.6972, -2.6792,
     & -2.6655, -2.6603, -2.6609, -2.6639, -2.6666, -2.6666, -2.6666,
     & -2.6666, -2.6666, -1.8820, -1.5541, -1.2206, -1.0311, -1.2242,
     & -2.5272, -2.7122, -2.6975, -2.6759, -2.6635, -2.6603, -2.6621,
     & -2.6655, -2.6655, -2.6655, -2.6655, -2.6655, -2.0318, -1.7012,
     & -1.3805, -1.1066, -1.0432, -1.8431, -2.7147, -2.7210, -2.6944,
     & -2.6728, -2.6621, -2.6605, -2.6633, -2.6633, -2.6633, -2.6633,
     & -2.6633, -2.1819, -1.8500, -1.5229, -1.2189, -1.0214, -1.3062,
     & -2.7765, -2.7617, -2.6944, -2.6728, -2.6621, -2.6605, -2.6633,
     & -2.6633, -2.6633, -2.6633, -2.6633/
      DATA ((SVCS(I,J,19,4),J=1,17),I=1,6)/
     & -1.4590, -1.2359,  -.8681, -2.8807, -3.1741, -3.1744, -3.1683,
     & -3.1673, -3.1697, -3.1726, -3.1742, -3.1726, -3.1902, -3.1902,
     & -3.1902, -3.1902, -3.1902, -1.5945, -1.3162,  -.9662, -2.0177,
     & -3.1670, -3.1850, -3.1741, -3.1679, -3.1679, -3.1711, -3.1744,
     & -3.1764, -3.1756, -3.1756, -3.1756, -3.1756, -3.1756, -1.7375,
     & -1.4310, -1.0900, -1.4692, -3.0264, -3.1977, -3.1864, -3.1727,
     & -3.1675, -3.1688, -3.1726, -3.1761, -3.1784, -3.1784, -3.1784,
     & -3.1784, -3.1784, -1.8843, -1.5646, -1.2283, -1.2545, -2.1972,
     & -3.2018, -3.2046, -3.1842, -3.1710, -3.1675, -3.1700, -3.1742,
     & -3.1779, -3.1779, -3.1779, -3.1779, -3.1779, -2.0329, -1.7064,
     & -1.4048, -1.2188, -1.5539, -3.1309, -3.2256, -3.2041, -3.1814,
     & -3.1696, -3.1678, -3.1712, -3.1756, -3.1756, -3.1756, -3.1756,
     & -3.1756, -2.1825, -1.8526, -1.5351, -1.2753, -1.2797, -2.4689,
     & -3.2775, -3.2304, -3.1814, -3.1696, -3.1678, -3.1712, -3.1756,
     & -3.1756, -3.1756, -3.1756, -3.1756/
      DATA ((SVCS(I,J,20,4),J=1,17),I=1,6)/
     & -1.5043, -1.4451, -1.0203, -3.6538, -3.6743, -3.6708, -3.6704,
     & -3.6735, -3.6776, -3.6807, -3.6821, -3.6805, -3.7009, -3.7009,
     & -3.7009, -3.7009, -3.7009, -1.6172, -1.4209, -1.0428, -3.5216,
     & -3.6819, -3.6770, -3.6710, -3.6712, -3.6749, -3.6792, -3.6824,
     & -3.6840, -3.6832, -3.6832, -3.6832, -3.6832, -3.6832, -1.7488,
     & -1.4836, -1.1285, -2.5502, -3.6849, -3.6893, -3.6765, -3.6708,
     & -3.6722, -3.6765, -3.6807, -3.6838, -3.6855, -3.6855, -3.6855,
     & -3.6855, -3.6855, -1.8899, -1.5909, -1.2476, -1.8133, -3.6267,
     & -3.7058, -3.6895, -3.6751, -3.6708, -3.6733, -3.6780, -3.6821,
     & -3.6853, -3.6853, -3.6853, -3.6853, -3.6853, -2.0358, -1.7196,
     & -1.4658, -1.5005, -2.7860, -3.7187, -3.7104, -3.6874, -3.6739,
     & -3.6710, -3.6745, -3.6792, -3.6834, -3.6834, -3.6834, -3.6834,
     & -3.6834, -2.1839, -1.8592, -1.5656, -1.4170, -1.9354, -3.7856,
     & -3.7434, -3.7025, -3.6739, -3.6710, -3.6745, -3.6792, -3.6834,
     & -3.6834, -3.6834, -3.6834, -3.6834/
      DATA ((SVCS(I,J,21,4),J=1,17),I=1,6)/
     & -1.6182, -1.9702, -1.4018, -4.1691, -4.1711, -4.1715, -4.1752,
     & -4.1799, -4.1837, -4.1862, -4.1873, -4.1860, -4.2063, -4.2063,
     & -4.2063, -4.2063, -4.2063, -1.6741, -1.6840, -1.2355, -4.1685,
     & -4.1772, -4.1722, -4.1724, -4.1766, -4.1813, -4.1851, -4.1876,
     & -4.1887, -4.1882, -4.1882, -4.1882, -4.1882, -4.1882, -1.7773,
     & -1.6158, -1.2253, -4.1197, -4.1884, -4.1789, -4.1724, -4.1733,
     & -4.1780, -4.1827, -4.1863, -4.1886, -4.1898, -4.1898, -4.1898,
     & -4.1898, -4.1898, -1.9042, -1.6570, -1.2959, -3.1735, -4.1985,
     & -4.1930, -4.1781, -4.1724, -4.1744, -4.1794, -4.1841, -4.1874,
     & -4.1896, -4.1896, -4.1896, -4.1896, -4.1896, -2.0429, -1.7526,
     & -1.6192, -2.2054, -4.1836, -4.2134, -4.1925, -4.1769, -4.1725,
     & -4.1755, -4.1807, -4.1852, -4.1884, -4.1884, -4.1884, -4.1884,
     & -4.1884, -2.1875, -1.8757, -1.6423, -1.7717, -3.5161, -4.2562,
     & -4.2118, -4.1824, -4.1725, -4.1755, -4.1807, -4.1852, -4.1884,
     & -4.1884, -4.1884, -4.1884, -4.1884/
      DATA ((SVCS(I,J,22,4),J=1,17),I=1,6)/
     & -1.9042, -3.2699, -2.3438, -4.6696, -4.6714, -4.6755, -4.6807,
     & -4.6851, -4.6880, -4.6899, -4.6908, -4.6987, -4.7091, -4.7091,
     & -4.7091, -4.7091, -4.7091, -1.8171, -2.3451, -1.7181, -4.6743,
     & -4.6726, -4.6726, -4.6770, -4.6821, -4.6863, -4.6891, -4.6908,
     & -4.6916, -4.6998, -4.6998, -4.6998, -4.6998, -4.6998, -1.8488,
     & -1.9477, -1.4680, -4.6803, -4.6801, -4.6733, -4.6737, -4.6782,
     & -4.6834, -4.6874, -4.6899, -4.6916, -4.6923, -4.6923, -4.6923,
     & -4.6923, -4.6923, -1.9399, -1.8230, -1.4175, -4.6684, -4.6942,
     & -4.6807, -4.6733, -4.6745, -4.6796, -4.6847, -4.6885, -4.6908,
     & -4.6923, -4.6923, -4.6923, -4.6923, -4.6923, -2.0607, -1.8356,
     & -1.9987, -3.8969, -4.7106, -4.6970, -4.6796, -4.6733, -4.6755,
     & -4.6807, -4.6857, -4.6893, -4.6914, -4.6914, -4.6914, -4.6914,
     & -4.6914, -2.1964, -1.9172, -1.8349, -2.6638, -4.7697, -4.7222,
     & -4.6877, -4.6727, -4.6755, -4.6807, -4.6857, -4.6893, -4.6914,
     & -4.6914, -4.6914, -4.6914, -4.6914/
      DATA ((SVCS(I,J,23,4),J=1,17),I=1,6)/
     & -2.6228, -5.1051, -4.0177, -5.1704, -5.1750, -5.1806, -5.1855,
     & -5.1890, -5.1911, -5.1923, -5.2012, -5.2052, -5.2105, -5.2105,
     & -5.2105, -5.2105, -5.2105, -2.1764, -3.9743, -2.9073, -5.1718,
     & -5.1722, -5.1765, -5.1820, -5.1866, -5.1898, -5.1917, -5.1930,
     & -5.2016, -5.2058, -5.2058, -5.2058, -5.2058, -5.2058, -2.0285,
     & -2.7804, -2.0764, -5.1792, -5.1738, -5.1733, -5.1778, -5.1832,
     & -5.1876, -5.1906, -5.1923, -5.1935, -5.2019, -5.2019, -5.2019,
     & -5.2019, -5.2019, -2.0298, -2.2395, -1.7226, -5.1904, -5.1828,
     & -5.1741, -5.1742, -5.1791, -5.1845, -5.1886, -5.1912, -5.1929,
     & -5.1939, -5.1939, -5.1939, -5.1939, -5.1939, -2.1057, -2.0441,
     & -2.9546, -5.1941, -5.2001, -5.1827, -5.1740, -5.1750, -5.1803,
     & -5.1856, -5.1894, -5.1918, -5.1933, -5.1933, -5.1933, -5.1933,
     & -5.1933, -2.2189, -2.0214, -2.3151, -4.7588, -5.2340, -5.1950,
     & -5.1748, -5.1713, -5.1803, -5.1856, -5.1894, -5.1918, -5.1933,
     & -5.1933, -5.1933, -5.1933, -5.1933/
      DATA ((SVCS(I,J,24,4),J=1,17),I=1,6)/
     & -4.4254, -5.6544, -4.6548, -5.6739, -5.6801, -5.6853, -5.6891,
     & -5.6916, -5.6931, -5.6940, -5.7065, -5.7085, -5.7112, -5.7112,
     & -5.7112, -5.7112, -5.7112, -3.0789, -5.6400, -4.6117, -5.6714,
     & -5.6755, -5.6813, -5.6863, -5.6899, -5.6921, -5.6936, -5.6944,
     & -5.7067, -5.7088, -5.7088, -5.7088, -5.7088, -5.7088, -2.4797,
     & -4.8005, -3.5677, -5.6739, -5.6728, -5.6770, -5.6826, -5.6874,
     & -5.6906, -5.6927, -5.6940, -5.6946, -5.7069, -5.7069, -5.7069,
     & -5.7069, -5.7069, -2.2554, -3.2857, -2.4889, -5.6838, -5.6751,
     & -5.6737, -5.6782, -5.6838, -5.6885, -5.6912, -5.6931, -5.6942,
     & -5.6949, -5.6949, -5.6949, -5.6949, -5.6949, -2.2185, -2.5674,
     & -4.7255, -5.7001, -5.6859, -5.6751, -5.6745, -5.6792, -5.6849,
     & -5.6891, -5.6919, -5.6936, -5.6946, -5.6946, -5.6946, -5.6946,
     & -5.6946, -2.2753, -2.2831, -3.5142, -5.7474, -5.7034, -5.6779,
     & -5.6712, -5.6750, -5.6849, -5.6891, -5.6919, -5.6936, -5.6946,
     & -5.6946, -5.6946, -5.6946, -5.6946/
      DATA ((SVCS(I,J,25,4),J=1,17),I=1,6)/
     & -7.1636, -6.1639, -5.1649, -6.1789, -6.1848, -6.1890, -6.1917,
     & -6.1934, -6.1944, -6.2028, -6.2092, -6.2102, -6.2115, -6.2115,
     & -6.2115, -6.2115, -6.2115, -5.3396, -6.1623, -5.1651, -6.1743,
     & -6.1804, -6.1859, -6.1898, -6.1922, -6.1938, -6.1946, -6.2030,
     & -6.2093, -6.2103, -6.2103, -6.2103, -6.2103, -6.2103, -3.6131,
     & -6.1586, -5.1533, -6.1721, -6.1757, -6.1816, -6.1869, -6.1905,
     & -6.1927, -6.1941, -6.1949, -6.2032, -6.2094, -6.2094, -6.2094,
     & -6.2094, -6.2094, -2.8221, -5.7447, -4.3408, -6.1760, -6.1732,
     & -6.1770, -6.1829, -6.1878, -6.1911, -6.1932, -6.1944, -6.1951,
     & -6.2033, -6.2033, -6.2033, -6.2033, -6.2033, -2.5018, -3.8811,
     & -5.2921, -6.1888, -6.1765, -6.1739, -6.1782, -6.1840, -6.1886,
     & -6.1917, -6.1935, -6.1946, -6.1953, -6.1953, -6.1953, -6.1953,
     & -6.1953, -2.4170, -2.9407, -5.3033, -6.2137, -6.1827, -6.1712,
     & -6.1734, -6.1802, -6.1886, -6.1917, -6.1935, -6.1946, -6.1953,
     & -6.1953, -6.1953, -6.1953, -6.1953/
      DATA ((SVCS(I,J,26,4),J=1,17),I=1,6)/
     & -7.6741, -6.6696, -5.6682, -6.6838, -6.6885, -6.6914, -6.6934,
     & -6.6946, -6.6951, -6.7073, -6.7105, -6.7110, -6.7119, -6.7119,
     & -6.7119, -6.7119, -6.7119, -7.6709, -6.6670, -5.6690, -6.6790,
     & -6.6851, -6.6893, -6.6921, -6.6938, -6.6949, -6.6953, -6.7074,
     & -6.7106, -6.7119, -6.7119, -6.7119, -6.7119, -6.7119, -6.4348,
     & -6.6692, -5.6741, -6.6745, -6.6805, -6.6861, -6.6902, -6.6925,
     & -6.6942, -6.6951, -6.7033, -6.7075, -6.7119, -6.7119, -6.7119,
     & -6.7119, -6.7119, -4.2456, -6.6985, -5.6749, -6.6728, -6.6757,
     & -6.6817, -6.6870, -6.6908, -6.6931, -6.6944, -6.6953, -6.7034,
     & -6.7119, -6.7119, -6.7119, -6.7119, -6.7119, -3.2136, -6.5847,
     & -5.7546, -6.6784, -6.6737, -6.6767, -6.6828, -6.6878, -6.6912,
     & -6.6934, -6.6946, -6.6953, -6.7119, -6.7119, -6.7119, -6.7119,
     & -6.7119, -2.7729, -4.5896, -5.7824, -6.6887, -6.6725, -6.6723,
     & -6.6786, -6.6848, -6.6912, -6.6934, -6.6946, -6.6953, -6.7119,
     & -6.7119, -6.7119, -6.7119, -6.7119/
      DATA ((SVCS(I,J,27,4),J=1,17),I=1,6)/
     & -8.1797, -7.1754, -6.1722, -7.1879, -7.1912, -7.1933, -7.1946,
     & -7.1952, -7.2034, -7.2096, -7.2112, -7.2114, -7.2119, -7.2119,
     & -7.2119, -7.2119, -7.2119, -8.1738, -7.1706, -6.1698, -7.1839,
     & -7.1888, -7.1918, -7.1937, -7.1948, -7.1955, -7.2035, -7.2096,
     & -7.2112, -7.2119, -7.2119, -7.2119, -7.2119, -7.2119, -8.1713,
     & -7.1697, -6.1729, -7.1789, -7.1850, -7.1895, -7.1923, -7.1940,
     & -7.1950, -7.1956, -7.2075, -7.2097, -7.2119, -7.2119, -7.2119,
     & -7.2119, -7.2119, -7.6571, -7.1797, -6.1820, -7.1745, -7.1803,
     & -7.1860, -7.1902, -7.1927, -7.1943, -7.1952, -7.1957, -7.2076,
     & -7.2119, -7.2119, -7.2119, -7.2119, -7.2119, -5.0013, -7.1824,
     & -6.2203, -7.1737, -7.1755, -7.1814, -7.1870, -7.1908, -7.1931,
     & -7.1946, -7.1954, -7.1958, -7.2119, -7.2119, -7.2119, -7.2119,
     & -7.2119, -3.6668, -7.1732, -6.2391, -7.1750, -7.1714, -7.1767,
     & -7.1838, -7.1889, -7.1931, -7.1946, -7.1954, -7.1958, -7.2119,
     & -7.2119, -7.2119, -7.2119, -7.2119/
      DATA ((SVCS(I,J,28,4),J=1,17),I=1,6)/
     & -8.6850, -7.6809, -6.6774, -7.6908, -7.6931, -7.6944, -7.6953,
     & -7.7035, -7.7076, -7.7107, -7.7115, -7.7119, -7.7119, -7.7119,
     & -7.7119, -7.7119, -7.7119, -8.6793, -7.6755, -6.6726, -7.6878,
     & -7.6914, -7.6936, -7.6946, -7.6955, -7.7036, -7.7076, -7.7107,
     & -7.7119, -7.7119, -7.7119, -7.7119, -7.7119, -7.7119, -8.6733,
     & -7.6714, -6.6712, -7.6836, -7.6887, -7.6919, -7.6938, -7.6949,
     & -7.6955, -7.7036, -7.7097, -7.7119, -7.7119, -7.7119, -7.7119,
     & -7.7119, -7.7119, -8.6590, -7.6716, -6.6765, -7.6786, -7.6849,
     & -7.6895, -7.6923, -7.6942, -7.6951, -7.6957, -7.7037, -7.7119,
     & -7.7119, -7.7119, -7.7119, -7.7119, -7.7119, -8.5871, -7.6689,
     & -6.6931, -7.6743, -7.6799, -7.6857, -7.6902, -7.6927, -7.6944,
     & -7.6953, -7.6957, -7.7119, -7.7119, -7.7119, -7.7119, -7.7119,
     & -7.7119, -5.9116, -7.6662, -6.7020, -7.6708, -7.6749, -7.6819,
     & -7.6881, -7.6912, -7.6944, -7.6953, -7.6957, -7.7119, -7.7119,
     & -7.7119, -7.7119, -7.7119, -7.7119/
      DATA ((SVCS(I,J,29,4),J=1,17),I=1,6)/
     & -9.1890, -8.1858, -7.1826, -8.1928, -8.1944, -8.1952, -8.1957,
     & -8.2076, -8.2097, -8.2113, -8.2119, -8.2119, -8.2119, -8.2119,
     & -8.2119, -8.2119, -8.2119, -9.1845, -8.1808, -7.1774, -8.1908,
     & -8.1932, -8.1946, -8.1954, -8.1959, -8.2077, -8.2097, -8.2119,
     & -8.2119, -8.2119, -8.2119, -8.2119, -8.2119, -8.2119, -9.1784,
     & -8.1754, -7.1730, -8.1877, -8.1913, -8.1935, -8.1948, -8.1955,
     & -8.1959, -8.2077, -8.2119, -8.2119, -8.2119, -8.2119, -8.2119,
     & -8.2119, -8.2119, -9.1673, -8.1716, -7.1727, -8.1834, -8.1885,
     & -8.1918, -8.1939, -8.1950, -8.1957, -8.1960, -8.2119, -8.2119,
     & -8.2119, -8.2119, -8.2119, -8.2119, -8.2119, -9.1562, -8.1667,
     & -7.1772, -8.1782, -8.1844, -8.1892, -8.1923, -8.1942, -8.1952,
     & -8.1957, -8.2119, -8.2119, -8.2119, -8.2119, -8.2119, -8.2119,
     & -8.2119, -9.1434, -8.1618, -7.1785, -8.1734, -8.1802, -8.1866,
     & -8.1907, -8.1934, -8.1952, -8.1957, -8.2119, -8.2119, -8.2119,
     & -8.2119, -8.2119, -8.2119, -8.2119/
      DATA ((SVCS(I,J,30,4),J=1,17),I=1,6)/
     & -9.6914, -8.6893, -7.6870, -8.6942, -8.6951, -8.6957, -8.7037,
     & -8.7097, -8.7108, -8.7116, -8.7119, -8.7119, -8.7119, -8.7119,
     & -8.7119, -8.7119, -8.7119, -9.6886, -8.6855, -7.6826, -8.6927,
     & -8.6944, -8.6953, -8.6959, -8.7038, -8.7098, -8.7108, -8.7119,
     & -8.7119, -8.7119, -8.7119, -8.7119, -8.7119, -8.7119, -9.6838,
     & -8.6803, -7.6772, -8.6906, -8.6931, -8.6946, -8.6955, -8.6959,
     & -8.7038, -8.7098, -8.7119, -8.7119, -8.7119, -8.7119, -8.7119,
     & -8.7119, -8.7119, -9.6777, -8.6761, -7.6733, -8.6874, -8.6912,
     & -8.6936, -8.6949, -8.6955, -8.6959, -8.7038, -8.7119, -8.7119,
     & -8.7119, -8.7119, -8.7119, -8.7119, -8.7119, -9.6716, -8.6713,
     & -7.6710, -8.6828, -8.6882, -8.6916, -8.6938, -8.6951, -8.6957,
     & -8.6959, -8.7119, -8.7119, -8.7119, -8.7119, -8.7119, -8.7119,
     & -8.7119, -9.6655, -8.6665, -7.6675, -8.6782, -8.6852, -8.6896,
     & -8.6927, -8.6947, -8.6957, -8.6959, -8.7119, -8.7119, -8.7119,
     & -8.7119, -8.7119, -8.7119, -8.7119/
      DATA ((SVCS(I,J,31,4),J=1,17),I=1,6)/
     &-10.1934, -9.1917, -8.1901, -9.1950, -9.1957, -9.1961, -9.2078,
     & -9.2108, -9.2113, -9.2119, -9.2119, -9.2119, -9.2119, -9.2119,
     & -9.2119, -9.2119, -9.2119,-10.1915, -9.1891, -8.1868, -9.1942,
     & -9.1952, -9.1958, -9.1961, -9.2078, -9.2108, -9.2119, -9.2119,
     & -9.2119, -9.2119, -9.2119, -9.2119, -9.2119, -9.2119,-10.1880,
     & -9.1850, -8.1822, -9.1927, -9.1944, -9.1953, -9.1959, -9.2038,
     & -9.2078, -9.2119, -9.2119, -9.2119, -9.2119, -9.2119, -9.2119,
     & -9.2119, -9.2119,-10.1850, -9.1812, -8.1769, -9.1904, -9.1931,
     & -9.1946, -9.1955, -9.1959, -9.2038, -9.2119, -9.2119, -9.2119,
     & -9.2119, -9.2119, -9.2119, -9.2119, -9.2119,-10.1820, -9.1772,
     & -8.1724, -9.1869, -9.1909, -9.1934, -9.1948, -9.1956, -9.1960,
     & -9.2119, -9.2119, -9.2119, -9.2119, -9.2119, -9.2119, -9.2119,
     & -9.2119,-10.1790, -9.1732, -8.1674, -9.1836, -9.1887, -9.1922,
     & -9.1941, -9.1953, -9.1960, -9.2119, -9.2119, -9.2119, -9.2119,
     & -9.2119, -9.2119, -9.2119, -9.2119/
      DATA ((SVCS(I,J,32,4),J=1,17),I=1,6)/
     & -8.6863, -8.6893, -8.6923, -9.6955, -9.6959, -9.7039, -9.7098,
     & -9.7113, -9.7116, -9.7119, -9.7119, -9.7119, -9.7119, -9.7119,
     & -9.7119, -9.7119, -9.7119, -8.6812, -8.6855, -8.6899, -9.6951,
     & -9.6957, -9.6962, -9.7039, -9.7098, -9.7113, -9.7119, -9.7119,
     & -9.7119, -9.7119, -9.7119, -9.7119, -9.7119, -9.7119, -8.6752,
     & -8.6803, -8.6866, -9.6940, -9.6953, -9.6957, -9.6962, -9.7078,
     & -9.7098, -9.7119, -9.7119, -9.7119, -9.7119, -9.7119, -9.7119,
     & -9.7119, -9.7119, -8.6703, -8.6761, -8.6817, -9.6925, -9.6944,
     & -9.6953, -9.6959, -9.6962, -9.7078, -9.7119, -9.7119, -9.7119,
     & -9.7119, -9.7119, -9.7119, -9.7119, -9.7119, -8.6654, -8.6713,
     & -8.6772, -9.6899, -9.6929, -9.6946, -9.6955, -9.6959, -9.6962,
     & -9.7119, -9.7119, -9.7119, -9.7119, -9.7119, -9.7119, -9.7119,
     & -9.7119, -8.6605, -8.6665, -8.6725, -9.6879, -9.6914, -9.6939,
     & -9.6951, -9.6956, -9.6962, -9.7119, -9.7119, -9.7119, -9.7119,
     & -9.7119, -9.7119, -9.7119, -9.7119/
      DATA ((SVCS(I,J,33,4),J=1,17),I=1,6)/
     & -9.1897, -9.1917, -9.1939,-10.1959,-10.2038,-10.2078,-10.2108,
     &-10.2116,-10.2119,-10.2119,-10.2119,-10.2119,-10.2119,-10.2119,
     &-10.2119,-10.2119,-10.2119, -9.1861, -9.1891, -9.1923,-10.1956,
     &-10.1960,-10.2039,-10.2078,-10.2108,-10.2119,-10.2119,-10.2119,
     &-10.2119,-10.2119,-10.2119,-10.2119,-10.2119,-10.2119, -9.1806,
     & -9.1850, -9.1898,-10.1950,-10.1957,-10.1961,-10.2039,-10.2098,
     &-10.2119,-10.2119,-10.2119,-10.2119,-10.2119,-10.2119,-10.2119,
     &-10.2119,-10.2119, -9.1762, -9.1812, -9.1860,-10.1940,-10.1951,
     &-10.1958,-10.1961,-10.2039,-10.2119,-10.2119,-10.2119,-10.2119,
     &-10.2119,-10.2119,-10.2119,-10.2119,-10.2119, -9.1718, -9.1772,
     & -9.1826,-10.1923,-10.1942,-10.1952,-10.1959,-10.1961,-10.2119,
     &-10.2119,-10.2119,-10.2119,-10.2119,-10.2119,-10.2119,-10.2119,
     &-10.2119, -9.1674, -9.1732, -9.1790,-10.1906,-10.1933,-10.1946,
     &-10.1957,-10.1883,-10.2119,-10.2119,-10.2119,-10.2119,-10.2119,
     &-10.2119,-10.2119,-10.2119,-10.2119/
      DATA ((SVCS(I,J,34,4),J=1,17),I=1,6)/
     & -9.6923, -9.6936, -9.6949,-10.6962,-10.7078,-10.7098,-10.7113,
     &-10.7119,-10.7119,-10.7119,-10.7119,-10.7119,-10.7119,-10.7119,
     &-10.7119,-10.7119,-10.7119, -9.6894, -9.6916, -9.6938,-10.6959,
     &-10.6962,-10.7079,-10.7098,-10.7119,-10.7119,-10.7119,-10.7119,
     &-10.7119,-10.7119,-10.7119,-10.7119,-10.7119,-10.7119, -9.6857,
     & -9.6887, -9.6921,-10.6955,-10.6959,-10.6962,-10.7079,-10.7119,
     &-10.7119,-10.7119,-10.7119,-10.7119,-10.7119,-10.7119,-10.7119,
     &-10.7119,-10.7119, -9.6829, -9.6862, -9.6895,-10.6949,-10.6957,
     &-10.6962,-10.6964,-10.7119,-10.7119,-10.7119,-10.7119,-10.7119,
     &-10.7119,-10.7119,-10.7119,-10.7119,-10.7119, -9.6801, -9.6835,
     & -9.6869,-10.6938,-10.6951,-10.6957,-10.6962,-10.7119,-10.7119,
     &-10.7119,-10.7119,-10.7119,-10.7119,-10.7119,-10.7119,-10.7119,
     &-10.7119, -9.6773, -9.6808, -9.6843,-10.6927,-10.6945,-10.6952,
     &-10.6960,-10.7119,-10.7119,-10.7119,-10.7119,-10.7119,-10.7119,
     &-10.7119,-10.7119,-10.7119,-10.7119/
      DATA ((SVCS(I,J,35,4),J=1,17),I=1,6)/
     &-10.1937,-10.1946,-10.1955,-11.2039,-11.2098,-11.2109,-11.2116,
     &-11.2119,-11.2119,-11.2119,-11.2119,-11.2119,-11.2119,-11.2119,
     &-11.2119,-11.2119,-11.2119,-10.1918,-10.1933,-10.1948,-11.1961,
     &-11.2039,-11.2099,-11.2109,-11.2119,-11.2119,-11.2119,-11.2119,
     &-11.2119,-11.2119,-11.2119,-11.2119,-11.2119,-11.2119,-10.1891,
     &-10.1913,-10.1937,-11.1959,-11.1962,-11.2039,-11.2099,-11.2119,
     &-11.2119,-11.2119,-11.2119,-11.2119,-11.2119,-11.2119,-11.2119,
     &-11.2119,-11.2119,-10.1872,-10.1895,-10.1918,-11.1955,-11.1960,
     &-11.1963,-11.2040,-11.2119,-11.2119,-11.2119,-11.2119,-11.2119,
     &-11.2119,-11.2119,-11.2119,-11.2119,-11.2119,-10.1853,-10.1876,
     &-10.1899,-11.1948,-11.1956,-11.1961,-11.1963,-11.2119,-11.2119,
     &-11.2119,-11.2119,-11.2119,-11.2119,-11.2119,-11.2119,-11.2119,
     &-11.2119,-10.1834,-10.1857,-10.1880,-11.1941,-11.1952,-11.1959,
     &-11.1886,-11.2119,-11.2119,-11.2119,-11.2119,-11.2119,-11.2119,
     &-11.2119,-11.2119,-11.2119,-11.2119/
      DATA ((SVCS(I,J,36,4),J=1,17),I=1,6)/
     &-10.6947,-10.6953,-10.6959,-11.7079,-11.7108,-11.7114,-11.7119,
     &-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,
     &-11.7119,-11.7119,-11.7119,-10.6937,-10.6944,-10.6955,-11.6964,
     &-11.7079,-11.7109,-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,
     &-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,-10.6916,
     &-10.6931,-10.6946,-11.6962,-11.7039,-11.7079,-11.7119,-11.7119,
     &-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,
     &-11.7119,-11.7119,-10.6896,-10.6916,-10.6936,-11.6959,-11.6962,
     &-11.7040,-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,
     &-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,-10.6876,-10.6901,
     &-10.6926,-11.6955,-11.6959,-11.6962,-11.7119,-11.7119,-11.7119,
     &-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,
     &-11.7119,-10.6856,-10.6886,-10.6916,-11.6951,-11.6956,-11.6884,
     &-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,-11.7119,
     &-11.7119,-11.7119,-11.7119,-11.7119/
      DATA ((SVCS(I,J,37,4),J=1,17),I=1,6)/
     &-11.1955,-11.1958,-11.1961,-12.2099,-12.2114,-12.2116,-12.2119,
     &-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,
     &-12.2119,-12.2119,-12.2119,-11.1945,-11.1952,-11.1959,-12.2040,
     &-12.2099,-12.2114,-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,
     &-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,-11.1934,
     &-11.1944,-11.1954,-12.1963,-12.2079,-12.2099,-12.2119,-12.2119,
     &-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,
     &-12.2119,-12.2119,-11.1926,-11.1936,-11.1946,-12.1961,-12.1963,
     &-12.2079,-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,
     &-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,-11.1918,-11.1928,
     &-11.1938,-12.1959,-12.1962,-12.1963,-12.2119,-12.2119,-12.2119,
     &-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,
     &-12.2119,-11.1910,-11.1920,-11.1930,-12.1957,-12.1961,-12.1847,
     &-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,-12.2119,
     &-12.2119,-12.2119,-12.2119,-12.2119/
      DATA ((SVCS(I,J,38,4),J=1,17),I=1,6)/
     &-11.6960,-11.6962,-11.6964,-12.7109,-12.7116,-12.7119,-12.7119,
     &-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,
     &-12.7119,-12.7119,-12.7119,-11.6952,-11.6957,-11.6962,-12.7079,
     &-12.7109,-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,
     &-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,-11.6945,
     &-11.6951,-11.6957,-12.7040,-12.7099,-12.7119,-12.7119,-12.7119,
     &-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,
     &-12.7119,-12.7119,-11.6937,-11.6945,-11.6953,-12.6964,-12.7040,
     &-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,
     &-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,-11.6929,-11.6939,
     &-11.6949,-12.6962,-12.6964,-12.7119,-12.7119,-12.7119,-12.7119,
     &-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,
     &-12.7119,-11.6921,-11.6933,-11.6945,-12.6960,-12.6888,-12.7119,
     &-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,-12.7119,
     &-12.7119,-12.7119,-12.7119,-12.7119/
      DATA ((SVCS(I,J,39,4),J=1,17),I=1,6)/
     &-12.1886,-12.1963,-12.2040,-13.2114,-13.2119,-13.2119,-13.2119,
     &-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,
     &-13.2119,-13.2119,-13.2119,-12.1959,-12.1961,-12.1963,-13.2099,
     &-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,
     &-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,-12.1953,
     &-12.1957,-12.1961,-13.2079,-13.2119,-13.2119,-13.2119,-13.2119,
     &-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,
     &-13.2119,-13.2119,-12.1948,-12.1953,-12.1958,-13.1964,-13.2119,
     &-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,
     &-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,-12.1943,-12.1949,
     &-12.1955,-13.1963,-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,
     &-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,
     &-13.2119,-12.1938,-12.1945,-12.1952,-13.1962,-13.2119,-13.2119,
     &-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,-13.2119,
     &-13.2119,-13.2119,-13.2119,-13.2119/
      DATA ((SVCS(I,J,40,4),J=1,17),I=1,6)/
     &-12.7001,-12.7040,-12.7079,-13.7116,-13.7119,-13.7119,-13.7119,
     &-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,
     &-13.7119,-13.7119,-13.7119,-12.6884,-12.6962,-12.7040,-13.7109,
     &-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,
     &-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,-12.6954,
     &-12.6959,-12.6964,-13.7099,-13.7119,-13.7119,-13.7119,-13.7119,
     &-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,
     &-13.7119,-13.7119,-12.6950,-12.6956,-12.6962,-13.7040,-13.7119,
     &-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,
     &-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,-12.6946,-12.6953,
     &-12.6960,-13.6964,-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,
     &-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,
     &-13.7119,-12.6942,-12.6950,-12.6958,-13.6888,-13.7119,-13.7119,
     &-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,-13.7119,
     &-13.7119,-13.7119,-13.7119,-13.7119/
      DATA ALPHA0/-3.,-3.,-3.,-3./
      SAVE SVCS
C
      EXP10(X)=EXP(2.30258509299405E0*X)                       
      LINE=M-N                                  
C     TEMPERATURE AND ELECTRON DENSITY INTERPOLATION        
      AT=LOG10(T)                                                              
      BTEMP=(AT-3.3979411)/.3010300+1.                     
      ITEMP=BTEMP                                       
C     ITEMP=(LOG10(T)-LOG10(2500.))/LOG10(2.)+1.                             
      ITEMP=MAX(MIN(ITEMP,5),1)
      WTTEMP=BTEMP-ITEMP                                   
      ANE=LOG10(XNE)                                                           
C     TO AVOID BAD EXTRAPOLATION AT LOW DENSITY
      ANE=MAX(10.D0,ANE)
      BNE=(ANE-10.)/.5+1.                  
      INE=BNE                      
      INE=MAX(MIN(INE,16),1)
      WTXNE=BNE-INE                             
      DO 21 I=1,40                                                
   21 PRALPH(I)=(1.-WTXNE)*(1.-WTTEMP)*SVCS(ITEMP  ,INE  ,I,LINE)+
     1               (1.-WTXNE)*WTTEMP*SVCS(ITEMP+1,INE  ,I,LINE)+           
     2               WTXNE*(1.-WTTEMP)*SVCS(ITEMP  ,INE+1,I,LINE)+
     3                    WTXNE*WTTEMP*SVCS(ITEMP+1,INE+1,I,LINE)       
C     NOW ALPHA INTERPOLATION                                             
      FO=1.25E-9*XNE**.66666667                                    
      DO 50 I=1,II                                         
      IF (DEL(I).NE.0.) GO TO 25                            
      PR(I)=EXP10(PRALPH(1))/FO                               
      GO TO 50                     
   25 AALP=LOG10(ABS(DEL(I))/FO)                                               
      BALP=(AALP-ALPHA0(LINE))/.2+1.          
      IALP=BALP                                       
      IF (IALP.GT.1) GO TO 30                 
      PR(I)=EXP10(PRALPH(1))/FO           
      GO TO 50                                                  
   30 IF (IALP.GT.39) GO TO 40                
      WT=BALP-IALP                        
      PR(I)=EXP10((1.-WT)*PRALPH(IALP)+WT*PRALPH(IALP+1))/FO    
      GO TO 50                                        
   40 PR(I)=EXP10(PRALPH(40)+2.5*(ALPHA0(LINE)+7.8-AALP))/FO      
   50 CONTINUE
      RETURN                       
      END  


